@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -87,6 +88,16 @@ static MagickBooleanType
 %
 %
 */
+
+static inline size_t CheckOverflowException(const Image *image,
+  const size_t extend)
+{
+  if ((extend*image->columns) < (size_t) image->columns)
+    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      image->filename);
+  return(extend*image->columns);
+}
+
 static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
@@ -110,6 +121,9 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ssize_t
     count;
 
+  size_t
+    length;
+
   unsigned char
     *pixels;
 
@@ -123,7 +137,8 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -138,7 +153,7 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   width=ReadBlobMSBLong(image);
   height=ReadBlobMSBLong(image);
-  if ((width == (unsigned long) ~0) || (height == (unsigned long) ~0))
+  if ((width == ~0UL) || (height == ~0UL))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   do
   {
@@ -151,13 +166,14 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    pixels=(unsigned char *) AcquireMagickMemory((size_t) (4*image->columns));
-    if (pixels == (unsigned char *) NULL)
+    length=CheckOverflowException(image,4);
+    pixels=(unsigned char *) AcquireMagickMemory(length);
+    if (pixels == (unsigned char *) NULL) 
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=0; y < (long) image->rows; y++)
     {
-      count=ReadBlob(image,(size_t) (4*image->columns),pixels);
-      if (count != (ssize_t) (4*image->columns))
+      count=ReadBlob(image,length,pixels);
+      if ((size_t) count != length)
         ThrowReaderException(CorruptImageError,"UnableToReadImageData");
       p=pixels;
       q=SetImagePixels(image,0,y,image->columns,1);
@@ -165,10 +181,10 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       for (x=0; x < (long) image->columns; x++)
       {
-        q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum((unsigned long) *p++));
-        q->red=ScaleCharToQuantum((unsigned long) *p++);
-        q->green=ScaleCharToQuantum((unsigned long) *p++);
-        q->blue=ScaleCharToQuantum((unsigned long) *p++);
+        q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
+        q->red=ScaleCharToQuantum(*p++);
+        q->green=ScaleCharToQuantum(*p++);
+        q->blue=ScaleCharToQuantum(*p++);
         if (q->opacity != OpaqueOpacity)
           image->matte=MagickTrue;
         q++;
@@ -200,13 +216,13 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
     width=ReadBlobMSBLong(image);
     height=ReadBlobMSBLong(image);
-    if ((width != (unsigned long) ~0) && (height != (unsigned long) ~0))
+    if ((width != ~0UL) && (height != ~0UL))
       {
         /*
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -220,7 +236,7 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
           }
       }
-  } while ((width != (unsigned long) ~0) && (height != (unsigned long) ~0));
+  } while ((width != ~0UL) && (height != ~0UL));
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -255,8 +271,8 @@ ModuleExport void RegisterAVSImage(void)
   entry=SetMagickInfo("AVS");
   entry->decoder=(DecoderHandler *) ReadAVSImage;
   entry->encoder=(EncoderHandler *) WriteAVSImage;
-  entry->description=AcquireString("AVS X image");
-  entry->module=AcquireString("AVS");
+  entry->description=ConstantString("AVS X image");
+  entry->module=ConstantString("AVS");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -338,7 +354,7 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -348,7 +364,8 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
     /*
       Write AVS header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     (void) WriteBlobMSBLong(image,image->columns);
     (void) WriteBlobMSBLong(image,image->rows);
     /*
@@ -369,8 +386,8 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
       q=pixels;
       for (x=0; x < (long) image->columns; x++)
       {
-        *q++=ScaleQuantumToChar(MaxRGB-(image->matte != MagickFalse ?
-          p->opacity : OpaqueOpacity));
+        *q++=ScaleQuantumToChar((Quantum) (QuantumRange-
+          (image->matte != MagickFalse ? p->opacity : OpaqueOpacity)));
         *q++=ScaleQuantumToChar(p->red);
         *q++=ScaleQuantumToChar(p->green);
         *q++=ScaleQuantumToChar(p->blue);
@@ -388,7 +405,7 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
           }
     }
     pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
