@@ -117,6 +117,9 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   TypeMetric
     metrics;
 
+  unsigned long
+    width;
+
   /*
     Initialize Image structure.
   */
@@ -172,10 +175,17 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     status=GetTypeMetrics(image,draw_info,&metrics);
     if (status == MagickFalse)
       ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
+    width=(unsigned long) (metrics.width+metrics.max_advance+0.5);
     if (*p != '\n')
-      if ((metrics.width+metrics.max_advance/2) < (double) image->columns)
+      if (width < image->columns)
         continue;
-    if ((s != (char *) NULL) && (*s != '\0'))
+    if (s == (char *) NULL)
+      {
+        s=p;
+        while ((isspace((int) ((unsigned char) *s)) == 0) && (*s != '\0'))
+          s++;
+    	}
+    if (*s != '\0')
       {
         *s='\n';
         p=s+1;
@@ -195,7 +205,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   */
   (void) CopyMagickString(draw_info->text,caption,length);
   (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
-    metrics.max_advance/4,metrics.ascent);
+    metrics.max_advance/4.0,metrics.ascent);
   if (draw_info->gravity == UndefinedGravity)
     draw_info->geometry=AcquireString(geometry);
   (void) AnnotateImage(image,draw_info);
