@@ -54,19 +54,87 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/option.h"
 #include "magick/quantize.h"
 #include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
+/*
+  Define declarations.
+*/
+#define MaximumLZWBits  12
+#define MaximumLZWCode  (1UL << MaximumLZWBits)
+
+/*
+  Typdef declarations.
+*/
+typedef struct _LZWCodeInfo
+{
+  unsigned char
+    buffer[280];
+
+  unsigned long
+    count,
+    bit;
+
+  MagickBooleanType
+    eof;
+} LZWCodeInfo;
+
+typedef struct _LZWStack
+{
+  unsigned long
+    *codes,
+    *index,
+    *top;
+} LZWStack;
+
+typedef struct _LZWInfo
+{
+  Image
+    *image;
+
+  LZWStack
+    *stack;
+
+  MagickBooleanType
+    genesis;
+
+  unsigned long
+    data_size,
+    maximum_data_value,
+    clear_code,
+    end_code,
+    bits,
+    first_code,
+    last_code,
+    maximum_code,
+    slot,
+    *table[2];
+
+  LZWCodeInfo
+    code_info;
+} LZWInfo;
+
 /*
   Forward declarations.
 */
-static ssize_t
-  ReadBlobBlock(Image *,unsigned char *);
+static int
+  GetNextLZWCode(LZWInfo *,const unsigned long);
 
 static MagickBooleanType
   WriteGIFImage(const ImageInfo *,Image *);
+
+static ssize_t
+  ReadBlobBlock(Image *,unsigned char *);
+
+static unsigned long
+  PopLZWStack(LZWStack *);
+
+static void
+  PushLZWStack(LZWStack *,const unsigned long),
+  ResetLZWInfo(LZWInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -89,19 +157,226 @@ static MagickBooleanType
 %
 %    o image: The address of a structure of type Image.
 %
-%    o opacity:  The colormap index associated with the transparent
-%      color.
-%
+%    o opacity:  The colormap index associated with the transparent color.
 %
 */
+
+static LZWInfo *RelinquishLZWInfo(LZWInfo *lzw_info)
+{
+  if (lzw_info->table[0] != (unsigned long *) NULL)
+    lzw_info->table[0]=(unsigned long *) RelinquishMagickMemory(
+      lzw_info->table[0]);
+  if (lzw_info->table[1] != (unsigned long *) NULL)
+    lzw_info->table[1]=(unsigned long *) RelinquishMagickMemory(
+      lzw_info->table[1]);
+  if (lzw_info->stack != (LZWStack *) NULL)
+    {
+      if (lzw_info->stack->codes != (unsigned long *) NULL)
+        lzw_info->stack->codes=(unsigned long *) RelinquishMagickMemory(
+          lzw_info->stack->codes);
+      lzw_info->stack=(LZWStack *) RelinquishMagickMemory(lzw_info->stack);
+    }
+  lzw_info=(LZWInfo *) RelinquishMagickMemory(lzw_info);
+  return((LZWInfo *) NULL);
+}
+
+static LZWInfo *AcquireLZWInfo(Image *image,const unsigned long data_size)
+{
+  LZWInfo
+    *lzw_info;
+
+  register long
+    i;
+
+  lzw_info=(LZWInfo *) AcquireMagickMemory(sizeof(*lzw_info));
+  if (lzw_info == (LZWInfo *) NULL)
+    return((LZWInfo *) NULL);
+  (void) ResetMagickMemory(lzw_info,0,sizeof(*lzw_info));
+  lzw_info->image=image;
+  lzw_info->data_size=data_size;
+  lzw_info->maximum_data_value=(1UL << data_size)-1;
+  lzw_info->clear_code=lzw_info->maximum_data_value+1;
+  lzw_info->end_code=lzw_info->maximum_data_value+2;
+  lzw_info->table[0]=(unsigned long *) AcquireMagickMemory(MaximumLZWCode*
+    sizeof(*lzw_info->table));
+  lzw_info->table[1]=(unsigned long *) AcquireMagickMemory(MaximumLZWCode*
+    sizeof(*lzw_info->table));
+  if ((lzw_info->table[0] == (unsigned long *) NULL) ||
+      (lzw_info->table[1] == (unsigned long *) NULL))
+    {
+      lzw_info=RelinquishLZWInfo(lzw_info);
+      return((LZWInfo *) NULL);
+    }
+  for (i=0; i <= (long) lzw_info->maximum_data_value; i++)
+  {
+    lzw_info->table[0][i]=0;
+    lzw_info->table[1][i]=i;
+  }
+  ResetLZWInfo(lzw_info);
+  lzw_info->code_info.buffer[0]='\0';
+  lzw_info->code_info.buffer[1]='\0';
+  lzw_info->code_info.count=2;
+  lzw_info->code_info.bit=8*lzw_info->code_info.count;
+  lzw_info->code_info.eof=MagickFalse;
+  lzw_info->genesis=MagickTrue;
+  lzw_info->stack=(LZWStack *) AcquireMagickMemory(sizeof(*lzw_info->stack));
+  if (lzw_info->stack == (LZWStack *) NULL)
+    {
+      lzw_info=RelinquishLZWInfo(lzw_info);
+      return((LZWInfo *) NULL);
+    }
+  lzw_info->stack->codes=(unsigned long *) AcquireMagickMemory(2*MaximumLZWCode*
+    sizeof(*lzw_info->stack->codes));
+  if (lzw_info->stack->codes == (unsigned long *) NULL)
+    {
+      lzw_info=RelinquishLZWInfo(lzw_info);
+      return((LZWInfo *) NULL);
+    }
+  lzw_info->stack->index=lzw_info->stack->codes;
+  lzw_info->stack->top=lzw_info->stack->codes+2*MaximumLZWCode;
+  return(lzw_info);
+}
+
+static int GetNextLZWCode(LZWInfo *lzw_info,const unsigned long bits)
+{
+  register long
+    i;
+
+  unsigned long
+    code;
+
+  if (((lzw_info->code_info.bit+bits) > (8*lzw_info->code_info.count)) &&
+      (lzw_info->code_info.eof == MagickFalse))
+    {
+      ssize_t
+        count;
+
+      lzw_info->code_info.buffer[0]=lzw_info->code_info.buffer[
+        lzw_info->code_info.count-2];
+      lzw_info->code_info.buffer[1]=lzw_info->code_info.buffer[
+        lzw_info->code_info.count-1];
+      lzw_info->code_info.bit-=8*(lzw_info->code_info.count-2);
+      lzw_info->code_info.count=2;
+      count=ReadBlobBlock(lzw_info->image,&lzw_info->code_info.buffer[
+        lzw_info->code_info.count]);
+      if (count >= 0)
+        lzw_info->code_info.count+=count;
+      else
+        lzw_info->code_info.eof=MagickTrue;
+    }
+  if ((lzw_info->code_info.bit+bits) > (8*lzw_info->code_info.count))
+    return(-1);
+  code=0;
+  for (i=0; i < (long) bits; i++)
+  {
+    code|=((lzw_info->code_info.buffer[lzw_info->code_info.bit/8] &
+      (1UL << (lzw_info->code_info.bit % 8))) != 0) << i;
+    lzw_info->code_info.bit++;
+  }
+  return(code);
+}
+
+static unsigned long PopLZWStack(LZWStack *stack_info)
+{
+  if (stack_info->index <= stack_info->codes)
+    return(-1);
+  stack_info->index--;
+  return(*stack_info->index);
+}
+
+static void PushLZWStack(LZWStack *stack_info,const unsigned long value)
+{
+  if (stack_info->index >= stack_info->top)
+    return;
+  *stack_info->index=value;
+  stack_info->index++;
+}
+
+static int ReadBlobLZWByte(LZWInfo *lzw_info)
+{
+  int
+    code;
+
+  ssize_t
+    count;
+
+  unsigned long
+    value;
+
+  if (lzw_info->stack->index != lzw_info->stack->codes)
+    return(PopLZWStack(lzw_info->stack));
+  if (lzw_info->genesis != MagickFalse)
+    {
+      lzw_info->genesis=MagickFalse;
+      do
+      {
+        lzw_info->first_code=GetNextLZWCode(lzw_info,lzw_info->bits);
+        lzw_info->last_code=lzw_info->first_code;
+      } while (lzw_info->first_code == lzw_info->clear_code);
+      return(lzw_info->first_code);
+    }
+  code=GetNextLZWCode(lzw_info,lzw_info->bits);
+  if (code < 0)
+    return(code);
+  if ((unsigned long) code == lzw_info->clear_code)
+    {
+      ResetLZWInfo(lzw_info);
+      return(ReadBlobLZWByte(lzw_info));
+    }
+  if ((unsigned long) code == lzw_info->end_code)
+    return(-1);
+  if ((unsigned long) code < lzw_info->slot)
+    value=code;
+  else
+    {
+      PushLZWStack(lzw_info->stack,lzw_info->first_code);
+      value=lzw_info->last_code;
+    }
+  count=0;
+  while (value > lzw_info->maximum_data_value)
+  {
+    if ((size_t) count > MaximumLZWCode)
+      return(-1);
+    count++;
+    if ((size_t) value > MaximumLZWCode)
+      return(-1);
+    PushLZWStack(lzw_info->stack,lzw_info->table[1][value]);
+    value=lzw_info->table[0][value];
+  }
+  lzw_info->first_code=lzw_info->table[1][value];
+  PushLZWStack(lzw_info->stack,lzw_info->first_code);
+  if (lzw_info->slot < MaximumLZWCode)
+    {
+      lzw_info->table[0][lzw_info->slot]=lzw_info->last_code;
+      lzw_info->table[1][lzw_info->slot]=lzw_info->first_code;
+      lzw_info->slot++;
+      if ((lzw_info->slot >= lzw_info->maximum_code) &&
+          (lzw_info->bits < MaximumLZWBits))
+        {
+          lzw_info->bits++;
+          lzw_info->maximum_code=1UL << lzw_info->bits;
+        }
+    }
+  lzw_info->last_code=code;
+  return(PopLZWStack(lzw_info->stack));
+}
+
+static void ResetLZWInfo(LZWInfo *lzw_info)
+{
+  lzw_info->bits=lzw_info->data_size+1;
+  lzw_info->maximum_code=1UL << lzw_info->bits;
+  lzw_info->slot=lzw_info->maximum_data_value+3;
+  lzw_info->genesis=MagickTrue;
+}
+
 static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
 {
-#define MaxStackSize  4096
-#define NullCode  (~0UL)
-
   IndexPacket
     index;
 
+  int
+    c;
+
   long
     offset,
     y;
@@ -118,36 +393,13 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
   register PixelPacket
     *q;
 
-  register unsigned char
-    *c;
-
-  register unsigned long
-    datum;
-
-  short
-    *prefix;
-
-  ssize_t
-    count;
+  LZWInfo
+    *lzw_info;
 
   unsigned char
-    *packet,
-    *pixel_stack,
-    *suffix,
-    *top_stack;
+    data_size;
 
   unsigned long
-    available,
-    bits,
-    clear,
-    code,
-    code_mask,
-    code_size,
-    data_size,
-    first,
-    end_of_information,
-    in_code,
-    old_code,
     pass;
 
   /*
@@ -157,158 +409,37 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  packet=(unsigned char *) AcquireMagickMemory(256*sizeof(*packet));
-  prefix=(short *) AcquireMagickMemory(MaxStackSize*sizeof(*prefix));
-  suffix=(unsigned char *) AcquireMagickMemory(MaxStackSize*sizeof(*suffix));
-  pixel_stack=(unsigned char *) AcquireMagickMemory((MaxStackSize+1)*
-    sizeof(*pixel_stack));
-  if ((packet == (unsigned char *) NULL) || (prefix == (short *) NULL) ||
-      (suffix == (unsigned char *) NULL) ||
-      (pixel_stack == (unsigned char *) NULL))
-    {
-      if (packet != (unsigned char *) NULL)
-        packet=(unsigned char *) RelinquishMagickMemory(packet);
-      if (prefix != (short *) NULL)
-        prefix=(short *) RelinquishMagickMemory(prefix);
-      if (suffix != (unsigned char *) NULL)
-        suffix=(unsigned char *) RelinquishMagickMemory(suffix);
-      if (pixel_stack != (unsigned char *) NULL)
-        pixel_stack=(unsigned char *) RelinquishMagickMemory(pixel_stack);
-      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-        image->filename);
-    }
-  /*
-    Initialize GIF data stream decoder.
-  */
-  data_size=(unsigned long) ReadBlobByte(image);
-  if (data_size > 8)
+  data_size=(unsigned char) ReadBlobByte(image);
+  if (data_size > MaximumLZWBits)
     ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
-  clear=1UL << data_size;
-  end_of_information=clear+1;
-  available=clear+2;
-  old_code=NullCode;
-  code_size=data_size+1;
-  code_mask=(1 << code_size)-1;
-  for (code=0; code < clear; code++)
-  {
-    prefix[code]=0;
-    suffix[code]=(unsigned char) code;
-  }
-  /*
-    Decode GIF pixel stream.
-  */
-  datum=0;
-  bits=0;
-  c=0;
-  count=0;
-  first=0;
-  offset=0;
+  lzw_info=AcquireLZWInfo(image,data_size);
+  if (lzw_info == (LZWInfo *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
   pass=0;
-  top_stack=pixel_stack;
+  offset=0;
   for (y=0; y < (long) image->rows; y++)
   {
-    q=SetImagePixels(image,0,offset,image->columns,1);
+    q=GetImagePixels(image,0,offset,image->columns,1);
     if (q == (PixelPacket *) NULL)
       break;
     indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; )
     {
-      if (top_stack == pixel_stack)
-        {
-          if (bits < code_size)
-            {
-              /*
-                Load bytes until there is enough bits for a code.
-              */
-              if (count == 0)
-                {
-                  /*
-                    Read a new data block.
-                  */
-                  count=ReadBlobBlock(image,packet);
-                  if (count == 0)
-                    break;
-                  c=packet;
-                }
-              datum+=(unsigned long) (*c) << bits;
-              bits+=8;
-              c++;
-              count--;
-              continue;
-            }
-          /*
-            Get the next code.
-          */
-          code=datum & code_mask;
-          datum>>=code_size;
-          bits-=code_size;
-          /*
-            Interpret the code
-          */
-          if ((code > available) || (code == end_of_information))
-            break;
-          if (code == clear)
-            {
-              /*
-                Reset decoder.
-              */
-              code_size=data_size+1;
-              code_mask=(1 << code_size)-1;
-              available=clear+2;
-              old_code=NullCode;
-              continue;
-            }
-          if (old_code == NullCode)
-            {
-              *top_stack++=suffix[code];
-              old_code=code;
-              first=code;
-              continue;
-            }
-          in_code=code;
-          if (code >= available)
-            {
-              *top_stack++=(unsigned char) first;
-              code=old_code;
-            }
-          while (code >= clear)
-          {
-            if ((top_stack-pixel_stack) >= MaxStackSize)
-              break;
-            *top_stack++=suffix[code];
-            code=(unsigned long) prefix[code];
-          }
-          first=(unsigned long) suffix[code];
-          /*
-            Add a new string to the string table,
-          */
-          if ((top_stack-pixel_stack) >= MaxStackSize)
-            break;
-          if (available >= MaxStackSize)
-            break;
-          *top_stack++=(unsigned char) first;
-          prefix[available]=(short) old_code;
-          suffix[available]=(unsigned char) first;
-          available++;
-          if (((available & code_mask) == 0) && (available < MaxStackSize))
-            {
-              code_size++;
-              code_mask+=available;
-            }
-          old_code=in_code;
-        }
-      /*
-        Pop a pixel off the pixel stack.
-      */
-      top_stack--;
-      index=ConstrainColormapIndex(image,*top_stack);
+      c=ReadBlobLZWByte(lzw_info);
+      if (c < 0)
+        break;
+      index=ConstrainColormapIndex(image,c);
+      q->red=image->colormap[index].red;
+      q->green=image->colormap[index].green;
+      q->blue=image->colormap[index].blue;
+      q->opacity=index == opacity ? TransparentOpacity : OpaqueOpacity;
       indexes[x]=index;
-      *q=image->colormap[index];
-      q->opacity=(Quantum) (index == opacity ? TransparentOpacity :
-        OpaqueOpacity);
       x++;
       q++;
     }
+    if (x < (long) image->columns)
+      break;
     if (image->interlace == NoInterlace)
       offset++;
     else
@@ -353,8 +484,6 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
       }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (x < (long) image->columns)
-      break;
     if (image->previous == (Image *) NULL)
       if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
           (QuantumTick(y,image->rows) != MagickFalse))
@@ -365,10 +494,7 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
             break;
         }
   }
-  pixel_stack=(unsigned char *) RelinquishMagickMemory(pixel_stack);
-  suffix=(unsigned char *) RelinquishMagickMemory(suffix);
-  prefix=(short *) RelinquishMagickMemory(prefix);
-  packet=(unsigned char *) RelinquishMagickMemory(packet);
+  lzw_info=RelinquishLZWInfo(lzw_info);
   if (y < (long) image->rows)
     ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
   return(MagickTrue);
@@ -517,7 +643,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   */
   number_bits=data_size;
   max_code=MaxCode(number_bits);
-  clear_code=((short) 1 << (data_size-1));
+  clear_code=((short) 1UL << (data_size-1));
   end_of_information_code=clear_code+1;
   free_code=clear_code+2;
   length=0;
@@ -547,7 +673,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
       */
       index=indexes[x] & 0xff;
       p++;
-      k=(int) (index << (MaxGIFBits-8))+waiting_code;
+      k=(long) (index << (MaxGIFBits-8))+waiting_code;
       if (k >= MaxHashTable)
         k-=MaxHashTable;
       next_pixel=MagickFalse;
@@ -870,7 +996,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   flag=(unsigned char) ReadBlobByte(image);
   background=(unsigned char) ReadBlobByte(image);
   c=(unsigned char) ReadBlobByte(image);  /* reserved */
-  global_colors=1 << (((unsigned long) flag & 0x07)+1);
+  global_colors=1UL << (((unsigned long) flag & 0x07)+1);
   global_colormap=(unsigned char *) AcquireMagickMemory((size_t)
     (3*Max(global_colors,256))*sizeof(*global_colormap));
   if (global_colormap == (unsigned char *) NULL)
@@ -897,8 +1023,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         count=ReadBlob(image,1,&c);
         if (count != 1)
           {
-            global_colormap=(unsigned char *)
-              RelinquishMagickMemory(global_colormap);
+            global_colormap=(unsigned char *) RelinquishMagickMemory(
+              global_colormap);
             ThrowReaderException(CorruptImageError,
               "UnableToReadExtensionBlock");
           }
@@ -912,7 +1038,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             while (ReadBlobBlock(image,header) != 0);
             dispose=(unsigned long) (header[0] >> 2);
             delay=(unsigned long) ((header[2] << 8) | header[1]);
-            if ((int) (header[0] & 0x01) == 0x01)
+            if ((long) (header[0] & 0x01) == 0x01)
               opacity=(long) header[3];
             break;
           }
@@ -972,8 +1098,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
-            global_colormap=(unsigned char *)
-              RelinquishMagickMemory(global_colormap);
+            global_colormap=(unsigned char *) RelinquishMagickMemory(
+              global_colormap);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
@@ -1002,7 +1128,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->colors=BitSet((int) flag,0x80) == 0 ? global_colors :
       1UL << ((unsigned long) (flag & 0x07)+1);
     if (opacity >= (long) image->colors)
-      image->colors=(unsigned long) opacity+1;
+      opacity=(-1);
     image->page.width=page.width;
     image->page.height=page.height;
     image->page.y=page.y;
@@ -1017,8 +1143,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     iterations=1;
     if ((image->columns == 0) || (image->rows == 0))
       {
-        global_colormap=(unsigned char *)
-          RelinquishMagickMemory(global_colormap);
+        global_colormap=(unsigned char *) RelinquishMagickMemory(
+          global_colormap);
         ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
       }
     /*
@@ -1026,8 +1152,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     if (AllocateImageColormap(image,image->colors) == MagickFalse)
       {
-        global_colormap=(unsigned char *)
-          RelinquishMagickMemory(global_colormap);
+        global_colormap=(unsigned char *) RelinquishMagickMemory(
+          global_colormap);
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
     if (BitSet((int) flag,0x80) == 0)
@@ -1042,10 +1168,13 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->colormap[i].green=ScaleCharToQuantum(*p++);
           image->colormap[i].blue=ScaleCharToQuantum(*p++);
           if (i == opacity)
-            image->colormap[i].opacity=TransparentOpacity;
+            {
+              image->colormap[i].opacity=TransparentOpacity;
+              image->transparent_color=image->colormap[opacity];
+            }
         }
-        image->background_color=
-          image->colormap[Min((unsigned long) background,image->colors-1)];
+        image->background_color=image->colormap[Min((unsigned long) background,
+         image->colors-1)];
       }
     else
       {
@@ -1055,19 +1184,19 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read local colormap.
         */
-        colormap=(unsigned char *)
-          AcquireMagickMemory((3*image->colors)*sizeof(*colormap));
+        colormap=(unsigned char *) AcquireMagickMemory((3*image->colors)*
+          sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
           {
-            global_colormap=(unsigned char *)
-              RelinquishMagickMemory(global_colormap);
+            global_colormap=(unsigned char *) RelinquishMagickMemory(
+              global_colormap);
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         count=ReadBlob(image,(3*image->colors)*sizeof(*colormap),colormap);
         if (count != (ssize_t) (3*image->colors))
           {
-            global_colormap=(unsigned char *)
-              RelinquishMagickMemory(global_colormap);
+            global_colormap=(unsigned char *) RelinquishMagickMemory(
+              global_colormap);
             colormap=(unsigned char *) RelinquishMagickMemory(colormap);
             ThrowReaderException(CorruptImageError,
               "InsufficientImageDataInFile");
@@ -1092,8 +1221,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status=DecodeImage(image,(Quantum) opacity);
     if ((image_info->ping == MagickFalse) && (status == MagickFalse))
       {
-        global_colormap=(unsigned char *)
-          RelinquishMagickMemory(global_colormap);
+        global_colormap=(unsigned char *) RelinquishMagickMemory(
+          global_colormap);
         ThrowReaderException(CorruptImageError,"CorruptImage");
       }
     if (image_info->number_scenes != 0)
@@ -1215,8 +1344,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
 
   long
     j,
-    opacity,
-    y;
+    opacity;
 
   ImageInfo
     *write_info;
@@ -1230,21 +1358,9 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   MagickOffsetType
     scene;
 
-  QuantizeInfo
-    quantize_info;
-
   RectangleInfo
     page;
 
-  register IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
-  register long
-    x;
-
   register long
     i;
 
@@ -1277,8 +1393,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   /*
     Allocate colormap.
   */
-  global_colormap=(unsigned char *)
-    AcquireMagickMemory(768*sizeof(*global_colormap));
+  global_colormap=(unsigned char *) AcquireMagickMemory(768*
+    sizeof(*global_colormap));
   colormap=(unsigned char *) AcquireMagickMemory(768*sizeof(*colormap));
   if ((global_colormap == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
@@ -1333,107 +1449,63 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     if (image_info->colorspace == UndefinedColorspace)
       (void) SetImageColorspace(image,RGBColorspace);
     opacity=(-1);
-    if ((image->storage_class == PseudoClass) &&
-        (image->matte != MagickFalse) && (image->colors <= 256))
+    if (image->matte == MagickFalse)
       {
-        /*
-          Identify transparent pixel index.
-        */
-        for (y=0; y < (long) image->rows; y++)
-        {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-          if (p == (const PixelPacket *) NULL)
-            break;
-          indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
-          {
-            if (p->opacity != OpaqueOpacity)
-              {
-                opacity=(long) indexes[x];
-                break;
-              }
-            p++;
-          }
-          if (x < (long) image->columns)
-            break;
-        }
-        for ( ; y < (long) image->rows; y++)
-        {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-          if (p == (const PixelPacket *) NULL)
-            break;
-          indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
-          {
-            if ((p->opacity != OpaqueOpacity) && (opacity != (long) indexes[x]))
-              {
-                opacity=(-1);
-                break;
-              }
-            p++;
-          }
-          if (x < (long) image->columns)
-            break;
-        }
+        if ((image->storage_class == DirectClass) || (image->colors > 256))
+          (void) SetImageType(image,PaletteType);
       }
-    if ((image->storage_class == DirectClass) || (image->colors > 256))
+    else
       {
-        /*
-          GIF requires that the image is colormapped.
-        */
-        GetQuantizeInfo(&quantize_info);
-        quantize_info.number_colors=256;
-        quantize_info.dither=IsPaletteImage(image,&image->exception) ==
-          MagickFalse ? MagickTrue : MagickFalse;
-        (void) QuantizeImage(&quantize_info,image);
-      }
-    if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
-        (image->matte != MagickFalse) && (opacity == -1))
-      {
-        size_t
-          length;
+        MagickRealType
+          alpha,
+          beta;
 
         /*
-          Set transparent pixel.
+          Identify transparent colormap index.
         */
-        GetQuantizeInfo(&quantize_info);
-        quantize_info.number_colors=255;
-        quantize_info.dither=IsPaletteImage(image,&image->exception) ==
-          MagickFalse ? MagickTrue : MagickFalse;
-        (void) QuantizeImage(&quantize_info,image);
-        opacity=(long) image->colors++;
-        length=(size_t) image->colors*sizeof(*image->colormap);
-        image->colormap=(PixelPacket *)
-          ResizeMagickMemory(image->colormap,length);
-        if (image->colormap == (PixelPacket *) NULL)
+        if ((image->storage_class == DirectClass) || (image->colors > 256))
+          (void) SetImageType(image,PaletteBilevelMatteType);
+        for (i=0; i < (long) image->colors; i++)
+          if (image->colormap[i].opacity != OpaqueOpacity)
+            {
+              if (opacity < 0)
+                {
+                  opacity=i;
+                  continue;
+                }
+              alpha=TransparentOpacity-(MagickRealType)
+                image->colormap[i].opacity;
+              beta=TransparentOpacity-(MagickRealType)
+                image->colormap[opacity].opacity;
+              if (alpha < beta)
+                opacity=i;
+            }
+        if (opacity == -1)
           {
-            global_colormap=(unsigned char *)
-              RelinquishMagickMemory(global_colormap);
-            colormap=(unsigned char *) RelinquishMagickMemory(colormap);
-            write_info=DestroyImageInfo(write_info);
-            ThrowWriterException(ResourceLimitError,
-              "MemoryAllocationFailed");
+            (void) SetImageType(image,PaletteBilevelMatteType);
+            for (i=0; i < (long) image->colors; i++)
+              if (image->colormap[i].opacity != OpaqueOpacity)
+                {
+                  if (opacity < 0)
+                    {
+                      opacity=i;
+                      continue;
+                    }
+                  alpha=TransparentOpacity-(MagickRealType)
+                    image->colormap[i].opacity;
+                  beta=TransparentOpacity-(MagickRealType)
+                    image->colormap[opacity].opacity;
+                  if (alpha < beta)
+                    opacity=i;
+                }
           }
-        (void) QueryColorDatabase("#abacadab",image->colormap+opacity,
-          &image->exception);
-        for (y=0; y < (long) image->rows; y++)
-        {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-          if (p == (const PixelPacket *) NULL)
-            break;
-          indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
+        if (opacity >= 0)
           {
-            if (p->opacity != OpaqueOpacity)
-              indexes[x]=(IndexPacket) opacity;
-            p++;
+            image->colormap[opacity].red=image->transparent_color.red;
+            image->colormap[opacity].green=image->transparent_color.green;
+            image->colormap[opacity].blue=image->transparent_color.blue;
           }
-          if (SyncImagePixels(image) == MagickFalse)
-            break;
-        }
       }
-    if (image->colormap == (PixelPacket *) NULL)
-      break;
     for (bits_per_pixel=1; bits_per_pixel < 8; bits_per_pixel++)
       if ((1UL << bits_per_pixel) >= image->colors)
         break;
@@ -1461,12 +1533,12 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         c|=(bits_per_pixel-1);   /* size of global colormap */
         (void) WriteBlobByte(image,(unsigned char) c);
         for (j=0; j < (long) image->colors; j++)
-          if (ColorMatch(&image->background_color,image->colormap+j))
+          if (IsColorEqual(&image->background_color,image->colormap+j))
             break;
         (void) WriteBlobByte(image,(unsigned char)
           (j == (long) image->colors ? 0 : j));  /* background color */
         (void) WriteBlobByte(image,(unsigned char) 0x00);  /* reserved */
-        length=(size_t) (3*(1 << bits_per_pixel));
+        length=(size_t) (3*(1UL << bits_per_pixel));
         (void) WriteBlob(image,length,colormap);
         for (j=0; j < 768; j++)
           global_colormap[j]=colormap[j];
@@ -1485,8 +1557,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobByte(image,(unsigned char) c);
         delay=100*image->delay/Max(image->ticks_per_second,1);
         (void) WriteBlobLSBShort(image,(unsigned short) delay);
-        (void) WriteBlobByte(image,(unsigned char)
-          (opacity >= 0 ? opacity : 0));
+        (void) WriteBlobByte(image,(unsigned char) (opacity >= 0 ? opacity :
+          0));
         (void) WriteBlobByte(image,(unsigned char) 0x00);
         if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
             (GetImageAttribute(image,"Comment") != (ImageAttribute *) NULL))
@@ -1569,8 +1641,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     status=EncodeImage(write_info,image,Max(bits_per_pixel,2)+1);
     if (status == MagickFalse)
       {
-        global_colormap=(unsigned char *)
-          RelinquishMagickMemory(global_colormap);
+        global_colormap=(unsigned char *) RelinquishMagickMemory(
+          global_colormap);
         colormap=(unsigned char *) RelinquishMagickMemory(colormap);
         write_info=DestroyImageInfo(write_info);
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
