@@ -48,6 +48,7 @@
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/list.h"
+#include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -207,9 +208,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -259,6 +258,22 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     image->columns=icon_info.width;
     image->rows=icon_info.height;
     image->depth=8;
+    if (image->debug != MagickFalse)
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule()," scene    = %ld",i);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"   size   = %ld",
+        icon_info.size);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"   width  = %d",
+        icon_file.directory[i].width);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"   height = %d",
+        icon_file.directory[i].height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"   colors = %ld",
+        icon_info.number_colors);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"   planes = %d",
+        icon_info.planes);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"   bpp    = %d",
+        icon_info.bits_per_pixel);
+    }
     if ((icon_info.number_colors != 0) || (icon_info.bits_per_pixel < 16))
       {
         image->storage_class=PseudoClass;
@@ -464,7 +479,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             q->green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             q->red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             if (image->matte != MagickFalse)
-              q->opacity=ScaleCharToQuantum((unsigned char)
+              q->opacity=QuantumRange-ScaleCharToQuantum((unsigned char)
                 ReadBlobByte(image));
             q++;
           }
@@ -714,8 +729,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -733,7 +747,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
   */
   (void) WriteBlobLSBShort(image,0);
   (void) WriteBlobLSBShort(image,1);
-  (void) WriteBlobLSBShort(image,scene);
+  (void) WriteBlobLSBShort(image,(unsigned char) scene);
   (void) ResetMagickMemory(&icon_info,0,sizeof(icon_info));
   scene=0;
   next=image;
