@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
@@ -54,6 +55,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/nt-feature.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #if defined(HasJBIG)
@@ -95,13 +97,10 @@ static MagickBooleanType
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadJBIGImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define MaxBufferSize  8192
-
   Image
     *image;
 
@@ -144,7 +143,8 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -168,13 +168,14 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   /*
     Read JBIG file.
   */
-  buffer=(unsigned char *) AcquireMagickMemory(MaxBufferSize);
+  buffer=(unsigned char *) AcquireMagickMemory(MagickMaxBufferSize*
+    sizeof(buffer));
   if (buffer == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   status=JBG_EAGAIN;
   do
   {
-    length=(long) ReadBlob(image,MaxBufferSize,buffer);
+    length=(long) ReadBlob(image,MagickMaxBufferSize,buffer);
     if (length == 0)
       break;
     p=buffer;
@@ -202,9 +203,9 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   image->colormap[0].red=0;
   image->colormap[0].green=0;
   image->colormap[0].blue=0;
-  image->colormap[1].red=MaxRGB;
-  image->colormap[1].green=MaxRGB;
-  image->colormap[1].blue=MaxRGB;
+  image->colormap[1].red=QuantumRange;
+  image->colormap[1].green=QuantumRange;
+  image->colormap[1].blue=QuantumRange;
   image->x_resolution=300;
   image->y_resolution=300;
   if (image_info->ping != MagickFalse)
@@ -300,30 +301,30 @@ ModuleExport void RegisterJBIGImage(void)
   entry->encoder=(EncoderHandler *) WriteJBIGImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString(JBIGDescription);
+  entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBG");
 #if defined(HasJBIG)
   entry->decoder=(DecoderHandler *) ReadJBIGImage;
   entry->encoder=(EncoderHandler *) WriteJBIGImage;
 #endif
-  entry->description=AcquireString(JBIGDescription);
+  entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBIG");
 #if defined(HasJBIG)
   entry->decoder=(DecoderHandler *) ReadJBIGImage;
   entry->encoder=(EncoderHandler *) WriteJBIGImage;
 #endif
-  entry->description=AcquireString(JBIGDescription);
+  entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("JBIG");
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -395,6 +396,9 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   double
     version;
 
+  IndexPacket
+    polarity;
+
   long
     y;
 
@@ -422,8 +426,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   unsigned char
     bit,
     byte,
-    *pixels,
-    polarity;
+    *pixels;
 
   unsigned long
     number_packets;
@@ -436,7 +439,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -447,7 +450,8 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
     /*
       Allocate pixel data.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     number_packets=((image->columns+7) >> 3)*image->rows;
     pixels=(unsigned char *) AcquireMagickMemory(number_packets);
     if (pixels == (unsigned char *) NULL)
@@ -455,11 +459,13 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
     /*
       Convert pixels to a bitmap.
     */
-    SetImageType(image,BilevelType);
-    polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+    (void) SetImageType(image,PaletteType);
+    (void) SetImageType(image,BilevelType);
+    polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
+      (QuantumRange/2) ? 1 : 0);
     if (image->colors == 2)
-      polarity=PixelIntensityToQuantum(&image->colormap[0]) >
-        PixelIntensityToQuantum(&image->colormap[1]);
+      polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) >
+        PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
     q=pixels;
     for (y=0; y < (long) image->rows; y++)
     {
@@ -542,7 +548,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
     jbg_enc_out(&jbig_info);
     jbg_enc_free(&jbig_info);
     pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
