@@ -17,7 +17,7 @@
 %                                 June 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,7 @@
 #include "magick/monitor.h"
 #include "magick/option.h"
 #include "magick/profile.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -111,7 +112,6 @@ static MagickBooleanType
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsJP2(const unsigned char *magick,const size_t length)
 {
@@ -147,7 +147,6 @@ static MagickBooleanType IsJP2(const unsigned char *magick,const size_t length)
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsJPC(const unsigned char *magick,const size_t length)
 {
@@ -204,8 +203,11 @@ static int BlobRead(jas_stream_obj_t *object,char *buffer,const int length)
   ssize_t
     count;
 
-  count=ReadBlob(((StreamManager *) object)->image,(size_t) length,
-    (void *) buffer);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  count=ReadBlob(source->image,(size_t) length,(void *) buffer);
   return((int) count);
 }
 
@@ -214,19 +216,30 @@ static int BlobWrite(jas_stream_obj_t *object,char *buffer,const int length)
   size_t
     count;
 
-  count=WriteBlob(((StreamManager *) object)->image,(size_t) length,
-    (void *) buffer);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  count=WriteBlob(source->image,(size_t) length,(void *) buffer);
   return((int) count);
 }
 
 static long BlobSeek(jas_stream_obj_t *object,long offset,int origin)
 {
-  return(SeekBlob(((StreamManager *) object)->image,offset,origin));
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  return(SeekBlob(source->image,offset,origin));
 }
 
 static int BlobClose(jas_stream_obj_t *object)
 {
-  CloseBlob(((StreamManager *) object)->image);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  CloseBlob(source->image);
   return(0);
 }
 
@@ -319,7 +332,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -332,7 +346,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize JPEG 2000 API.
   */
-  jas_init();
   jp2_stream=JP2StreamManager(image);
   if (jp2_stream == (jas_stream_t *) NULL)
     ThrowReaderException(DelegateError,"UnableToManageJP2Stream");
@@ -408,10 +421,10 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   image->rows=jas_image_height(jp2_image);
   for (i=0; i < (long) number_components; i++)
   {
-    if (((jas_image_cmptwidth(jp2_image,components[i]) *
-          jas_image_cmpthstep(jp2_image,components[i])) != image->columns) ||
-        ((jas_image_cmptheight(jp2_image,components[i]) *
-          jas_image_cmptvstep(jp2_image,components[i])) != image->rows) ||
+    if ((((jas_image_cmptwidth(jp2_image,components[i])*
+          jas_image_cmpthstep(jp2_image,components[i])) != (long) image->columns)) ||
+        (((jas_image_cmptheight(jp2_image,components[i])*
+          jas_image_cmptvstep(jp2_image,components[i])) != (long) image->rows)) ||
         (jas_image_cmpttlx(jp2_image,components[i]) != 0) ||
         (jas_image_cmpttly(jp2_image,components[i]) != 0) ||
         (jas_image_cmptsgnd(jp2_image,components[i]) != MagickFalse))
@@ -437,7 +450,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   for (i=0; i < (long) number_components; i++)
   {
     maximum_component_depth=Max(jas_image_cmptprec(jp2_image,components[i]),
-      maximum_component_depth);
+      (long) maximum_component_depth);
     pixels[i]=jas_matrix_create(1,((unsigned int) image->columns)/x_step[i]);
     if (pixels[i] == (jas_matrix_t *) NULL)
       {
@@ -453,7 +466,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   {
     scale[i]=1;
     if (jas_image_cmptprec(jp2_image,components[i]) < 16)
-      scale[i]=(1 << (16-jas_image_cmptprec(jp2_image,components[i])))+1;
+      scale[i]=(1 << (16-jas_image_cmptprec(jp2_image,components[i])));
   }
   if (number_components == 1)
     {
@@ -535,8 +548,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
             jas_matrix_getv(pixels[1],x/x_step[1])));
           q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
             jas_matrix_getv(pixels[2],x/x_step[2])));
-          q->opacity=MaxRGB-ScaleShortToQuantum((unsigned short) (scale[3]*
-            jas_matrix_getv(pixels[3],x/x_step[3])));
+          q->opacity=QuantumRange-ScaleShortToQuantum((unsigned short)
+            (scale[3]*jas_matrix_getv(pixels[3],x/x_step[3])));
           q++;
         }
         break;
@@ -568,13 +581,13 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
           (jas_iccprof_save(icc_profile,icc_stream) == 0) &&
           (jas_stream_flush(icc_stream) == 0))
         {
-          jas_stream_memobj_t
-            *blob;
-
           StringInfo
             *icc_profile,
             *profile;
 
+          jas_stream_memobj_t
+            *blob;
+
           /*
             Extract the icc profile, handle errors without much noise.
           */
@@ -584,7 +597,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
               "Profile: ICC, %lu bytes",(unsigned long) blob->len_);
           profile=AcquireStringInfo(blob->len_);
           SetStringInfoDatum(profile,blob->buf_);
-          icc_profile=GetImageProfile(image,"icc");
+          icc_profile=(StringInfo *) GetImageProfile(image,"icc");
           if (icc_profile == (StringInfo *) NULL)
             SetImageProfile(image,"icc",profile);
           else
@@ -629,8 +642,8 @@ ModuleExport void RegisterJP2Image(void)
     *entry;
 
   entry=SetMagickInfo("JP2");
-  entry->description=AcquireString("JPEG-2000 File Format Syntax");
-  entry->module=AcquireString("JP2");
+  entry->description=ConstantString("JPEG-2000 File Format Syntax");
+  entry->module=ConstantString("JP2");
   entry->magick=(MagickHandler *) IsJP2;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
@@ -641,8 +654,8 @@ ModuleExport void RegisterJP2Image(void)
 #endif
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JPC");
-  entry->description=AcquireString("JPEG-2000 Code Stream Syntax");
-  entry->module=AcquireString("JPC");
+  entry->description=ConstantString("JPEG-2000 Code Stream Syntax");
+  entry->module=ConstantString("JPC");
   entry->magick=(MagickHandler *) IsJPC;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
@@ -653,8 +666,8 @@ ModuleExport void RegisterJP2Image(void)
 #endif
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JPX");
-  entry->description=AcquireString("JPEG-2000 File Format Syntax");
-  entry->module=AcquireString("JPX");
+  entry->description=ConstantString("JPEG-2000 File Format Syntax");
+  entry->module=ConstantString("JPX");
   entry->magick=(MagickHandler *) IsJPC;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
@@ -665,8 +678,8 @@ ModuleExport void RegisterJP2Image(void)
 #endif
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PGX");
-  entry->description=AcquireString("JPEG-2000 VM Format");
-  entry->module=AcquireString("PGX");
+  entry->description=ConstantString("JPEG-2000 VM Format");
+  entry->module=ConstantString("PGX");
   entry->magick=(MagickHandler *) IsJPC;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
@@ -675,6 +688,9 @@ ModuleExport void RegisterJP2Image(void)
   entry->decoder=(DecoderHandler *) ReadJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
+#if defined(HasJP2)
+  jas_init();
+#endif
 }
 
 /*
@@ -701,6 +717,9 @@ ModuleExport void UnregisterJP2Image(void)
   (void) UnregisterMagickInfo("JP2");
   (void) UnregisterMagickInfo("JPC");
   (void) UnregisterMagickInfo("PGX");
+#if defined(HasJP2)
+  jas_cleanup();
+#endif
 }
 
 #if defined(HasJP2)
@@ -729,7 +748,6 @@ ModuleExport void UnregisterJP2Image(void)
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
 {
@@ -780,16 +798,15 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
   /*
     Intialize JPEG 2000 API.
   */
-  (void) SetImageColorspace(image,RGBColorspace);
-  if (jas_init() != 0)
-    ThrowWriterException(DelegateError,"UnableToCreateImage");
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   jp2_stream=JP2StreamManager(image);
   if (jp2_stream == (jas_stream_t *) NULL)
     ThrowWriterException(DelegateError,"UnableToManageJP2Stream");
@@ -805,7 +822,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     component_info[i].vstep=1;
     component_info[i].width=(unsigned int) image->columns;
     component_info[i].height=(unsigned int) image->rows;
-    component_info[i].prec=(unsigned int) image->depth <= 8 ? 8 : 16;
+    component_info[i].prec=(unsigned int) Min(image->depth,16);
     component_info[i].sgnd=MagickFalse;
   }
   jp2_image=jas_image_create(number_components,component_info,
@@ -855,40 +872,25 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    if (image->depth <= 8)
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if (number_components == 1)
-          jas_matrix_setv(pixels[0],x,
-            ScaleQuantumToChar(PixelIntensityToQuantum(p)));
-        else
-          {
-            jas_matrix_setv(pixels[0],x,ScaleQuantumToChar(p->red));
-            jas_matrix_setv(pixels[1],x,ScaleQuantumToChar(p->green));
-            jas_matrix_setv(pixels[2],x,ScaleQuantumToChar(p->blue));
-            if (number_components > 3)
-              jas_matrix_setv(pixels[3],x,
-                ScaleQuantumToChar(MaxRGB-p->opacity));
-          }
-        p++;
-      }
-    else
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if (number_components == 1)
-          jas_matrix_setv(pixels[0],x,
-            ScaleQuantumToShort(PixelIntensityToQuantum(p)));
-        else
-          {
-            jas_matrix_setv(pixels[0],x,ScaleQuantumToShort(p->red));
-            jas_matrix_setv(pixels[1],x,ScaleQuantumToShort(p->green));
-            jas_matrix_setv(pixels[2],x,ScaleQuantumToShort(p->blue));
-            if (number_components > 3)
-              jas_matrix_setv(pixels[3],x,
-                ScaleQuantumToShort(MaxRGB-p->opacity));
-          }
-        p++;
-      }
+    for (x=0; x < (long) image->columns; x++)
+    {
+      if (number_components == 1)
+        jas_matrix_setv(pixels[0],x,ScaleQuantumToShort(
+          PixelIntensityToQuantum(p)) >> (16-Min(image->depth,16)));
+      else
+        {
+          jas_matrix_setv(pixels[0],x,ScaleQuantumToShort(p->red) >> (16-
+            Min(image->depth,16)));
+          jas_matrix_setv(pixels[1],x,ScaleQuantumToShort(p->green) >> (16-
+            Min(image->depth,16)));
+          jas_matrix_setv(pixels[2],x,ScaleQuantumToShort(p->blue) >> (16-
+            Min(image->depth,16)));
+          if (number_components > 3)
+            jas_matrix_setv(pixels[3],x,ScaleQuantumToShort((Quantum)
+              (QuantumRange-p->opacity)) >> (16-Min(image->depth,16)));
+        }
+      p++;
+    }
     for (i=0; i < (long) number_components; i++)
       (void) jas_image_writecmpt(jp2_image,(short) i,0,(unsigned int) y,
         (unsigned int) image->columns,1,pixels[i]);
