@@ -437,11 +437,6 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
     *iptc_profile,
     *profile;
 
-#ifdef GET_ONLY_IPTC_DATA
-  unsigned char
-    tag[MaxTextExtent];
-#endif
-
   /*
     Determine length of binary data stored here.
   */
@@ -451,28 +446,11 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
     return(MagickTrue);
   length-=2;
   tag_length=0;
-#ifdef GET_ONLY_IPTC_DATA
-  *tag='\0';
-#endif
   error_manager=(ErrorManager *) jpeg_info->client_data;
   image=error_manager->image;
   iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
   if (iptc_profile == (StringInfo *) NULL)
     {
-#ifdef GET_ONLY_IPTC_DATA
-      /*
-        Find the beginning of the IPTC portion of the binary data.
-      */
-      for (*tag='\0'; length > 0; )
-      {
-        *tag=GetCharacter(jpeg_info);
-        *(tag+1)=GetCharacter(jpeg_info);
-        length-=2;
-        if ((*tag == 0x1c) && (*(tag+1) == 0x02))
-          break;
-      }
-      tag_length=2;
-#else
       /*
         Validate that this was written as a Photoshop resource format slug.
       */
@@ -485,7 +463,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
       if (LocaleCompare(magick,"Photoshop ") != 0)
         {
           /*
-            Not a ICC profile, return.
+            Not a IPTC profile, return.
           */
           for (i=0; i < (long) length; i++)
             (void) GetCharacter(jpeg_info);
@@ -500,7 +478,6 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
         return(MagickTrue);
       length-=4;
       tag_length=0;
-#endif
     }
   if (length == 0)
     return(MagickTrue);
@@ -712,8 +689,10 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       return((Image *) NULL);
     }
   /*
-    Initialize image structure.
+    Initialize JPEG parameters.
   */
+  (void) ResetMagickMemory(&jpeg_info,0,sizeof(jpeg_info));
+  (void) ResetMagickMemory(&jpeg_error,0,sizeof(jpeg_error));
   jpeg_info.err=jpeg_std_error(&jpeg_error);
   jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) EmitMessage;
   jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
@@ -738,7 +717,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   jpeg_set_marker_processor(&jpeg_info,IPTC_MARKER,ReadIPTCProfile);
   for (i=1; i < 16; i++)
     if ((i != 2) && (i != 13) && (i != 14))
-      jpeg_set_marker_processor(&jpeg_info,(int) i+JPEG_APP0,ReadProfile);
+      jpeg_set_marker_processor(&jpeg_info,(int) (JPEG_APP0+i),ReadProfile);
   i=jpeg_read_header(&jpeg_info,MagickTrue);
   if ((image_info->colorspace == YCbCrColorspace) ||
       (image_info->colorspace == Rec601YCbCrColorspace) ||
@@ -792,8 +771,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     }
 #if (JPEG_LIB_VERSION >= 61) && defined(D_PROGRESSIVE_SUPPORTED)
 #ifdef D_LOSSLESS_SUPPORTED
-  image->interlace=
-    jpeg_info.process == JPROC_PROGRESSIVE ? PlaneInterlace : NoInterlace;
+  image->interlace=jpeg_info.process == JPROC_PROGRESSIVE ?
+    PlaneInterlace : NoInterlace;
   image->compression=jpeg_info.process == JPROC_LOSSLESS ?
     LosslessJPEGCompression : JPEGCompression;
   if (jpeg_info.data_precision > 8)
@@ -801,8 +780,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       "12-bit JPEG not supported. Reducing pixel data to 8 bits","`%s'",
       image->filename);
 #else
-  image->interlace=jpeg_info.progressive_mode != 0 ? PlaneInterlace :
-    NoInterlace;
+  image->interlace=jpeg_info.progressive_mode != 0 ?
+    PlaneInterlace : NoInterlace;
   image->compression=JPEGCompression;
 #endif
 #else
@@ -816,13 +795,14 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       */
       jpeg_info.quantize_colors=MagickTrue;
       jpeg_info.desired_number_of_colors=(int) image_info->colors;
-      if (AllocateImageColormap(image,image_info->colors) == MagickFalse)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   (void) jpeg_start_decompress(&jpeg_info);
   image->columns=jpeg_info.output_width;
   image->rows=jpeg_info.output_height;
   image->depth=(unsigned long) jpeg_info.data_precision;
+  if ((image_info->colors != 0) && (image_info->colors <= 256))
+    if (AllocateImageColormap(image,image_info->colors) == MagickFalse)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if ((jpeg_info.output_components == 1) &&
       (jpeg_info.quantize_colors == MagickFalse))
     {
@@ -1417,10 +1397,20 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
         unsigned long
           roundup;
 
-        tag_length=14;
-        (void) CopyMagickMemory(custom_profile->datum,"Photoshop 3.0 ",
-          tag_length);
-        custom_profile->datum[13]=0x00;
+        if (LocaleNCompare((char *) profile->datum,"8BIM",4) == 0)
+          {
+            (void) CopyMagickMemory(custom_profile->datum,"Photoshop 3.0\0",14);
+            tag_length=14;
+          }
+        else
+          {
+            (void) CopyMagickMemory(custom_profile->datum,
+              "Photoshop 3.0\08BIM\04\04\0\0\0\0",24);
+            custom_profile->datum[13]=0x00;
+            custom_profile->datum[24]=profile->length >> 8;
+            custom_profile->datum[25]=profile->length & 0xff;
+            tag_length=26;
+          }
         for (i=0; i < (long) profile->length; i+=65500L)
         {
           length=Min(profile->length-i,65500L);
@@ -1565,6 +1555,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   /*
     Initialize JPEG parameters.
   */
+  (void) ResetMagickMemory(&jpeg_info,0,sizeof(jpeg_info));
+  (void) ResetMagickMemory(&jpeg_error,0,sizeof(jpeg_error));
   jpeg_info.client_data=(void *) image;
   jpeg_info.err=jpeg_std_error(&jpeg_error);
   jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) EmitMessage;
@@ -1628,9 +1620,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           jpeg_info.in_color_space=JCS_YCbCr;
           break;
         }
-      if (image_info->colorspace != LogColorspace)
-    if (image_info->colorspace == UndefinedColorspace)
-          (void) SetImageColorspace(image,RGBColorspace);
+      if (image_info->colorspace == UndefinedColorspace)
+        (void) SetImageColorspace(image,RGBColorspace);
       break;
     }
   }
@@ -1670,7 +1661,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       /*
         Perform optimization only if available memory resources permit it.
       */
-      status=AcquireMagickResource(AreaResource,number_pixels);
+      status=AcquireMagickResource(MemoryResource,8*number_pixels);
       if (status != MagickFalse)
         {
           const char
@@ -1681,7 +1672,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
               (LocaleCompare(option,"true") == 0))
             jpeg_info.optimize_coding=MagickTrue;
         }
-      RelinquishMagickResource(AreaResource,number_pixels);
+      RelinquishMagickResource(MemoryResource,8*number_pixels);
     }
 #if (JPEG_LIB_VERSION >= 61) && defined(C_PROGRESSIVE_SUPPORTED)
   if ((LocaleCompare(image_info->magick,"PJPEG") == 0) ||
