@@ -152,7 +152,7 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       status=GetMultilineTypeMetrics(image,draw_info,&metrics);
       for ( ; status != MagickFalse; draw_info->pointsize*=2)
       {
-        width=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
+        width=(unsigned long) (metrics.width+metrics.max_advance+0.5);
         height=(unsigned long) (metrics.height+0.5);
         if (((image->columns != 0) && (width >= image->columns)) ||
             ((image->rows != 0) && (height >= image->rows)))
@@ -161,7 +161,7 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       }
       for ( ; status != MagickFalse; draw_info->pointsize--)
       {
-        width=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
+        width=(unsigned long) (metrics.width+metrics.max_advance+0.5);
         height=(unsigned long) (metrics.height+0.5);
         if ((image->columns != 0) && (width <= image->columns) &&
            ((image->rows == 0) || (height <= image->rows)))
@@ -182,7 +182,7 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       return((Image *) NULL);
     }
   if (image->columns == 0)
-    image->columns=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
+    image->columns=(unsigned long) (metrics.width+metrics.max_advance+0.5);
   if (draw_info->gravity == UndefinedGravity)
     {
       (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
