@@ -191,7 +191,7 @@ static int stringnicmp(const char *p,const char *q,size_t n)
     p++;
     q++;
   }
-  return(toupper(*p)-toupper(*q));
+  return(toupper((int) *p)-toupper((int) *q));
 }
 
 static int convertHTMLcodes(char *s, int len)
@@ -229,7 +229,7 @@ static int convertHTMLcodes(char *s, int len)
         if (html_codes[i].len <= len)
           if (stringnicmp(s, html_codes[i].code, html_codes[i].len) == 0)
             {
-              (void) CopyMagickString(s+1,s+html_codes[i].len,strlen(s+html_codes[i].len));
+              (void) CopyMagickString(s+1,s+html_codes[i].len,strlen(s+html_codes[i].len)+1);
               *s = html_codes[i].val;
               return html_codes[i].len-1;
             }
@@ -321,13 +321,14 @@ static long parse8BIM(Image *ifile, Image *ofile)
     currentpos;
 
   TokenInfo
-    token_info;
+    *token_info;
 
   dataset = 0;
   recnum = 0;
   line = (char *) AcquireMagickMemory(inputlen);
   name = token = (char *)NULL;
   savedpos = 0;
+  token_info=AcquireTokenInfo();
   while(super_fgets(&line,&inputlen,ifile)!=NULL)
   {
     state=0;
@@ -335,9 +336,8 @@ static long parse8BIM(Image *ifile, Image *ofile)
 
     token = (char *) AcquireMagickMemory(inputlen);
     newstr = (char *) AcquireMagickMemory(inputlen);
-    while (Tokenizer(&token_info, 0, token, inputlen, line,
-          (char *) "", (char *) "=",
-      (char *) "\"", 0, &brkused,&next,&quoted)==0)
+    while (Tokenizer(token_info, 0, token, inputlen, line, (char *) "",
+      (char *) "=", (char *) "\"", 0, &brkused,&next,&quoted)==0)
     {
       if (state == 0)
         {
@@ -351,7 +351,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
 
           state=0;
           next=0;
-          while(Tokenizer(&token_info, 0, newstr, inputlen, token, (char *) "",
+          while(Tokenizer(token_info, 0, newstr, inputlen, token, (char *) "",
             (char *) "#", (char *) "", 0, &brkused, &next, &quoted)==0)
           {
             switch (state)
@@ -389,7 +389,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
 
             next=0;
             len = strlen(token);
-            while (Tokenizer(&token_info,0, newstr, inputlen, token, (char *) "",
+            while (Tokenizer(token_info,0, newstr, inputlen, token, (char *) "",
               (char *) "&", (char *) "", 0, &brkused, &next, &quoted)==0)
             {
               if (brkused && next > 0)
@@ -424,7 +424,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
                     outputlen++;
                   }
                 WriteBlobString(ofile,"8BIM");
-                WriteBlobMSBShort(ofile,recnum);
+                WriteBlobMSBShort(ofile,(unsigned short) recnum);
                 outputlen += 6;
                 if (name)
                   nlen = strlen(name);
@@ -469,7 +469,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
                   {
                     WriteBlobByte(ofile,0x1c);
                     WriteBlobByte(ofile,dataset);
-                    WriteBlobByte(ofile,recnum & 255);
+                    WriteBlobByte(ofile,(unsigned char) (recnum & 0xff));
                     WriteBlobMSBShort(ofile,len);
                     outputlen += 5;
                     next=0;
@@ -485,6 +485,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
     newstr=(char *) RelinquishMagickMemory(newstr);
     name=(char *) RelinquishMagickMemory(name);
   }
+  token_info=DestroyTokenInfo(token_info);
   line=(char *) RelinquishMagickMemory(line);
   if (savedolen > 0)
     {
@@ -580,13 +581,14 @@ static long parse8BIMW(Image *ifile, Image *ofile)
     currentpos;
 
   TokenInfo
-    token_info;
+    *token_info;
 
   dataset = 0;
   recnum = 0;
   line = (char *) AcquireMagickMemory(inputlen);
   name = token = (char *)NULL;
   savedpos = 0;
+  token_info=AcquireTokenInfo();
   while(super_fgets_w(&line,&inputlen,ifile)!=NULL)
   {
     state=0;
@@ -594,7 +596,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
 
     token = (char *) AcquireMagickMemory(inputlen);
     newstr = (char *) AcquireMagickMemory(inputlen);
-    while (Tokenizer(&token_info, 0, token, inputlen, line,
+    while (Tokenizer(token_info, 0, token, inputlen, line,
           (char *) "", (char *) "=",
       (char *) "\"", 0, &brkused,&next,&quoted)==0)
     {
@@ -610,7 +612,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
 
           state=0;
           next=0;
-          while(Tokenizer(&token_info, 0, newstr, inputlen, token, (char *) "",
+          while(Tokenizer(token_info, 0, newstr, inputlen, token, (char *) "",
             (char *) "#", (char *) "", 0, &brkused, &next, &quoted)==0)
           {
             switch (state)
@@ -648,7 +650,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
 
             next=0;
             len = strlen(token);
-            while (Tokenizer(&token_info,0, newstr, inputlen, token, (char *) "",
+            while (Tokenizer(token_info,0, newstr, inputlen, token, (char *) "",
               (char *) "&", (char *) "", 0, &brkused, &next, &quoted)==0)
             {
               if (brkused && next > 0)
@@ -683,7 +685,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                     outputlen++;
                   }
                 WriteBlobString(ofile,"8BIM");
-                WriteBlobMSBShort(ofile,recnum);
+                WriteBlobMSBShort(ofile,(unsigned short) recnum);
                 outputlen += 6;
                 if (name)
                   nlen = strlen(name);
@@ -728,8 +730,8 @@ static long parse8BIMW(Image *ifile, Image *ofile)
                   {
                     WriteBlobByte(ofile,0x1c);
                     WriteBlobByte(ofile,dataset);
-                    WriteBlobByte(ofile,recnum & 255);
-                    WriteBlobMSBShort(ofile,len);
+                    WriteBlobByte(ofile,(unsigned char) (recnum & 0xff));
+                    WriteBlobMSBShort(ofile,(unsigned short) len);
                     outputlen += 5;
                     next=0;
                     outputlen += len;
@@ -744,6 +746,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
     newstr=(char *) RelinquishMagickMemory(newstr);
     name=(char *) RelinquishMagickMemory(name);
   }
+  token_info=DestroyTokenInfo(token_info);
   line=(char *) RelinquishMagickMemory(line);
   if (savedolen > 0)
     {
@@ -799,7 +802,7 @@ static int jpeg_transfer_1(Image *ifile, Image *ofile)
   c = ReadBlobByte(ifile);
   if (c == EOF)
     return EOF;
-  WriteBlobByte(ofile,c);
+  WriteBlobByte(ofile,(unsigned char) c);
   return c;
 }
 
@@ -874,7 +877,7 @@ static int jpeg_nextmarker(Image *ifile, Image *ofile)
       return M_EOI; /* we hit EOF */
     else
       if (c != 0xff)
-        WriteBlobByte(ofile,c);
+        WriteBlobByte(ofile,(unsigned char) c);
   } while (c != 0xff);
 
   /* get marker byte, swallowing possible padding */
@@ -944,7 +947,7 @@ static int jpeg_embed(Image *ifile, Image *ofile, Image *iptc)
         if (marker != M_APP13)
           {
             WriteBlobByte(ofile,0xff);
-            WriteBlobByte(ofile,marker);
+            WriteBlobByte(ofile,(unsigned char) marker);
           }
       }
 
@@ -1185,7 +1188,6 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             WriteBlobByte(buff,c);
           }
 #else
-#define MaxBufferSize  65541
           long
             i;
 
@@ -1198,11 +1200,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           ssize_t
             count;
 
-          buffer=(unsigned char *) AcquireMagickMemory(MaxBufferSize);
+          buffer=(unsigned char *) AcquireMagickMemory(MagickMaxBufferSize*
+            sizeof(*buffer));
           if (buffer != (unsigned char *) NULL)
             {
               i=0;
-              while ((length=ReadBlob(image,MaxBufferSize,buffer)) != 0)
+              while ((length=ReadBlob(image,MagickMaxBufferSize,buffer)) != 0)
               {
                 count=0;
                 for (i=0; i < (long) length; i+=count)
@@ -1269,12 +1272,13 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
       AttachBlob(buff->blob,blob,length);
       /* write out the header - length field patched below */
-      WriteBlob(buff,12,(unsigned char *) "8BIM\04\04\0\0\0\0\0\0");
+      WriteBlob(buff,11,(unsigned char *) "8BIM\04\04\0\0\0\0\01");
+      WriteBlobByte(buff,0xe0);
       if (LocaleCompare(image_info->magick,"IPTCTEXT") == 0)
         {
           length=parse8BIM(image, buff);
           if (length & 1)
-            WriteBlobByte(buff,0x0);
+            WriteBlobByte(buff,0x00);
         }
       else if (LocaleCompare(image_info->magick,"IPTCWTEXT") == 0)
         {
@@ -2136,8 +2140,6 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if (image->profiles == (void *) NULL)
-    return(MagickFalse);
   length=0;
   if (LocaleCompare(image_info->magick,"8BIM") == 0)
     {
