@@ -57,6 +57,7 @@
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -286,7 +287,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -330,9 +332,9 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         bluebits=ReadBlobByte(image);  /* # of bits of blue */
         ReadBlobByte(image);  /* reserved by Palm */
         ReadBlobByte(image);  /* reserved by Palm */
-        transpix.red=MaxRGB*ReadBlobByte(image)/31;
-        transpix.green=MaxRGB*ReadBlobByte(image)/63;
-        transpix.blue=MaxRGB*ReadBlobByte(image)/31;
+        transpix.red=QuantumRange*ReadBlobByte(image)/31;
+        transpix.green=QuantumRange*ReadBlobByte(image)/63;
+        transpix.blue=QuantumRange*ReadBlobByte(image)/31;
       }
     if (bits_per_pixel == 8)
       {
@@ -343,11 +345,11 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
             {
               ReadBlobByte(image);
               image->colormap[255-i].red=
-                ScaleCharToQuantum(ReadBlobByte(image));
+                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               image->colormap[255-i].green=
-                ScaleCharToQuantum(ReadBlobByte(image));
+                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               image->colormap[255-i].blue=
-                ScaleCharToQuantum(ReadBlobByte(image));
+                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
           }
         }
       else
@@ -419,7 +421,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
                     one_row[i+bit]=lastrow[i+bit];
                 }
               }
-              CopyMagickMemory(lastrow, one_row, bytes_per_row);
+              (void) CopyMagickMemory(lastrow, one_row, bytes_per_row);
             }
         }
       ptr=one_row;
@@ -433,9 +435,9 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
           {
             color16=(*ptr++ << 8);
             color16|=(*ptr++);
-            q->red=(MaxRGB*((color16 >> 11) & 0x1f))/0x1f;
-            q->green=(MaxRGB*((color16 >> 5) & 0x3f))/0x3f;
-            q->blue=(MaxRGB*((color16 >> 0) & 0x1f))/0x1f;
+            q->red=(QuantumRange*((color16 >> 11) & 0x1f))/0x1f;
+            q->green=(QuantumRange*((color16 >> 5) & 0x3f))/0x3f;
+            q->blue=(QuantumRange*((color16 >> 0) & 0x1f))/0x1f;
             q->opacity=OpaqueOpacity;
             q++;
           }
@@ -496,7 +498,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
           Allocate next image structure. Copied from coders/pnm.c
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             DestroyImageList(image);
             return((Image *) NULL);
@@ -547,8 +549,8 @@ ModuleExport void RegisterPALMImage(void)
   entry->decoder=(DecoderHandler *) ReadPALMImage;
   entry->encoder=(EncoderHandler *) WritePALMImage;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Palm pixmap");
-  entry->module=AcquireString("PALM");
+  entry->description=ConstantString("Palm pixmap");
+  entry->module=ConstantString("PALM");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -671,7 +673,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&exception);
   if (status == MagickFalse)
     return(status);
@@ -700,15 +702,15 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
       }
     if (bits_per_pixel > 8)
       flags|=PALM_IS_DIRECT_COLOR;
-    (void) WriteBlobMSBShort(image,image->columns);  /* width */
-    (void) WriteBlobMSBShort(image,image->rows );  /* height */
+    (void) WriteBlobMSBShort(image,(unsigned short) image->columns);  /* width */
+    (void) WriteBlobMSBShort(image,(unsigned short) image->rows );  /* height */
     bytes_per_row=((image->columns+(16/bits_per_pixel-1))/(16/bits_per_pixel))*2;
-    (void) WriteBlobMSBShort(image, bytes_per_row);
+    (void) WriteBlobMSBShort(image,(unsigned short) bytes_per_row);
     if ((image->compression == RLECompression) ||
         (image->compression == FaxCompression))
       flags|=PALM_IS_COMPRESSED_FLAG;
     (void) WriteBlobMSBShort(image, flags);
-    (void) WriteBlobByte(image, bits_per_pixel);
+    (void) WriteBlobByte(image,(unsigned char) bits_per_pixel);
     if (bits_per_pixel > 1)
       version=1;
     if ((image->compression == RLECompression) ||
@@ -716,7 +718,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
       version=2;
     (void) WriteBlobByte(image,version);
     (void) WriteBlobMSBShort(image,0);  /* nextDepthOffset */
-    (void) WriteBlobByte(image,transparentIndex);
+    (void) WriteBlobByte(image,(unsigned char) transparentIndex);
     if (image->compression == RLECompression)
       (void) WriteBlobByte(image,PALM_COMPRESSION_RLE);
     else
@@ -743,11 +745,12 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
             quantize_info.dither=IsPaletteImage(image,&image->exception);
             quantize_info.number_colors=image->colors;
             (void) QuantizeImage(&quantize_info,image);
-            WriteBlobMSBShort(image,image->colors);
+            WriteBlobMSBShort(image,(unsigned short) image->colors);
             for (count = 0; count < image->colors; count++)
             {
               WriteBlobByte(image,count);
-              WriteBlobByte(image,ScaleQuantumToChar(image->colormap[count].red));
+              WriteBlobByte(image,ScaleQuantumToChar(
+                image->colormap[count].red));
               WriteBlobByte(image,
                 ScaleQuantumToChar(image->colormap[count].green));
               WriteBlobByte(image,
@@ -791,8 +794,8 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
         {
           for (x=0; x < (int) image->columns; x++)
           {
-            color16=(((31*p->red)/MaxRGB) << 11) | (((63*p->green)/MaxRGB) << 5) |
-              ((31*p->blue)/MaxRGB);
+            color16=(((31*p->red)/QuantumRange) << 11) | (((63*p->green)/QuantumRange) << 5) |
+              ((31*p->blue)/QuantumRange);
             if (p->opacity == TransparentOpacity)
               {
                 transpix.red=p->red;
@@ -840,8 +843,8 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
             while ((one_row[++x] == byte) && (count < 255) &&
                    (x < (long) bytes_per_row))
               count++;
-            (void) WriteBlobByte(image,count);
-            (void) WriteBlobByte(image,byte);
+            (void) WriteBlobByte(image,(unsigned char) count);
+            (void) WriteBlobByte(image,(unsigned char) byte);
           }
         }
       else
@@ -875,26 +878,26 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
         SeekBlob(image,currentOffset+6,SEEK_SET);
         (void) WriteBlobMSBShort(image,flags);
         SeekBlob(image,currentOffset+12,SEEK_SET);
-        (void) WriteBlobByte(image,transparentIndex);  /* trans index */
+        (void) WriteBlobByte(image,(unsigned char) transparentIndex);  /* trans index */
       }
     if (bits_per_pixel == 16)
       {
         SeekBlob(image,currentOffset+20,SEEK_SET);
         WriteBlobByte(image,0);  /* reserved by Palm */
-        WriteBlobByte(image,(31*transpix.red)/MaxRGB);
-        WriteBlobByte(image,(63*transpix.green)/MaxRGB);
-        WriteBlobByte(image,(31*transpix.blue)/MaxRGB);
+        WriteBlobByte(image,(unsigned char) ((31*transpix.red)/QuantumRange));
+        WriteBlobByte(image,(unsigned char) ((63*transpix.green)/QuantumRange));
+        WriteBlobByte(image,(unsigned char) ((31*transpix.blue)/QuantumRange));
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)  /* fill in size now */
       {
         SeekBlob(image,currentOffset+offset,SEEK_SET);
-        (void) WriteBlobMSBShort(image,GetBlobSize(image)-currentOffset-offset);
+        (void) WriteBlobMSBShort(image,(unsigned short) (GetBlobSize(image)-currentOffset-offset));
       }
     if (one_row != (unsigned char *) NULL) 
       one_row=(unsigned char *) RelinquishMagickMemory(one_row);
     if (lastrow != (unsigned char *) NULL) 
       lastrow=(unsigned char *) RelinquishMagickMemory(lastrow);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     /* padding to 4 byte word */
     for (cc = (GetBlobSize(image))%4; cc > 0; cc--)
@@ -904,12 +907,12 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
     /* write nextDepthOffset and return to end of image */
     SeekBlob(image,currentOffset+10,SEEK_SET);
     nextDepthOffset=(unsigned long) ((GetBlobSize(image)-currentOffset)/4);
-    (void) WriteBlobMSBShort(image,nextDepthOffset);
+    (void) WriteBlobMSBShort(image,(unsigned short) nextDepthOffset);
     currentOffset=(MagickOffsetType) GetBlobSize(image);
     SeekBlob(image,currentOffset,SEEK_SET);
     image=SyncNextImageInList(image);
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   return(MagickTrue);
 }
