@@ -59,9 +59,10 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/option.h"
+#include "magick/quantum.h"
 #include "magick/resource_.h"
 #include "magick/resize.h"
-#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -84,6 +85,103 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I n v o k e P o s t s r i p t D e l e g a t e                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  InvokePostscriptDelegate() executes the postscript interpreter with the
+%  specified command.
+%
+%  The format of the InvokePostscriptDelegate method is:
+%
+%      MagickBooleanType InvokePostscriptDelegate(
+%        const MagickBooleanType verbose,const char *command)
+%
+%  A description of each parameter follows:
+%
+%    o verbose: A value other than zero displays the command prior to
+%      executing it.
+%
+%    o command: The address of a character string containing the command to
+%      execute.
+%
+*/
+static MagickBooleanType InvokePostscriptDelegate(
+  const MagickBooleanType verbose,const char *command)
+{
+#if defined(HasGS) || defined(__WINDOWS__)
+  char
+    **argv;
+
+  const GhostscriptVectors
+    *gs_func;
+
+  gs_main_instance
+    *interpreter;
+
+  int
+    argc,
+    code,
+    status;
+
+  register long
+    i;
+
+#if defined(__WINDOWS__)
+  gs_func=NTGhostscriptDLLVectors();
+#else
+  GhostscriptVectors
+    gs_func_struct;
+
+  gs_func=(&gs_func_struct);
+  gs_func_struct.exit=gsapi_exit;
+  gs_func_struct.init_with_args=gsapi_init_with_args;
+  gs_func_struct.new_instance=gsapi_new_instance;
+  gs_func_struct.run_string=gsapi_run_string;
+  gs_func_struct.delete_instance=gsapi_delete_instance;
+#endif
+  if (gs_func == (GhostscriptVectors *) NULL)
+    return(SystemCommand(verbose,command) == 0 ? MagickFalse : MagickTrue);
+  if (verbose != MagickFalse)
+    {
+      (void) fputs("[ghostscript library]",stdout);
+      (void) fputs(strchr(command,' '),stdout);
+    }
+  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+  if (status < 0)
+    return(MagickFalse);
+  argv=StringToArgv(command,&argc);
+  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+  if (status == 0)
+    status=(gs_func->run_string)
+      (interpreter,"systemdict /start get exec\n",0,&code);
+  (gs_func->exit)(interpreter);
+  (gs_func->delete_instance)(interpreter);
+#if defined(__WINDOWS__)
+  NTGhostscriptUnLoadDLL();
+#endif
+  for (i=0; i < (long) argc; i++)
+    argv[i]=(char *) RelinquishMagickMemory(argv[i]);
+  argv=(char **) RelinquishMagickMemory(argv);
+  if ((status == 0) || (status <= -100))
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Ghostscript returns status %d, exit code %d",status,code);
+      return(MagickFalse);
+    }
+  return(MagickTrue);
+#else
+  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I s P D F                                                                 %
 %                                                                             %
 %                                                                             %
@@ -148,6 +246,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define DeviceCMYK  "DeviceCMYK"
 #define MediaBox  "MediaBox"
 #define RenderPostscriptText  "  Rendering Postscript...  "
+#define TrimBox  "TrimBox"
 
   char
     command[MaxTextExtent],
@@ -158,6 +257,9 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     input_filename[MaxTextExtent],
     postscript_filename[MaxTextExtent];
 
+  const char
+    *option;
+
   const DelegateInfo
     *delegate_info;
 
@@ -173,6 +275,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   MagickBooleanType
     cmyk,
+    cropbox,
+    trimbox,
     status;
 
   PointInfo
@@ -195,9 +299,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count;
 
   unsigned long
-    height,
-    scene,
-    width;
+    scene;
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
@@ -248,7 +350,15 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine page geometry from the PDF media box.
   */
-  cmyk=MagickFalse;
+  cmyk=image_info->colorspace == CMYKColorspace ? MagickTrue : MagickFalse;
+  cropbox=MagickFalse;
+  option=GetImageOption(image_info,"pdf:use-cropbox");
+  if (option != (const char *) NULL)
+    cropbox=IsMagickTrue(option);
+  trimbox=MagickFalse;
+  option=GetImageOption(image_info,"pdf:use-trimbox");
+  if (option != (const char *) NULL)
+    trimbox=IsMagickTrue(option);
   count=0;
   (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
@@ -273,7 +383,12 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     if (LocaleNCompare(DeviceCMYK,command,strlen(DeviceCMYK)) == 0)
       cmyk=MagickTrue;
-    if (LocaleNCompare(CropBox,command,strlen(CropBox)) == 0)
+    if ((bounds.x1 != 0.0) || (bounds.y1 != 0.0) || (bounds.x2 != 0.0) ||
+        (bounds.y2 != 0.0))
+      continue;
+    count=0;
+    if ((cropbox != MagickFalse) &&
+        (LocaleNCompare(CropBox,command,strlen(CropBox)) == 0))
       {
         /*
           Note region defined by crop box.
@@ -284,6 +399,18 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           count=(ssize_t) sscanf(command,"CropBox[%lf %lf %lf %lf",
             &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
       }
+    if ((trimbox != MagickFalse) &&
+        (LocaleNCompare(TrimBox,command,strlen(TrimBox)) == 0))
+      {
+        /*
+          Note region defined by trim box.
+        */
+        count=(ssize_t) sscanf(command,"TrimBox [%lf %lf %lf %lf",
+          &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+        if (count != 4)
+          count=(ssize_t) sscanf(command,"TrimBox[%lf %lf %lf %lf",
+            &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+      }
     if (LocaleNCompare(MediaBox,command,strlen(MediaBox)) == 0)
       {
         /*
@@ -300,12 +427,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Set PDF render geometry.
     */
-    width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-    height=(unsigned long) (bounds.y2-bounds.y1+0.5);
-    if (width > page.width)
-      page.width=width;
-    if (height > page.height)
-      page.height=height;
+    page.width=(unsigned long) (bounds.x2-bounds.x1+0.5);
+    page.height=(unsigned long) (bounds.y2-bounds.y1+0.5);
   }
   CloseBlob(image);
   if (image_info->colorspace == RGBColorspace)
@@ -351,6 +474,10 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   *options='\0';
+  if (cropbox != MagickFalse)
+    (void) ConcatenateMagickString(options,"-dUseCropBox ",MaxTextExtent);
+  if (trimbox != MagickFalse)
+    (void) ConcatenateMagickString(options,"-dUseTrimBox ",MaxTextExtent);
   read_info=CloneImageInfo(image_info);
   *read_info->magick='\0';
   if (read_info->number_scenes != 0)
@@ -443,6 +570,7 @@ ModuleExport void RegisterPDFImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Encapsulated Portable Document Format");
   entry->module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
@@ -452,6 +580,7 @@ ModuleExport void RegisterPDFImage(void)
   entry->magick=(MagickHandler *) IsPDF;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Portable Document Format");
   entry->module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
@@ -541,7 +670,7 @@ static char *EscapeParenthesis(const char *text)
 static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
 {
 #define CFormat  "/Filter [ /%s ]\n"
-#define ObjectsPerImage  12
+#define ObjectsPerImage  14
 
   char
     buffer[MaxTextExtent],
@@ -563,10 +692,10 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     y;
 
   Image
+    *next,
     *tile_image;
 
   MagickBooleanType
-    matte,
     status;
 
   MagickOffsetType
@@ -574,6 +703,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     scene,
     *xref;
 
+  MagickRealType
+    version;
+
   MagickSizeType
     number_pixels;
 
@@ -634,40 +766,6 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  compression=image->compression;
-  switch (compression)
-  {
-#if !defined(HasJPEG)
-    case JPEGCompression:
-    {
-      compression=RLECompression;
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JPEGLibraryIsNotAvailable","`%s'",
-        image->filename);
-      break;
-    }
-#endif
-#if !defined(HasJP2)
-    case JPEG2000Compression:
-    {
-      compression=RLECompression;
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JP2LibraryIsNotAvailable","`%s'",image->filename);
-      break;
-    }
-#endif
-#if !defined(HasZLIB)
-    case ZipCompression:
-    {
-      compression=RLECompression;
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"ZipLibraryIsNotAvailable","`%s'",image->filename);
-      break;
-    }
-#endif
-    default:
-      break;
-  }
   /*
     Allocate X ref memory.
   */
@@ -679,13 +777,15 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     Write Info object.
   */
   object=0;
-  if (compression == JPEG2000Compression)
-    (void) WriteBlobString(image,"%PDF-1.5 \n");
-  else
-    if (image->matte != MagickFalse)
-      (void) WriteBlobString(image,"%PDF-1.4 \n");
+  version=1.3f;
+  for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
+    if (next->compression == JPEG2000Compression)
+      version=Max(version,1.5);
     else
-      (void) WriteBlobString(image,"%PDF-1.3 \n");
+      if (next->matte != MagickFalse)
+        version=Max(version,1.4);
+  (void) FormatMagickString(buffer,MaxTextExtent,"%%PDF-%.1f \n",version);
+  (void) WriteBlobString(image,buffer);
   xref[object++]=TellBlob(image);
   info_id=object;
   (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
@@ -748,8 +848,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         kid_image=GetNextImageInList(kid_image);
       }
-      xref=(MagickOffsetType *)
-        ResizeMagickMemory(xref,(size_t) (count+2048)*sizeof(*xref));
+      xref=(MagickOffsetType *) ResizeMagickMemory(xref,(size_t) (count+2048)*
+        sizeof(*xref));
       if (xref == (MagickOffsetType *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
@@ -762,6 +862,54 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
+    compression=image->compression;
+    switch (compression)
+    {
+      case FaxCompression:
+      case Group4Compression:
+      {
+        if ((IsMonochromeImage(image,&image->exception) == MagickFalse) ||
+            (image->matte != MagickFalse))
+          compression=RLECompression;
+        break;
+      }
+#if !defined(HasJPEG)
+      case JPEGCompression:
+      {
+        compression=RLECompression;
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          MissingDelegateError,"JPEGLibraryIsNotAvailable","`%s'",
+          image->filename);
+        break;
+      }
+#endif
+#if !defined(HasJP2)
+      case JPEG2000Compression:
+      {
+        compression=RLECompression;
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          MissingDelegateError,"JP2LibraryIsNotAvailable","`%s'",image->filename);
+        break;
+      }
+#endif
+#if !defined(HasZLIB)
+      case ZipCompression:
+      {
+        compression=RLECompression;
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          MissingDelegateError,"ZipLibraryIsNotAvailable","`%s'",
+          image->filename);
+        break;
+      }
+#endif
+      default:
+        break;
+    }
+    if (compression == JPEG2000Compression)
+      {
+        if (image_info->colorspace == UndefinedColorspace)
+  	      (void) SetImageColorspace(image,RGBColorspace);
+      }
     /*
       Scale relative to dots-per-inch.
     */
@@ -802,6 +950,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PDF") == 0))
           (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
+    (void) ConcatenateMagickString(page_geometry,">",MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
@@ -835,18 +984,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,
       "/Font << /F%lu %lu 0 R >>\n",image->scene,object+4);
     (void) WriteBlobString(image,buffer);
-    matte=image->matte;
-    if (IsGrayImage(image,&image->exception) != MagickFalse)
-      matte=MagickFalse;
-    if (compression == JPEG2000Compression)
-      {
-        if (image_info->colorspace == UndefinedColorspace)
-  	      (void) SetImageColorspace(image,RGBColorspace);
-        matte=MagickFalse;
-      }
     (void) FormatMagickString(buffer,MaxTextExtent,
-      "/XObject << /Im%lu %lu 0 R >>\n",image->scene,object+
-      (matte != MagickFalse ? 7 : 5));
+      "/XObject << /Im%lu %lu 0 R >>\n",image->scene,object+5);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/ProcSet %lu 0 R >>\n",
       object+3);
@@ -862,7 +1001,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       object+1);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Thumb %lu 0 R\n",
-      object+(matte != MagickFalse ? 10 : 8));
+      object+8);
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,">>\n");
     (void) WriteBlobString(image,"endobj\n");
@@ -932,7 +1071,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     if ((image->storage_class == DirectClass) || (image->colors > 256))
       (void) CopyMagickString(buffer,"[ /PDF /Text /ImageC",MaxTextExtent);
     else
-      if (compression == FaxCompression)
+      if ((compression == FaxCompression) || (compression == Group4Compression))
         (void) CopyMagickString(buffer,"[ /PDF /Text /ImageB",MaxTextExtent);
       else
         (void) CopyMagickString(buffer,"[ /PDF /Text /ImageI",MaxTextExtent);
@@ -955,149 +1094,6 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"/Encoding /MacRomanEncoding\n");
     (void) WriteBlobString(image,">>\n");
     (void) WriteBlobString(image,"endobj\n");
-    if (matte != MagickFalse)
-      {
-        /*
-          Write softmask object.
-        */
-        xref[object++]=TellBlob(image);
-        (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
-        (void) WriteBlobString(image,buffer);
-        (void) WriteBlobString(image,"<<\n");
-        (void) WriteBlobString(image,"/Type /XObject\n");
-        (void) WriteBlobString(image,"/Subtype /Image\n");
-        (void) FormatMagickString(buffer,MaxTextExtent,"/Name /Ma%lu\n",
-          image->scene);
-        (void) WriteBlobString(image,buffer);
-        switch (compression)
-        {
-          case NoCompression:
-          {
-            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,
-              "ASCII85Decode");
-            break;
-          }
-          case LZWCompression:
-          {
-            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"LZWDecode");
-            break;
-          }
-          case ZipCompression:
-          {
-            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,
-              "FlateDecode");
-            break;
-          }
-          default:
-          {
-            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,
-              "RunLengthDecode");
-            break;
-          }
-        }
-        (void) WriteBlobString(image,buffer);
-        (void) FormatMagickString(buffer,MaxTextExtent,"/Width %lu\n",
-          image->columns);
-        (void) WriteBlobString(image,buffer);
-        (void) FormatMagickString(buffer,MaxTextExtent,"/Height %lu\n",
-          image->rows);
-        (void) WriteBlobString(image,buffer);
-        (void) WriteBlobString(image,"/ColorSpace /DeviceGray\n");
-        (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
-          compression == FaxCompression ? 1 : 8);
-        (void) WriteBlobString(image,buffer);
-        (void) FormatMagickString(buffer,MaxTextExtent,"/Length %lu 0 R\n",
-          object+1);
-        (void) WriteBlobString(image,buffer);
-        (void) WriteBlobString(image,">>\n");
-        (void) WriteBlobString(image,"stream\n");
-        offset=TellBlob(image);
-        number_pixels=(MagickSizeType) image->columns*image->rows;
-        switch (compression)
-        {
-          case RLECompression:
-          default:
-          {
-            /*
-              Allocate pixel array.
-            */
-            length=(size_t) number_pixels;
-            pixels=(unsigned char *) AcquireMagickMemory(length);
-            if (pixels == (unsigned char *) NULL)
-              {
-                image=DestroyImage(image);
-                ThrowWriterException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              }
-            /*
-              Dump Runlength encoded pixels.
-            */
-            q=pixels;
-            for (y=0; y < (long) image->rows; y++)
-            {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
-                &image->exception);
-              if (p == (const PixelPacket *) NULL)
-                break;
-              for (x=0; x < (long) image->columns; x++)
-              {
-                *q++=ScaleQuantumToChar(QuantumRange-p->opacity);
-                p++;
-              }
-            }
-#if defined(HasZLIB)
-            if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,pixels);
-            else
-#endif
-              if (compression == LZWCompression)
-                status=LZWEncodeImage(image,length,pixels);
-              else
-                status=PackbitsEncodeImage(image,length,pixels);
-            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-            if (status == MagickFalse)
-              {
-                CloseBlob(image);
-                return(MagickFalse);
-              }
-            break;
-          }
-          case NoCompression:
-          {
-            /*
-              Dump uncompressed PseudoColor packets.
-            */
-            Ascii85Initialize(image);
-            for (y=0; y < (long) image->rows; y++)
-            {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
-                &image->exception);
-              if (p == (const PixelPacket *) NULL)
-                break;
-              for (x=0; x < (long) image->columns; x++)
-              {
-                Ascii85Encode(image,ScaleQuantumToChar(QuantumRange-p->opacity));
-                p++;
-              }
-            }
-            Ascii85Flush(image);
-            break;
-          }
-        }
-        offset=TellBlob(image)-offset;
-        (void) WriteBlobString(image,"\nendstream\n");
-        (void) WriteBlobString(image,"endobj\n");
-        /*
-          Write Length object.
-        */
-        xref[object++]=TellBlob(image);
-        (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
-        (void) WriteBlobString(image,buffer);
-        (void) FormatMagickString(buffer,MaxTextExtent,"%lu\n",
-          (unsigned long) offset);
-        (void) WriteBlobString(image,buffer);
-        (void) WriteBlobString(image,"endobj\n");
-      }
     /*
       Write XObject object.
     */
@@ -1148,12 +1144,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
          break;
       }
       case FaxCompression:
+      case Group4Compression:
       {
         (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
           MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
-          "/DecodeParms [ << >> << /K %s /Columns %ld /Rows %ld >> ]\n",
+          "/DecodeParms [ << /K %s /Columns %ld /Rows %ld >> ]\n",
           CCITTParam,image->columns,image->rows);
         break;
       }
@@ -1174,12 +1171,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       object+2);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
-      compression == FaxCompression ? 1 : 8);
+      (compression == FaxCompression) || (compression == Group4Compression) ?
+      1 : 8);
     (void) WriteBlobString(image,buffer);
-    if (matte != MagickFalse)
+    if (image->matte != MagickFalse)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"/SMask %lu 0 R\n",
-          object-2);
+          object+7);
         (void) WriteBlobString(image,buffer);
       }
     (void) FormatMagickString(buffer,MaxTextExtent,"/Length %lu 0 R\n",
@@ -1191,14 +1189,14 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     number_pixels=(MagickSizeType) image->columns*image->rows;
     if ((4*number_pixels) != (MagickSizeType) ((size_t) (4*number_pixels)))
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    if ((compression == FaxCompression) ||
+    if ((compression == FaxCompression) || (compression == Group4Compression) ||
         ((image_info->type != TrueColorType) &&
          (IsGrayImage(image,&image->exception) != MagickFalse)))
-
       {
         switch (compression)
         {
           case FaxCompression:
+          case Group4Compression:
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
               {
@@ -1532,6 +1530,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       (void) CopyMagickString(buffer,"/DeviceCMYK\n",MaxTextExtent);
     else
       if ((compression == FaxCompression) ||
+          (compression == Group4Compression) ||
           ((image_info->type != TrueColorType) &&
            (IsGrayImage(image,&image->exception) != MagickFalse)))
           (void) CopyMagickString(buffer,"/DeviceGray\n",MaxTextExtent);
@@ -1598,12 +1597,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         break;
       }
       case FaxCompression:
+      case Group4Compression:
       {
         (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
           MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
-          "/DecodeParms [ << >> << /K %s /Columns %lu /Rows %lu >> ]\n",
+          "/DecodeParms [ << /K %s /Columns %lu /Rows %lu >> ]\n",
           CCITTParam,image->columns,image->rows);
         break;
       }
@@ -1625,7 +1625,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       object-1);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
-      compression == FaxCompression ? 1 : 8);
+      (compression == FaxCompression) || (compression == Group4Compression) ?
+      1 : 8);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Length %lu 0 R\n",
       object+1);
@@ -1635,17 +1636,21 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     offset=TellBlob(image);
     number_pixels=(MagickSizeType) tile_image->columns*tile_image->rows;
     if ((compression == FaxCompression) ||
+        (compression == Group4Compression) ||
         ((image_info->type != TrueColorType) &&
          (IsGrayImage(tile_image,&image->exception) != MagickFalse)))
       {
         switch (compression)
         {
           case FaxCompression:
+          case Group4Compression:
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
-              (void) HuffmanEncodeImage(image_info,tile_image);
-            else
-              (void) Huffman2DEncodeImage(image_info,tile_image);
+              {
+                (void) HuffmanEncodeImage(image_info,tile_image);
+                break;
+              }
+            (void) Huffman2DEncodeImage(image_info,tile_image);
             break;
           }
           case JPEGCompression:
@@ -1928,7 +1933,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,"<<\n");
     if ((image->storage_class != DirectClass) && (image->colors <= 256) &&
-        (compression != FaxCompression))
+        (compression != FaxCompression) && (compression != Group4Compression))
       {
         if (compression == NoCompression)
           (void) WriteBlobString(image,"/Filter [ /ASCII85Decode ]\n");
@@ -1972,6 +1977,152 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       (unsigned long) offset);
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,"endobj\n");
+    /*
+      Write softmask object.
+    */
+    xref[object++]=TellBlob(image);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
+    (void) WriteBlobString(image,buffer);
+    if (image->matte != MagickFalse)
+      {
+        (void) WriteBlobString(image,"<<\n");
+        (void) WriteBlobString(image,"/Type /XObject\n");
+        (void) WriteBlobString(image,"/Subtype /Image\n");
+        (void) FormatMagickString(buffer,MaxTextExtent,"/Name /Ma%lu\n",
+          image->scene);
+        (void) WriteBlobString(image,buffer);
+        switch (compression)
+        {
+          case NoCompression:
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,
+              "ASCII85Decode");
+            break;
+          }
+          case LZWCompression:
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"LZWDecode");
+            break;
+          }
+          case ZipCompression:
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,
+              "FlateDecode");
+            break;
+          }
+          default:
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,CFormat,
+              "RunLengthDecode");
+            break;
+          }
+        }
+        (void) WriteBlobString(image,buffer);
+        (void) FormatMagickString(buffer,MaxTextExtent,"/Width %lu\n",
+          image->columns);
+        (void) WriteBlobString(image,buffer);
+        (void) FormatMagickString(buffer,MaxTextExtent,"/Height %lu\n",
+          image->rows);
+        (void) WriteBlobString(image,buffer);
+        (void) WriteBlobString(image,"/ColorSpace /DeviceGray\n");
+        (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
+          (compression == FaxCompression) || (compression == Group4Compression)
+          ? 1 : 8);
+        (void) WriteBlobString(image,buffer);
+        (void) FormatMagickString(buffer,MaxTextExtent,"/Length %lu 0 R\n",
+          object+1);
+        (void) WriteBlobString(image,buffer);
+        (void) WriteBlobString(image,">>\n");
+        (void) WriteBlobString(image,"stream\n");
+        offset=TellBlob(image);
+        number_pixels=(MagickSizeType) image->columns*image->rows;
+        switch (compression)
+        {
+          case RLECompression:
+          default:
+          {
+            /*
+              Allocate pixel array.
+            */
+            length=(size_t) number_pixels;
+            pixels=(unsigned char *) AcquireMagickMemory(length*
+              sizeof(*pixels));
+            if (pixels == (unsigned char *) NULL)
+              {
+                image=DestroyImage(image);
+                ThrowWriterException(ResourceLimitError,
+                  "MemoryAllocationFailed");
+              }
+            /*
+              Dump Runlength encoded pixels.
+            */
+            q=pixels;
+            for (y=0; y < (long) image->rows; y++)
+            {
+              p=AcquireImagePixels(image,0,y,image->columns,1,
+                &image->exception);
+              if (p == (const PixelPacket *) NULL)
+                break;
+              for (x=0; x < (long) image->columns; x++)
+              {
+                *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
+                p++;
+              }
+            }
+#if defined(HasZLIB)
+            if (compression == ZipCompression)
+              status=ZLIBEncodeImage(image,length,pixels);
+            else
+#endif
+              if (compression == LZWCompression)
+                status=LZWEncodeImage(image,length,pixels);
+              else
+                status=PackbitsEncodeImage(image,length,pixels);
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            if (status == MagickFalse)
+              {
+                CloseBlob(image);
+                return(MagickFalse);
+              }
+            break;
+          }
+          case NoCompression:
+          {
+            /*
+              Dump uncompressed PseudoColor packets.
+            */
+            Ascii85Initialize(image);
+            for (y=0; y < (long) image->rows; y++)
+            {
+              p=AcquireImagePixels(image,0,y,image->columns,1,
+                &image->exception);
+              if (p == (const PixelPacket *) NULL)
+                break;
+              for (x=0; x < (long) image->columns; x++)
+              {
+                Ascii85Encode(image,ScaleQuantumToChar((Quantum) (QuantumRange-
+                  p->opacity)));
+                p++;
+              }
+            }
+            Ascii85Flush(image);
+            break;
+          }
+        }
+        offset=TellBlob(image)-offset;
+        (void) WriteBlobString(image,"\nendstream\n");
+      }
+    (void) WriteBlobString(image,"endobj\n");
+    /*
+      Write Length object.
+    */
+    xref[object++]=TellBlob(image);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
+    (void) WriteBlobString(image,buffer);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%lu\n",(unsigned long)
+      offset);
+    (void) WriteBlobString(image,buffer);
+    (void) WriteBlobString(image,"endobj\n");
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
