@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -57,6 +57,7 @@
 #include "magick/monitor.h"
 #include "magick/profile.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -135,168 +136,168 @@ typedef struct _PICTRectangle
 static const PICTCode
   codes[] =
   {
-    /* 0x00 */ { (char *) "NOP", 0, (char *) "nop" },
-    /* 0x01 */ { (char *) "Clip", 0, (char *) "clip" },
-    /* 0x02 */ { (char *) "BkPat", 8, (char *) "background pattern" },
-    /* 0x03 */ { (char *) "TxFont", 2, (char *) "text font (word)" },
-    /* 0x04 */ { (char *) "TxFace", 1, (char *) "text face (byte)" },
-    /* 0x05 */ { (char *) "TxMode", 2, (char *) "text mode (word)" },
-    /* 0x06 */ { (char *) "SpExtra", 4, (char *) "space extra (fixed point)" },
-    /* 0x07 */ { (char *) "PnSize", 4, (char *) "pen size (point)" },
-    /* 0x08 */ { (char *) "PnMode", 2, (char *) "pen mode (word)" },
-    /* 0x09 */ { (char *) "PnPat", 8, (char *) "pen pattern" },
-    /* 0x0a */ { (char *) "FillPat", 8, (char *) "fill pattern" },
-    /* 0x0b */ { (char *) "OvSize", 4, (char *) "oval size (point)" },
-    /* 0x0c */ { (char *) "Origin", 4, (char *) "dh, dv (word)" },
-    /* 0x0d */ { (char *) "TxSize", 2, (char *) "text size (word)" },
-    /* 0x0e */ { (char *) "FgColor", 4, (char *) "foreground color (longword)" },
-    /* 0x0f */ { (char *) "BkColor", 4, (char *) "background color (longword)" },
-    /* 0x10 */ { (char *) "TxRatio", 8, (char *) "numerator (point), denominator (point)" },
-    /* 0x11 */ { (char *) "Version", 1, (char *) "version (byte)" },
-    /* 0x12 */ { (char *) "BkPixPat", 0, (char *) "color background pattern" },
-    /* 0x13 */ { (char *) "PnPixPat", 0, (char *) "color pen pattern" },
-    /* 0x14 */ { (char *) "FillPixPat", 0, (char *) "color fill pattern" },
-    /* 0x15 */ { (char *) "PnLocHFrac", 2, (char *) "fractional pen position" },
-    /* 0x16 */ { (char *) "ChExtra", 2, (char *) "extra for each character" },
-    /* 0x17 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x18 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x19 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x1a */ { (char *) "RGBFgCol", 6, (char *) "RGB foreColor" },
-    /* 0x1b */ { (char *) "RGBBkCol", 6, (char *) "RGB backColor" },
-    /* 0x1c */ { (char *) "HiliteMode", 0, (char *) "hilite mode flag" },
-    /* 0x1d */ { (char *) "HiliteColor", 6, (char *) "RGB hilite color" },
-    /* 0x1e */ { (char *) "DefHilite", 0, (char *) "Use default hilite color" },
-    /* 0x1f */ { (char *) "OpColor", 6, (char *) "RGB OpColor for arithmetic modes" },
-    /* 0x20 */ { (char *) "Line", 8, (char *) "pnLoc (point), newPt (point)" },
-    /* 0x21 */ { (char *) "LineFrom", 4, (char *) "newPt (point)" },
-    /* 0x22 */ { (char *) "ShortLine", 6, (char *) "pnLoc (point, dh, dv (-128 .. 127))" },
-    /* 0x23 */ { (char *) "ShortLineFrom", 2, (char *) "dh, dv (-128 .. 127)" },
-    /* 0x24 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x25 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x26 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x27 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x28 */ { (char *) "LongText", 0, (char *) "txLoc (point), count (0..255), text" },
-    /* 0x29 */ { (char *) "DHText", 0, (char *) "dh (0..255), count (0..255), text" },
-    /* 0x2a */ { (char *) "DVText", 0, (char *) "dv (0..255), count (0..255), text" },
-    /* 0x2b */ { (char *) "DHDVText", 0, (char *) "dh, dv (0..255), count (0..255), text" },
-    /* 0x2c */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x2d */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x2e */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x2f */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x30 */ { (char *) "frameRect", 8, (char *) "rect" },
-    /* 0x31 */ { (char *) "paintRect", 8, (char *) "rect" },
-    /* 0x32 */ { (char *) "eraseRect", 8, (char *) "rect" },
-    /* 0x33 */ { (char *) "invertRect", 8, (char *) "rect" },
-    /* 0x34 */ { (char *) "fillRect", 8, (char *) "rect" },
-    /* 0x35 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x36 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x37 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x38 */ { (char *) "frameSameRect", 0, (char *) "rect" },
-    /* 0x39 */ { (char *) "paintSameRect", 0, (char *) "rect" },
-    /* 0x3a */ { (char *) "eraseSameRect", 0, (char *) "rect" },
-    /* 0x3b */ { (char *) "invertSameRect", 0, (char *) "rect" },
-    /* 0x3c */ { (char *) "fillSameRect", 0, (char *) "rect" },
-    /* 0x3d */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x3e */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x3f */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x40 */ { (char *) "frameRRect", 8, (char *) "rect" },
-    /* 0x41 */ { (char *) "paintRRect", 8, (char *) "rect" },
-    /* 0x42 */ { (char *) "eraseRRect", 8, (char *) "rect" },
-    /* 0x43 */ { (char *) "invertRRect", 8, (char *) "rect" },
-    /* 0x44 */ { (char *) "fillRRrect", 8, (char *) "rect" },
-    /* 0x45 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x46 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x47 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x48 */ { (char *) "frameSameRRect", 0, (char *) "rect" },
-    /* 0x49 */ { (char *) "paintSameRRect", 0, (char *) "rect" },
-    /* 0x4a */ { (char *) "eraseSameRRect", 0, (char *) "rect" },
-    /* 0x4b */ { (char *) "invertSameRRect", 0, (char *) "rect" },
-    /* 0x4c */ { (char *) "fillSameRRect", 0, (char *) "rect" },
-    /* 0x4d */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x4e */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x4f */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x50 */ { (char *) "frameOval", 8, (char *) "rect" },
-    /* 0x51 */ { (char *) "paintOval", 8, (char *) "rect" },
-    /* 0x52 */ { (char *) "eraseOval", 8, (char *) "rect" },
-    /* 0x53 */ { (char *) "invertOval", 8, (char *) "rect" },
-    /* 0x54 */ { (char *) "fillOval", 8, (char *) "rect" },
-    /* 0x55 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x56 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x57 */ { (char *) "reserved", 8, (char *) "reserved for Apple use" },
-    /* 0x58 */ { (char *) "frameSameOval", 0, (char *) "rect" },
-    /* 0x59 */ { (char *) "paintSameOval", 0, (char *) "rect" },
-    /* 0x5a */ { (char *) "eraseSameOval", 0, (char *) "rect" },
-    /* 0x5b */ { (char *) "invertSameOval", 0, (char *) "rect" },
-    /* 0x5c */ { (char *) "fillSameOval", 0, (char *) "rect" },
-    /* 0x5d */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x5e */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x5f */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x60 */ { (char *) "frameArc", 12, (char *) "rect, startAngle, arcAngle" },
-    /* 0x61 */ { (char *) "paintArc", 12, (char *) "rect, startAngle, arcAngle" },
-    /* 0x62 */ { (char *) "eraseArc", 12, (char *) "rect, startAngle, arcAngle" },
-    /* 0x63 */ { (char *) "invertArc", 12, (char *) "rect, startAngle, arcAngle" },
-    /* 0x64 */ { (char *) "fillArc", 12, (char *) "rect, startAngle, arcAngle" },
-    /* 0x65 */ { (char *) "reserved", 12, (char *) "reserved for Apple use" },
-    /* 0x66 */ { (char *) "reserved", 12, (char *) "reserved for Apple use" },
-    /* 0x67 */ { (char *) "reserved", 12, (char *) "reserved for Apple use" },
-    /* 0x68 */ { (char *) "frameSameArc", 4, (char *) "rect, startAngle, arcAngle" },
-    /* 0x69 */ { (char *) "paintSameArc", 4, (char *) "rect, startAngle, arcAngle" },
-    /* 0x6a */ { (char *) "eraseSameArc", 4, (char *) "rect, startAngle, arcAngle" },
-    /* 0x6b */ { (char *) "invertSameArc", 4, (char *) "rect, startAngle, arcAngle" },
-    /* 0x6c */ { (char *) "fillSameArc", 4, (char *) "rect, startAngle, arcAngle" },
-    /* 0x6d */ { (char *) "reserved", 4, (char *) "reserved for Apple use" },
-    /* 0x6e */ { (char *) "reserved", 4, (char *) "reserved for Apple use" },
-    /* 0x6f */ { (char *) "reserved", 4, (char *) "reserved for Apple use" },
-    /* 0x70 */ { (char *) "framePoly", 0, (char *) "poly" },
-    /* 0x71 */ { (char *) "paintPoly", 0, (char *) "poly" },
-    /* 0x72 */ { (char *) "erasePoly", 0, (char *) "poly" },
-    /* 0x73 */ { (char *) "invertPoly", 0, (char *) "poly" },
-    /* 0x74 */ { (char *) "fillPoly", 0, (char *) "poly" },
-    /* 0x75 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x76 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x77 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x78 */ { (char *) "frameSamePoly", 0, (char *) "poly (NYI)" },
-    /* 0x79 */ { (char *) "paintSamePoly", 0, (char *) "poly (NYI)" },
-    /* 0x7a */ { (char *) "eraseSamePoly", 0, (char *) "poly (NYI)" },
-    /* 0x7b */ { (char *) "invertSamePoly", 0, (char *) "poly (NYI)" },
-    /* 0x7c */ { (char *) "fillSamePoly", 0, (char *) "poly (NYI)" },
-    /* 0x7d */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x7e */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x7f */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x80 */ { (char *) "frameRgn", 0, (char *) "region" },
-    /* 0x81 */ { (char *) "paintRgn", 0, (char *) "region" },
-    /* 0x82 */ { (char *) "eraseRgn", 0, (char *) "region" },
-    /* 0x83 */ { (char *) "invertRgn", 0, (char *) "region" },
-    /* 0x84 */ { (char *) "fillRgn", 0, (char *) "region" },
-    /* 0x85 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x86 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x87 */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x88 */ { (char *) "frameSameRgn", 0, (char *) "region (NYI)" },
-    /* 0x89 */ { (char *) "paintSameRgn", 0, (char *) "region (NYI)" },
-    /* 0x8a */ { (char *) "eraseSameRgn", 0, (char *) "region (NYI)" },
-    /* 0x8b */ { (char *) "invertSameRgn", 0, (char *) "region (NYI)" },
-    /* 0x8c */ { (char *) "fillSameRgn", 0, (char *) "region (NYI)" },
-    /* 0x8d */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x8e */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x8f */ { (char *) "reserved", 0, (char *) "reserved for Apple use" },
-    /* 0x90 */ { (char *) "BitsRect", 0, (char *) "copybits, rect clipped" },
-    /* 0x91 */ { (char *) "BitsRgn", 0, (char *) "copybits, rgn clipped" },
-    /* 0x92 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x93 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x94 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x95 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x96 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x97 */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x98 */ { (char *) "PackBitsRect", 0, (char *) "packed copybits, rect clipped" },
-    /* 0x99 */ { (char *) "PackBitsRgn", 0, (char *) "packed copybits, rgn clipped" },
-    /* 0x9a */ { (char *) "DirectBitsRect", 0, (char *) "PixMap, srcRect, dstRect, mode, PixData" },
-    /* 0x9b */ { (char *) "DirectBitsRgn", 0, (char *) "PixMap, srcRect, dstRect, mode, maskRgn, PixData" },
-    /* 0x9c */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x9d */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x9e */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0x9f */ { (char *) "reserved", -1, (char *) "reserved for Apple use" },
-    /* 0xa0 */ { (char *) "ShortComment", 2, (char *) "kind (word)" },
-    /* 0xa1 */ { (char *) "LongComment", 0, (char *) "kind (word), size (word), data" }
+    /* 0x00 */ { "NOP", 0, "nop" },
+    /* 0x01 */ { "Clip", 0, "clip" },
+    /* 0x02 */ { "BkPat", 8, "background pattern" },
+    /* 0x03 */ { "TxFont", 2, "text font (word)" },
+    /* 0x04 */ { "TxFace", 1, "text face (byte)" },
+    /* 0x05 */ { "TxMode", 2, "text mode (word)" },
+    /* 0x06 */ { "SpExtra", 4, "space extra (fixed point)" },
+    /* 0x07 */ { "PnSize", 4, "pen size (point)" },
+    /* 0x08 */ { "PnMode", 2, "pen mode (word)" },
+    /* 0x09 */ { "PnPat", 8, "pen pattern" },
+    /* 0x0a */ { "FillPat", 8, "fill pattern" },
+    /* 0x0b */ { "OvSize", 4, "oval size (point)" },
+    /* 0x0c */ { "Origin", 4, "dh, dv (word)" },
+    /* 0x0d */ { "TxSize", 2, "text size (word)" },
+    /* 0x0e */ { "FgColor", 4, "foreground color (longword)" },
+    /* 0x0f */ { "BkColor", 4, "background color (longword)" },
+    /* 0x10 */ { "TxRatio", 8, "numerator (point), denominator (point)" },
+    /* 0x11 */ { "Version", 1, "version (byte)" },
+    /* 0x12 */ { "BkPixPat", 0, "color background pattern" },
+    /* 0x13 */ { "PnPixPat", 0, "color pen pattern" },
+    /* 0x14 */ { "FillPixPat", 0, "color fill pattern" },
+    /* 0x15 */ { "PnLocHFrac", 2, "fractional pen position" },
+    /* 0x16 */ { "ChExtra", 2, "extra for each character" },
+    /* 0x17 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x18 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x19 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x1a */ { "RGBFgCol", 6, "RGB foreColor" },
+    /* 0x1b */ { "RGBBkCol", 6, "RGB backColor" },
+    /* 0x1c */ { "HiliteMode", 0, "hilite mode flag" },
+    /* 0x1d */ { "HiliteColor", 6, "RGB hilite color" },
+    /* 0x1e */ { "DefHilite", 0, "Use default hilite color" },
+    /* 0x1f */ { "OpColor", 6, "RGB OpColor for arithmetic modes" },
+    /* 0x20 */ { "Line", 8, "pnLoc (point), newPt (point)" },
+    /* 0x21 */ { "LineFrom", 4, "newPt (point)" },
+    /* 0x22 */ { "ShortLine", 6, "pnLoc (point, dh, dv (-128 .. 127))" },
+    /* 0x23 */ { "ShortLineFrom", 2, "dh, dv (-128 .. 127)" },
+    /* 0x24 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x25 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x26 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x27 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x28 */ { "LongText", 0, "txLoc (point), count (0..255), text" },
+    /* 0x29 */ { "DHText", 0, "dh (0..255), count (0..255), text" },
+    /* 0x2a */ { "DVText", 0, "dv (0..255), count (0..255), text" },
+    /* 0x2b */ { "DHDVText", 0, "dh, dv (0..255), count (0..255), text" },
+    /* 0x2c */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x2d */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x2e */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x2f */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x30 */ { "frameRect", 8, "rect" },
+    /* 0x31 */ { "paintRect", 8, "rect" },
+    /* 0x32 */ { "eraseRect", 8, "rect" },
+    /* 0x33 */ { "invertRect", 8, "rect" },
+    /* 0x34 */ { "fillRect", 8, "rect" },
+    /* 0x35 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x36 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x37 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x38 */ { "frameSameRect", 0, "rect" },
+    /* 0x39 */ { "paintSameRect", 0, "rect" },
+    /* 0x3a */ { "eraseSameRect", 0, "rect" },
+    /* 0x3b */ { "invertSameRect", 0, "rect" },
+    /* 0x3c */ { "fillSameRect", 0, "rect" },
+    /* 0x3d */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x3e */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x3f */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x40 */ { "frameRRect", 8, "rect" },
+    /* 0x41 */ { "paintRRect", 8, "rect" },
+    /* 0x42 */ { "eraseRRect", 8, "rect" },
+    /* 0x43 */ { "invertRRect", 8, "rect" },
+    /* 0x44 */ { "fillRRrect", 8, "rect" },
+    /* 0x45 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x46 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x47 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x48 */ { "frameSameRRect", 0, "rect" },
+    /* 0x49 */ { "paintSameRRect", 0, "rect" },
+    /* 0x4a */ { "eraseSameRRect", 0, "rect" },
+    /* 0x4b */ { "invertSameRRect", 0, "rect" },
+    /* 0x4c */ { "fillSameRRect", 0, "rect" },
+    /* 0x4d */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x4e */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x4f */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x50 */ { "frameOval", 8, "rect" },
+    /* 0x51 */ { "paintOval", 8, "rect" },
+    /* 0x52 */ { "eraseOval", 8, "rect" },
+    /* 0x53 */ { "invertOval", 8, "rect" },
+    /* 0x54 */ { "fillOval", 8, "rect" },
+    /* 0x55 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x56 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x57 */ { "reserved", 8, "reserved for Apple use" },
+    /* 0x58 */ { "frameSameOval", 0, "rect" },
+    /* 0x59 */ { "paintSameOval", 0, "rect" },
+    /* 0x5a */ { "eraseSameOval", 0, "rect" },
+    /* 0x5b */ { "invertSameOval", 0, "rect" },
+    /* 0x5c */ { "fillSameOval", 0, "rect" },
+    /* 0x5d */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x5e */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x5f */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x60 */ { "frameArc", 12, "rect, startAngle, arcAngle" },
+    /* 0x61 */ { "paintArc", 12, "rect, startAngle, arcAngle" },
+    /* 0x62 */ { "eraseArc", 12, "rect, startAngle, arcAngle" },
+    /* 0x63 */ { "invertArc", 12, "rect, startAngle, arcAngle" },
+    /* 0x64 */ { "fillArc", 12, "rect, startAngle, arcAngle" },
+    /* 0x65 */ { "reserved", 12, "reserved for Apple use" },
+    /* 0x66 */ { "reserved", 12, "reserved for Apple use" },
+    /* 0x67 */ { "reserved", 12, "reserved for Apple use" },
+    /* 0x68 */ { "frameSameArc", 4, "rect, startAngle, arcAngle" },
+    /* 0x69 */ { "paintSameArc", 4, "rect, startAngle, arcAngle" },
+    /* 0x6a */ { "eraseSameArc", 4, "rect, startAngle, arcAngle" },
+    /* 0x6b */ { "invertSameArc", 4, "rect, startAngle, arcAngle" },
+    /* 0x6c */ { "fillSameArc", 4, "rect, startAngle, arcAngle" },
+    /* 0x6d */ { "reserved", 4, "reserved for Apple use" },
+    /* 0x6e */ { "reserved", 4, "reserved for Apple use" },
+    /* 0x6f */ { "reserved", 4, "reserved for Apple use" },
+    /* 0x70 */ { "framePoly", 0, "poly" },
+    /* 0x71 */ { "paintPoly", 0, "poly" },
+    /* 0x72 */ { "erasePoly", 0, "poly" },
+    /* 0x73 */ { "invertPoly", 0, "poly" },
+    /* 0x74 */ { "fillPoly", 0, "poly" },
+    /* 0x75 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x76 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x77 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x78 */ { "frameSamePoly", 0, "poly (NYI)" },
+    /* 0x79 */ { "paintSamePoly", 0, "poly (NYI)" },
+    /* 0x7a */ { "eraseSamePoly", 0, "poly (NYI)" },
+    /* 0x7b */ { "invertSamePoly", 0, "poly (NYI)" },
+    /* 0x7c */ { "fillSamePoly", 0, "poly (NYI)" },
+    /* 0x7d */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x7e */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x7f */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x80 */ { "frameRgn", 0, "region" },
+    /* 0x81 */ { "paintRgn", 0, "region" },
+    /* 0x82 */ { "eraseRgn", 0, "region" },
+    /* 0x83 */ { "invertRgn", 0, "region" },
+    /* 0x84 */ { "fillRgn", 0, "region" },
+    /* 0x85 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x86 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x87 */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x88 */ { "frameSameRgn", 0, "region (NYI)" },
+    /* 0x89 */ { "paintSameRgn", 0, "region (NYI)" },
+    /* 0x8a */ { "eraseSameRgn", 0, "region (NYI)" },
+    /* 0x8b */ { "invertSameRgn", 0, "region (NYI)" },
+    /* 0x8c */ { "fillSameRgn", 0, "region (NYI)" },
+    /* 0x8d */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x8e */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x8f */ { "reserved", 0, "reserved for Apple use" },
+    /* 0x90 */ { "BitsRect", 0, "copybits, rect clipped" },
+    /* 0x91 */ { "BitsRgn", 0, "copybits, rgn clipped" },
+    /* 0x92 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x93 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x94 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x95 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x96 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x97 */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x98 */ { "PackBitsRect", 0, "packed copybits, rect clipped" },
+    /* 0x99 */ { "PackBitsRgn", 0, "packed copybits, rgn clipped" },
+    /* 0x9a */ { "DirectBitsRect", 0, "PixMap, srcRect, dstRect, mode, PixData" },
+    /* 0x9b */ { "DirectBitsRgn", 0, "PixMap, srcRect, dstRect, mode, maskRgn, PixData" },
+    /* 0x9c */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x9d */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x9e */ { "reserved", -1, "reserved for Apple use" },
+    /* 0x9f */ { "reserved", -1, "reserved for Apple use" },
+    /* 0xa0 */ { "ShortComment", 2, "kind (word)" },
+    /* 0xa1 */ { "LongComment", 0, "kind (word), size (word), data" }
   };
 
 /*
@@ -462,11 +463,12 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
   /*
     Allocate pixel and scanline buffer.
   */
-  pixels=(unsigned char *) AcquireMagickMemory(row_bytes*image->rows);
+  pixels=(unsigned char *) AcquireMagickMemory(row_bytes*image->rows*
+    sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     return((unsigned char *) NULL);
   (void) ResetMagickMemory(pixels,0,row_bytes*image->rows);
-  scanline=(unsigned char *) AcquireMagickMemory(row_bytes);
+  scanline=(unsigned char *) AcquireMagickMemory(row_bytes*sizeof(scanline));
   if (scanline == (unsigned char *) NULL)
     return((unsigned char *) NULL);
   if (bytes_per_line < 8)
@@ -593,7 +595,7 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(scanline != (unsigned char *) NULL);
   assert(pixels != (unsigned char *) NULL);
   count=0;
@@ -674,12 +676,12 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   length=(q-pixels);
   if (bytes_per_line > 200)
     {
-      (void) WriteBlobMSBShort(image,length);
+      (void) WriteBlobMSBShort(image,(unsigned short) length);
       length+=2;
     }
   else
     {
-      (void) WriteBlobByte(image,(long) length);
+      (void) WriteBlobByte(image,(unsigned char) length);
       length++;
     }
   while (q != pixels)
@@ -814,7 +816,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -1033,12 +1036,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               return((Image *) NULL);
             DestroyBlob(tile_image);
             tile_image->blob=CloneBlobInfo((BlobInfo *) NULL);
-            if ((code == 0x9a) || (code == 0x9b) || (bytes_per_line & 0x8000))
+            if ((code == 0x9a) || (code == 0x9b) ||
+                ((bytes_per_line & 0x8000) != 0))
               {
                 ReadPixmap(pixmap);
                 tile_image->depth=pixmap.component_size;
-                tile_image->matte=
-                  pixmap.component_count == 4 ? MagickTrue : MagickFalse;
+                tile_image->matte=pixmap.component_count == 4 ?
+                  MagickTrue : MagickFalse;
                 if (tile_image->matte != MagickFalse)
                   image->matte=tile_image->matte;
               }
@@ -1080,11 +1084,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   {
                     for (i=0; i < (long) tile_image->colors; i++)
                     {
-                      tile_image->colormap[i].red=(Quantum) (MaxRGB-
+                      tile_image->colormap[i].red=(Quantum) (QuantumRange-
                         tile_image->colormap[i].red);
-                      tile_image->colormap[i].green=(Quantum) (MaxRGB-
+                      tile_image->colormap[i].green=(Quantum) (QuantumRange-
                         tile_image->colormap[i].green);
-                      tile_image->colormap[i].blue=(Quantum) (MaxRGB-
+                      tile_image->colormap[i].blue=(Quantum) (QuantumRange-
                         tile_image->colormap[i].blue);
                     }
                   }
@@ -1139,28 +1143,31 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       {
                         i=(*p++);
                         j=(*p);
-                        q->red=ScaleCharToQuantum((i & 0x7c) << 1);
-                        q->green=ScaleCharToQuantum(((i & 0x03) << 6) |
-                          ((j & 0xe0) >> 2));
-                        q->blue=ScaleCharToQuantum((j & 0x1f) << 3);
+                        q->red=ScaleCharToQuantum((unsigned char)
+                          ((i & 0x7c) << 1));
+                        q->green=ScaleCharToQuantum((unsigned char)
+                          (((i & 0x03) << 6) | ((j & 0xe0) >> 2)));
+                        q->blue=ScaleCharToQuantum((unsigned char)
+                          ((j & 0x1f) << 3));
                       }
                     else
                       if (tile_image->matte == MagickFalse)
                         {
                           q->red=ScaleCharToQuantum(*p);
-                          q->green=
-                            ScaleCharToQuantum(*(p+tile_image->columns));
-                          q->blue=
-                            ScaleCharToQuantum(*(p+2*tile_image->columns));
+                          q->green=ScaleCharToQuantum(
+                            *(p+tile_image->columns));
+                          q->blue=ScaleCharToQuantum(
+                            *(p+2*tile_image->columns));
                         }
                       else
                         {
-                          q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p));
+                          q->opacity=(Quantum) (QuantumRange-
+                            ScaleCharToQuantum(*p));
                           q->red=ScaleCharToQuantum(*(p+tile_image->columns));
-                          q->green=(Quantum)
-                            ScaleCharToQuantum(*(p+2*tile_image->columns));
-                          q->blue=
-                            ScaleCharToQuantum(*(p+3*tile_image->columns));
+                          q->green=(Quantum) ScaleCharToQuantum(
+                            *(p+2*tile_image->columns));
+                          q->blue=ScaleCharToQuantum(
+                            *(p+3*tile_image->columns));
                         }
                   }
                 p++;
@@ -1333,6 +1340,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           Max(image->columns,tile_image->columns),
           Max(image->rows,tile_image->rows));
         (void) TransformImage(&image,(char *) NULL,geometry);
+        if (image_info->colorspace == UndefinedColorspace)
+          SetImageColorspace(image,tile_image->colorspace);
         (void) CompositeImage(image,CopyCompositeOp,tile_image,frame.left,
           frame.right);
         image->compression=tile_image->compression;
@@ -1400,17 +1409,17 @@ ModuleExport void RegisterPICTImage(void)
   entry->decoder=(DecoderHandler *) ReadPICTImage;
   entry->encoder=(EncoderHandler *) WritePICTImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
+  entry->description=ConstantString("Apple Macintosh QuickDraw/PICT");
   entry->magick=(MagickHandler *) IsPICT;
-  entry->module=AcquireString("PICT");
+  entry->module=ConstantString("PICT");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PICT");
   entry->decoder=(DecoderHandler *) ReadPICTImage;
   entry->encoder=(EncoderHandler *) WritePICTImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
+  entry->description=ConstantString("Apple Macintosh QuickDraw/PICT");
   entry->magick=(MagickHandler *) IsPICT;
-  entry->module=AcquireString("PICT");
+  entry->module=ConstantString("PICT");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1478,6 +1487,9 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
 #define PictPICTOp  0x98
 #define PictVersion  0x11
 
+  const StringInfo
+    *profile;
+
   double
     x_resolution,
     y_resolution;
@@ -1515,9 +1527,6 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   size_t
     count;
 
-  StringInfo
-    *profile;
-
   unsigned char
     *buffer,
     *packed_scanline,
@@ -1540,13 +1549,14 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->columns > 65535L) || (image->rows > 65535L))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize image info.
   */
@@ -1635,7 +1645,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0x1f2);
-      (void) WriteBlobMSBShort(image,profile->length+4);
+      (void) WriteBlobMSBShort(image,(unsigned short) (profile->length+4));
       (void) WriteBlobString(image,"8BIM");
       (void) WriteBlob(image,profile->length,profile->datum);
     }
@@ -1644,7 +1654,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
-      (void) WriteBlobMSBShort(image,profile->length+4);
+      (void) WriteBlobMSBShort(image,(unsigned short) (profile->length+4));
       (void) WriteBlobMSBLong(image,0x00000000UL);
       (void) WriteBlob(image,profile->length,profile->datum);
       (void) WriteBlobMSBShort(image,0xa1);
@@ -1684,7 +1694,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       DestroyBlob(jpeg_image);
       jpeg_image->blob=CloneBlobInfo((BlobInfo *) NULL);
       jpeg_info=CloneImageInfo(image_info);
-      (void) strcpy(jpeg_info->magick,"JPEG");
+      (void) CopyMagickString(jpeg_info->magick,"JPEG",MaxTextExtent);
       length=0;
       blob=(unsigned char *) ImageToBlob(jpeg_info,jpeg_image,&length,
         &image->exception);
@@ -1709,8 +1719,8 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBLong(image,0x00000000UL);
       (void) WriteBlobMSBLong(image,0x00400000UL);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBShort(image,image->rows);
-      (void) WriteBlobMSBShort(image,image->columns);
+      (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
+      (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,768);
       (void) WriteBlobMSBShort(image,0x0000);
@@ -1723,8 +1733,8 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBLong(image,0x706C0000UL);
       (void) WriteBlobMSBLong(image,0x00000000UL);
       (void) WriteBlobMSBShort(image,768);
-      (void) WriteBlobMSBShort(image,image->columns);
-      (void) WriteBlobMSBShort(image,image->rows);
+      (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
+      (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
       (void) WriteBlobMSBShort(image,(unsigned short) x_resolution);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) y_resolution);
@@ -1789,13 +1799,13 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,(unsigned short) (image->colors-1));
       for (i=0; i < (long) image->colors; i++)
       {
-        (void) WriteBlobMSBShort(image,i);
-        (void) WriteBlobMSBShort(image,
-          ScaleQuantumToShort(image->colormap[i].red));
-        (void) WriteBlobMSBShort(image,
-          ScaleQuantumToShort(image->colormap[i].green));
-        (void) WriteBlobMSBShort(image,
-          ScaleQuantumToShort(image->colormap[i].blue));
+        (void) WriteBlobMSBShort(image,(unsigned short) i);
+        (void) WriteBlobMSBShort(image,ScaleQuantumToShort(
+          image->colormap[i].red));
+        (void) WriteBlobMSBShort(image,ScaleQuantumToShort(
+          image->colormap[i].green));
+        (void) WriteBlobMSBShort(image,ScaleQuantumToShort(
+          image->colormap[i].blue));
       }
     }
   /*
@@ -1873,7 +1883,8 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
             *green++=ScaleQuantumToChar(p->green);
             *blue++=ScaleQuantumToChar(p->blue);
             if (image->matte != MagickFalse)
-              *opacity++=ScaleQuantumToChar(MaxRGB-p->opacity);
+              *opacity++=ScaleQuantumToChar((Quantum)
+                (QuantumRange-p->opacity));
             p++;
           }
           count+=EncodeImage(image,scanline,bytes_per_line & 0x7FFF,
@@ -1893,7 +1904,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   (void) WriteBlobMSBShort(image,PictEndOfPictureOp);
   offset=TellBlob(image);
   (void) SeekBlob(image,512,SEEK_SET);
-  (void) WriteBlobMSBShort(image,(unsigned long) offset);
+  (void) WriteBlobMSBShort(image,(unsigned short) offset);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   packed_scanline=(unsigned char *) RelinquishMagickMemory(packed_scanline);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
