@@ -132,8 +132,6 @@ static MagickBooleanType IsPNM(const unsigned char *magick,const size_t length)
 
 static unsigned long PNMInteger(Image *image,const unsigned int base)
 {
-#define P7Comment  "END_OF_COMMENTS\n"
-
   char
     *comment;
 
@@ -190,12 +188,6 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
   } while (isdigit(c) == MagickFalse);
   if (comment != (char *) NULL)
     {
-      if (strlen(comment) > strlen(P7Comment))
-        {
-          p-=strlen(P7Comment);
-          if (LocaleCompare(p,P7Comment) == 0)
-            *p='\0';
-        }
       (void) SetImageAttribute(image,"Comment",comment);
       comment=(char *) RelinquishMagickMemory(comment);
     }
@@ -356,7 +348,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           while (isspace((int) ((unsigned char) c)) != 0)
             c=ReadBlobByte(image);
           p=value;
-          while (isalnum(c))
+          while (isalnum(c) || (c == '_'))
           {
             if ((size_t) (p-value) < (MaxTextExtent/2))
               *p++=c;
@@ -390,6 +382,10 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 }
               if (LocaleCompare(value,"RGB_ALPHA") == 0)
                 image->matte=MagickTrue;
+              if (LocaleCompare(value,"CMYK") == 0)
+                image->colorspace=CMYKColorspace;
+              if (LocaleCompare(value,"CMYK_ALPHA") == 0)
+                image->matte=MagickTrue;
             }
           if (LocaleCompare(keyword,"width") == 0)
             image->columns=(unsigned long) atol(value);
@@ -668,10 +664,13 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Convert PNM raster image to pixel packets.
         */
         packet_size=(size_t) (image->depth <= 8 ? 3 : 6);
-        pixels=(unsigned char *)
-          AcquireMagickMemory(packet_size*image->columns*sizeof(*pixels));
+        if (image->colorspace == CMYKColorspace)
+          packet_size+=(size_t) (image->depth <= 8 ? 1 : 2);
+        pixels=(unsigned char *) AcquireMagickMemory(packet_size*
+          image->columns*sizeof(*pixels));
         if (pixels == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) ResetMagickMemory(&pixel,0,sizeof(LongPixelPacket));
         for (y=0; y < (long) image->rows; y++)
         {
           count=ReadBlob(image,packet_size*image->columns,pixels);
@@ -735,6 +734,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
       case '7':
       {
+        IndexPacket
+          *indexes;
+
         unsigned long
           pixel;
 
@@ -746,6 +748,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           packet_size=1;
         if (image->matte != MagickFalse)
           packet_size++;
+        if (image->colorspace == CMYKColorspace)
+          packet_size++;
         if (image->depth > 8)
           packet_size*=2;
         length=packet_size*image->columns*sizeof(*pixels);
@@ -761,6 +765,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
+          indexes=GetIndexes(image);
           if (grayscale != MagickFalse)
             {
               if (image->depth <= 8)
@@ -770,10 +775,16 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   q->red=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
                   q->green=q->red;
                   q->blue=q->red;
+                  if (image->colorspace == CMYKColorspace)
+                    {
+                      PushCharPixel(pixel,p);
+                      indexes[x]=(IndexPacket) ScaleAnyToQuantum(pixel,
+                        (1UL << image->depth)-1);
+                    }
                   if (image->matte != MagickFalse)
                     {
                       PushCharPixel(pixel,p);
-                      q->opacity=ScaleAnyToQuantum(pixel,
+                      q->opacity=QuantumRange-ScaleAnyToQuantum(pixel,
                         (1UL << image->depth)-1);
                     }
                   q++;
@@ -785,10 +796,16 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   q->red=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
                   q->green=q->red;
                   q->blue=q->red;
+                  if (image->colorspace == CMYKColorspace)
+                    {
+                      PushShortPixel(pixel,p);
+                      indexes[x]=(IndexPacket) ScaleAnyToQuantum(pixel,
+                        (1UL << image->depth)-1);
+                    }
                   if (image->matte != MagickFalse)
                     {
                       PushShortPixel(pixel,p);
-                      q->opacity=ScaleAnyToQuantum(pixel,
+                      q->opacity=QuantumRange-ScaleAnyToQuantum(pixel,
                         (1UL << image->depth)-1);
                     }
                   q++;
@@ -804,10 +821,17 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 q->green=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
                 PushCharPixel(pixel,p);
                 q->blue=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                if (image->colorspace == CMYKColorspace)
+                  {
+                    PushCharPixel(pixel,p);
+                    indexes[x]=(IndexPacket) ScaleAnyToQuantum(pixel,
+                      (1UL << image->depth)-1);
+                  }
                 if (image->matte != MagickFalse)
                   {
                     PushCharPixel(pixel,p);
-                    q->opacity=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                    q->opacity=QuantumRange-ScaleAnyToQuantum(pixel,
+                      (1UL << image->depth)-1);
                   }
                 q++;
               }
@@ -820,10 +844,17 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 q->green=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
                 PushShortPixel(pixel,p);
                 q->blue=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                if (image->colorspace == CMYKColorspace)
+                  {
+                    PushShortPixel(pixel,p);
+                    indexes[x]=(IndexPacket) ScaleAnyToQuantum(pixel,
+                      (1UL << image->depth)-1);
+                  }
                 if (image->matte != MagickFalse)
                   {
                     PushShortPixel(pixel,p);
-                    q->opacity=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                    q->opacity=QuantumRange-ScaleAnyToQuantum(pixel,
+                      (1UL << image->depth)-1);
                   }
                 q++;
               }
@@ -922,13 +953,8 @@ ModuleExport void RegisterPNMImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("P7");
-  entry->decoder=(DecoderHandler *) ReadPNMImage;
-  entry->encoder=(EncoderHandler *) WritePNMImage;
-  entry->description=ConstantString("Xv thumbnail format");
-  entry->module=ConstantString("PNM");
-  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PAM");
+  entry->decoder=(DecoderHandler *) ReadPNMImage;
   entry->encoder=(EncoderHandler *) WritePNMImage;
   entry->description=ConstantString("Common 2-dimensional bitmap format");
   entry->module=ConstantString("PNM");
@@ -981,7 +1007,6 @@ ModuleExport void RegisterPNMImage(void)
 */
 ModuleExport void UnregisterPNMImage(void)
 {
-  (void) UnregisterMagickInfo("P7");
   (void) UnregisterMagickInfo("PAM");
   (void) UnregisterMagickInfo("PBM");
   (void) UnregisterMagickInfo("PGM");
@@ -1195,13 +1220,13 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PBM image.
         */
+        (void) SetImageType(image,PaletteType);
         (void) SetImageType(image,BilevelType);
-        polarity=(IndexPacket)
-          (PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2));
+        polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
+          (QuantumRange/2) ? 1 : 0);
         if (image->colors == 2)
-          polarity=(IndexPacket)
-            (PixelIntensityToQuantum(&image->colormap[0]) <
-             PixelIntensityToQuantum(&image->colormap[1]));
+          polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
+             PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
         i=0;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -1342,13 +1367,14 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PBM image.
         */
+        (void) SetImageType(image,PaletteType);
         (void) SetImageType(image,BilevelType);
-        polarity=(IndexPacket)
-          (PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2));
+        polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
+          (QuantumRange/2) ? 1 : 0);
         if (image->colors == 2)
           polarity=(IndexPacket)
             (PixelIntensityToQuantum(&image->colormap[0]) <
-             PixelIntensityToQuantum(&image->colormap[1]));
+             PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -1516,7 +1542,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
                   PopCharPixel(pixel,q);
                   if (image->matte != MagickFalse)
                     {
-                      pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                      pixel=ScaleQuantumToAny((Quantum) (QuantumRange-
+                        p->opacity),(1UL << depth)-1UL);
                       PopCharPixel(pixel,q);
                     }
                   p++;
@@ -1529,7 +1556,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
                   PopShortPixel(pixel,q);
                   if (image->matte != MagickFalse)
                     {
-                      pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                      pixel=ScaleQuantumToAny((Quantum) (QuantumRange-
+                        p->opacity),(1UL << depth)-1UL);
                       PopShortPixel(pixel,q);
                     }
                   p++;
@@ -1547,7 +1575,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
                 PopCharPixel(pixel,q);
                 if (image->matte != MagickFalse)
                   {
-                    pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                    pixel=ScaleQuantumToAny((Quantum) (QuantumRange-p->opacity),
+                      (1UL << depth)-1UL);
                     PopCharPixel(pixel,q);
                   }
                 p++;
@@ -1563,7 +1592,8 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
                 PopShortPixel(pixel,q);
                 if (image->matte != MagickFalse)
                   {
-                    pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                    pixel=ScaleQuantumToAny((Quantum) (QuantumRange-p->opacity),
+                      (1UL << depth)-1UL);
                     PopShortPixel(pixel,q);
                   }
                 p++;
