@@ -78,6 +78,103 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I n v o k e P o s t s r i p t D e l e g a t e                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  InvokePostscriptDelegate() executes the postscript interpreter with the
+%  specified command.
+%
+%  The format of the InvokePostscriptDelegate method is:
+%
+%      MagickBooleanType InvokePostscriptDelegate(
+%        const MagickBooleanType verbose,const char *command)
+%
+%  A description of each parameter follows:
+%
+%    o verbose: A value other than zero displays the command prior to
+%      executing it.
+%
+%    o command: The address of a character string containing the command to
+%      execute.
+%
+*/
+static MagickBooleanType InvokePostscriptDelegate(
+  const MagickBooleanType verbose,const char *command)
+{
+#if defined(HasGS) || defined(__WINDOWS__)
+  char
+    **argv;
+
+  const GhostscriptVectors
+    *gs_func;
+
+  gs_main_instance
+    *interpreter;
+
+  int
+    argc,
+    code,
+    status;
+
+  register long
+    i;
+
+#if defined(__WINDOWS__)
+  gs_func=NTGhostscriptDLLVectors();
+#else
+  GhostscriptVectors
+    gs_func_struct;
+
+  gs_func=(&gs_func_struct);
+  gs_func_struct.exit=gsapi_exit;
+  gs_func_struct.init_with_args=gsapi_init_with_args;
+  gs_func_struct.new_instance=gsapi_new_instance;
+  gs_func_struct.run_string=gsapi_run_string;
+  gs_func_struct.delete_instance=gsapi_delete_instance;
+#endif
+  if (gs_func == (GhostscriptVectors *) NULL)
+    return(SystemCommand(verbose,command) == 0 ? MagickFalse : MagickTrue);
+  if (verbose != MagickFalse)
+    {
+      (void) fputs("[ghostscript library]",stdout);
+      (void) fputs(strchr(command,' '),stdout);
+    }
+  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+  if (status < 0)
+    return(MagickFalse);
+  argv=StringToArgv(command,&argc);
+  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+  if (status == 0)
+    status=(gs_func->run_string)
+      (interpreter,"systemdict /start get exec\n",0,&code);
+  (gs_func->exit)(interpreter);
+  (gs_func->delete_instance)(interpreter);
+#if defined(__WINDOWS__)
+  NTGhostscriptUnLoadDLL();
+#endif
+  for (i=0; i < (long) argc; i++)
+    argv[i]=(char *) RelinquishMagickMemory(argv[i]);
+  argv=(char **) RelinquishMagickMemory(argv);
+  if ((status == 0) || (status <= -100))
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Ghostscript returns status %d, exit code %d",status,code);
+      return(MagickFalse);
+    }
+  return(MagickTrue);
+#else
+  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I s P S                                                                   %
 %                                                                             %
 %                                                                             %
@@ -247,13 +344,11 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   unsigned long
     columns,
-    height,
     language_level,
     length,
     pages,
     rows,
-    scene,
-    width;
+    scene;
 
   /*
     Open image file.
@@ -332,7 +427,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   xml_profile=(StringInfo *) NULL;
   language_level=1;
   skip=MagickFalse;
-  cmyk=MagickFalse;
+  cmyk=image_info->colorspace == CMYKColorspace ? MagickTrue : MagickFalse;
   pages=(~0UL);
   p=command;
   for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
@@ -403,12 +498,12 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (LocaleCompare(token,"cyan") == 0)
         cmyk=MagickTrue;
     }
-    /*
-      Note region defined by bounding box.
-    */
     if ((bounds.x1 != 0.0) || (bounds.y1 != 0.0) || (bounds.x2 != 0.0) ||
         (bounds.y2 != 0.0))
       continue;
+    /*
+      Note region defined by bounding box.
+    */
     count=0;
     if (LocaleNCompare(BoundingBox,command,strlen(BoundingBox)) == 0)
       count=(ssize_t) sscanf(command,BoundingBox " %lf %lf %lf %lf",&bounds.x1,
@@ -430,12 +525,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Set Postscript render geometry.
     */
-    width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-    height=(unsigned long) (bounds.y2-bounds.y1+0.5);
-    if (width > page.width)
-      page.width=width;
-    if (height > page.height)
-      page.height=height;
+    page.width=(unsigned long) (bounds.x2-bounds.x1+0.5);
+    page.height=(unsigned long) (bounds.y2-bounds.y1+0.5);
   }
   CloseBlob(image);
   if (image_info->colorspace == RGBColorspace)
@@ -609,6 +700,7 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString(
    "Encapsulated PostScript Interchange format");
   entry->module=ConstantString("PS");
@@ -620,6 +712,7 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Encapsulated PostScript");
   entry->module=ConstantString("PS");
   (void) RegisterMagickInfo(entry);
@@ -640,6 +733,7 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString(
     "Encapsulated PostScript Interchange format");
   entry->module=ConstantString("PS");
@@ -651,6 +745,7 @@ ModuleExport void RegisterPSImage(void)
   entry->module=ConstantString("PS");
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("PostScript");
   (void) RegisterMagickInfo(entry);
 }
@@ -1115,6 +1210,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PS") == 0))
           (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
+    (void) ConcatenateMagickString(page_geometry,">",MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
@@ -1220,13 +1316,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Dump image as bitmap.
             */
+            (void) SetImageType(preview_image,PaletteType);
             (void) SetImageType(preview_image,BilevelType);
             polarity=(IndexPacket) (PixelIntensityToQuantum(
-              &preview_image->colormap[0]) < (QuantumRange/2));
+              &preview_image->colormap[0]) < (QuantumRange/2) ? 1 : 0);
             if (preview_image->colors == 2)
-              polarity=(IndexPacket)
-                (PixelIntensityToQuantum(&preview_image->colormap[0]) >
-                 PixelIntensityToQuantum(&preview_image->colormap[1]));
+              polarity=(IndexPacket) (
+                PixelIntensityToQuantum(&preview_image->colormap[0]) >
+                PixelIntensityToQuantum(&preview_image->colormap[1]) ? 1 : 0);
             (void) FormatMagickString(buffer,MaxTextExtent,
               "%%%%BeginPreview: %lu %lu %lu %lu\n%%  ",preview_image->columns,
               preview_image->rows,1L,(((preview_image->columns+7) >> 3)*
@@ -1407,13 +1504,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Dump image as bitmap.
             */
+            (void) SetImageType(image,PaletteType);
             (void) SetImageType(image,BilevelType);
             polarity=(IndexPacket) (PixelIntensityToQuantum(
-              &image->colormap[0]) < (QuantumRange/2));
+              &image->colormap[0]) < (QuantumRange/2) ? 1 : 0);
             if (image->colors == 2)
-              polarity=(IndexPacket)
-                (PixelIntensityToQuantum(&image->colormap[1]) <
-                 PixelIntensityToQuantum(&image->colormap[0]));
+              polarity=(IndexPacket) (
+                PixelIntensityToQuantum(&image->colormap[1]) <
+                PixelIntensityToQuantum(&image->colormap[0]) ? 1 : 0);
             count=0;
             for (y=0; y < (long) image->rows; y++)
             {
