@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,6 +50,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -228,7 +229,8 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      q->red=(Quantum) (QuantumRange-ScaleCharToQuantum(ReadBlobByte(image)));
+      q->red=(Quantum) (QuantumRange-ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
       q++;
     }
     if ((image->columns % 2) != 0)
@@ -238,7 +240,8 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      q->green=(Quantum) (QuantumRange-ScaleCharToQuantum(ReadBlobByte(image)));
+      q->green=(Quantum) (QuantumRange-ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
       q++;
     }
     if ((image->columns % 2) != 0)
@@ -249,7 +252,8 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; x++)
     {
-      q->blue=(Quantum) (QuantumRange-ScaleCharToQuantum(ReadBlobByte(image)));
+      q->blue=(Quantum) (QuantumRange-ScaleCharToQuantum((unsigned char)
+        ReadBlobByte(image)));
       q++;
     }
     if ((image->columns % 2) != 0)
@@ -261,8 +265,8 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         for (x=0; x < (long) image->columns; x++)
         {
-          indexes[x]=(IndexPacket) (QuantumRange-
-            ScaleCharToQuantum(ReadBlobByte(image)));
+          indexes[x]=(IndexPacket) (QuantumRange-ScaleCharToQuantum(
+            (unsigned char) ReadBlobByte(image)));
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -318,8 +322,8 @@ ModuleExport void RegisterSCTImage(void)
   entry->decoder=(DecoderHandler *) ReadSCTImage;
   entry->magick=(MagickHandler *) IsSCT;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Scitex HandShake");
-  entry->module=AcquireString("SCT");
+  entry->description=ConstantString("Scitex HandShake");
+  entry->module=ConstantString("SCT");
   (void) RegisterMagickInfo(entry);
 }
 
