@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color.h"
@@ -77,10 +78,17 @@ typedef struct _SGIInfo
 
   unsigned long
     minimum_value,
-    maximum_value;
+    maximum_value,
+    sans;
+
+  char
+    name[80];
+
+  unsigned long
+    pixel_format;
 
   unsigned char
-    filler[492];
+    filler[404];
 } SGIInfo;
 
 /*
@@ -149,40 +157,43 @@ static MagickBooleanType IsSGI(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 
-static void SGIDecode(const unsigned long width,const size_t bytes_per_pixel,
-  unsigned char *max_packets,unsigned char *pixels)
+static MagickBooleanType SGIDecode(const size_t bytes_per_pixel,
+  long number_packets,unsigned char *packets,long number_pixels,
+  unsigned char *pixels)
 {
   register unsigned char
     *p,
     *q;
 
-  register long
-    i;
-
   ssize_t
     count;
 
   unsigned long
     pixel;
 
-  p=max_packets;
+  p=packets;
   q=pixels;
   if (bytes_per_pixel == 2)
     {
-      for (i=0; i < (long) width; )
+      for ( ; number_pixels > 0; )
       {
+        if (number_packets-- == 0)
+          return(MagickFalse);
         pixel=(unsigned long) (*p++) << 8;
         pixel|=(*p++);
         count=(ssize_t) (pixel & 0x7f);
-        i+=count;
         if (count == 0)
           break;
+        if (count > number_pixels)
+          return(MagickFalse);
+        number_pixels-=count;
         if ((pixel & 0x80) != 0)
           for ( ; count != 0; count--)
           {
+            if (number_packets-- == 0)
+              return(MagickFalse);
             *q=(*p++);
             *(q+1)=(*p++);
             q+=8;
@@ -193,29 +204,39 @@ static void SGIDecode(const unsigned long width,const size_t bytes_per_pixel,
             pixel|=(*p++);
             for ( ; count != 0; count--)
             {
+              if (number_packets-- == 0)
+                return(MagickFalse);
               *q=(unsigned char) (pixel >> 8);
               *(q+1)=(unsigned char) pixel;
               q+=8;
             }
           }
       }
-      return;
+      return(MagickTrue);
     }
-  for (i=0; i < (long) width; )
+  for ( ; number_pixels > 0; )
   {
+    if (number_packets-- == 0)
+      return(MagickFalse);
     pixel=(unsigned long) (*p++);
     count=(ssize_t) (pixel & 0x7f);
     if (count == 0)
       break;
-    i+=count;
+    if (count > number_pixels)
+      return(MagickFalse);
+    number_pixels-=count;
     if ((pixel & 0x80) != 0)
       for ( ; count != 0; count--)
       {
+        if (number_packets-- == 0)
+          return(MagickFalse);
         *q=(*p++);
         q+=4;
       }
     else
       {
+        if (number_packets-- == 0)
+          return(MagickFalse);
         pixel=(unsigned long) (*p++);
         for ( ; count != 0; count--)
         {
@@ -224,6 +245,7 @@ static void SGIDecode(const unsigned long width,const size_t bytes_per_pixel,
         }
       }
   }
+  return(MagickTrue);
 }
 
 static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -295,16 +317,39 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (iris_info.magic != 0x01DA)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     iris_info.storage=(unsigned char) ReadBlobByte(image);
-    if ((int) iris_info.storage == 0x01)
-      image->compression=RLECompression;
+    switch (iris_info.storage)
+    {
+      case 0x00: image->compression=NoCompression; break;
+      case 0x01: image->compression=RLECompression; break;
+      default:
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
     iris_info.bytes_per_pixel=(unsigned char) ReadBlobByte(image);
+    if ((iris_info.bytes_per_pixel == 0) || (iris_info.bytes_per_pixel > 2))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     iris_info.dimension=ReadBlobMSBShort(image);
     iris_info.columns=ReadBlobMSBShort(image);
     iris_info.rows=ReadBlobMSBShort(image);
     iris_info.depth=ReadBlobMSBShort(image);
+    if ((iris_info.depth == 0) || (iris_info.depth > 4))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    iris_info.minimum_value=ReadBlobMSBLong(image);
+    iris_info.maximum_value=ReadBlobMSBLong(image);
+    iris_info.sans=ReadBlobMSBLong(image);
+    (void) ReadBlob(image,sizeof(iris_info.name),(unsigned char *)
+      iris_info.name);
+    iris_info.name[sizeof(iris_info.name)-1]='\0';
+    if (*iris_info.name != '\0')
+      (void) SetImageAttribute(image,"label",iris_info.name);
+    iris_info.pixel_format=ReadBlobMSBLong(image);
+    if (iris_info.pixel_format != 0)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    count=ReadBlob(image,sizeof(iris_info.filler),iris_info.filler);
     image->columns=iris_info.columns;
     image->rows=iris_info.rows;
-    image->depth=(unsigned long) (iris_info.depth <= 8 ? 8 : QuantumDepth);
+    image->depth=Min(iris_info.depth,QuantumDepth);
+    if (iris_info.pixel_format == 0)
+      image->depth=Min(8*iris_info.bytes_per_pixel,QuantumDepth);
     if (iris_info.depth < 3)
       {
         image->storage_class=PseudoClass;
@@ -313,9 +358,6 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((image_info->ping != MagickFalse)  && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    iris_info.minimum_value=ReadBlobMSBLong(image);
-    iris_info.maximum_value=ReadBlobMSBLong(image);
-    count=ReadBlob(image,sizeof(iris_info.filler),iris_info.filler);
     /*
       Allocate SGI pixels.
     */
@@ -324,8 +366,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((4*bytes_per_pixel*number_pixels) != ((MagickSizeType) (size_t)
         (4*bytes_per_pixel*number_pixels)))
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    iris_pixels=(unsigned char *)
-      AcquireMagickMemory(4*bytes_per_pixel*iris_info.columns*iris_info.rows);
+    iris_pixels=(unsigned char *) AcquireMagickMemory(4*bytes_per_pixel*
+      iris_info.columns*iris_info.rows);
     if (iris_pixels == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((int) iris_info.storage != 0x01)
@@ -336,17 +378,16 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read standard image format.
         */
-        scanline=(unsigned char *)
-          AcquireMagickMemory(bytes_per_pixel*iris_info.columns);
+        scanline=(unsigned char *) AcquireMagickMemory(bytes_per_pixel*
+          iris_info.columns);
         if (scanline == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        for (z=0; z < (int) iris_info.depth; z++)
+        for (z=0; z < (long) iris_info.depth; z++)
         {
           p=iris_pixels+bytes_per_pixel*z;
           for (y=0; y < (long) iris_info.rows; y++)
           {
-            count=ReadBlob(image,bytes_per_pixel*iris_info.columns,
-              scanline);
+            count=ReadBlob(image,bytes_per_pixel*iris_info.columns,scanline);
             if (EOFBlob(image) != MagickFalse)
               break;
             if (bytes_per_pixel == 2)
@@ -373,7 +414,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           *offsets;
 
         unsigned char
-          *max_packets;
+          *packets;
 
         unsigned int
           data_order;
@@ -386,35 +427,38 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         offsets=(ssize_t *) AcquireMagickMemory((size_t) iris_info.rows*
           iris_info.depth*sizeof(*offsets));
-        max_packets=(unsigned char *)
-          AcquireMagickMemory(4*(size_t) iris_info.columns+10);
+        packets=(unsigned char *) AcquireMagickMemory(4*(size_t)
+          iris_info.columns+10);
         runlength=(unsigned long *) AcquireMagickMemory(iris_info.rows*
           iris_info.depth*sizeof(*runlength));
         if ((offsets == (ssize_t *) NULL) ||
-            (max_packets == (unsigned char *) NULL) ||
+            (packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
           offsets[i]=(ssize_t) ReadBlobMSBLong(image);
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
+        {
           runlength[i]=ReadBlobMSBLong(image);
+          if (runlength[i] > (4*(size_t) iris_info.columns+10))
+            ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        }
         /*
           Check data order.
         */
         offset=0;
-        data_order=MagickFalse;
-        for (y=0; ((y < (long) iris_info.rows) && (data_order == MagickFalse)); y++)
-          for (z=0; ((z < (int) iris_info.depth) && (data_order == MagickFalse)); z++)
+        data_order=0;
+        for (y=0; ((y < (long) iris_info.rows) && (data_order == 0)); y++)
+          for (z=0; ((z < (long) iris_info.depth) && (data_order == 0)); z++)
           {
             if (offsets[y+z*iris_info.rows] < offset)
-              data_order=MagickTrue;
+              data_order=1;
             offset=offsets[y+z*iris_info.rows];
           }
-        offset=(ssize_t) (512+4*bytes_per_pixel*2*(iris_info.rows*
-          iris_info.depth));
+        offset=TellBlob(image);
         if (data_order == 1)
           {
-            for (z=0; z < (int) iris_info.depth; z++)
+            for (z=0; z < (long) iris_info.depth; z++)
             {
               p=iris_pixels;
               for (y=0; y < (long) iris_info.rows; y++)
@@ -425,12 +469,15 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
                 count=ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
-                  max_packets);
+                  packets);
                 if (EOFBlob(image) != MagickFalse)
                   break;
                 offset+=runlength[y+z*iris_info.rows];
-                SGIDecode(iris_info.columns,bytes_per_pixel,max_packets,
+                status=SGIDecode(bytes_per_pixel,runlength[y+z*iris_info.rows]/
+                  bytes_per_pixel,packets,iris_info.columns,
                   p+bytes_per_pixel*z);
+                if (status == MagickFalse)
+                  ThrowReaderException(CorruptImageError,"ImproperImageHeader");
                 p+=(iris_info.columns*4*bytes_per_pixel);
               }
             }
@@ -444,7 +491,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             p=iris_pixels;
             for (y=0; y < (long) iris_info.rows; y++)
             {
-              for (z=0; z < (int) iris_info.depth; z++)
+              for (z=0; z < (long) iris_info.depth; z++)
               {
                 if (offset != offsets[y+z*iris_info.rows])
                   {
@@ -452,19 +499,22 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
                 count=ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
-                  max_packets);
+                  packets);
                 if (EOFBlob(image) != MagickFalse)
                   break;
                 offset+=runlength[y+z*iris_info.rows];
-                SGIDecode(iris_info.columns,bytes_per_pixel,max_packets,
+                status=SGIDecode(bytes_per_pixel,runlength[y+z*iris_info.rows]/
+                  bytes_per_pixel,packets,iris_info.columns,
                   p+bytes_per_pixel*z);
+                if (status == MagickFalse)
+                  ThrowReaderException(CorruptImageError,"ImproperImageHeader");
               }
               p+=(iris_info.columns*4*bytes_per_pixel);
             }
             (void) SeekBlob(image,position,SEEK_SET);
           }
         runlength=(unsigned long *) RelinquishMagickMemory(runlength);
-        max_packets=(unsigned char *) RelinquishMagickMemory(max_packets);
+        packets=(unsigned char *) RelinquishMagickMemory(packets);
         offsets=(ssize_t *) RelinquishMagickMemory(offsets);
       }
     /*
@@ -797,6 +847,9 @@ static size_t SGIEncode(unsigned char *pixels,size_t length,
 
 static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
 {
+  const ImageAttribute
+    *attribute;
+
   long
     y,
     z;
@@ -849,6 +902,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     */
     if (image_info->colorspace == UndefinedColorspace)
       (void) SetImageColorspace(image,RGBColorspace);
+    (void) ResetMagickMemory(&iris_info,0,sizeof(iris_info));
     iris_info.magic=0x01DA;
     if (image->compression == NoCompression)
       iris_info.storage=(unsigned char) 0x00;
@@ -873,8 +927,6 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
       }
     iris_info.minimum_value=0;
     iris_info.maximum_value=(unsigned long) ScaleQuantumToChar(QuantumRange);
-    for (i=0; i < (int) sizeof(iris_info.filler); i++)
-      iris_info.filler[i]=(unsigned char) 0;
     /*
       Write SGI header.
     */
@@ -887,6 +939,14 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobMSBShort(image,iris_info.depth);
     (void) WriteBlobMSBLong(image,iris_info.minimum_value);
     (void) WriteBlobMSBLong(image,iris_info.maximum_value);
+    (void) WriteBlobMSBLong(image,iris_info.sans);
+    attribute=GetImageAttribute(image,"label");
+    if (attribute != (const ImageAttribute *) NULL)
+      (void) CopyMagickString(iris_info.name,attribute->value,
+        sizeof(iris_info.name));
+    (void) WriteBlob(image,sizeof(iris_info.name),(unsigned char *)
+      iris_info.name);
+    (void) WriteBlobMSBLong(image,iris_info.pixel_format);
     (void) WriteBlob(image,sizeof(iris_info.filler),iris_info.filler);
     /*
       Allocate SGI pixels.
@@ -894,8 +954,8 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     number_pixels=(MagickSizeType) image->columns*image->rows;
     if ((4*number_pixels) != ((MagickSizeType) (size_t) (4*number_pixels)))
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    iris_pixels=(unsigned char *)
-      AcquireMagickMemory(4*(size_t) image->columns*image->rows);
+    iris_pixels=(unsigned char *) AcquireMagickMemory(4*(size_t) image->columns*
+      image->rows);
     if (iris_pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     /*
@@ -937,7 +997,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           AcquireMagickMemory((size_t) iris_info.columns);
         if (scanline == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-        for (z=0; z < (int) iris_info.depth; z++)
+        for (z=0; z < (long) iris_info.depth; z++)
         {
           q=iris_pixels+z;
           for (y=0; y < (long) iris_info.rows; y++)
