@@ -59,7 +59,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/option.h"
-#include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/profile.h"
 #include "magick/resize.h"
 #include "magick/splay-tree.h"
@@ -67,6 +67,7 @@
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/utility.h"
 #include "magick/version.h"
 #if defined(HasTIFF)
 # if defined(HAVE_TIFFCONF_H)
@@ -178,7 +179,7 @@ static MagickBooleanType ReadProfile(Image *image,const char *name,
   i=0;
   if ((LocaleCompare(name,"icc") != 0) && (LocaleCompare(name,"xmp") != 0))
     {
-      for (i=0; i < length; i+=2)
+      for (i=0; i < (length-4); i+=2)
         if (LocaleNCompare((char *) (datum+i),"8BIM",4) == 0)
           break;
       if (i == length)
@@ -188,7 +189,7 @@ static MagickBooleanType ReadProfile(Image *image,const char *name,
       if (length < 4)
         return(MagickFalse);
     }
-  profile=AcquireStringInfo(length);
+  profile=AcquireStringInfo((size_t) length);
   SetStringInfoDatum(profile,datum+i);
   status=SetImageProfile(image,name,profile);
   profile=DestroyStringInfo(profile);
@@ -311,8 +312,13 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
   char
     *text;
 
+  const char
+    *option;
+
   float
     *chromaticity,
+    x_position,
+    y_position,
     x_resolution,
     y_resolution;
 
@@ -330,6 +336,9 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
   MagickSizeType
     number_pixels;
 
+  QuantumInfo
+    quantum_info;
+
   QuantumType
     quantum_type;
 
@@ -340,9 +349,6 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
   register PixelPacket
     *q;
 
-  size_t
-    pad;
-
   TIFF
     *tiff;
 
@@ -361,6 +367,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     pages,
     photometric,
     *sample_info,
+    sample_format,
     samples_per_pixel,
     units,
     value;
@@ -407,23 +414,10 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  if (image_info->number_scenes != 0)
-    while (image->scene < image_info->scene)
-    {
-      /*
-        Skip to next image.
-      */
-      image->scene++;
-      status=TIFFReadDirectory(tiff) != 0 ? MagickTrue : MagickFalse;
-      if (status == MagickFalse)
-        {
-          TIFFClose(tiff);
-          ThrowReaderException(CorruptImageError,"UnableToReadSubimage");
-        }
-    }
   debug=IsEventLogging();
   do
   {
+    GetQuantumInfo(image_info,&quantum_info);
     if (image_info->verbose != MagickFalse)
       TIFFPrintDirectory(tiff,stdout,MagickFalse);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_COMPRESSION,&compress_tag);
@@ -433,6 +427,13 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_FILLORDER,&endian);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_PLANARCONFIG,&interlace);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_BITSPERSAMPLE,&bits_per_sample);
+    (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_SAMPLEFORMAT,&sample_format);
+    if (sample_format == SAMPLEFORMAT_UINT)
+      quantum_info.format=UnsignedQuantumFormat;
+    if (sample_format == SAMPLEFORMAT_INT)
+      quantum_info.format=SignedQuantumFormat;
+    if (sample_format == SAMPLEFORMAT_IEEEFP)
+      quantum_info.format=FloatingPointQuantumFormat;
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_MINSAMPLEVALUE,&min_sample_value);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_MAXSAMPLEVALUE,&max_sample_value);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_PHOTOMETRIC,&photometric);
@@ -459,7 +460,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           case PHOTOMETRIC_MINISWHITE:
           {
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: MINISBLACK");
+              "Photometric: MINISWHITE");
             break;
           }
           case PHOTOMETRIC_PALETTE:
@@ -509,6 +510,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_YRESOLUTION,&y_resolution);
     image->x_resolution=x_resolution;
     image->y_resolution=y_resolution;
+    x_position=(float) image->page.x/x_resolution;
+    y_position=(float) image->page.y/y_resolution;
+    (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_XPOSITION,&x_position);
+    (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_YPOSITION,&y_position);
+    image->page.x=(long) (x_position*x_resolution+0.5);
+    image->page.y=(long) (y_position*y_resolution+0.5);
     image->orientation=(OrientationType) orientation;
     chromaticity=(float *) NULL;
     (void) TIFFGetField(tiff,TIFFTAG_WHITEPOINT,&chromaticity);
@@ -535,7 +542,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
 #endif
 #if defined(TIFFTAG_PHOTOSHOP)
     if (TIFFGetField(tiff,TIFFTAG_PHOTOSHOP,&length,&profile) == 1)
-      (void) ReadProfile(image,"iptc",profile,(long) length);
+      (void) ReadProfile(image,"8bim",profile,(long) length);
 #endif
 #if defined(TIFFTAG_RICHTIFFIPTC)
     if (TIFFGetField(tiff,TIFFTAG_RICHTIFFIPTC,&length,&profile) == 1)
@@ -582,16 +589,28 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           image->matte=MagickTrue;
       }
     else
-      for (i=0 ; i < (long) extra_samples; i++)
+      for (i=0; i < (long) extra_samples; i++)
       {
+        if (samples_per_pixel > 3)
+          {
+            image->matte=MagickTrue;
+            associated_alpha=MagickFalse;
+          }
         if (sample_info[i] == EXTRASAMPLE_UNASSALPHA)
-          image->matte=MagickTrue;
+          {
+            image->matte=MagickTrue;
+            associated_alpha=MagickFalse;
+          }
         if (sample_info[i] == EXTRASAMPLE_ASSOCALPHA)
           {
-            associated_alpha=MagickTrue;
             image->matte=MagickTrue;
+            associated_alpha=MagickTrue;
           }
       }
+    option=GetImageOption(image_info,"tiff:alpha");
+    if (option != (const char *) NULL)
+      associated_alpha=LocaleCompare(option,"associate") ? MagickTrue :
+        MagickFalse;
     if ((samples_per_pixel <= 2) && (TIFFIsTiled(tiff) == MagickFalse) &&
         (image->compression != JPEGCompression) &&
         ((photometric == PHOTOMETRIC_MINISBLACK) ||
@@ -620,6 +639,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       (void) SetImageAttribute(image,"Timestamp",text);
     if (TIFFGetField(tiff,TIFFTAG_SOFTWARE,&text) == 1)
       (void) SetImageAttribute(image,"Software",text);
+    if (TIFFGetField(tiff,TIFFTAG_HOSTCOMPUTER,&text) == 1)
+      (void) SetImageAttribute(image,"HostComputer",text);
     if (TIFFGetField(tiff,TIFFTAG_DOCUMENTNAME,&text) == 1)
       (void) SetImageAttribute(image,"Document",text);
     if (TIFFGetField(tiff,TIFFTAG_MAKE,&text) == 1)
@@ -681,9 +702,10 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             {
               case PHOTOMETRIC_MINISBLACK:
               {
-                for (i=0; i < image->colors; i++)
+                for (i=0; i < (long) image->colors; i++)
                 {
-                  image->colormap[i].red=ScaleAnyToQuantum(i,image->colors-1);
+                  image->colormap[i].red=ScaleAnyToQuantum((unsigned long) i,
+                    image->colors-1);
                   image->colormap[i].green=image->colormap[i].red;
                   image->colormap[i].blue=image->colormap[i].red;
                   image->colormap[i].opacity=OpaqueOpacity;
@@ -693,7 +715,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
               case PHOTOMETRIC_MINISWHITE:
               default:
               {
-                for (i=0; i < image->colors; i++)
+                for (i=0; i < (long) image->colors; i++)
                 {
                   image->colormap[i].red=
                     ScaleAnyToQuantum(image->colors-i-1,image->colors-1);
@@ -726,36 +748,36 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
                 for (i=0; i < (long) image->colors; i++)
                 {
                   image->colormap[i].red=ScaleAnyToQuantum(red_colormap[i],
-                    range);
+                    (unsigned long) range);
                   image->colormap[i].green=ScaleAnyToQuantum(green_colormap[i],
-                    range);
+                    (unsigned long) range);
                   image->colormap[i].blue=ScaleAnyToQuantum(blue_colormap[i],
-                    range);
+                    (unsigned long) range);
                 }
                 break;
               }
             }
           }
         quantum_type=IndexQuantum;
-        pad=samples_per_pixel-1;
+        quantum_info.pad=(size_t) samples_per_pixel-1;
         if (image->matte != MagickFalse)
           {
             if (image->storage_class != PseudoClass)
               {
                 quantum_type=GrayAlphaQuantum;
-                pad=samples_per_pixel-2;
+                quantum_info.pad=(size_t) samples_per_pixel-2;
               }
             else
               {
                 quantum_type=IndexAlphaQuantum;
-                pad=samples_per_pixel-2;
+                quantum_info.pad=(size_t) samples_per_pixel-2;
               }
           }
         else
           if (image->storage_class != PseudoClass)
             {
               quantum_type=GrayQuantum;
-              pad=samples_per_pixel-1;
+              quantum_info.pad=(size_t) samples_per_pixel-1;
             }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -771,7 +793,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           /*
             Transfer image scanline.
           */
-          (void) ExportQuantumPixels(image,quantum_type,pad,scanline);
+          (void) ExportQuantumPixels(image,&quantum_info,quantum_type,scanline);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -799,26 +821,26 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             TIFFClose(tiff);
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
+        quantum_info.pad=(size_t) samples_per_pixel-3;
         quantum_type=RGBQuantum;
-        pad=samples_per_pixel-3;
         if (image->colorspace == CMYKColorspace)
           {
             if (image->matte == MagickFalse)
               {
                 quantum_type=CMYKQuantum;
-                pad=samples_per_pixel-4;
+                quantum_info.pad=(size_t) samples_per_pixel-4;
               }
             else
               {
                 quantum_type=CMYKAQuantum;
-                pad=samples_per_pixel-5;
+                quantum_info.pad=(size_t) samples_per_pixel-5;
               }
           }
         else
           if (image->matte != MagickFalse)
             {
               quantum_type=RGBAQuantum;
-              pad=samples_per_pixel-4;
+              quantum_info.pad=(size_t) samples_per_pixel-4;
             }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -831,7 +853,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,quantum_type,pad,scanline);
+          (void) ExportQuantumPixels(image,&quantum_info,quantum_type,scanline);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -862,8 +884,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             TIFFClose(tiff);
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        pixels=(uint32 *)
-          AcquireMagickMemory((size_t) number_pixels*sizeof(uint32));
+        pixels=(uint32 *) AcquireMagickMemory((size_t)
+          number_pixels*sizeof(uint32));
         if (pixels == (uint32 *) NULL)
           {
             TIFFClose(tiff);
@@ -883,7 +905,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             {
               if (TIFFReadRGBAStrip(tiff,(tstrip_t) y,pixels) == 0)
                 break;
-              i=(long) Min(rows_per_strip,image->rows-y);
+              i=(long) Min((long) rows_per_strip,(long) image->rows-y);
             }
           i--;
           p=pixels+image->columns*i;
@@ -893,8 +915,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             q->green=ScaleCharToQuantum((unsigned char) TIFFGetG(*p));
             q->blue=ScaleCharToQuantum((unsigned char) TIFFGetB(*p));
             if (image->matte != MagickFalse)
-              q->opacity=(Quantum)
-                ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
+              q->opacity=ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
             p++;
             q++;
           }
@@ -1029,8 +1050,8 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             TIFFClose(tiff);
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        pixels=(uint32 *)
-          AcquireMagickMemory((size_t) number_pixels*sizeof(*pixels));
+        pixels=(uint32 *) AcquireMagickMemory((size_t) number_pixels*
+          sizeof(*pixels));
         if (pixels == (uint32 *) NULL)
           {
             TIFFClose(tiff);
@@ -1054,8 +1075,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             q->green=ScaleCharToQuantum((unsigned char) TIFFGetG(*p));
             q->blue=ScaleCharToQuantum((unsigned char) TIFFGetB(*p));
             if (image->matte != MagickFalse)
-              q->opacity=(Quantum)
-                ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
+              q->opacity=ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
             p--;
             q--;
           }
@@ -1090,11 +1110,11 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            gamma=((MagickRealType) QuantumRange-q->opacity)/QuantumRange;
+            gamma=QuantumScale*((MagickRealType) QuantumRange-q->opacity);
             gamma=1.0/(fabs(gamma) <= MagickEpsilon ? 1.0 : gamma);
-            q->red=(Quantum) (gamma*q->red);
-            q->green=(Quantum) (gamma*q->green);
-            q->blue=(Quantum) (gamma*q->blue);
+            q->red=RoundToQuantum(gamma*q->red);
+            q->green=RoundToQuantum(gamma*q->green);
+            q->blue=RoundToQuantum(gamma*q->blue);
             q++;
           }
           if (SyncImagePixels(image) == MagickFalse)
@@ -1195,6 +1215,7 @@ ModuleExport void RegisterTIFFImage(void)
   entry->adjoin=MagickFalse;
   entry->endian_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Digital Negative");
   entry->module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
@@ -1206,6 +1227,7 @@ ModuleExport void RegisterTIFFImage(void)
   entry->adjoin=MagickFalse;
   entry->endian_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Pyramid encoded TIFF");
   entry->module=ConstantString("TIFF");
   (void) RegisterMagickInfo(entry);
@@ -1214,12 +1236,13 @@ ModuleExport void RegisterTIFFImage(void)
   entry->decoder=(DecoderHandler *) ReadTIFFImage;
   entry->encoder=(EncoderHandler *) WriteTIFFImage;
 #endif
-  entry->description=ConstantString(TIFFDescription);
-  if (*version != '\0')
-    entry->version=ConstantString(version);
   entry->endian_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->stealth=MagickTrue;
+  entry->thread_support=MagickFalse;
+  entry->description=ConstantString(TIFFDescription);
+  if (*version != '\0')
+    entry->version=ConstantString(version);
   entry->module=ConstantString("TIFF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TIFF");
@@ -1228,11 +1251,12 @@ ModuleExport void RegisterTIFFImage(void)
   entry->encoder=(EncoderHandler *) WriteTIFFImage;
 #endif
   entry->magick=(MagickHandler *) IsTIFF;
+  entry->endian_support=MagickTrue;
+  entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString(TIFFDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
-  entry->endian_support=MagickTrue;
-  entry->seekable_stream=MagickTrue;
   entry->module=ConstantString("TIFF");
   (void) RegisterMagickInfo(entry);
 }
@@ -1457,11 +1481,15 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
 #endif
 
   const char
+    *mode,
     *option;
 
   const ImageAttribute
     *attribute;
 
+  CompressionType
+    compression;
+
   long
     y;
 
@@ -1472,6 +1500,9 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
   MagickOffsetType
     scene;
 
+  QuantumInfo
+    quantum_info;
+
   register const PixelPacket
     *p;
 
@@ -1509,7 +1540,13 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
   tiff_exception=(&image->exception);
   (void) TIFFSetErrorHandler((TIFFErrorHandler) TIFFErrors);
   (void) TIFFSetWarningHandler((TIFFErrorHandler) TIFFWarnings);
-  tiff=TIFFClientOpen(image->filename,"w",(thandle_t) image,TIFFReadBlob,
+  switch (image_info->endian)
+  {
+    case LSBEndian: mode="wl"; break;
+    case MSBEndian: mode="wb"; break;
+    default: mode="w"; break;
+  }
+  tiff=TIFFClientOpen(image->filename,mode,(thandle_t) image,TIFFReadBlob,
     TIFFWriteBlob,TIFFSeekBlob,TIFFCloseBlob,TIFFGetBlobSize,TIFFMapBlob,
     TIFFUnmapBlob);
   if (tiff == (TIFF *) NULL)
@@ -1521,6 +1558,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     /*
       Initialize TIFF fields.
     */
+    GetQuantumInfo(image_info,&quantum_info);
     if (LocaleCompare(image_info->magick,"PTIF") == 0)
       if (GetPreviousImageInList(image) != (Image *) NULL)
         (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_REDUCEDIMAGE);
@@ -1531,33 +1569,75 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         (void) TIFFSetField(tiff,TIFFTAG_TILEWIDTH,image->extract_info.width);
         (void) TIFFSetField(tiff,TIFFTAG_TILELENGTH,image->extract_info.height);
       }
-    compress_tag=COMPRESSION_NONE;
+    compression=image->compression;
     switch (image->compression)
     {
       case FaxCompression:
+      case Group4Compression:
       {
-        if ((IsMonochromeImage(image,&image->exception) != MagickFalse) &&
-            (image->matte == MagickFalse))
-          compress_tag=COMPRESSION_CCITTFAX3;
+        if (IsMonochromeImage(image,&image->exception) == MagickFalse)
+          compression=RLECompression;
+#if !defined(CCITT_SUPPORT)
+        compression=RLECompression;
+#endif
+        break;
+      }
+      case JPEGCompression:
+      {
+#if !defined(HasJPEG) || !defined(YCBCR_SUPPORT)
+        compression=RLECompression;
+#else
+        (void) SetImageDepth(image,8);
+        (void) TIFFSetField(tiff,TIFFTAG_JPEGQUALITY,
+          image->quality == UndefinedCompressionQuality ? 75 : image->quality);
+        if (image->colorspace == RGBColorspace)
+          (void) TIFFSetField(tiff,TIFFTAG_JPEGCOLORMODE,JPEGCOLORMODE_RGB);
+        if (bits_per_sample == 12)
+          (void) TIFFSetField(tiff,TIFFTAG_JPEGTABLESMODE,JPEGTABLESMODE_QUANT);
+#endif
+        break;
+      }
+      case LZWCompression:
+      {
+#if !defined(LZW_SUPPORT)
+        compression=RLECompression;
+#endif
+        break;
+      }
+      case ZipCompression:
+      {
+#if !defined(ZIP_SUPPORT)
+        compression=RLECompression;
+#endif
+        break;
+      }
+      default:
+        break;
+    }
+#if !defined(PACKBITS_SUPPORT)
+    if (compression == RLECompression)
+      compression=NoCompression;
+#endif
+    switch (compression)
+    {
+      case FaxCompression:
+      {
+        compress_tag=COMPRESSION_CCITTFAX3;
         break;
       }
       case Group4Compression:
       {
-        if ((IsMonochromeImage(image,&image->exception) != MagickFalse) &&
-            (image->matte == MagickFalse))
-          compress_tag=COMPRESSION_CCITTFAX4;
+        compress_tag=COMPRESSION_CCITTFAX4;
         break;
       }
       case JPEGCompression:
       {
         compress_tag=COMPRESSION_JPEG;
-        (void) SetImageDepth(image,8);
         break;
       }
       case LZWCompression:
       {
-        if (image_info->compression == LZWCompression)
-          compress_tag=COMPRESSION_LZW;  /* LZW compression must be explicit */
+        compress_tag=COMPRESSION_LZW;
         break;
       }
       case RLECompression:
@@ -1570,12 +1650,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         compress_tag=COMPRESSION_ADOBE_DEFLATE;
         break;
       }
+      case NoCompression:
       default:
       {
         compress_tag=COMPRESSION_NONE;
         break;
       }
     }
+    (void) TIFFSetField(tiff,TIFFTAG_COMPRESSION,compress_tag);
     (void) TIFFSetField(tiff,TIFFTAG_BITSPERSAMPLE,image->depth);
     if (((image_info->colorspace == UndefinedColorspace) &&
          (image->colorspace == CMYKColorspace)) ||
@@ -1647,56 +1729,29 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         (void) TIFFSetField(tiff,TIFFTAG_EXTRASAMPLES,extra_samples,
           &sample_info);
       }
-    switch (image->compression)
-    {
-      case NoCompression: compress_tag=COMPRESSION_NONE; break;
-#if defined(CCITT_SUPPORT)
-      case FaxCompression:
-      {
-        if (IsMonochromeImage(image,&image->exception) != MagickFalse)
-          compress_tag=COMPRESSION_CCITTFAX3;
-        break;
-      }
-      case Group4Compression:
-      {
-        if ((IsMonochromeImage(image,&image->exception) != MagickFalse) &&
-            (image->matte == MagickFalse))
-          compress_tag=COMPRESSION_CCITTFAX4;
-        break;
-      }
-#endif
-#if defined(YCBCR_SUPPORT)
-      case JPEGCompression:
-      {
-        compress_tag=COMPRESSION_JPEG;
-        (void) TIFFSetField(tiff,TIFFTAG_JPEGQUALITY,
-          image->quality == UndefinedCompressionQuality ? 75 : image->quality);
-        if (image->colorspace == RGBColorspace)
-          (void) TIFFSetField(tiff,TIFFTAG_JPEGCOLORMODE,JPEGCOLORMODE_RGB);
-        if (bits_per_sample == 12)
-          (void) TIFFSetField(tiff,TIFFTAG_JPEGTABLESMODE,JPEGTABLESMODE_QUANT);
-        break;
-      }
-#endif
-#if defined(LZW_SUPPORT)
-      case LZWCompression:
-      {
-        if (image_info->compression == LZWCompression)
-          compress_tag=COMPRESSION_LZW;  /* LZW compression must be explicit */
-        break;
-      }
-#endif
-#if defined(PACKBITS_SUPPORT)
-      case RLECompression:
-        compress_tag=COMPRESSION_PACKBITS; break;
-#endif
-#if defined(ZIP_SUPPORT)
-      case ZipCompression: compress_tag=COMPRESSION_ADOBE_DEFLATE; break;
-#endif
-      default: break;
-    }
     (void) TIFFSetField(tiff,TIFFTAG_PHOTOMETRIC,photometric);
-    (void) TIFFSetField(tiff,TIFFTAG_COMPRESSION,compress_tag);
+    switch (quantum_info.format)
+    {
+      case FloatingPointQuantumFormat:
+      {
+        (void) TIFFSetField(tiff,TIFFTAG_SAMPLEFORMAT,SAMPLEFORMAT_IEEEFP);
+        (void) TIFFSetField(tiff,TIFFTAG_SMINSAMPLEVALUE,quantum_info.minimum);
+        (void) TIFFSetField(tiff,TIFFTAG_SMAXSAMPLEVALUE,quantum_info.maximum);
+        break;
+      }
+      case SignedQuantumFormat:
+      {
+        (void) TIFFSetField(tiff,TIFFTAG_SAMPLEFORMAT,SAMPLEFORMAT_INT);
+        break;
+      }
+      case UnsignedQuantumFormat:
+      {
+        (void) TIFFSetField(tiff,TIFFTAG_SAMPLEFORMAT,SAMPLEFORMAT_UINT);
+        break;
+      }
+      default:
+        break;
+    }
     switch (image->endian)
     {
       case LSBEndian:
@@ -1732,7 +1787,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       rows_per_strip=(unsigned long) Max(TIFFDefaultStripSize(tiff,-1),1);
     option=GetImageOption(image_info,"tiff:rows-per-strip");
     if (option != (const char *) NULL)
-      rows_per_strip=strtol(option,(char **) NULL,10);
+      rows_per_strip=(unsigned long) strtol(option,(char **) NULL,10);
     switch (compress_tag)
     {
       case COMPRESSION_JPEG:
@@ -1776,7 +1831,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         break;
       }
     }
-    if ((image->x_resolution != 0) && (image->y_resolution != 0))
+    if ((image->x_resolution != 0.0) && (image->y_resolution != 0.0))
       {
         unsigned short
           units;
@@ -1792,6 +1847,16 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         (void) TIFFSetField(tiff,TIFFTAG_RESOLUTIONUNIT,(uint16) units);
         (void) TIFFSetField(tiff,TIFFTAG_XRESOLUTION,image->x_resolution);
         (void) TIFFSetField(tiff,TIFFTAG_YRESOLUTION,image->y_resolution);
+        if ((image->page.x != 0) || (image->page.y != 0))
+          {
+            /*
+              Set image position.
+            */
+            (void) TIFFSetField(tiff,TIFFTAG_XPOSITION,(float) image->page.x/
+              image->x_resolution);
+            (void) TIFFSetField(tiff,TIFFTAG_YPOSITION,(float) image->page.y/
+              image->y_resolution);
+          }
       }
     if (image->chromaticity.white_point.x != 0.0)
       {
@@ -1836,13 +1901,6 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
 #endif
           if (LocaleCompare(name,"iptc") == 0)
             {
-#if defined(TIFFTAG_PHOTOSHOP)
-              uint32
-                length;
-
-              length=profile->length+(profile->length & 0x01);
-              (void) TIFFSetField(tiff,TIFFTAG_PHOTOSHOP,length,profile->datum);
-#else
               size_t
                 length;
 
@@ -1857,6 +1915,15 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               (void) TIFFSetField(tiff,TIFFTAG_RICHTIFFIPTC,
                 (uint32) iptc_profile->length/4,iptc_profile->datum);
               iptc_profile=DestroyStringInfo(iptc_profile);
+            }
+          if (LocaleCompare(name,"8bim") == 0)
+            {
+#if defined(TIFFTAG_PHOTOSHOP)
+              uint32
+                length;
+
+              length=(uint32) (profile->length+(profile->length & 0x01));
+              (void) TIFFSetField(tiff,TIFFTAG_PHOTOSHOP,length,profile->datum);
 #endif
             }
           if (LocaleCompare(name,"tiff:37724") == 0)
@@ -1874,6 +1941,9 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       }
     if (image->orientation != UndefinedOrientation)
       (void) TIFFSetField(tiff,TIFFTAG_ORIENTATION,(uint16) image->orientation);
+    attribute=GetImageAttribute(image,"HostComputer");
+    if (attribute != (const ImageAttribute *) NULL)
+      (void) TIFFSetField(tiff,TIFFTAG_HOSTCOMPUTER,attribute->value);
     attribute=GetImageAttribute(image,"Artist");
     if (attribute != (const ImageAttribute *) NULL)
       (void) TIFFSetField(tiff,TIFFTAG_ARTIST,attribute->value);
@@ -1931,9 +2001,11 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               if (p == (const PixelPacket *) NULL)
                 break;
               if (image->matte == MagickFalse)
-                (void) ImportQuantumPixels(image,RGBQuantum,0,scanline);
+                (void) ImportQuantumPixels(image,&quantum_info,RGBQuantum,
+                  scanline);
               else
-                (void) ImportQuantumPixels(image,RGBAQuantum,0,scanline);
+                (void) ImportQuantumPixels(image,&quantum_info,RGBAQuantum,
+                  scanline);
               if (TIFFWritePixels(tiff,(char *) scanline,y,0,image) < 0)
                 break;
               if (image->previous == (Image *) NULL)
@@ -1960,7 +2032,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              (void) ImportQuantumPixels(image,RedQuantum,0,scanline);
+              (void) ImportQuantumPixels(image,&quantum_info,RedQuantum,
+                scanline);
               if (TIFFWritePixels(tiff,(char *) scanline,y,0,image) < 0)
                 break;
             }
@@ -1977,7 +2050,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              (void) ImportQuantumPixels(image,GreenQuantum,0,scanline);
+              (void) ImportQuantumPixels(image,&quantum_info,GreenQuantum,
+                scanline);
               if (TIFFWritePixels(tiff,(char *) scanline,y,1,image) < 0)
                 break;
             }
@@ -1994,7 +2068,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              (void) ImportQuantumPixels(image,BlueQuantum,0,scanline);
+              (void) ImportQuantumPixels(image,&quantum_info,BlueQuantum,
+                scanline);
               if (TIFFWritePixels(tiff,(char *) scanline,y,2,image) < 0)
                 break;
             }
@@ -2012,7 +2087,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                (void) ImportQuantumPixels(image,AlphaQuantum,0,scanline);
+                (void) ImportQuantumPixels(image,&quantum_info,AlphaQuantum,
+                  scanline);
                 if (TIFFWritePixels(tiff,(char *) scanline,y,3,image) < 0)
                   break;
               }
@@ -2041,9 +2117,11 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           if (p == (const PixelPacket *) NULL)
             break;
           if (image->matte == MagickFalse)
-            (void) ImportQuantumPixels(image,CMYKQuantum,0,scanline);
+            (void) ImportQuantumPixels(image,&quantum_info,CMYKQuantum,
+              scanline);
           else
-            (void) ImportQuantumPixels(image,CMYKAQuantum,0,scanline);
+            (void) ImportQuantumPixels(image,&quantum_info,CMYKAQuantum,
+              scanline);
           if (TIFFWritePixels(tiff,(char *) scanline,y,0,image) < 0)
             break;
           if (image->previous == (Image *) NULL)
@@ -2068,12 +2146,11 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         /*
           Colormapped TIFF image.
         */
-        red=(unsigned short *) AcquireMagickMemory(65536*sizeof(*red));
-        green=(unsigned short *) AcquireMagickMemory(65536*sizeof(*green));
-        blue=(unsigned short *) AcquireMagickMemory(65536*sizeof(*blue));
-        if ((red == (unsigned short *) NULL) ||
-            (green == (unsigned short *) NULL) ||
-            (blue == (unsigned short *) NULL))
+        red=(uint16 *) AcquireMagickMemory(65536*sizeof(*red));
+        green=(uint16 *) AcquireMagickMemory(65536*sizeof(*green));
+        blue=(uint16 *) AcquireMagickMemory(65536*sizeof(*blue));
+        if ((red == (uint16 *) NULL) || (green == (uint16 *) NULL) ||
+            (blue == (uint16 *) NULL))
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         /*
           Initialize TIFF colormap.
@@ -2105,15 +2182,19 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           if (image->matte != MagickFalse)
              {
                if (photometric != PHOTOMETRIC_PALETTE)
-                 (void) ImportQuantumPixels(image,GrayAlphaQuantum,0,scanline);
+                 (void) ImportQuantumPixels(image,&quantum_info,
+                   GrayAlphaQuantum,scanline);
                else
-                 (void) ImportQuantumPixels(image,IndexAlphaQuantum,0,scanline);
+                 (void) ImportQuantumPixels(image,&quantum_info,
+                   IndexAlphaQuantum,scanline);
              }
            else
              if (photometric != PHOTOMETRIC_PALETTE)
-               (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
+               (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,
+                 scanline);
              else
-               (void) ImportQuantumPixels(image,IndexQuantum,0,scanline);
+               (void) ImportQuantumPixels(image,&quantum_info,IndexQuantum,
+                 scanline);
           if (TIFFWritePixels(tiff,(char *) scanline,y,0,image) < 0)
             break;
           if (image->previous == (Image *) NULL)
