@@ -54,8 +54,8 @@
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/type.h"
-#include "wand/magick-wand.h"
-#if defined(HasTTF)
+#include "wand/MagickWand.h"
+#if defined(HasFREETYPE)
 #if defined(HAVE_FT2BUILD_H)
 #  include <ft2build.h>
 #endif
@@ -140,7 +140,7 @@ static MagickBooleanType IsTTF(const unsigned char *magick,const size_t length)
   return(MagickFalse);
 }
 
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -284,7 +284,7 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
-#endif /* HasTTF */
+#endif /* HasFREETYPE */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -322,8 +322,19 @@ ModuleExport void RegisterTTFImage(void)
   (void) FormatMagickString(version,MaxTextExtent,"Freetype %d.%d.%d",
     FREETYPE_MAJOR,FREETYPE_MINOR,FREETYPE_PATCH);
 #endif
+  entry=SetMagickInfo("DFONT");
+#if defined(HasFREETYPE)
+  entry->decoder=(DecoderHandler *) ReadTTFImage;
+#endif
+  entry->magick=(MagickHandler *) IsTTF;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString("Multi-face font package");
+  if (*version != '\0')
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("TTF");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PFA");
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
   entry->decoder=(DecoderHandler *) ReadTTFImage;
 #endif
   entry->magick=(MagickHandler *) IsPFA;
@@ -334,7 +345,7 @@ ModuleExport void RegisterTTFImage(void)
   entry->module=ConstantString("TTF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PFB");
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
   entry->decoder=(DecoderHandler *) ReadTTFImage;
 #endif
   entry->magick=(MagickHandler *) IsPFA;
@@ -345,7 +356,7 @@ ModuleExport void RegisterTTFImage(void)
   entry->module=ConstantString("TTF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("OTF");
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
   entry->decoder=(DecoderHandler *) ReadTTFImage;
 #endif
   entry->magick=(MagickHandler *) IsTTF;
@@ -356,7 +367,7 @@ ModuleExport void RegisterTTFImage(void)
   entry->module=ConstantString("TTF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TTC");
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
   entry->decoder=(DecoderHandler *) ReadTTFImage;
 #endif
   entry->magick=(MagickHandler *) IsTTF;
@@ -367,7 +378,7 @@ ModuleExport void RegisterTTFImage(void)
   entry->module=ConstantString("TTF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TTF");
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
   entry->decoder=(DecoderHandler *) ReadTTFImage;
 #endif
   entry->magick=(MagickHandler *) IsTTF;
@@ -406,4 +417,5 @@ ModuleExport void UnregisterTTFImage(void)
   (void) UnregisterMagickInfo("PFA");
   (void) UnregisterMagickInfo("PFB");
   (void) UnregisterMagickInfo("PFA");
+  (void) UnregisterMagickInfo("DFONT");
 }
