@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -59,6 +59,7 @@
 #include "magick/monitor.h"
 #include "magick/option.h"
 #include "magick/pixel-private.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -157,10 +158,6 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
     *p,
     text[MaxTextExtent];
 
-  double
-    dx_resolution,
-    dy_resolution;
-
   DrawInfo
     *draw_info;
 
@@ -174,6 +171,9 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  PointInfo
+    delta;
+
   RectangleInfo
     page;
 
@@ -200,8 +200,8 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   /*
     Set the page geometry.
   */
-  dx_resolution=DefaultResolution;
-  dy_resolution=DefaultResolution;
+  delta.x=DefaultResolution;
+  delta.y=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       GeometryInfo
@@ -226,9 +226,8 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
     Initialize Image structure.
   */
   image->columns=(unsigned long)
-    (((page.width*image->x_resolution)/dx_resolution)+0.5);
-  image->rows=(unsigned long)
-    (((page.height*image->y_resolution)/dy_resolution)+0.5);
+    (((page.width*image->x_resolution)/delta.x)+0.5);
+  image->rows=(unsigned long) (((page.height*image->y_resolution)/delta.y)+0.5);
   image->page.x=0;
   image->page.y=0;
   texture=(Image *) NULL;
@@ -249,7 +248,6 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   */
   SetImageBackgroundColor(image);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->fill=image_info->pen;
   (void) CloneString(&draw_info->text,image_info->filename);
   (void) FormatMagickString(geometry,MaxTextExtent,"0x0%+ld%+ld",page.x,page.y);
   (void) CloneString(&draw_info->geometry,geometry);
@@ -269,7 +267,7 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
       Annotate image with text.
     */
     (void) ConcatenateString(&draw_info->text,text);
-    (void) ConcatenateString(&draw_info->text,"\\n");
+    (void) ConcatenateString(&draw_info->text,"\n");
     offset+=(long) (metrics.ascent-metrics.descent);
     if (image->previous == (Image *) NULL)
       if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
@@ -375,6 +373,9 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
+  IndexPacket
+    *indexes;
+
   long
     x,
     y;
@@ -385,12 +386,6 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  Quantum
-    *scale;
-
-  register long
-    i;
-
   register PixelPacket
     *q;
 
@@ -418,62 +413,43 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   (void) ReadBlobString(image,text);
-  count=(ssize_t) sscanf(text+32,"%lu,%lu,%lu,%s",&image->columns,&image->rows,
-    &max_value,format);
+  count=(ssize_t) sscanf(text+32,"%lu,%lu,%lu,%s",&image->columns,
+    &image->rows,&max_value,format);
   if ((LocaleNCompare(text,"# ImageMagick pixel enumeration:",32) != 0) ||
       (count != 4))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   if (LocaleNCompare(format,"CMYK",4) == 0)
     image->colorspace=CMYKColorspace;
-  image->matte=(MagickBooleanType) (strchr(format,'A') != (char *) NULL);
-  scale=(Quantum *) NULL;
-  if (max_value != QuantumRange)
-    {
-      /*
-        Compute pixel scaling table.
-      */
-      scale=(Quantum *)
-        AcquireMagickMemory((size_t) (max_value+1)*sizeof(*scale));
-      if (scale == (Quantum *) NULL)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      for (i=0; i <= (long) max_value; i++)
-        scale[i]=ScaleAnyToQuantum(i,max_value);
-    }
-  pixel.opacity=OpaqueOpacity;
+  image->matte=strchr(format,'A') != (char *) NULL ? MagickTrue : MagickFalse;
   while (ReadBlobString(image,text) != (char *) NULL)
   {
-    if (image->matte == MagickFalse)
-      {
-        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu)",&x,&y,&pixel.red,
-          &pixel.green,&pixel.blue);
-        if (count != 5)
-          continue;
-      }
+    if ((image->colorspace == CMYKColorspace) && (image->matte != MagickFalse))
+      count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu,%lu",&x,&y,
+        &pixel.red,&pixel.green,&pixel.blue,&pixel.index,&pixel.opacity);
     else
-      {
-        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu)",&x,&y,
+      if (image->matte != MagickFalse)
+        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu",&x,&y,
           &pixel.red,&pixel.green,&pixel.blue,&pixel.opacity);
-        if (count != 6)
-          continue;
-      }
-    if (scale != (Quantum *) NULL)
-      {
-        pixel.red=scale[pixel.red];
-        pixel.green=scale[pixel.green];
-        pixel.blue=scale[pixel.blue];
-        pixel.opacity=scale[pixel.opacity];
-      }
+      else
+        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu",&x,&y,
+          &pixel.red,&pixel.green,&pixel.blue);
+    if (count < 5)
+      continue;
     q=SetImagePixels(image,x,y,1,1);
     if (q == (PixelPacket *) NULL)
       continue;
-    q->red=(Quantum) pixel.red;
-    q->green=(Quantum) pixel.green;
-    q->blue=(Quantum) pixel.blue;
-    q->opacity=(Quantum) pixel.opacity;
+    q->red=ScaleAnyToQuantum(pixel.red,max_value);
+    q->green=ScaleAnyToQuantum(pixel.green,max_value);
+    q->blue=ScaleAnyToQuantum(pixel.blue,max_value);
+    if (image->colorspace == CMYKColorspace)
+      {
+        indexes=GetIndexes(image);
+        *indexes=ScaleAnyToQuantum(pixel.index,max_value);
+      }
+    if (image->matte != MagickFalse)
+      q->opacity=ScaleAnyToQuantum(pixel.opacity,max_value);
     (void) SyncImagePixels(image);
   }
-  if (scale != (Quantum *) NULL)
-    scale=(Quantum *) RelinquishMagickMemory(scale);
   return(GetFirstImageInList(image));
 }
 
@@ -510,15 +486,15 @@ ModuleExport void RegisterTXTImage(void)
   entry->encoder=(EncoderHandler *) WriteTXTImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Text");
-  entry->module=AcquireString("TXT");
+  entry->description=ConstantString("Text");
+  entry->module=ConstantString("TXT");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TXT");
   entry->decoder=(DecoderHandler *) ReadTXTImage;
   entry->encoder=(EncoderHandler *) WriteTXTImage;
-  entry->description=AcquireString("Text");
+  entry->description=ConstantString("Text");
   entry->magick=(MagickHandler *) IsTXT;
-  entry->module=AcquireString("TXT");
+  entry->module=ConstantString("TXT");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -600,6 +576,9 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
   register long
     x;
 
+  unsigned long
+    quantum_range;
+
   /*
     Open output image file.
   */
@@ -612,16 +591,18 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image->colorspace == CMYKColorspace)
-    (void) strcpy(format,"CMYK");
+  if (image->colorspace != CMYKColorspace)
+    (void) CopyMagickString(format,"RGB",MaxTextExtent);
   else
-    (void) strcpy(format,"RGB");
+    (void) CopyMagickString(format,"CMYK",MaxTextExtent);
   if (image->matte != MagickFalse)
     (void) ConcatenateMagickString(format,"A",MaxTextExtent);
+  quantum_range=(1UL << image->depth)-1;
+  if (image->depth == 32)
+     quantum_range=4294967295UL;
   (void) FormatMagickString(buffer,MaxTextExtent,
     "# ImageMagick pixel enumeration: %lu,%lu,%lu,%s\n",image->columns,
-    image->rows,image->depth == QuantumDepth ? (unsigned long) QuantumRange :
-    (1L << image->depth)-1,format);
+    image->rows,quantum_range,format);
   (void) WriteBlobString(image,buffer);
   scene=0;
   do
@@ -629,7 +610,6 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to TXT raster pixels.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
     GetMagickPixelPacket(image,&pixel);
     for (y=0; y < (long) image->rows; y++)
     {
@@ -642,11 +622,25 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
         (void) FormatMagickString(buffer,MaxTextExtent,"%ld,%ld: ",x,y);
         (void) WriteBlobString(image,buffer);
         SetMagickPixelPacket(p,indexes+x,&pixel);
-        GetColorTuple(&pixel,MagickFalse,tuple);
-        (void) ConcatenateMagickString(tuple," ",MaxTextExtent);
-        (void) WriteBlobString(image,tuple);
-        (void) QueryColorname(image,p,SVGCompliance,tuple,&image->exception);
-        (void) WriteBlobString(image,tuple);
+        (void) FormatMagickString(buffer,MaxTextExtent,"(%g,%g,%g",pixel.red,
+          pixel.green,pixel.blue);
+        (void) WriteBlobString(image,buffer);
+        if (pixel.colorspace == CMYKColorspace)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,",%g",pixel.index);
+            (void) WriteBlobString(image,buffer);
+          }
+        if (pixel.matte != MagickFalse)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,",%g",pixel.opacity);
+            (void) WriteBlobString(image,buffer);
+          }
+        (void) FormatMagickString(buffer,MaxTextExtent,")  ");
+        (void) WriteBlobString(image,buffer);
+        (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickFalse,
+          tuple,&image->exception);
+        (void) FormatMagickString(buffer,MaxTextExtent,"%-45.45s ",tuple);
+        (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"\n");
         p++;
       }
