@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,8 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel-private.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -92,8 +94,8 @@ ModuleExport void RegisterUILImage(void)
   entry=SetMagickInfo("UIL");
   entry->encoder=(EncoderHandler *) WriteUILImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("X-Motif UIL table");
-  entry->module=AcquireString("UIL");
+  entry->description=ConstantString("X-Motif UIL table");
+  entry->module=ConstantString("UIL");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -168,6 +170,9 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
     status,
     transparent;
 
+  MagickPixelPacket
+    pixel;
+
   MagickSizeType
     number_pixels;
 
@@ -197,11 +202,12 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   transparent=MagickFalse;
   i=0;
   p=(const PixelPacket *) NULL;
@@ -243,7 +249,7 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
             }
           }
         }
-      SetImageType(image,PaletteType);
+      (void) SetImageType(image,PaletteType);
       colors=image->colors;
       if (transparent)
         {
@@ -279,16 +285,21 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
   (void) FormatMagickString(buffer,MaxTextExtent,
     "value\n  %s_ct : color_table(\n",basename);
   (void) WriteBlobString(image,buffer);
+  GetMagickPixelPacket(image,&pixel);
   for (i=0; i < (long) colors; i++)
   {
     /*
       Define UIL color.
     */
-    (void) QueryColorname(image,image->colormap+i,X11Compliance,name,
+    SetMagickPixelPacket(image->colormap+i,(IndexPacket *) NULL,&pixel);
+    pixel.colorspace=RGBColorspace;
+    pixel.depth=8;
+    pixel.opacity=OpaqueOpacity;
+    (void) QueryMagickColorname(image,&pixel,X11Compliance,MagickTrue,name,
       &image->exception);
-    if (transparent)
+    if (transparent != MagickFalse)
       if (i == (long) (colors-1))
-        (void) strcpy(name,"None");
+        (void) CopyMagickString(name,"None",MaxTextExtent);
     /*
       Write UIL color.
     */
@@ -305,8 +316,8 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
         "    background color = '%s'",symbol);
     else
       (void) FormatMagickString(buffer,MaxTextExtent,
-        "    color('%s',%s) = '%s'",name,
-        PixelIntensityToQuantum(image->colormap+i) < ((MaxRGB+1.0)/2.0) ?
+        "    color('%s',%s) = '%s'",name,(double)
+        PixelIntensityToQuantum(image->colormap+i) < ((QuantumRange+1.0)/2.0) ?
         "background" : "foreground",symbol);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"%s",
