@@ -64,7 +64,7 @@
 #include "magick/string_.h"
 #include "magick/type.h"
 #include "magick/version.h"
-#include "wand/magick-wand.h"
+#include "wand/MagickWand.h"
 
 #if defined(HasWMF) || defined(HasWMFlite)
 
@@ -532,7 +532,7 @@ static void ipa_bmp_draw(wmfAPI *API, wmfBMP_Draw_t *bmp_draw)
   QueryColorDatabase( "white", &white, &exception );
 
   if ( ddata->image_info->texture ||
-       !(ColorMatch(&ddata->image_info->background_color,&white)) ||
+       !(IsColorEqual(&ddata->image_info->background_color,&white)) ||
        ddata->image_info->background_color.opacity != OpaqueOpacity )
   {
     MagickPixelPacket
@@ -1134,7 +1134,7 @@ static void ipa_region_frame(wmfAPI * API, wmfPolyRectangle_t * poly_rect)
       draw_fill_color_string(WmfDrawingWand,"none");
       util_set_brush(API, poly_rect->dc, BrushApplyStroke);
 
-      for (i = 0; i < poly_rect->count; i++)
+      for (i = 0; i < (long) poly_rect->count; i++)
         {
           DrawRectangle(WmfDrawingWand,
                          XC(poly_rect->TL[i].x), YC(poly_rect->TL[i].y),
@@ -1163,7 +1163,7 @@ static void ipa_region_paint(wmfAPI * API, wmfPolyRectangle_t * poly_rect)
       draw_stroke_color_string(WmfDrawingWand,"none");
       util_set_brush(API, poly_rect->dc, BrushApplyFill);
 
-      for (i = 0; i < poly_rect->count; i++)
+      for (i = 0; i < (long) poly_rect->count; i++)
         {
           DrawRectangle(WmfDrawingWand,
                          XC(poly_rect->TL[i].x), YC(poly_rect->TL[i].y),
@@ -1200,7 +1200,7 @@ static void ipa_region_clip(wmfAPI *API, wmfPolyRectangle_t *poly_rect)
         ddata->clip_path_id);
       DrawPushClipPath(WmfDrawingWand,clip_path_id);
       (void) PushDrawingWand(WmfDrawingWand);
-      for (i = 0; i < poly_rect->count; i++)
+      for (i = 0; i < (long) poly_rect->count; i++)
         {
           DrawRectangle(WmfDrawingWand,
                          XC(poly_rect->TL[i].x), YC(poly_rect->TL[i].y),
@@ -1369,7 +1369,7 @@ static void ipa_draw_text(wmfAPI * API, wmfDrawText_t * draw_text)
       ImageInfo
         *image_info;
 
-      image_info = CloneImageInfo((ImageInfo *) NULL);
+      image_info = AcquireImageInfo();
       CloneString(&image_info->font, WMF_FONT_PSNAME(font));
       image_info->pointsize = pointsize;
       GetDrawInfo(image_info, &draw_info);
@@ -1993,7 +1993,7 @@ static double util_pointsize( wmfAPI* API, wmfFont* font, char* str, double font
   double
     pointsize = 0;
 
-  image_info = CloneImageInfo((ImageInfo *) NULL);
+  image_info = AcquireImageInfo();
   CloneString(&image_info->font, WMF_FONT_PSNAME(font));
   image_info->pointsize = font_height;
   GetDrawInfo(image_info, &draw_info);
@@ -2114,7 +2114,7 @@ static float lite_font_stringwidth( wmfAPI* API, wmfFont* font, char* str)
   orig_y_resolution = image->y_resolution;
   orig_resolution_units = image->units;
 
-  image_info = CloneImageInfo((ImageInfo *) NULL);
+  image_info = AcquireImageInfo();
   CloneString(&image_info->font, WMF_FONT_PSNAME(font));
   image_info->pointsize = 12;
   GetDrawInfo(image_info, &draw_info);
