@@ -17,7 +17,7 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -52,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -268,7 +269,7 @@ static CompositeOperator GIMPBlendModeToCompositeOperator( unsigned long blendMo
 %
 %
 */
-static char *ReadBlobStringWithLongSize(Image *image,char *string)
+static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max)
 {
   int
     c;
@@ -281,10 +282,11 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string)
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  assert(max != 0);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   length = ReadBlobMSBLong(image);
-  for (i=0; i < (long) length; i++)
+  for (i=0; i < (long) Min(length, max-1); i++)
   {
     c=ReadBlobByte(image);
     if (c == EOF)
@@ -292,6 +294,7 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string)
     string[i]=(char) c;
   }
   string[i]='\0';
+  (void) SeekBlob(image, length-i, SEEK_CUR);
   return(string);
 }
 
@@ -320,13 +323,13 @@ static int load_tile (Image* image, Image* tile_image, XCFDocInfo* inDocInfo,
       q->red    = ScaleCharToQuantum(*graydata);
       q->green  = ScaleCharToQuantum(*graydata);
       q->blue    = ScaleCharToQuantum(*graydata);
-      q->opacity  = ScaleCharToQuantum(255-inLayerInfo->opacity);
+      q->opacity  = ScaleCharToQuantum((unsigned char) (255-inLayerInfo->opacity));
       graydata++;
     } else if ( inDocInfo->image_type == GIMP_RGB ) {
       q->red    = ScaleCharToQuantum(xcfdata->red);
       q->green  = ScaleCharToQuantum(xcfdata->green);
       q->blue    = ScaleCharToQuantum(xcfdata->blue);
-      q->opacity  = (Quantum) ((int) xcfdata->opacity==0 ? TransparentOpacity : ScaleCharToQuantum(255-inLayerInfo->opacity));
+      q->opacity  = (Quantum) ((int) xcfdata->opacity==0 ? TransparentOpacity : ScaleCharToQuantum((unsigned char) (255-inLayerInfo->opacity)));
 
       xcfdata++;
     };
@@ -402,15 +405,15 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
               data = *xcfdata++;
               switch (i) {
                 case 0:
-                q->red    = ScaleCharToQuantum(data);
-                q->green  = q->red;
-                q->blue    = q->red;
-                q->opacity  = ScaleCharToQuantum(255-inLayerInfo->opacity);
+                q->red=ScaleCharToQuantum(data);
+                q->green=q->red;
+                q->blue=q->red;
+                q->opacity=ScaleCharToQuantum((unsigned char) (255-inLayerInfo->opacity));
                 break;
-                case 1: q->green= ScaleCharToQuantum(data);  break;
-                case 2: q->blue = ScaleCharToQuantum(data);  break;
+                case 1: q->green=ScaleCharToQuantum(data);  break;
+                case 2: q->blue=ScaleCharToQuantum(data);  break;
                 case 3:
-                  q->opacity = (Quantum) (ScaleCharToQuantum(255-data));
+                  q->opacity=(Quantum) (ScaleCharToQuantum((unsigned char) (255UL-data)));
                 break;
               }
               q++;
@@ -453,19 +456,19 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
                   q->red=ScaleCharToQuantum(data);
                   q->green=q->red;
                   q->blue=q->red;
-                  q->opacity=ScaleCharToQuantum(255-inLayerInfo->opacity);
+                  q->opacity=ScaleCharToQuantum((unsigned char) (255-inLayerInfo->opacity));
                   break;
                 case 1: q->green=ScaleCharToQuantum(data);  break;
                 case 2: q->blue=ScaleCharToQuantum(data);  break;
                 case 3:
-                  q->opacity = (Quantum) (ScaleCharToQuantum(255-data));
+                  q->opacity=(Quantum) (ScaleCharToQuantum((unsigned char) (255-data)));
                   break;
               }
               q++;
             }
           }
       }
-      SyncImagePixels(tile_image);
+      (void) SyncImagePixels(tile_image);
     }
   xcfodata=(unsigned char *) RelinquishMagickMemory(xcfodata);
   return MagickTrue;
@@ -693,7 +696,8 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
   outLayer->width = ReadBlobMSBLong(image);
   outLayer->height = ReadBlobMSBLong(image);
   outLayer->type = ReadBlobMSBLong(image);
-  (void) ReadBlobStringWithLongSize(image, outLayer->name);
+  (void) ReadBlobStringWithLongSize(image, outLayer->name,
+    sizeof(outLayer->name));
 
   /* allocate the image for this layer */
   outLayer->image=CloneImage(image,outLayer->width, outLayer->height,MagickTrue,
@@ -703,7 +707,7 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
 
   /* read the layer properties! */
   foundPropEnd = 0;
-  while ( foundPropEnd  == MagickFalse ) {
+  while ( (foundPropEnd == MagickFalse) && (EOFBlob(image) == MagickFalse) ) {
   PropType    prop_type = (PropType) ReadBlobMSBLong(image);
   unsigned long  prop_size = ReadBlobMSBLong(image);
 
@@ -777,7 +781,7 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
       ssize_t amount;
 
       /* read over it... */
-      while (prop_size > 0)
+      while ((prop_size > 0) && (EOFBlob(image) == MagickFalse))
         {
         amount = (ssize_t) Min (16, prop_size);
         amount = ReadBlob(image, (size_t) amount, (unsigned char *) &buf);
@@ -791,9 +795,11 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
     }
   }
 
+  if (foundPropEnd == MagickFalse)
+    return(MagickFalse);
   /* clear the image based on the layer opacity */
   outLayer->image->background_color.opacity=
-    ScaleCharToQuantum(255-outLayer->opacity);    
+    ScaleCharToQuantum((unsigned char) (255-outLayer->opacity));    
   SetImageBackgroundColor(outLayer->image);
 
   /* set the compositing mode */
@@ -908,7 +914,8 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -1099,7 +1106,8 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*float  factor = (float) */ (void) ReadBlobMSBLong(image);
         /* unsigned long digits =  */ (void) ReadBlobMSBLong(image);
         for (i=0; i<5; i++)
-         (void) ReadBlobStringWithLongSize(image, unit_string);
+         (void) ReadBlobStringWithLongSize(image, unit_string,
+           sizeof(unit_string));
       }
      break;
 
@@ -1109,7 +1117,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         long amount;
 
       /* read over it... */
-      while (prop_size > 0)
+      while ((prop_size > 0) && (EOFBlob(image) == MagickFalse))
       {
         amount = (long) Min (16, prop_size);
         amount = (long) ReadBlob(image,(size_t) amount, (unsigned char *) &buf);
@@ -1122,6 +1130,10 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   }
 
+  
+  if (foundPropEnd == MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+
   if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0)) {
   ; /* do nothing, we were just pinging! */
   } else {
@@ -1256,9 +1268,14 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /* now reverse the order of the layers as they are put
            into subimages
         */
-        image->next=layer_info[number_layers-2].image;
-        layer_info[number_layers-2].image->previous=image;
-        for (j=number_layers-2; j>=0; j--)
+        j=number_layers-2;
+        image->next=layer_info[j].image;
+        layer_info[j].image->previous=image;
+        layer_info[j].image->page.x = layer_info[j].offset_x;
+        layer_info[j].image->page.y = layer_info[j].offset_y;
+        layer_info[j].image->page.width = layer_info[j].width;
+        layer_info[j].image->page.height = layer_info[j].height;
+        for (j=number_layers-3; j>=0; j--)
         {
           if (j > 0)
             layer_info[j].image->next=layer_info[j-1].image;
@@ -1357,8 +1374,8 @@ ModuleExport void RegisterXCFImage(void)
   entry=SetMagickInfo("XCF");
   entry->decoder=(DecoderHandler *) ReadXCFImage;
   entry->magick=(MagickHandler *) IsXCF;
-  entry->description=AcquireString("GIMP image");
-  entry->module=AcquireString("XCF");
+  entry->description=ConstantString("GIMP image");
+  entry->module=ConstantString("XCF");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
 }
