@@ -54,12 +54,14 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel-private.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/resize.h"
 #include "magick/resource_.h"
-#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/threshold.h"
 #include "magick/utility.h"
 
 
@@ -275,7 +277,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (count == 4)
       break;
   }
-  if ((count != 4) || (width > 2) || (image->columns == 0) ||
+  if ((count != 4) || (width > 10) || (image->columns == 0) ||
       (image->rows == 0) || (image->colors == 0))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   image->depth=16;
@@ -509,7 +511,8 @@ ModuleExport void UnregisterXPMImage(void)
 %
 %  The format of the WritePICONImage method is:
 %
-%      MagickBooleanType WritePICONImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePICONImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -517,7 +520,6 @@ ModuleExport void UnregisterXPMImage(void)
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   Image *image)
@@ -583,6 +585,9 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
     status,
     transparent;
 
+  MagickPixelPacket
+    pixel;
+
   RectangleInfo
     geometry;
 
@@ -713,14 +718,21 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   (void) FormatMagickString(buffer,MaxTextExtent,"\"%lu %lu %lu %ld\",\n",
     picon->columns,picon->rows,colors,characters_per_pixel);
   (void) WriteBlobString(image,buffer);
+  GetMagickPixelPacket(image,&pixel);
   for (i=0; i < (long) colors; i++)
   {
     /*
       Define XPM color.
     */
-    picon->colormap[i].opacity=OpaqueOpacity;
-    (void) QueryColorname(picon,picon->colormap+i,XPMCompliance,name,
-      &picon->exception);
+    SetMagickPixelPacket(picon->colormap+i,(IndexPacket *) NULL,&pixel);
+    pixel.colorspace=RGBColorspace;
+    pixel.depth=8;
+    pixel.opacity=OpaqueOpacity;
+    (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickFalse,name,
+      &image->exception);
+    if (LocaleNCompare(name,"rgb",3) == 0)
+      (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickTrue,name,
+        &image->exception);
     if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
@@ -826,11 +838,14 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
   long
     j,
     k,
+    opacity,
     y;
 
   MagickBooleanType
-    status,
-    transparent;
+    status;
+
+  MagickPixelPacket
+    pixel;
 
   register const PixelPacket
     *p;
@@ -842,12 +857,8 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
     i,
     x;
 
-  register PixelPacket
-    *q;
-
   unsigned long
-    characters_per_pixel,
-    colors;
+    characters_per_pixel;
 
   /*
     Open output image file.
@@ -861,99 +872,109 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-    if (image_info->colorspace == UndefinedColorspace)
+  if (image_info->colorspace == UndefinedColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
-  transparent=MagickFalse;
-  if (image->storage_class == PseudoClass)
+  opacity=(-1);
+  if (image->matte == MagickFalse)
     {
-      CompressImageColormap(image);
-      if (image->matte != MagickFalse)
-        transparent=MagickTrue;
+      if ((image->storage_class == DirectClass) || (image->colors > 256))
+        (void) SetImageType(image,PaletteType);
     }
   else
     {
+      MagickRealType
+        alpha,
+        beta;
+
       /*
-        Convert DirectClass to PseudoClass image.
+        Identify transparent colormap index.
       */
-      if (image->matte != MagickFalse)
-        {
-          /*
-            Map all the transparent pixels.
-          */
-          for (y=0; y < (long) image->rows; y++)
+      if ((image->storage_class == DirectClass) || (image->colors > 256))
+        (void) SetImageType(image,PaletteBilevelMatteType);
+      for (i=0; i < (long) image->colors; i++)
+        if (image->colormap[i].opacity != OpaqueOpacity)
           {
-            q=GetImagePixels(image,0,y,image->columns,1);
-            if (q == (PixelPacket *) NULL)
-              break;
-            for (x=0; x < (long) image->columns; x++)
-            {
-              if (q->opacity == TransparentOpacity)
-                transparent=MagickTrue;
-              else
-                q->opacity=OpaqueOpacity;
-              q++;
-            }
-            if (SyncImagePixels(image) == MagickFalse)
-              break;
+            if (opacity < 0)
+              {
+                opacity=i;
+                continue;
+              }
+            alpha=TransparentOpacity-(MagickRealType)
+              image->colormap[i].opacity;
+            beta=TransparentOpacity-(MagickRealType)
+              image->colormap[opacity].opacity;
+            if (alpha < beta)
+              opacity=i;
           }
-        }
-      (void) SetImageType(image,PaletteType);
-    }
-  colors=image->colors;
-  if (transparent != MagickFalse)
-    {
-      colors++;
-      image->colormap=(PixelPacket *) ResizeMagickMemory(image->colormap,
-        (size_t) colors*sizeof(*image->colormap));
-      if (image->colormap == (PixelPacket *) NULL)
-        ThrowWriterException(ResourceLimitError,"MemoryAllocationError");
-      for (y=0; y < (long) image->rows; y++)
-      {
-        q=GetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-          break;
-        indexes=GetIndexes(image);
-        for (x=0; x < (long) image->columns; x++)
+      if (opacity == -1)
         {
-          if (q->opacity == TransparentOpacity)
-            indexes[x]=(IndexPacket) image->colors;
-          q++;
+          (void) SetImageType(image,PaletteBilevelMatteType);
+          for (i=0; i < (long) image->colors; i++)
+            if (image->colormap[i].opacity != OpaqueOpacity)
+              {
+                if (opacity < 0)
+                  {
+                    opacity=i;
+                    continue;
+                  }
+                alpha=TransparentOpacity-(MagickRealType)
+                  image->colormap[i].opacity;
+                beta=TransparentOpacity-(MagickRealType)
+                  image->colormap[opacity].opacity;
+                if (alpha < beta)
+                  opacity=i;
+              }
+        }
+      if (opacity >= 0)
+        {
+          image->colormap[opacity].red=image->transparent_color.red;
+          image->colormap[opacity].green=image->transparent_color.green;
+          image->colormap[opacity].blue=image->transparent_color.blue;
         }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-      }
     }
   /*
     Compute the character per pixel.
   */
   characters_per_pixel=1;
-  for (k=MaxCixels; (long) colors > k; k*=MaxCixels)
+  for (k=MaxCixels; (long) image->colors > k; k*=MaxCixels)
     characters_per_pixel++;
   /*
     XPM header.
   */
   (void) WriteBlobString(image,"/* XPM */\n");
   GetPathComponent(image->filename,BasePath,basename);
+  if (isalnum((int) ((unsigned char) *basename)) == 0)
+    {
+      (void) FormatMagickString(buffer,MaxTextExtent,"xpm_%s",basename);
+      (void) CopyMagickString(basename,buffer,MaxTextExtent);
+    }
+  for (i=0; basename[i] != '\0'; i++)
+    if (isalpha((int) ((unsigned char) basename[i])) == 0)
+      basename[i]='_';
   (void) FormatMagickString(buffer,MaxTextExtent,
     "static char *%s[] = {\n",basename);
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,"/* columns rows colors chars-per-pixel */\n");
   (void) FormatMagickString(buffer,MaxTextExtent,"\"%lu %lu %lu %ld\",\n",
-    image->columns,image->rows,colors,characters_per_pixel);
+    image->columns,image->rows,image->colors,characters_per_pixel);
   (void) WriteBlobString(image,buffer);
-  for (i=0; i < (long) colors; i++)
+  GetMagickPixelPacket(image,&pixel);
+  for (i=0; i < (long) image->colors; i++)
   {
     /*
       Define XPM color.
     */
-    image->colormap[i].opacity=OpaqueOpacity;
-    (void) QueryColorname(image,image->colormap+i,XPMCompliance,name,
+    SetMagickPixelPacket(image->colormap+i,(IndexPacket *) NULL,&pixel);
+    pixel.colorspace=RGBColorspace;
+    pixel.depth=8;
+    pixel.opacity=OpaqueOpacity;
+    (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickFalse,name,
       &image->exception);
-    if (transparent != MagickFalse)
-      {
-        if (i == (long) (colors-1))
-          (void) CopyMagickString(name,"None",MaxTextExtent);
-      }
+    if (LocaleNCompare(name,"rgb",3) == 0)
+      (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickTrue,name,
+        &image->exception);
+    if (i == opacity)
+      (void) CopyMagickString(name,"None",MaxTextExtent);
     /*
       Write XPM color.
     */
@@ -965,8 +986,8 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
       symbol[j]=Cixel[k];
     }
     symbol[j]='\0';
-    (void) FormatMagickString(buffer,MaxTextExtent,"\"%s c %s\",\n",
-      symbol,name);
+    (void) FormatMagickString(buffer,MaxTextExtent,"\"%s c %s\",\n",symbol,
+      name);
     (void) WriteBlobString(image,buffer);
   }
   /*
