@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -44,8 +44,11 @@
 */
 #include "magick/studio.h"
 #include "magick/annotate.h"
+#include "magick/attribute.h"
+#include "magick/cache-view.h"
 #include "magick/client.h"
 #include "magick/color.h"
+#include "magick/color-private.h"
 #include "magick/composite.h"
 #include "magick/composite-private.h"
 #include "magick/constitute.h"
@@ -57,6 +60,7 @@
 #include "magick/geometry.h"
 #include "magick/image-private.h"
 #include "magick/log.h"
+#include "magick/quantum.h"
 #include "magick/resource_.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -64,7 +68,7 @@
 #include "magick/type.h"
 #include "magick/utility.h"
 #include "magick/xwindow-private.h"
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
 #if defined(__MINGW32__)
 #  undef interface
 #endif
@@ -158,7 +162,6 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
 {
   char
     primitive[MaxTextExtent],
-    *text,
     **textlist;
 
   DrawInfo
@@ -190,25 +193,17 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     height,
     number_lines;
 
-  /*
-    Translate any embedded format characters (e.g. %f).
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->signature == MagickSignature);
   if (draw_info->text == (char *) NULL)
     return(MagickFalse);
   if (*draw_info->text == '\0')
-    return(MagickFalse);
-  text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
-  if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  textlist=StringToList(text);
-  text=(char *) RelinquishMagickMemory(text);
+    return(MagickTrue);
+  textlist=StringToList(draw_info->text);
   if (textlist == (char **) NULL)
     return(MagickFalse);
   length=strlen(textlist[0]);
@@ -216,10 +211,6 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     if (strlen(textlist[i]) > length)
       length=strlen(textlist[i]);
   number_lines=(unsigned long) i;
-  text=(char *) AcquireMagickMemory(length+MaxTextExtent);
-  if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
   annotate=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   SetGeometry(image,&geometry);
@@ -229,15 +220,16 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
       (void) ParsePageGeometry(image,annotate_info->geometry,&geometry);
       (void) ParseGeometry(annotate_info->geometry,&geometry_info);
     }
-  image->storage_class=DirectClass;
+  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+    return(MagickFalse);
   status=MagickTrue;
   for (i=0; textlist[i] != (char *) NULL; i++)
   {
     /*
       Position text relative to image.
     */
-    annotate_info->affine.tx=geometry_info.xi;
-    annotate_info->affine.ty=geometry_info.psi;
+    annotate_info->affine.tx=geometry_info.xi-image->page.x;
+    annotate_info->affine.ty=geometry_info.psi-image->page.y;
     (void) CloneString(&annotate->text,textlist[i]);
     (void) GetTypeMetrics(image,annotate,&metrics);
     height=(unsigned long) (metrics.ascent-metrics.descent+0.5);
@@ -397,7 +389,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
           "rectangle 0,0 %g,%ld",metrics.width,height);
         (void) CloneString(&undercolor_info->primitive,primitive);
         (void) DrawImage(image,undercolor_info);
-        DestroyDrawInfo(undercolor_info);
+        (void) DestroyDrawInfo(undercolor_info);
       }
     annotate_info->affine.tx=offset.x;
     annotate_info->affine.ty=offset.y;
@@ -438,14 +430,13 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   */
   annotate_info=DestroyDrawInfo(annotate_info);
   annotate=DestroyDrawInfo(annotate);
-  text=(char *) RelinquishMagickMemory(text);
   for (i=0; textlist[i] != (char *) NULL; i++)
     textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
   textlist=(char **) RelinquishMagickMemory(textlist);
   return(status);
 }
 
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -503,7 +494,7 @@ static int GetOneCharacter(const unsigned char *text,size_t *length)
   return((int) c);
 }
 
-static unsigned short *EncodeSJIS(const Image *image,const char *text,
+static unsigned long *EncodeSJIS(const Image *image,const char *text,
   size_t *count)
 {
   int
@@ -512,21 +503,21 @@ static unsigned short *EncodeSJIS(const Image *image,const char *text,
   register const char
     *p;
 
-  register unsigned short
+  register unsigned long
     *q;
 
   size_t
     length;
 
-  unsigned short
+  unsigned long
     *encoding;
 
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned short *) NULL);
+    return((unsigned long *) NULL);
   length=strlen(text)+MaxTextExtent;
-  encoding=(unsigned short *) AcquireMagickMemory(length*sizeof(*encoding));
-  if (encoding == (unsigned short *) NULL)
+  encoding=(unsigned long *) AcquireMagickMemory(length*sizeof(*encoding));
+  if (encoding == (unsigned long *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       image->filename);
   q=encoding;
@@ -541,7 +532,7 @@ static unsigned short *EncodeSJIS(const Image *image,const char *text,
           *q++=(unsigned char) *p;
         break;
       }
-    *q=(unsigned short) c;
+    *q=(unsigned long) c;
     q++;
   }
   *count=q-encoding;
@@ -549,7 +540,7 @@ static unsigned short *EncodeSJIS(const Image *image,const char *text,
 }
 #endif
 
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -578,24 +569,24 @@ static unsigned short *EncodeSJIS(const Image *image,const char *text,
 %
 %
 */
-static unsigned short *EncodeText(const Image *image,const char *text,
+static unsigned long *EncodeText(const Image *image,const char *text,
   size_t *count)
 {
   register const char
     *p;
 
-  register unsigned short
+  register unsigned long
     *q;
 
-  unsigned short
+  unsigned long
     *encoding;
 
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned short *) NULL);
-  encoding=(unsigned short *)
+    return((unsigned long *) NULL);
+  encoding=(unsigned long *)
     AcquireMagickMemory((strlen(text)+MaxTextExtent)*sizeof(*encoding));
-  if (encoding == (unsigned short *) NULL)
+  if (encoding == (unsigned long *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       image->filename);
   q=encoding;
@@ -606,7 +597,7 @@ static unsigned short *EncodeText(const Image *image,const char *text,
 }
 #endif
 
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -624,19 +615,18 @@ static unsigned short *EncodeText(const Image *image,const char *text,
 %
 %  The format of the EncodeUnicode function is:
 %
-%      unicode=EncodeUnicode(const Image *image,const unsigned char *text,
-%        size_t count)
+%      unsigned short *EncodeUnicode(const Image *image,
+%        const unsigned char *text,size_t count)
 %
 %  A description of each parameter follows:
 %
 %    o unicode:  EncodeUnicode() returns a pointer to an unsigned short array
-%      array representing the encoded version of the ASCII string.
+%      representing the encoded version of the ASCII string.
 %
 %    o text: The text.
 %
 %    o count: return the number of characters generated by the encoding.
 %
-%
 */
 
 static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
@@ -690,7 +680,7 @@ static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
   return((long) c);
 }
 
-static unsigned short *EncodeUnicode(const Image *image,const char *text,
+static unsigned long *EncodeUnicode(const Image *image,const char *text,
   size_t *count)
 {
   long
@@ -699,21 +689,21 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
   register const char
     *p;
 
-  register unsigned short
+  register unsigned long
     *q;
 
   size_t
     length;
 
-  unsigned short
+  unsigned long
     *unicode;
 
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned short *) NULL);
+    return((unsigned long *) NULL);
   length=strlen(text)+MaxTextExtent;
-  unicode=(unsigned short *) AcquireMagickMemory(length*sizeof(*unicode));
-  if (unicode == (unsigned short *) NULL)
+  unicode=(unsigned long *) AcquireMagickMemory(length*sizeof(*unicode));
+  if (unicode == (unsigned long *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       image->filename);
   q=unicode;
@@ -728,7 +718,7 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
           *q++=(unsigned char) *p;
         break;
       }
-    *q=(unsigned short) c;
+    *q=(unsigned long) c;
     q++;
   }
   *count=q-unicode;
@@ -781,7 +771,6 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   const DrawInfo *draw_info,TypeMetric *metrics)
 {
   char
-    *text,
     **textlist;
 
   double
@@ -805,7 +794,7 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->text != (char *) NULL);
   assert(draw_info->signature == MagickSignature);
@@ -813,18 +802,10 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
     return(MagickFalse);
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   annotate_info->text=(char *) RelinquishMagickMemory(annotate_info->text);
-  /*
-    Substitute values for formating characters (%x, etc).
-  */
-  text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
-  if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
   /*
     Convert newlines to multiple lines of text.
   */
-  textlist=StringToList(text);
-  text=(char *) RelinquishMagickMemory(text);
+  textlist=StringToList(draw_info->text);
   if (textlist == (char **) NULL)
     return(MagickFalse);
   annotate_info->render=MagickFalse;
@@ -834,19 +815,20 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   /*
     Find the widest of the text lines.
   */
-  annotate_info->text = textlist[0];
+  annotate_info->text=textlist[0];
   status=RenderType(image,annotate_info,&offset,metrics);
-  max_width = metrics->width;
+  max_width=metrics->width;
   for (i=1; textlist[i] != (char *) NULL; i++)
   {
-    annotate_info->text = textlist[i];
+    annotate_info->text=textlist[i];
     status=RenderType(image,annotate_info,&offset,metrics);
     if (metrics->width > max_width)
       max_width=metrics->width;
   }
-  number_lines= i;
+  number_lines=(unsigned long) i;
   metrics->width=max_width;
-  metrics->height=number_lines*(long) (metrics->ascent-metrics->descent+0.5)+1;
+  metrics->height=(double) number_lines*(long) (metrics->ascent-
+    metrics->descent+0.5)+1;
   /*
     Free resources.
   */
@@ -912,7 +894,7 @@ MagickExport MagickBooleanType GetTypeMetrics(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->text != (char *) NULL);
   assert(draw_info->signature == MagickSignature);
@@ -1048,7 +1030,7 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
 %
 */
 
-#if defined(HasTTF)
+#if defined(HasFREETYPE)
 static int TraceCubicBezier(FT_Vector *p,FT_Vector *q,FT_Vector *to,
   DrawInfo *draw_info)
 {
@@ -1132,6 +1114,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       image;
   } GlyphInfo;
 
+  const ImageAttribute
+    *attribute;
+
   DrawInfo
     *annotate_info;
 
@@ -1150,6 +1135,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   FT_Face
     face;
 
+  FT_Int32
+    flags;
+
   FT_Library
     library;
 
@@ -1205,7 +1193,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       0, 0
     };
 
-  unsigned short
+  unsigned long
     *text;
 
   /*
@@ -1220,6 +1208,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     args.pathname=draw_info->font;
   else
     args.pathname=draw_info->font+1;
+  face=(FT_Face) NULL;
   status=FT_Open_Face(library,&args,draw_info->face,&face);
   if (status != 0)
     {
@@ -1249,8 +1238,10 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         encoding_type=ft_encoding_gb2312;
       if (LocaleCompare(encoding,"Johab") == 0)
         encoding_type=ft_encoding_johab;
+#if defined(ft_encoding_latin_1)
       if (LocaleCompare(encoding,"Latin-1") == 0)
         encoding_type=ft_encoding_latin_1;
+#endif
       if (LocaleCompare(encoding,"Latin-2") == 0)
         encoding_type=ft_encoding_latin_2;
       if (LocaleCompare(encoding,"None") == 0)
@@ -1295,7 +1286,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->descent=(double) face->size->metrics.descender/64.0;
   metrics->width=0;
   metrics->height=(double) face->size->metrics.height/64.0;
-  metrics->max_advance=(double) face->size->metrics.max_advance/64.0;
+  metrics->max_advance=0.0;
+  if (face->size->metrics.max_advance > MagickEpsilon)
+    metrics->max_advance=(double) face->size->metrics.max_advance/64.0;
   metrics->bounds.x1=0.0;
   metrics->bounds.y1=metrics->descent;
   metrics->bounds.x2=metrics->ascent+metrics->descent;
@@ -1348,7 +1341,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       break;
     }
   }
-  if (text == (unsigned short *) NULL)
+  if (text == (unsigned long *) NULL)
     {
       (void) FT_Done_Face(face);
       (void) FT_Done_FreeType(library);
@@ -1365,6 +1358,11 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       encoding != (char *) NULL ? encoding : "none",
       draw_info->encoding != (char *) NULL ? draw_info->encoding : "none",
       draw_info->pointsize);
+  flags=FT_LOAD_NO_BITMAP;
+  attribute=GetImageAttribute(image,"type:hinting");
+  if ((attribute != (const ImageAttribute *) NULL) &&
+      (LocaleCompare(attribute->value,"off") == 0))
+    flags|=FT_LOAD_NO_HINTING;
   glyph.id=0;
   glyph.image=NULL;
   last_glyph.id=0;
@@ -1378,7 +1376,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   (void) QueryColorDatabase("#000000ff",&annotate_info->fill,&image->exception);
   (void) CloneString(&annotate_info->primitive,"path '");
-  if (image->matte == MagickFalse)
+  if ((draw_info->render != MagickFalse) && (image->matte == MagickFalse))
     SetImageOpacity(image,OpaqueOpacity);
   for (i=0; i < (long) length; i++)
   {
@@ -1429,6 +1427,13 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
     if (draw_info->render != MagickFalse)
       {
+        long
+          x_offset,
+          y_offset;
+
+        ViewInfo
+          *image_view;
+
         /*
           Rasterize the glyph.
         */
@@ -1437,55 +1442,57 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         if (status != 0)
           continue;
         bitmap=(FT_BitmapGlyph) glyph.image;
-        image->storage_class=DirectClass;
+        if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+          return(MagickFalse);
         point.x=offset->x+bitmap->left;
         point.y=offset->y-bitmap->top;
         p=bitmap->bitmap.buffer;
+        image_view=OpenCacheView(image);
         for (y=0; y < (long) bitmap->bitmap.rows; y++)
         {
-          if (((long) (point.y+y+0.5) < 0) ||
-              ((long) (point.y+y+0.5) >= (long) image->rows))
+          x_offset=(long) (point.x+0.5);
+          y_offset=(long) (point.y+y+0.5);
+          if ((y_offset < 0) || (y_offset >= (long) image->rows))
             {
               p+=bitmap->bitmap.width;
               continue;
             }
-          q=GetImagePixels(image,(long) (point.x+0.5),(long) (point.y+y+0.5),
-            bitmap->bitmap.width,1);
+          q=GetCacheView(image_view,x_offset,y_offset,bitmap->bitmap.width,1);
           active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
           for (x=0; x < (long) bitmap->bitmap.width; x++)
           {
-            if (((long) (point.x+x+0.5) < 0) || (*p == 0) ||
-                ((unsigned long) (point.x+x+0.5) >= image->columns))
+            x_offset=(long) (point.x+x+0.5);
+            if ((*p == 0) || (x_offset < 0) ||
+                (x_offset >= (long) image->columns))
               {
                 p++;
                 q++;
                 continue;
               }
-            if (draw_info->text_antialias)
-              fill_opacity=(MagickRealType) (*p)/255.0;
-            else
-              fill_opacity=(*p) > 127 ? 1.0 : 0.0;
+            fill_opacity=(MagickRealType) (*p)/255.0;
+            if (draw_info->text_antialias == MagickFalse)
+              fill_opacity=fill_opacity > 0.5 ? 1.0 : 0.0;
             if (active == MagickFalse)
-              q=GetImagePixels(image,(long) (point.x+x+0.5),(long)
-                (point.y+y+0.5),1,1);
+              q=GetCacheView(image_view,x_offset,y_offset,1,1);
             if (q == (PixelPacket *) NULL)
               {
                 p++;
                 q++;
                 continue;
               }
-            fill_color=GetFillColor(draw_info,(long) (point.x+x+0.5),(long)
-              (point.y+y+0.5));
-            fill_opacity=MaxRGB-fill_opacity*(MaxRGB-fill_color.opacity);
+            fill_color=GetFillColor(draw_info,x_offset,y_offset);
+            fill_opacity=QuantumRange-fill_opacity*(QuantumRange-
+              fill_color.opacity);
             MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
             if (active == MagickFalse)
-              (void) SyncImagePixels(image);
+              (void) SyncCacheView(image_view);
             p++;
             q++;
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncCacheView(image_view) == MagickFalse)
             break;
         }
+        image_view=CloseCacheView(image_view);
       }
     origin.x+=face->glyph->advance.x;
     if (origin.x > metrics->width)
@@ -1499,32 +1506,39 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y1/=64.0;
   metrics->bounds.x2/=64.0;
   metrics->bounds.y2/=64.0;
-  if (draw_info->render != MagickFalse)
-    if ((draw_info->stroke.opacity != TransparentOpacity) ||
-        (draw_info->stroke_pattern != (Image *) NULL))
-      {
-        /*
-          Draw text stroke.
-        */
-        annotate_info->affine.tx=offset->x;
-        annotate_info->affine.ty=offset->y;
-        (void) ConcatenateString(&annotate_info->primitive,"'");
-        (void) DrawImage(image,annotate_info);
+  if ((draw_info->stroke.opacity != TransparentOpacity) ||
+      (draw_info->stroke_pattern != (Image *) NULL))
+    {
+      metrics->bounds.x1-=draw_info->stroke_width/2.0;
+      metrics->bounds.y1-=draw_info->stroke_width/2.0;
+      metrics->bounds.x2+=draw_info->stroke_width/2.0;
+      metrics->bounds.y2+=draw_info->stroke_width/2.0;
+      if (draw_info->render != MagickFalse)
+        {
+          /*
+            Draw text stroke.
+          */
+          annotate_info->affine.tx=offset->x;
+          annotate_info->affine.ty=offset->y;
+          (void) ConcatenateString(&annotate_info->primitive,"'");
+          (void) DrawImage(image,annotate_info);
+        }
       }
   if (glyph.id != 0)
     FT_Done_Glyph(glyph.image);
   /*
     Free resources.
   */
-  text=(unsigned short *) RelinquishMagickMemory(text);
+  text=(unsigned long *) RelinquishMagickMemory(text);
   annotate_info=DestroyDrawInfo(annotate_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
   return(MagickTrue);
 }
 #else
-static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
-  const char *encoding,const PointInfo *offset,TypeMetric *metrics)
+static MagickBooleanType RenderFreetype(Image *image,
+  const DrawInfo *magick_unused(draw_info),const char *magick_unused(encoding),
+  const PointInfo *magick_unused(offset),TypeMetric *magick_unused(metrics))
 {
   ThrowBinaryException(MissingDelegateError,"FreeTypeLibraryIsNotAvailable",
     image->filename)
@@ -1605,8 +1619,7 @@ static MagickBooleanType RenderPostscript(Image *image,
     *file;
 
   Image
-    *annotate_image,
-    *pattern;
+    *annotate_image;
 
   ImageInfo
     *annotate_info;
@@ -1661,9 +1674,9 @@ static MagickBooleanType RenderPostscript(Image *image,
   /*
     Sample to compute bounding box.
   */
-  identity=(MagickBooleanType)
-    ((draw_info->affine.sx == draw_info->affine.sy) &&
-     (draw_info->affine.rx == 0.0) && (draw_info->affine.ry == 0.0));
+  identity=(draw_info->affine.sx == draw_info->affine.sy) &&
+    (draw_info->affine.rx == 0.0) && (draw_info->affine.ry == 0.0) ?
+    MagickTrue : MagickFalse;
   extent.x=0.0;
   extent.y=0.0;
   for (i=0; i <= (long) (strlen(draw_info->text)+2); i++)
@@ -1700,7 +1713,7 @@ static MagickBooleanType RenderPostscript(Image *image,
   (void) fclose(file);
   (void) FormatMagickString(geometry,MaxTextExtent,"%ldx%ld+0+0!",
     (long) (extent.x+0.5),(long) (extent.y+0.5));
-  annotate_info=CloneImageInfo((ImageInfo *) NULL);
+  annotate_info=AcquireImageInfo();
   (void) FormatMagickString(annotate_info->filename,MaxTextExtent,
     "ps:%s",filename);
   (void) CloneString(&annotate_info->page,geometry);
@@ -1769,6 +1782,9 @@ static MagickBooleanType RenderPostscript(Image *image,
       PixelPacket
         fill_color;
 
+      ViewInfo
+        *annotate_view;
+
       /*
         Render fill color.
       */
@@ -1777,30 +1793,27 @@ static MagickBooleanType RenderPostscript(Image *image,
       if (annotate_image->matte == MagickFalse)
         SetImageOpacity(annotate_image,OpaqueOpacity);
       fill_color=draw_info->fill;
-      pattern=draw_info->fill_pattern;
+      annotate_view=OpenCacheView(annotate_image);
       for (y=0; y < (long) annotate_image->rows; y++)
       {
-        q=GetImagePixels(annotate_image,0,y,annotate_image->columns,1);
+        q=GetCacheView(annotate_view,0,y,annotate_image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
         for (x=0; x < (long) annotate_image->columns; x++)
         {
-          if (pattern != (Image *) NULL)
-            fill_color=AcquireOnePixel(pattern,
-              (long) (x-pattern->extract_info.x) % pattern->columns,
-              (long) (y-pattern->extract_info.y) % pattern->rows,
-              &image->exception);
-          q->opacity=(Quantum) (MaxRGB-(((MaxRGB-(MagickRealType)
-            PixelIntensityToQuantum(q))*(MaxRGB-fill_color.opacity))/
-            MaxRGB)+0.5);
+          fill_color=GetFillColor(draw_info,x,y);
+          q->opacity=(Quantum) (QuantumRange-(((QuantumRange-(MagickRealType)
+            PixelIntensityToQuantum(q))*(QuantumRange-fill_color.opacity))/
+            QuantumRange)+0.5);
           q->red=fill_color.red;
           q->green=fill_color.green;
           q->blue=fill_color.blue;
           q++;
         }
-        if (SyncImagePixels(annotate_image) == MagickFalse)
+        if (SyncCacheView(annotate_view) == MagickFalse)
           break;
       }
+      annotate_view=CloseCacheView(annotate_view);
       (void) CompositeImage(image,OverCompositeOp,annotate_image,
         (long) (offset->x+0.5),(long) (offset->y-(metrics->ascent+
         metrics->descent)+0.5));
