@@ -47,6 +47,8 @@
 #include "magick/blob.h"
 #include "magick/draw.h"
 #include "magick/exception-private.h"
+#include "magick/fx-private.h"
+#include "magick/geometry.h"
 #include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/option.h"
@@ -104,6 +106,48 @@ MagickExport MagickBooleanType CloneImageAttributes(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(clone_image != (const Image *) NULL);
   assert(clone_image->signature == MagickSignature);
+  image->compression=clone_image->compression;
+  image->quality=clone_image->quality;
+  image->depth=clone_image->depth;
+  image->background_color=clone_image->background_color;
+  image->border_color=clone_image->border_color;
+  image->matte_color=clone_image->matte_color;
+  image->transparent_color=clone_image->transparent_color;
+  image->gamma=clone_image->gamma;
+  image->chromaticity=clone_image->chromaticity;
+  image->rendering_intent=clone_image->rendering_intent;
+  image->black_point_compensation=clone_image->black_point_compensation;
+  image->units=clone_image->units;
+  image->montage=(char *) NULL;
+  image->directory=(char *) NULL;
+  (void) CloneString(&image->geometry,clone_image->geometry);
+  image->offset=clone_image->offset;
+  image->x_resolution=clone_image->x_resolution;
+  image->y_resolution=clone_image->y_resolution;
+  image->page=clone_image->page;
+  image->extract_info=clone_image->extract_info;
+  image->bias=clone_image->bias;
+  image->blur=clone_image->blur;
+  image->fuzz=clone_image->fuzz;
+  image->filter=clone_image->filter;
+  image->interlace=clone_image->interlace;
+  image->interpolate=clone_image->interpolate;
+  image->endian=clone_image->endian;
+  image->gravity=clone_image->gravity;
+  image->compose=clone_image->compose;
+  image->scene=clone_image->scene;
+  image->orientation=clone_image->orientation;
+  image->dispose=clone_image->dispose;
+  image->delay=clone_image->delay;
+  image->ticks_per_second=clone_image->ticks_per_second;
+  image->iterations=clone_image->iterations;
+  image->total_colors=clone_image->total_colors;
+  image->taint=clone_image->taint;
+  image->progress_monitor=clone_image->progress_monitor;
+  image->client_data=clone_image->client_data;
+  image->start_loop=clone_image->start_loop;
+  image->error=clone_image->error;
+  image->signature=clone_image->signature;
   if (clone_image->attributes == (void *) NULL)
     return(MagickTrue);
   ResetImageAttributeIterator(clone_image);
@@ -276,7 +320,6 @@ MagickExport MagickBooleanType FormatImageAttribute(Image *image,
 %    o key:  These character strings are the name of an image attribute to
 %      return.
 %
-%
 */
 
 static MagickBooleanType GenerateIPTCAttribute(const Image *image,
@@ -322,8 +365,8 @@ static MagickBooleanType GenerateIPTCAttribute(const Image *image,
           {
             (void) CopyMagickString(message,(char *) profile->datum+i+5,
               length+1);
-            ConcatenateString(&attribute,message);
-            ConcatenateString(&attribute,";");
+            (void) ConcatenateString(&attribute,message);
+            (void) ConcatenateString(&attribute,";");
             message=(char *) RelinquishMagickMemory(message);
           }
       }
@@ -1302,6 +1345,18 @@ static int GenerateEXIFAttribute(const Image *image,const char *specification)
 MagickExport const ImageAttribute *GetImageAttribute(const Image *image,
   const char *key)
 {
+  ExceptionInfo
+    *exception;
+
+  FxInfo
+    *fx_info;
+
+  MagickRealType
+    alpha;
+
+  MagickStatusType
+    status;
+
   register const ImageAttribute
     *p;
 
@@ -1309,34 +1364,90 @@ MagickExport const ImageAttribute *GetImageAttribute(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if (image->attributes == (void *) NULL)
-   return((const ImageAttribute *) NULL);
+  p=(const ImageAttribute *) NULL;
   if (key == (const char *) NULL)
     {
       ResetSplayTreeIterator((SplayTreeInfo *) image->attributes);
-      p=(const ImageAttribute *)
-        GetNextValueInSplayTree((SplayTreeInfo *) image->attributes);
+      p=(const ImageAttribute *) GetNextValueInSplayTree((SplayTreeInfo *)
+        image->attributes);
       return(p);
     }
-  p=(const ImageAttribute *)
-    GetValueFromSplayTree((SplayTreeInfo *) image->attributes,key);
-  if (p != (const ImageAttribute *) NULL)
-    return(p);
+  if (image->attributes != (void *) NULL)
+    {
+      p=(const ImageAttribute *) GetValueFromSplayTree((SplayTreeInfo *)
+        image->attributes,key);
+      if (p != (const ImageAttribute *) NULL)
+        return(p);
+    }
   if (LocaleNCompare("iptc:",key,5) == 0)
     {
-      if (GenerateIPTCAttribute(image,key) == MagickTrue)
+      if (GenerateIPTCAttribute(image,key) != MagickFalse)
         return(GetImageAttribute(image,key));
     }
   if (LocaleNCompare("8bim:",key,5) == 0)
     {
-      if (Generate8BIMAttribute(image,key) == MagickTrue)
+      if (Generate8BIMAttribute(image,key) != MagickFalse)
         return(GetImageAttribute(image,key));
     }
   if (LocaleNCompare("exif:",key,5) == 0)
     {
-      if (GenerateEXIFAttribute(image,key) == MagickTrue)
+      if (GenerateEXIFAttribute(image,key) != MagickFalse)
         return(GetImageAttribute(image,key));
     }
+  exception=(&((Image *) image)->exception);
+  if (LocaleNCompare("fx:",key,3) == 0)
+    {
+      fx_info=AcquireFxInfo(image,key+3);  
+      status=FxEvaluateExpression(fx_info,&alpha,exception);
+      fx_info=DestroyFxInfo(fx_info);
+      if (status != MagickFalse)
+        {
+          char
+            value[MaxTextExtent];
+
+          (void) FormatMagickString(value,MaxTextExtent,"%g",alpha);
+          (void) SetImageAttribute((Image *) image,key,value);
+          return(GetImageAttribute(image,key));
+        }
+    }
+  if (LocaleNCompare("pixel:",key,6) == 0)
+    {
+      MagickPixelPacket
+        pixel;
+
+      GetMagickPixelPacket(image,&pixel);
+      fx_info=AcquireFxInfo(image,key+6);  
+      status=FxEvaluateChannelExpression(fx_info,RedChannel,&alpha,exception);
+      pixel.red=QuantumRange*alpha;
+      status|=FxEvaluateChannelExpression(fx_info,GreenChannel,&alpha,
+        exception);
+      pixel.green=QuantumRange*alpha;
+      status|=FxEvaluateChannelExpression(fx_info,BlueChannel,&alpha,exception);
+      pixel.blue=QuantumRange*alpha;
+      if (image->matte != MagickFalse)
+        {
+          status|=FxEvaluateChannelExpression(fx_info,OpacityChannel,&alpha,
+            exception);
+          pixel.opacity=QuantumRange*(1.0-alpha);
+        }
+      if (image->colorspace == CMYKColorspace)
+        {
+          status|=FxEvaluateChannelExpression(fx_info,BlackChannel,&alpha,
+            exception);
+          pixel.index=QuantumRange*alpha;
+        }
+      fx_info=DestroyFxInfo(fx_info);
+      if (status != MagickFalse)
+        {
+          char
+            name[MaxTextExtent];
+
+          (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickFalse,
+            name,exception);
+          (void) SetImageAttribute((Image *) image,key,name);
+          return(GetImageAttribute(image,key));
+        }
+    }
   return(p);
 }
 
@@ -1365,7 +1476,6 @@ MagickExport const ImageAttribute *GetImageAttribute(const Image *image,
 %
 %    o image: The image.
 %
-%
 */
 MagickExport const ImageAttribute *GetImageClippingPathAttribute(Image *image)
 {
@@ -1503,7 +1613,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
           char
             format[MaxTextExtent];
 
-          FormatSize(GetBlobSize(image),format);
+          FormatMagickSize(GetBlobSize(image),format);
           (void) FormatMagickString(attribute,MaxTextExtent,"%s",format);
           break;
         }
@@ -1759,22 +1869,6 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
     p++;
     switch (*p)
     {
-      case 'O':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%+ld%+ld",image->page.x,
-          image->page.y);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'P':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu",image->page.width,
-          image->page.height);
-        while (*q != '\0')
-          q++;
-        break;
-      }
       case 'b':
       {
         char
@@ -1790,7 +1884,7 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         (void) FormatMagickString(format,MaxTextExtent,"%lu",
           (unsigned long) length);
         if (length != (MagickSizeType) ((size_t) length))
-          FormatSize(length,format);
+          FormatMagickSize(length,format);
         (void) ConcatenateMagickString(q,format,MaxTextExtent);
         while (*q != '\0')
           q++;
@@ -1868,10 +1962,8 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Image geometry.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
+        q+=FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
           image->page.width,image->page.height,image->page.x,image->page.y);
-        while (*q != '\0')
-          q++;
         break;
       }
       case 'h':
@@ -1879,10 +1971,8 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Image height.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",
           image->rows != 0 ? image->rows : image->magick_rows);
-        while (*q != '\0')
-          q++;
         break;
       }
       case 'i':
@@ -1899,10 +1989,8 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Number of unique colors.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",
           GetNumberColors(image,(FILE *) NULL,&image->exception));
-        while (*q != '\0')
-          q++;
         break;
       }
       case 'l':
@@ -1926,15 +2014,22 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         q+=strlen(image->magick);
         break;
       }
+      case 'M':
+      {
+        /*
+          Image magick filename.
+        */
+        (void) CopyMagickString(q,image->magick_filename,MaxTextExtent);
+        q+=strlen(image->magick_filename);
+        break;
+      }
       case 'n':
       {
         /*
           Number of images in the list.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",(unsigned long)
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",(unsigned long)
           GetImageListLength(image));
-        while (*q != '\0')
-          q++;
         break;
       }
       case 'o':
@@ -1970,9 +2065,7 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Image depth.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->depth);
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",image->depth);
         break;
       }
       case 'r':
@@ -1999,11 +2092,10 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Image scene number.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->scene);
-        if (text_info->number_scenes != 0)
-          (void) FormatMagickString(q,MaxTextExtent,"%lu",text_info->scene);
-        while (*q != '\0')
-          q++;
+        if (text_info->number_scenes == 0)
+          q+=FormatMagickString(q,MaxTextExtent,"%lu",image->scene);
+        else
+          q+=FormatMagickString(q,MaxTextExtent,"%lu",text_info->scene);
         break;
       }
       case 'u':
@@ -2021,10 +2113,8 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Image width.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",
           image->columns != 0 ? image->columns : image->magick_columns);
-        while (*q != '\0')
-          q++;
         break;
       }
       case 'x':
@@ -2032,10 +2122,8 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Image horizontal resolution.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%g %s",image->x_resolution,
+        q+=FormatMagickString(q,MaxTextExtent,"%g %s",image->x_resolution,
           MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-        while (*q != '\0')
-          q++;
         break;
       }
       case 'y':
@@ -2043,10 +2131,8 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Image vertical resolution.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%g %s",image->y_resolution,
+        q+=FormatMagickString(q,MaxTextExtent,"%g %s",image->y_resolution,
           MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-        while (*q != '\0')
-          q++;
         break;
       }
       case 'z':
@@ -2054,10 +2140,39 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         /*
           Unique filename.
         */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          GetImageDepth(image,&image->exception));
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",GetImageDepth(image,
+          &image->exception));
+        break;
+      }
+      case 'D':
+      {
+        /*
+          Image vertical resolution.
+        */
+        q+=FormatMagickString(q,MaxTextExtent,"%s",MagickOptionToMnemonic(
+          MagickDisposeOptions,(long) image->dispose));
+        break;
+      }
+      case 'O':
+      {
+        q+=FormatMagickString(q,MaxTextExtent,"%+ld%+ld",image->page.x,
+          image->page.y);
+        break;
+      }
+      case 'P':
+      {
+        q+=FormatMagickString(q,MaxTextExtent,"%lux%lu",image->page.width,
+          image->page.height);
+        break;
+      }
+      case 'Q':
+      {
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",image->quality);
+        break;
+      }
+      case 'T':
+      {
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",image->delay);
         break;
       }
       case '[':
@@ -2065,6 +2180,12 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         char
           key[MaxTextExtent];
 
+        const char
+          *value;
+
+        long
+          depth;
+
         MagickOffsetType
           offset;
 
@@ -2073,9 +2194,18 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
         */
         if (strchr(p,']') == (char *) NULL)
           break;
+        depth=1;
         p++;
-        for (i=0; (i < (long) MaxTextExtent) && (*p != ']'); i++)
+        for (i=0; (i < (MaxTextExtent-1L)) && (*p != '\0'); i++)
+        {
+          if (*p == '[')
+            depth++;
+          if (*p == ']')
+            depth--;
+          if (depth <= 0)
+            break;
           key[i]=(*p++);
+        }
         key[i]='\0';
         attribute=GetImageAttribute(image,key);
         if (attribute != (const ImageAttribute *) NULL)
@@ -2118,28 +2248,46 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
                           break;
                         q=translate_text+strlen(translate_text);
                       }
-                    (void) FormatMagickString(q,MaxTextExtent,"%s=%s\n",
+                    q+=FormatMagickString(q,MaxTextExtent,"%s=%s\n",
                       attribute->key,attribute->value);
-                    q+=offset;
                   }
                 attribute=GetNextImageAttribute(image);
               }
             }
         attribute=GetImageInfoAttribute(text_info,image,key);
-        if (attribute == (const ImageAttribute *) NULL)
-          break;
-        offset=(MagickOffsetType) strlen(attribute->value);
-        if ((size_t) (q-translate_text+offset+1) >= length)
+        if (attribute != (const ImageAttribute *) NULL)
           {
-            length+=offset;
-            translate_text=(char *) ResizeMagickMemory(translate_text,
-              (length+MaxTextExtent)*sizeof(*translate_text));
-            if (translate_text == (char *) NULL)
-              break;
-            q=translate_text+strlen(translate_text);
+            offset=(MagickOffsetType) strlen(attribute->value);
+            if ((size_t) (q-translate_text+offset+1) >= length)
+              {
+                length+=offset;
+                translate_text=(char *) ResizeMagickMemory(translate_text,
+                  (length+MaxTextExtent)*sizeof(*translate_text));
+                if (translate_text == (char *) NULL)
+                  break;
+                q=translate_text+strlen(translate_text);
+              }
+            (void) CopyMagickString(q,attribute->value,length+MaxTextExtent);
+            q+=offset;
+          }
+        if (image_info == (ImageInfo *) NULL)
+          break;
+        value=GetImageOption(image_info,key);
+        if (value != (char *) NULL)
+          {
+            offset=(MagickOffsetType) strlen(value);
+            if ((size_t) (q-translate_text+offset+1) >= length)
+              {
+                length+=offset;
+                translate_text=(char *) ResizeMagickMemory(translate_text,
+                  (length+MaxTextExtent)*sizeof(*translate_text));
+                if (translate_text == (char *) NULL)
+                  break;
+                q=translate_text+strlen(translate_text);
+              }
+            (void) CopyMagickString(q,value,length+MaxTextExtent);
+            q+=offset;
           }
-        (void) CopyMagickString(q,attribute->value,length+MaxTextExtent);
-        q+=offset;
         break;
       }
       case '@':
@@ -2151,10 +2299,8 @@ MagickExport char *InterpretImageAttributes(const ImageInfo *image_info,
           Image bounding box.
         */
         page=GetImageBoundingBox(image,&image->exception);
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
+        q+=FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
           page.width,page.height,page.x,page.y);
-        while (*q != '\0')
-          q++;
         break;
       }
       case '#':
@@ -2268,7 +2414,7 @@ static void *DestroyAttribute(void *attribute)
   return(RelinquishMagickMemory(p));
 }
 
-MagickExport  MagickBooleanType SetImageAttribute(Image *image,const char *key,
+MagickExport MagickBooleanType SetImageAttribute(Image *image,const char *key,
   const char *value)
 {
   ImageAttribute
@@ -2277,6 +2423,9 @@ MagickExport  MagickBooleanType SetImageAttribute(Image *image,const char *key,
   MagickBooleanType
     status;
 
+  MagickStatusType
+    flags;
+
   register const char
     *q;
 
@@ -2309,12 +2458,53 @@ MagickExport  MagickBooleanType SetImageAttribute(Image *image,const char *key,
   if (*q != '\0')
     attribute->value=ConstantString(value);
   else
-    attribute->value=InterpretImageAttributes((ImageInfo *) NULL,image,value);
+    {
+      ImageInfo
+        *image_info;
+
+      image_info=AcquireImageInfo();
+      attribute->value=InterpretImageAttributes(image_info,image,value);
+      image_info=DestroyImageInfo(image_info);
+    }
   attribute->compression=MagickFalse;
   status=AddValueToSplayTree((SplayTreeInfo *) image->attributes,attribute->key,
     attribute);
-  if (LocaleCompare(key,"delay") == 0)
-    image->delay=(unsigned long) atol(value);
+  switch (*key)
+  {
+    case 'D':
+    case 'd':
+    {
+      if (LocaleCompare(key,"delay") == 0)
+        {
+          image->delay=(unsigned long) atol(value);
+          break;
+        }
+      if (LocaleCompare(key,"dispose") == 0)
+        {
+          image->dispose=(DisposeType) ParseMagickOption(MagickDisposeOptions,
+            MagickFalse,value);
+          break;
+        }
+      break;
+    }
+    case 'P':
+    case 'p':
+    {
+      if (LocaleCompare(key,"page") == 0)
+        {
+          char
+            *geometry;
+
+          geometry=GetPageGeometry(key);
+          flags=ParseAbsoluteGeometry(geometry,&image->page);
+          geometry=(char *) RelinquishMagickMemory(geometry);
+          break;
+        }
+      break;
+    }
+    default:
+      break;
+  }
   return(status);
 }
 
@@ -2346,7 +2536,6 @@ MagickExport  MagickBooleanType SetImageAttribute(Image *image,const char *key,
 %
 %    o rows: The image height.
 %
-%
 */
 static char *TracePSClippath(unsigned char *blob,size_t length,
   const unsigned long magick_unused(columns),
@@ -2572,7 +2761,6 @@ static char *TracePSClippath(unsigned char *blob,size_t length,
 %
 %    o rows: The image height.
 %
-%
 */
 static char *TraceSVGClippath(unsigned char *blob,size_t length,
   const unsigned long columns,const unsigned long rows)
