@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,13 +50,13 @@
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt-feature.h"
 #include "magick/option.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "magick/splay-tree.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
 
 /*
   Define declarations.
@@ -67,7 +67,7 @@
   Declare coder map.
 */
 static const char
-  *CoderMap = (char *)
+  *CoderMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<codermap>"
     "  <coder magick=\"8BIM\" name=\"META\" />"
@@ -84,6 +84,8 @@ static const char
     "  <coder magick=\"C\" name=\"GRAY\" />"
     "  <coder magick=\"CUR\" name=\"ICON\" />"
     "  <coder magick=\"DCX\" name=\"PCX\" />"
+    "  <coder magick=\"DFONT\" name=\"TTF\" />"
+    "  <coder magick=\"DNG\" name=\"TIFF\" />"
     "  <coder magick=\"EMF\" name=\"EMF\" />"
     "  <coder magick=\"EPDF\" name=\"PDF\" />"
     "  <coder magick=\"EPI\" name=\"PS\" />"
@@ -127,6 +129,7 @@ static const char
     "  <coder magick=\"OTF\" name=\"TTF\" />"
     "  <coder magick=\"P7\" name=\"PNM\" />"
     "  <coder magick=\"PAL\" name=\"UYVY\" />"
+    "  <coder magick=\"PAM\" name=\"PNM\" />"
     "  <coder magick=\"PBM\" name=\"PNM\" />"
     "  <coder magick=\"PCDS\" name=\"PCD\" />"
     "  <coder magick=\"PCT\" name=\"PICT\" />"
@@ -202,11 +205,10 @@ static MagickBooleanType
 %
 %      DestroyCoderList(void)
 %
-%
 */
 MagickExport void DestroyCoderList(void)
 {
-  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
+  AcquireSemaphoreInfo(&coder_semaphore);
   if (coder_list != (SplayTreeInfo *) NULL)
     coder_list=DestroySplayTree(coder_list);
   instantiate_coder=MagickFalse;
@@ -238,14 +240,10 @@ MagickExport void DestroyCoderList(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const CoderInfo *GetCoderInfo(const char *name,
   ExceptionInfo *exception)
 {
-  register const CoderInfo
-    *p;
-
   assert(exception != (ExceptionInfo *) NULL);
   if ((coder_list == (SplayTreeInfo *) NULL) ||
       (instantiate_coder == MagickFalse))
@@ -259,20 +257,7 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
       ResetSplayTreeIterator(coder_list);
       return((const CoderInfo *) GetNextValueInSplayTree(coder_list));
     }
-  /*
-    Search for requested coder.
-  */
-  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
-  ResetSplayTreeIterator(coder_list);
-  p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
-  while (p != (const CoderInfo *) NULL)
-  {
-    if (LocaleCompare(name,p->magick) == 0)
-      break;
-    p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
-  }
-  RelinquishSemaphoreInfo(coder_semaphore);
-  return(p);
+  return((const CoderInfo *) GetValueFromSplayTree(coder_list,name));
 }
 
 /*
@@ -286,22 +271,20 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetCoderInfoList() returns any coders that match the specified pattern.
-%
+%  GetCoderInfoList() returns any coder_map that match the specified pattern.
 %  The format of the GetCoderInfoList function is:
 %
 %      const CoderInfo **GetCoderInfoList(const char *pattern,
-%        unsigned long *number_coders,ExceptionInfo *exception)
+%        unsigned long *number_coder_map,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
-%    o number_coders:  This integer returns the number of coders in the list.
+%    o number_coder_map:  This integer returns the number of coder_map in the list.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 static int CoderInfoCompare(const void *x,const void *y)
@@ -318,10 +301,10 @@ static int CoderInfoCompare(const void *x,const void *y)
 }
 
 MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
-  unsigned long *number_coders,ExceptionInfo *exception)
+  unsigned long *number_coder_map,ExceptionInfo *exception)
 {
   const CoderInfo
-    **coders;
+    **coder_map;
 
   register const CoderInfo
     *p;
@@ -334,33 +317,33 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
   */
   assert(pattern != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
-  assert(number_coders != (unsigned long *) NULL);
-  *number_coders=0;
+  assert(number_coder_map != (unsigned long *) NULL);
+  *number_coder_map=0;
   p=GetCoderInfo("*",exception);
   if (p == (const CoderInfo *) NULL)
     return((const CoderInfo **) NULL);
-  coders=(const CoderInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfNodesInSplayTree(coder_list)+1)*sizeof(*coders));
-  if (coders == (const CoderInfo **) NULL)
+  coder_map=(const CoderInfo **) AcquireMagickMemory((size_t)
+    (GetNumberOfNodesInSplayTree(coder_list)+1)*sizeof(*coder_map));
+  if (coder_map == (const CoderInfo **) NULL)
     return((const CoderInfo **) NULL);
   /*
     Generate coder list.
   */
-  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
+  AcquireSemaphoreInfo(&coder_semaphore);
   ResetSplayTreeIterator(coder_list);
   p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   for (i=0; p != (const CoderInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      coders[i++]=p;
+      coder_map[i++]=p;
     p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   }
   RelinquishSemaphoreInfo(coder_semaphore);
-  qsort((void *) coders,(size_t) i,sizeof(*coders),CoderInfoCompare);
-  coders[i]=(CoderInfo *) NULL;
-  *number_coders=(unsigned long) i;
-  return(coders);
+  qsort((void *) coder_map,(size_t) i,sizeof(*coder_map),CoderInfoCompare);
+  coder_map[i]=(CoderInfo *) NULL;
+  *number_coder_map=(unsigned long) i;
+  return(coder_map);
 }
 
 /*
@@ -374,18 +357,18 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetCoderList() returns any coders that match the specified pattern.
+%  GetCoderList() returns any coder_map that match the specified pattern.
 %
 %  The format of the GetCoderList function is:
 %
-%      char **GetCoderList(const char *pattern,unsigned long *number_coders,
+%      char **GetCoderList(const char *pattern,unsigned long *number_coder_map,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
-%    o number_coders:  This integer returns the number of coders in the list.
+%    o number_coder_map:  This integer returns the number of coder_map in the list.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -393,20 +376,20 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
 
 static int CoderCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
 MagickExport char **GetCoderList(const char *pattern,
-  unsigned long *number_coders,ExceptionInfo *exception)
+  unsigned long *number_coder_map,ExceptionInfo *exception)
 {
   char
-    **coders;
+    **coder_map;
 
   register const CoderInfo
     *p;
@@ -419,33 +402,33 @@ MagickExport char **GetCoderList(const char *pattern,
   */
   assert(pattern != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
-  assert(number_coders != (unsigned long *) NULL);
-  *number_coders=0;
+  assert(number_coder_map != (unsigned long *) NULL);
+  *number_coder_map=0;
   p=GetCoderInfo("*",exception);
   if (p == (const CoderInfo *) NULL)
     return((char **) NULL);
-  coders=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfNodesInSplayTree(coder_list)+1)*sizeof(*coders));
-  if (coders == (char **) NULL)
+  coder_map=(char **) AcquireMagickMemory((size_t)
+    (GetNumberOfNodesInSplayTree(coder_list)+1)*sizeof(*coder_map));
+  if (coder_map == (char **) NULL)
     return((char **) NULL);
   /*
     Generate coder list.
   */
-  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
+  AcquireSemaphoreInfo(&coder_semaphore);
   ResetSplayTreeIterator(coder_list);
   p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   for (i=0; p != (const CoderInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      coders[i++]=ConstantString(AcquireString(p->name));
+      coder_map[i++]=ConstantString(p->name);
     p=(const CoderInfo *) GetNextValueInSplayTree(coder_list);
   }
   RelinquishSemaphoreInfo(coder_semaphore);
-  qsort((void *) coders,(size_t) i,sizeof(*coders),CoderCompare);
-  coders[i]=(char *) NULL;
-  *number_coders=(unsigned long) i;
-  return(coders);
+  qsort((void *) coder_map,(size_t) i,sizeof(*coder_map),CoderCompare);
+  coder_map[i]=(char *) NULL;
+  *number_coder_map=(unsigned long) i;
+  return(coder_map);
 }
 
 /*
@@ -475,7 +458,7 @@ static MagickBooleanType InitializeCoderList(ExceptionInfo *exception)
   if ((coder_list == (SplayTreeInfo *) NULL) &&
       (instantiate_coder == MagickFalse))
     {
-      coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
+      AcquireSemaphoreInfo(&coder_semaphore);
       if ((coder_list == (SplayTreeInfo *) NULL) &&
           (instantiate_coder == MagickFalse))
         {
@@ -510,7 +493,6 @@ static MagickBooleanType InitializeCoderList(ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListCoderInfo(FILE *file,
   ExceptionInfo *exception)
@@ -528,15 +510,15 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
     i;
 
   unsigned long
-    number_coders;
+    number_coder_map;
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  coder_info=GetCoderInfoList("*",&number_coders,exception);
+  coder_info=GetCoderInfoList("*",&number_coder_map,exception);
   if (coder_info == (const CoderInfo **) NULL)
     return(MagickFalse);
   path=(const char *) NULL;
-  for (i=0; i < (long) number_coders; i++)
+  for (i=0; i < (long) number_coder_map; i++)
   {
     if (coder_info[i]->stealth != MagickFalse)
       continue;
@@ -591,7 +573,6 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 static void *DestroyCoderNode(void *coder_info)
@@ -612,22 +593,25 @@ static void *DestroyCoderNode(void *coder_info)
 static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
-  char
-    keyword[MaxTextExtent],
-    *q,
-    *token;
-
   CoderInfo
     *coder_info = (CoderInfo *) NULL;
 
+  const char
+    *attribute;
+
   MagickBooleanType
     status;
 
+  XMLTreeInfo
+    *coder,
+    *coder_map,
+    *include;
+
   /*
     Load the coder map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading coder file \"%s\" ...",filename);
+    "Loading coder map \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (coder_list == (SplayTreeInfo *) NULL)
@@ -641,134 +625,70 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
+  coder_map=NewXMLTree(xml,exception);
+  if (coder_map == (XMLTreeInfo *) NULL)
+    return(MagickFalse);
   status=MagickTrue;
-  token=AcquireString((char *) xml);
-  for (q=(char *) xml; *q != '\0'; )
+  include=GetXMLTreeChild(coder_map,"include");
+  while (include != (XMLTreeInfo *) NULL)
   {
     /*
-      Interpret XML.
+      Process include element.
     */
-    GetMagickToken(q,&q,token);
-    if (*token == '\0')
-      break;
-    (void) CopyMagickString(keyword,token,MaxTextExtent);
-    if (LocaleNCompare(keyword,"<!--",4) == 0)
+    attribute=GetXMLTreeAttribute(include,"file");
+    if (attribute != (const char *) NULL)
       {
-        /*
-          Comment element.
-        */
-        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
-        continue;
-      }
-    if (LocaleCompare(keyword,"<include") == 0)
-      {
-        /*
-          Include element.
-        */
-        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
-        {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
-          if (*token != '=')
-            continue;
-          GetMagickToken(q,&q,token);
-          if (LocaleCompare(keyword,"file") == 0)
-            {
-              if (depth > 200)
-                (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeNodeNestedTooDeeply","`%s'",token);
-              else
-                {
-                  char
-                    path[MaxTextExtent],
-                    *xml;
-
-                  GetPathComponent(filename,HeadPath,path);
-                  if (*path != '\0')
-                    (void) ConcatenateMagickString(path,DirectorySeparator,
-                      MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
-                  if (LoadCoderList(xml,path,depth+1,exception) == MagickFalse)
-                    status=MagickFalse;
-                  xml=(char *) RelinquishMagickMemory(xml);
-                }
-            }
-        }
-        continue;
-      }
-    if (LocaleCompare(keyword,"<coder") == 0)
-      {
-        /*
-          Allocate memory for the coder list.
-        */
-        coder_info=(CoderInfo *) AcquireMagickMemory(sizeof(*coder_info));
-        if (coder_info == (CoderInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
-        (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
-        coder_info->path=ConstantString(AcquireString(filename));
-        coder_info->signature=MagickSignature;
-        continue;
-      }
-    if (coder_info == (CoderInfo *) NULL)
-      continue;
-    if (LocaleCompare(keyword,"/>") == 0)
-      {
-        status=AddValueToSplayTree(coder_list,
-          ConstantString(AcquireString(coder_info->magick)),coder_info);
-        if (status == MagickFalse)
+        if (depth > 200)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed","`%s'",
-            coder_info->magick);
-        coder_info=(CoderInfo *) NULL;
-      }
-    GetMagickToken(q,(char **) NULL,token);
-    if (*token != '=')
-      continue;
-    GetMagickToken(q,&q,token);
-    GetMagickToken(q,&q,token);
-    switch (*keyword)
-    {
-      case 'M':
-      case 'm':
-      {
-        if (LocaleCompare((char *) keyword,"magick") == 0)
+            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",attribute);
+        else
           {
-            coder_info->magick=ConstantString(AcquireString(token));
-            break;
+            char
+              path[MaxTextExtent],
+              *xml;
+
+            GetPathComponent(filename,HeadPath,path);
+            if (*path != '\0')
+              (void) ConcatenateMagickString(path,DirectorySeparator,
+                MaxTextExtent);
+            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
+            xml=FileToString(path,~0,exception);
+            status=LoadCoderList(xml,path,depth+1,exception);
+            xml=(char *) RelinquishMagickMemory(xml);
           }
-        break;
       }
-      case 'N':
-      case 'n':
-      {
-        if (LocaleCompare((char *) keyword,"name") == 0)
-          {
-            coder_info->name=ConstantString(AcquireString(token));
-            break;
-          }
-        break;
-      }
-      case 'S':
-      case 's':
-      {
-        if (LocaleCompare((char *) keyword,"stealth") == 0)
-          {
-            coder_info->stealth=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
-            break;
-          }
-        break;
-      }
-      default:
-        break;
-    }
+    include=GetNextXMLTreeTag(include);
   }
-  token=(char *) RelinquishMagickMemory(token);
-  if (coder_list == (SplayTreeInfo *) NULL)
-    return(MagickFalse);
+  coder=GetXMLTreeChild(coder_map,"coder");
+  while (coder != (XMLTreeInfo *) NULL)
+  {
+    /*
+      Process coder element.
+    */
+    coder_info=(CoderInfo *) AcquireMagickMemory(sizeof(*coder_info));
+    if (coder_info == (CoderInfo *) NULL)
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",filename);
+    (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
+    coder_info->path=ConstantString(filename);
+    coder_info->signature=MagickSignature;
+    attribute=GetXMLTreeAttribute(coder,"magick");
+    if (attribute != (const char *) NULL)
+      coder_info->magick=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(coder,"name");
+    if (attribute != (const char *) NULL)
+      coder_info->name=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(coder,"stealth");
+    if (attribute != (const char *) NULL)
+      coder_info->stealth=IsMagickTrue(attribute);
+    status=AddValueToSplayTree(coder_list,ConstantString(coder_info->magick),
+      coder_info);
+    if (status == MagickFalse)
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
+    coder=GetNextXMLTreeTag(coder);
+  }
+  coder_map=DestroyXMLTree(coder_map);
   return(status);
 }
 
@@ -797,14 +717,13 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadCoderLists(const char *filename,
   ExceptionInfo *exception)
 {
-  char
-    path[MaxTextExtent];
-
+#if defined(UseEmbeddableMagick)
+  return(LoadCoderList(CoderMap,"built-in",0,exception));
+#else
   const StringInfo
     *option;
 
@@ -815,12 +734,10 @@ static MagickBooleanType LoadCoderLists(const char *filename,
     status;
 
   status=MagickFalse;
-  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadCoderList((const char *) option->datum,option->path,0,
       exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
@@ -828,10 +745,7 @@ static MagickBooleanType LoadCoderLists(const char *filename,
   options=DestroyConfigureOptions(options);
   if ((coder_list == (SplayTreeInfo *) NULL) || 
       (GetNumberOfNodesInSplayTree(coder_list) == 0))
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-        "UnableToOpenConfigureFile","`%s'",path);
-      status|=LoadCoderList(CoderMap,"built-in",0,exception);
-    }
+    status|=LoadCoderList(CoderMap,"built-in",0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
