@@ -138,6 +138,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     }
   }
   image->colorspace=colorspace;
+  (void) SetImagePixels(image,0,0,image->columns,1);
   switch (colorspace)
   {
     case CMYKColorspace:
@@ -158,8 +159,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           if (SetImageStorageClass(image,DirectClass) == MagickFalse)
             return(MagickFalse);
         }
-      if (SyncImageCache(image) == MagickFalse)
-        return(MagickFalse);
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
@@ -189,30 +188,28 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     }
     case GRAYColorspace:
     {
-      IndexPacket
-        index;
-
       /*
         Convert RGB to GRAY colorspace.
       */
       if (IsGrayImage(image,&image->exception) != MagickFalse)
         return(MagickTrue);
-      if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-          image->filename);
+      if (image->storage_class == PseudoClass)
+        for (i=0; i < (long) image->colors; i++)
+        {
+          image->colormap[i].red=PixelIntensityToQuantum(image->colormap+i);
+          image->colormap[i].green=image->colormap[i].red;
+          image->colormap[i].blue=image->colormap[i].red;
+        }
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          index=PixelIntensityToQuantum(q);
-          q->red=index;
-          q->green=index;
-          q->blue=index;
-          indexes[x]=index;
+          q->red=PixelIntensityToQuantum(q);
+          q->green=q->red;
+          q->blue=q->red;
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -256,7 +253,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
-      image->colorspace=RGBColorspace;
       return(MagickTrue);
     }
     case HSLColorspace:
@@ -295,7 +291,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
-      image->colorspace=RGBColorspace;
       return(MagickTrue);
     }
     case HWBColorspace:
@@ -334,7 +329,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
-      image->colorspace=RGBColorspace;
       return(MagickTrue);
     }
     case LogColorspace:
@@ -850,8 +844,11 @@ MagickExport MagickBooleanType SetImageColorspace(Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(image->colorspace != UndefinedColorspace);
-  assert(colorspace != UndefinedColorspace);
+  if (colorspace == UndefinedColorspace)
+    {
+      image->colorspace=UndefinedColorspace;
+      return(MagickTrue);
+    }
   if (image->colorspace == colorspace)
     return(MagickTrue);
   if ((colorspace == RGBColorspace) || (colorspace == TransparentColorspace))
@@ -994,7 +991,6 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(image->colorspace != UndefinedColorspace);
   switch (colorspace)
   {
     case GRAYColorspace:
@@ -1002,6 +998,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
     case Rec709LumaColorspace:
     case RGBColorspace:
     case TransparentColorspace:
+    case UndefinedColorspace:
       return(MagickTrue);
     default:
       break;
@@ -1046,15 +1043,14 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
       }
       image->colorspace=RGBColorspace;
-      if (SyncImageCache(image) == MagickFalse)
-        return(MagickFalse);
+      (void) SetImagePixels(image,0,0,image->columns,1);
       return(y == (long) image->rows ? MagickTrue : MagickFalse);
     }
     case HSBColorspace:
     {
       double
+        brightness,
         hue,
-        luminosity,
         saturation;
 
       /*
@@ -1076,8 +1072,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         {
           hue=(double) (QuantumScale*q->red);
           saturation=(double) (QuantumScale*q->green);
-          luminosity=(double) (QuantumScale*q->blue);
-          HSBTransform(hue,saturation,luminosity,&q->red,&q->green,&q->blue);
+          brightness=(double) (QuantumScale*q->blue);
+          HSBTransform(hue,saturation,brightness,&q->red,&q->green,&q->blue);
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -1183,6 +1179,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
+      image->colorspace=RGBColorspace;
       break;
     }
     case LogColorspace:
