@@ -42,12 +42,14 @@
 */
 #include "magick/studio.h"
 #include "magick/attribute.h"
+#include "magick/cache-view.h"
 #include "magick/client.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/composite.h"
 #include "magick/composite-private.h"
 #include "magick/constitute.h"
+#include "magick/draw.h"
 #include "magick/fx.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
@@ -78,11 +80,14 @@
 %  CompositeImageChannel() returns the second image composited onto the first
 %  at the specified offsets.
 %
-%  The format of the CompositeImage method is:
+%  The format of the CompositeImageChannel method is:
 %
+%      MagickBooleanType CompositeImage(Image *image,
+%        const CompositeOperator compose,Image *composite_image,
+%        const long x_offset,const long y_offset)
 %      MagickBooleanType CompositeImageChannel(Image *image,
 %        const ChannelType channel,const CompositeOperator compose,
-%        const Image *composite_image,const long x_offset,const long y_offset)
+%        Image *composite_image,const long x_offset,const long y_offset)
 %
 %  A description of each parameter follows:
 %
@@ -100,7 +105,6 @@
 %
 %    o y_offset: The row offset of the composited image.
 %
-%
 */
 
 static inline MagickRealType Add(const MagickRealType p,const MagickRealType q)
@@ -928,7 +932,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
 }
 
 MagickExport MagickBooleanType CompositeImageChannel(Image *image,
-  const ChannelType channel,const CompositeOperator compose,
+  const ChannelType magick_unused(channel),const CompositeOperator compose,
   const Image *composite_image,const long x_offset,const long y_offset)
 {
   const ImageAttribute
@@ -946,6 +950,9 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   GeometryInfo
     geometry_info;
 
+  Image
+    *displace_image;
+
   IndexPacket
     *composite_indexes,
     *indexes;
@@ -982,6 +989,10 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   register PixelPacket
     *q;
 
+  ViewInfo
+    *composite_view,
+    *image_view;
+
   /*
     Prepare composite image.
   */
@@ -995,6 +1006,7 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
+  displace_image=(Image *) NULL;
   amount=0.5;
   destination_dissolve=1.0;
   modify_outside_overlay=MagickFalse;
@@ -1028,9 +1040,6 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     }
     case DisplaceCompositeOp:
     {
-      Image
-        *displace_image;
-
       MagickPixelPacket
         pixel;
 
@@ -1046,6 +1055,11 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
       register PixelPacket
         *r;
 
+      ViewInfo
+        *composite_view,
+        *displace_view,
+        *image_view;
+
       /*
         Allocate the displace image.
       */
@@ -1071,18 +1085,21 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         Shift image pixels as defined by a displacement map.
       */
       GetMagickPixelPacket(displace_image,&pixel);
+      image_view=OpenCacheView(image);
+      displace_view=OpenCacheView(displace_image);
+      composite_view=OpenCacheView(composite_image);
       for (y=0; y < (long) composite_image->rows; y++)
       {
         if (((y+y_offset) < 0) || ((y+y_offset) >= (long) image->rows))
           continue;
-        p=AcquireImagePixels(composite_image,0,y,composite_image->columns,1,
+        p=AcquireCacheView(composite_view,0,y,composite_image->columns,1,
           &image->exception);
-        q=GetImagePixels(image,0,y+y_offset,image->columns,1);
-        r=GetImagePixels(displace_image,0,y,displace_image->columns,1);
+        q=GetCacheView(image_view,0,y+y_offset,image->columns,1);
+        r=GetCacheView(displace_view,0,y,displace_image->columns,1);
         if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL) ||
             (r == (PixelPacket *) NULL))
           break;
-        displace_indexes=GetIndexes(displace_image);
+        displace_indexes=GetCacheViewIndexes(displace_view);
         q+=x_offset;
         for (x=0; x < (long) composite_image->columns; x++)
         {
@@ -1094,22 +1111,28 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
             }
           x_displace=(horizontal_scale*(PixelIntensityToQuantum(p)-
             (((MagickRealType) QuantumRange+1.0)/2)))/
-            (((MagickRealType) QuantumRange+1.0)/
-            2.0);
-          y_displace=x_displace;
+            (((MagickRealType) QuantumRange+1.0)/2.0);
           if (composite_image->matte != MagickFalse)
-            y_displace=(vertical_scale*(p->opacity-(((MagickRealType)
-              QuantumRange+1.0)/2)))/(((MagickRealType) QuantumRange+1.0)/2);
-          pixel=InterpolateColor(image,(double) (x_offset+x+x_displace),
-            (double) (y_offset+y+y_displace),&image->exception);
+            y_displace=(-vertical_scale*(p->opacity-(((MagickRealType)
+              QuantumRange+1.0)/2.0)))/(((MagickRealType) QuantumRange+1.0)/2);
+          else
+            y_displace=(vertical_scale*(PixelIntensityToQuantum(p)-
+              (((MagickRealType) QuantumRange+1.0)/2)))/
+              (((MagickRealType) QuantumRange+1.0)/2.0);
+          pixel=InterpolatePixelColor(image,image_view,image->interpolate,
+            (double) (x_offset+x+x_displace),(double) (y_offset+y+y_displace),
+            &image->exception);
           SetPixelPacket(&pixel,r,displace_indexes+x);
           p++;
           q++;
           r++;
         }
-        if (SyncImagePixels(displace_image) == MagickFalse)
+        if (SyncCacheView(displace_view) == MagickFalse)
           break;
       }
+      composite_view=CloseCacheView(composite_view);
+      displace_view=CloseCacheView(displace_view);
+      image_view=CloseCacheView(image_view);
       composite_image=displace_image;
       break;
     }
@@ -1212,6 +1235,8 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   midpoint=((MagickRealType) QuantumRange+1.0)/2;
   GetMagickPixelPacket(composite_image,&source);
   GetMagickPixelPacket(image,&destination);
+  image_view=OpenCacheView(image);
+  composite_view=OpenCacheView(composite_image);
   for (y=0; y < (long) image->rows; y++)
   {
     if (modify_outside_overlay == MagickFalse)
@@ -1228,19 +1253,19 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     p=(PixelPacket *) NULL;
     if ((y >= y_offset) && ((y-y_offset) < (long) composite_image->rows))
       {
-        p=AcquireImagePixels(composite_image,0,y-y_offset,
-          composite_image->columns,1,&image->exception);
+        p=AcquireCacheView(composite_view,0,y-y_offset,composite_image->columns,
+          1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
         pixels=p;
         if (x_offset < 0)
           p-=x_offset;
       }
-    q=GetImagePixels(image,0,y,image->columns,1);
+    q=GetCacheView(image_view,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
-    composite_indexes=GetIndexes(composite_image);
+    indexes=GetCacheViewIndexes(image_view);
+    composite_indexes=GetCacheViewIndexes(composite_view);
     for (x=0; x < (long) image->columns; x++)
     {
       if (modify_outside_overlay == MagickFalse)
@@ -1680,8 +1705,12 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         p=pixels;
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncCacheView(image_view) == MagickFalse)
       break;
   }
+  composite_view=CloseCacheView(composite_view);
+  image_view=CloseCacheView(image_view);
+  if (displace_image != (Image * ) NULL)
+    displace_image=DestroyImage(displace_image);
   return(MagickTrue);
 }
