@@ -64,6 +64,7 @@
 #include "magick/stream.h"
 #include "magick/string_.h"
 #include "magick/timer.h"
+#include "magick/transform.h"
 #include "magick/utility.h"
 
 static SemaphoreInfo
@@ -212,7 +213,6 @@ MagickExport void DestroyConstitute(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -295,7 +295,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     magick_filename[MaxTextExtent];
 
   const char
-    *option;
+    *value;
 
   const DelegateInfo
     *delegate_info;
@@ -352,8 +352,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         status;
 
       image=AllocateImage(read_info);
-      if (image == (Image *) NULL)
-        return((Image *) NULL);
       (void) CopyMagickString(image->filename,read_info->filename,
         MaxTextExtent);
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
@@ -418,8 +416,12 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       (void) CopyMagickString(image->filename,read_info->filename,
         MaxTextExtent);
       *read_info->filename='\0';
+      if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
+        AcquireSemaphoreInfo(&constitute_semaphore);
       (void) InvokeDelegate(read_info,image,read_info->magick,(char *) NULL,
         exception);
+      if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
+        RelinquishSemaphoreInfo(constitute_semaphore);
       image=DestroyImageList(image);
       read_info->temporary=MagickTrue;
       (void) SetImageInfo(read_info,MagickFalse,exception);
@@ -507,6 +509,17 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       MaxTextExtent);
     if (IsBlobTemporary(image) != MagickFalse)
       (void) CopyMagickString(next->filename,filename,MaxTextExtent);
+    if (next->magick_columns == 0)
+      next->magick_columns=next->columns;
+    if (next->magick_rows == 0)
+      next->magick_rows=next->rows;
+    attribute=GetImageAttribute(next,"EXIF:Orientation");
+    if ((attribute != (const ImageAttribute *) NULL) &&
+        (attribute->value != (char *) NULL))
+      {
+        next->orientation=(OrientationType) atol(attribute->value);
+        (void) DeleteImageAttribute(next,"EXIF:Orientation");
+      }
     attribute=GetImageAttribute(next,"EXIF:XResolution");
     if ((attribute != (const ImageAttribute *) NULL) &&
         (attribute->value != (char *) NULL))
@@ -542,16 +555,16 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         (void) DeleteImageAttribute(next,"EXIF:ResolutionUnit");
       }
     (void) SyncImageOptions(image_info,next);
-    option=GetImageOption(read_info,"comment");
-    if (option != (const char *) NULL)
+    value=GetImageOption(read_info,"comment");
+    if (value != (const char *) NULL)
       {
         (void) DeleteImageAttribute(next,"comment");
-        (void) SetImageAttribute(next,"comment",option);
+        (void) SetImageAttribute(next,"comment",value);
       }
-    option=GetImageOption(read_info,"delay");
-    if (option != (const char *) NULL)
+    value=GetImageOption(read_info,"delay");
+    if (value != (const char *) NULL)
       {
-        flags=ParseGeometry(option,&geometry_info);
+        flags=ParseGeometry(value,&geometry_info);
         if ((flags & GreaterValue) != 0)
           {
             if (next->delay > (unsigned long) (geometry_info.rho+0.5))
@@ -569,40 +582,70 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         if ((flags & SigmaValue) != 0)
           next->ticks_per_second=(unsigned long) (geometry_info.sigma+0.5);
       }
-    option=GetImageOption(read_info,"dispose");
-    if (option != (const char *) NULL)
+    value=GetImageOption(read_info,"dispose");
+    if (value != (const char *) NULL)
       next->dispose=(DisposeType) ParseMagickOption(MagickDisposeOptions,
-        MagickFalse,option);
-    option=GetImageOption(read_info,"label");
-    if (option != (const char *) NULL)
+        MagickFalse,value);
+    value=GetImageOption(read_info,"label");
+    if (value != (const char *) NULL)
       {
         (void) DeleteImageAttribute(next,"label");
-        (void) SetImageAttribute(next,"label",option);
+        (void) SetImageAttribute(next,"label",value);
       }
     if (next->page.width == 0)
       next->page.width=next->columns;
     if (next->page.height == 0)
       next->page.height=next->rows;
-    option=GetImageOption(read_info,"page");
-    if ((option != (const char *) NULL) &&
+    value=GetImageOption(read_info,"page");
+    if ((value != (const char *) NULL) &&
         (LocaleCompare(next->magick,"TEXT") != 0))
       {
         char
           *geometry;
 
-        geometry=GetPageGeometry(option);
+        geometry=GetPageGeometry(value);
         flags=ParseAbsoluteGeometry(geometry,&next->page);
         geometry=(char *) RelinquishMagickMemory(geometry);
       }
-    if (next->magick_columns == 0)
-      next->magick_columns=next->columns;
-    if (next->magick_rows == 0)
-      next->magick_rows=next->rows;
+    if ((read_info->extract != (char *) NULL) &&
+        (read_info->stream == (StreamHandler) NULL))
+      {
+        RectangleInfo
+          geometry;
+
+        flags=ParseAbsoluteGeometry(read_info->extract,&geometry);
+        if ((next->columns != geometry.width) ||
+            (next->rows != geometry.height))
+          {
+            if (((flags & XValue) != 0) || ((flags & YValue) != 0))
+              {
+                Image
+                  *crop_image;
+
+                crop_image=CropImage(next,&geometry,exception);
+                if (crop_image != (Image *) NULL)
+                  ReplaceImageInList(&next,crop_image);
+              }
+            else
+              if (((flags & WidthValue) != 0) || ((flags & HeightValue) != 0))
+                {
+                  Image
+                    *size_image;
+
+                  flags=ParseSizeGeometry(next,read_info->extract,&geometry);
+                  size_image=ResizeImage(next,geometry.width,geometry.height,
+                    next->filter,next->blur,exception);
+                  if (size_image != (Image *) NULL)
+                    ReplaceImageInList(&next,size_image);
+                }
+          }
+      }
     if (read_info->verbose != MagickFalse)
       (void) IdentifyImage(next,stdout,MagickFalse);
+    image=next;
   }
   read_info=DestroyImageInfo(read_info);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -940,8 +983,12 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
             Process the image with delegate.
           */
           *write_info->filename='\0';
+          if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
+            AcquireSemaphoreInfo(&constitute_semaphore);
           status=InvokeDelegate(write_info,image,(char *) NULL,
             write_info->magick,&image->exception);
+          if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
+            RelinquishSemaphoreInfo(constitute_semaphore);
           (void) CopyMagickString(image->filename,filename,MaxTextExtent);
         }
       else
