@@ -62,6 +62,7 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
 
 /*
   Define declarations.
@@ -78,13 +79,13 @@ static const char
     "  <delegate decode=\"autotrace\" stealth=\"True\" command='\"autotrace\" -output-format svg -output-file \"%o\" \"%i\"' />"
     "  <delegate decode=\"browse\" stealth=\"True\" spawn=\"True\" command='\"htmlview\" http://www.imagemagick.org/'  />"
     "  <delegate decode=\"cgm\" command='\"ralcgm\" -d ps -oC < \"%i\" > \"%o\" 2>/dev/null' />"
-    "  <delegate decode=\"crw\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
-    "  <delegate decode=\"dcr\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
-    "  <delegate decode=\"mrw\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
-    "  <delegate decode=\"nef\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
-    "  <delegate decode=\"orf\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
-    "  <delegate decode=\"raf\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
-    "  <delegate decode=\"x3f\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"crw\" thread-support=\"False\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"dcr\" thread-support=\"False\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"mrw\" thread-support=\"False\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"nef\" thread-support=\"False\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"orf\" thread-support=\"False\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"raf\" thread-support=\"False\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"x3f\" thread-support=\"False\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
     "  <delegate decode=\"dvi\" command='\"dvips\" -q -o \"%o\" \"%i\"' />"
     "  <delegate decode=\"edit\" stealth=\"True\" command='\"xterm\" -title \"Edit Image Comment\" -e vi \"%o\"' />"
     "  <delegate decode=\"emf\" command='\"wmf2eps\" -o \"%o\" \"%i\"' />"
@@ -635,6 +636,39 @@ MagickExport long GetDelegateMode(const DelegateInfo *delegate_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   G e t D e l e g a t e T h r e a d S u p p o r t                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetDelegateThreadSupport() returns MagickTrue if the delegate supports
+%  threads.
+%
+%  The format of the GetDelegateThreadSupport method is:
+%
+%      MagickBooleanType GetDelegateThreadSupport(
+%        const DelegateInfo *delegate_info)
+%
+%  A description of each parameter follows:
+%
+%    o delegate_info:  The delegate info.
+%
+*/
+MagickExport MagickBooleanType GetDelegateThreadSupport(
+  const DelegateInfo *delegate_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(delegate_info != (DelegateInfo *) NULL);
+  assert(delegate_info->signature == MagickSignature);
+  return(delegate_info->thread_support);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   I n i t i a l i z e D e l e g a t e L i s t                               %
 %                                                                             %
 %                                                                             %
@@ -699,6 +733,69 @@ static MagickBooleanType InitializeDelegateList(ExceptionInfo *exception)
 %
 %
 */
+
+static void CopyDelegateFile(const char *source,const char *destination)
+{
+  int
+    destination_file,
+    source_file,
+    status;
+
+  size_t
+    length;
+
+  ssize_t
+    count;
+
+  struct stat
+    file_info;
+
+  unsigned char
+    *buffer;
+
+  /*
+    Return if destination file already exists and is not empty.
+  */
+  assert(source != (const char *) NULL);
+  assert(destination != (char *) NULL);
+  status=stat(destination,&file_info);
+  if ((status == 0) && (file_info.st_size != 0))
+    return;
+  /*
+    Copy source file to destination.
+  */
+  destination_file=open(destination,O_WRONLY | O_BINARY | O_CREAT);
+  if (destination_file == -1)
+    return;
+  source_file=open(source,O_RDONLY | O_BINARY);
+  if (source_file == -1)
+    {
+      (void) close(destination_file);
+      return;
+    }
+  buffer=(unsigned char *) AcquireMagickMemory(MagickMaxBufferSize*
+    sizeof(*buffer));
+  if (buffer == (unsigned char *) NULL)
+    {
+      (void) close(source_file);
+      (void) close(destination_file);
+      return;
+    }
+  for (length=0; ; )
+  {
+    count=read(source_file,buffer,MagickMaxBufferSize);
+    if (count <= 0)
+      break;
+    length=(size_t) count;
+    count=write(destination_file,buffer,length);
+    if ((size_t) count != length)
+      break;
+  }
+  (void) close(destination_file);
+  (void) close(source_file);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+}
+
 MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   Image *image,const char *decode,const char *encode,ExceptionInfo *exception)
 {
@@ -855,12 +952,6 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   for (i=0; commands[i] != (char *) NULL; i++)
   {
     status=AcquireUniqueSymbolicLink(output_filename,image_info->filename);
-    if (status == MagickFalse)
-      {
-        ThrowFileException(exception,FileOpenError,
-          "UnableToCreateTemporaryFile",output_filename);
-        break;
-      }
     if (AcquireUniqueFilename(image_info->unique) == MagickFalse)
       {
         ThrowFileException(exception,FileOpenError,
@@ -895,6 +986,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
     (void) RelinquishUniqueFileResource(image->filename);
     (void) RelinquishUniqueFileResource(image_info->unique);
     (void) RelinquishUniqueFileResource(image_info->zero);
+    CopyDelegateFile(image_info->filename,output_filename);
     (void) RelinquishUniqueFileResource(image_info->filename);
     if (status != MagickFalse)
       {
@@ -922,104 +1014,6 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n v o k e P o s t s r i p t D e l e g a t e                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  InvokePostscriptDelegate() executes the postscript interpreter with the
-%  specified command.
-%
-%  The format of the InvokePostscriptDelegate method is:
-%
-%      MagickBooleanType InvokePostscriptDelegate(
-%        const MagickBooleanType verbose,const char *command)
-%
-%  A description of each parameter follows:
-%
-%    o verbose: A value other than zero displays the command prior to
-%      executing it.
-%
-%    o command: The address of a character string containing the command to
-%      execute.
-%
-%
-*/
-MagickExport MagickBooleanType InvokePostscriptDelegate(
-  const MagickBooleanType verbose,const char *command)
-{
-#if defined(HasGS) || defined(__WINDOWS__)
-  char
-    **argv;
-
-  const GhostscriptVectors
-    *gs_func;
-
-  gs_main_instance
-    *interpreter;
-
-  int
-    argc,
-    code,
-    status;
-
-  register long
-    i;
-
-#if defined(__WINDOWS__)
-  gs_func=NTGhostscriptDLLVectors();
-#else
-  GhostscriptVectors
-    gs_func_struct;
-
-  gs_func=(&gs_func_struct);
-  gs_func_struct.exit=gsapi_exit;
-  gs_func_struct.init_with_args=gsapi_init_with_args;
-  gs_func_struct.new_instance=gsapi_new_instance;
-  gs_func_struct.run_string=gsapi_run_string;
-  gs_func_struct.delete_instance=gsapi_delete_instance;
-#endif
-  if (gs_func == (GhostscriptVectors *) NULL)
-    return(SystemCommand(verbose,command));
-  if (verbose != MagickFalse)
-    {
-      (void) fputs("[ghostscript library]",stdout);
-      (void) fputs(strchr(command,' '),stdout);
-    }
-  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
-  if (status < 0)
-    return(MagickFalse);
-  argv=StringToArgv(command,&argc);
-  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
-  if (status == 0)
-    status=(gs_func->run_string)
-      (interpreter,"systemdict /start get exec\n",0,&code);
-  (gs_func->exit)(interpreter);
-  (gs_func->delete_instance)(interpreter);
-#if defined(__WINDOWS__)
-  NTGhostscriptUnLoadDLL();
-#endif
-  for (i=0; i < (long) argc; i++)
-    argv[i]=(char *) RelinquishMagickMemory(argv[i]);
-  argv=(char **) RelinquishMagickMemory(argv);
-  if ((status == 0) || (status <= -100))
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-        "Ghostscript returns status %d, exit code %d",status,code);
-      return(MagickFalse);
-    }
-  return(MagickTrue);
-#else
-  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
-#endif
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %  L i s t D e l e g a t e I n f o                                            %
 %                                                                             %
 %                                                                             %
@@ -1143,22 +1137,25 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
 static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
-  char
-    keyword[MaxTextExtent],
-    *q,
-    *token;
+  const char
+    *attribute;
 
   DelegateInfo
     *delegate_info = (DelegateInfo *) NULL;
 
-  MagickStatusType
+  MagickBooleanType
     status;
 
+  XMLTreeInfo
+    *delegate,
+    *delegate_map,
+    *include;
+
   /*
     Load the delegate map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading delegate file \"%s\" ...",filename);
+    "Loading delegate map \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (delegate_list == (LinkedListInfo *) NULL)
@@ -1171,190 +1168,114 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
+  delegate_map=NewXMLTree(xml,exception);
+  if (delegate_map == (XMLTreeInfo *) NULL)
+    return(MagickFalse);
   status=MagickTrue;
-  token=AcquireString(xml);
-  for (q=(char *) xml; *q != '\0'; )
+  include=GetXMLTreeChild(delegate_map,"include");
+  while (include != (XMLTreeInfo *) NULL)
   {
     /*
-      Interpret XML.
+      Process include element.
     */
-    GetMagickToken(q,&q,token);
-    if (*token == '\0')
-      break;
-    (void) CopyMagickString(keyword,token,MaxTextExtent);
-    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+    attribute=GetXMLTreeAttribute(include,"file");
+    if (attribute != (const char *) NULL)
       {
-        /*
-          Doctype element.
-        */
-        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
-        continue;
-      }
-    if (LocaleNCompare(keyword,"<!--",4) == 0)
-      {
-        /*
-          Comment element.
-        */
-        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
-        continue;
-      }
-    if (LocaleCompare(keyword,"<include") == 0)
-      {
-        /*
-          Include element.
-        */
-        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
-        {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
-          if (*token != '=')
-            continue;
-          GetMagickToken(q,&q,token);
-          if (LocaleCompare(keyword,"file") == 0)
-            {
-              if (depth > 200)
-                (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
-              else
-                {
-                  char
-                    path[MaxTextExtent],
-                    *xml;
-
-                  GetPathComponent(filename,HeadPath,path);
-                  if (*path != '\0')
-                    (void) ConcatenateMagickString(path,DirectorySeparator,
-                      MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path,~0,exception);
-                  status|=LoadDelegateList(xml,path,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
-                }
-            }
-        }
-        continue;
-      }
-    if (LocaleCompare(keyword,"<delegate") == 0)
-      {
-        /*
-          Allocate memory for the delegate list.
-        */
-        delegate_info=(DelegateInfo *)
-          AcquireMagickMemory(sizeof(*delegate_info));
-        if (delegate_info == (DelegateInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
-        (void) ResetMagickMemory(delegate_info,0,sizeof(*delegate_info));
-        delegate_info->path=ConstantString(filename);
-        delegate_info->signature=MagickSignature;
-        continue;
-      }
-    if (delegate_info == (DelegateInfo *) NULL)
-      continue;
-    if (LocaleCompare(keyword,"/>") == 0)
-      {
-        status=AppendValueToLinkedList(delegate_list,delegate_info);
-        if (status == MagickFalse)
+        if (depth > 200)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed","`%s'",
-            delegate_info->commands);
-        delegate_info=(DelegateInfo *) NULL;
-      }
-    GetMagickToken(q,(char **) NULL,token);
-    if (*token != '=')
-      continue;
-    GetMagickToken(q,&q,token);
-    GetMagickToken(q,&q,token);
-    switch (*keyword)
-    {
-      case 'C':
-      case 'c':
-      {
-        if (LocaleCompare((char *) keyword,"command") == 0)
+            DelegateError,"IncludeElementNestedTooDeeply","`%s'",filename);
+        else
           {
             char
-              *commands;
+              path[MaxTextExtent],
+              *xml;
 
-            commands=AcquireString(token);
-#if defined(__WINDOWS__)
-            if (strchr(commands,'@') != (char *) NULL)
-              {
-                char
-                  path[MaxTextExtent];
-
-                NTGhostscriptEXE(path,MaxTextExtent);
-                SubstituteString((char **) &commands,"@PSDelegate@",path);
-              }
-#endif
-            delegate_info->commands=commands;
-            break;
+            GetPathComponent(filename,HeadPath,path);
+            if (*path != '\0')
+              (void) ConcatenateMagickString(path,DirectorySeparator,
+                MaxTextExtent);
+            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
+            xml=FileToString(path,~0,exception);
+            status=LoadDelegateList(xml,path,depth+1,exception);
+            xml=(char *) RelinquishMagickMemory(xml);
           }
-        break;
       }
-      case 'D':
-      case 'd':
-      {
-        if (LocaleCompare((char *) keyword,"decode") == 0)
-          {
-            delegate_info->decode=ConstantString(token);
-            delegate_info->mode=1;
-            break;
-          }
-        break;
-      }
-      case 'E':
-      case 'e':
-      {
-        if (LocaleCompare((char *) keyword,"encode") == 0)
-          {
-            delegate_info->encode=ConstantString(token);
-            delegate_info->mode=(-1);
-            break;
-          }
-        break;
-      }
-      case 'M':
-      case 'm':
-      {
-        if (LocaleCompare((char *) keyword,"mode") == 0)
-          {
-            delegate_info->mode=1;
-            if (LocaleCompare(token,"bi") == 0)
-              delegate_info->mode=0;
-            else
-              if (LocaleCompare(token,"encode") == 0)
-                delegate_info->mode=(-1);
-            break;
-          }
-        break;
-      }
-      case 'S':
-      case 's':
-      {
-        if (LocaleCompare((char *) keyword,"spawn") == 0)
-          {
-            delegate_info->spawn=LocaleCompare(token,"True") == 0 ? MagickTrue :
-              MagickFalse;
-            break;
-          }
-        if (LocaleCompare((char *) keyword,"stealth") == 0)
-          {
-            delegate_info->stealth=LocaleCompare(token,"True") == 0 ?
-              MagickTrue : MagickFalse;
-            break;
-          }
-        break;
-      }
-      default:
-        break;
-    }
+    include=GetNextXMLTreeTag(include);
   }
-  token=(char *) RelinquishMagickMemory(token);
-  if (delegate_list == (LinkedListInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  delegate=GetXMLTreeChild(delegate_map,"delegate");
+  while (delegate != (XMLTreeInfo *) NULL)
+  {
+    const char
+      *attribute;
+
+    /*
+      Process delegate element.
+    */
+    delegate_info=(DelegateInfo *) AcquireMagickMemory(sizeof(*delegate_info));
+    if (delegate_info == (DelegateInfo *) NULL)
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",filename);
+    (void) ResetMagickMemory(delegate_info,0,sizeof(*delegate_info));
+    delegate_info->path=ConstantString(filename);
+    delegate_info->signature=MagickSignature;
+    attribute=GetXMLTreeAttribute(delegate,"command");
+    if (attribute != (const char *) NULL)
+      {
+        char
+          *commands;
+
+        commands=ConstantString(attribute);
+#if defined(__WINDOWS__)
+        if (strchr(commands,'@') != (char *) NULL)
+          {
+            char
+              path[MaxTextExtent];
+
+            NTGhostscriptEXE(path,MaxTextExtent);
+            SubstituteString((char **) &commands,"@PSDelegate@",path);
+            SubstituteString((char **) &commands,"\\","/");
+          }
+#endif
+        delegate_info->commands=commands;
+      }
+    attribute=GetXMLTreeAttribute(delegate,"decode");
+    if (attribute != (const char *) NULL)
+      {
+        delegate_info->decode=ConstantString(attribute);
+        delegate_info->mode=1;
+      }
+    attribute=GetXMLTreeAttribute(delegate,"encode");
+    if (attribute != (const char *) NULL)
+      {
+        delegate_info->encode=ConstantString(attribute);
+        delegate_info->mode=(-1);
+      }
+    attribute=GetXMLTreeAttribute(delegate,"mode");
+    if (attribute != (const char *) NULL)
+      {
+        delegate_info->mode=1;
+        if (LocaleCompare(attribute,"bi") == 0)
+          delegate_info->mode=0;
+        if (LocaleCompare(attribute,"encode") == 0)
+          delegate_info->mode=(-1);
+      }
+    attribute=GetXMLTreeAttribute(delegate,"spawn");
+    if (attribute != (const char *) NULL)
+      delegate_info->spawn=IsMagickTrue(attribute);
+    attribute=GetXMLTreeAttribute(delegate,"stealth");
+    if (attribute != (const char *) NULL)
+      delegate_info->stealth=IsMagickTrue(attribute);
+    attribute=GetXMLTreeAttribute(delegate,"thread-support");
+    if (attribute != (const char *) NULL)
+      delegate_info->thread_support=IsMagickTrue(attribute);
+    status=AppendValueToLinkedList(delegate_list,delegate_info);
+    if (status == MagickFalse)
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
+    delegate=GetNextXMLTreeTag(delegate);
+  }
+  delegate_map=DestroyXMLTree(delegate_map);
+  return(status);
 }
 
 /*
