@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,10 +13,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image effects methods.
+  MagickCore image effects methods.
 */
-#ifndef _MAGICK_EFFECT_H
-#define _MAGICK_EFFECT_H
+#ifndef _MAGICKCORE_EFFECT_H
+#define _MAGICKCORE_EFFECT_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -34,42 +34,44 @@ typedef enum
 } NoiseType;
 
 extern MagickExport Image
-  *AdaptiveThresholdImage(const Image *,const unsigned long,const unsigned long,    const long,ExceptionInfo *),
-  *AddNoiseImage(const Image *,const NoiseType,ExceptionInfo *),
-  *BlurImage(const Image *,const double,const double,ExceptionInfo *),
-  *BlurImageChannel(const Image *,const ChannelType channel,const double,
+  *AdaptiveBlurImage(const Image *,const double,const double,ExceptionInfo *),
+  *AdaptiveBlurImageChannel(const Image *,const ChannelType,const double,
     const double,ExceptionInfo *),
+  *AdaptiveSharpenImage(const Image *,const double,const double,
+     ExceptionInfo *),
+  *AdaptiveSharpenImageChannel(const Image *,const ChannelType,const double,
+    const double,ExceptionInfo *),
+  *AddNoiseImage(const Image *,const NoiseType,ExceptionInfo *),
+  *AddNoiseImageChannel(const Image *,const ChannelType,const NoiseType,
+    ExceptionInfo *),
+  *BlurImage(const Image *,const double,const double,ExceptionInfo *),
+  *BlurImageChannel(const Image *,const ChannelType,const double,const double,
+    ExceptionInfo *),
   *DespeckleImage(const Image *,ExceptionInfo *),
   *EdgeImage(const Image *,const double,ExceptionInfo *),
   *EmbossImage(const Image *,const double,const double,ExceptionInfo *),
   *GaussianBlurImage(const Image *,const double,const double,ExceptionInfo *),
-  *GaussianBlurImageChannel(const Image *,const ChannelType channel,
-    const double,const double,ExceptionInfo *),
+  *GaussianBlurImageChannel(const Image *,const ChannelType,const double,
+    const double,ExceptionInfo *),
   *MedianFilterImage(const Image *,const double,ExceptionInfo *),
   *MotionBlurImage(const Image *,const double,const double,const double,
     ExceptionInfo *),
   *PreviewImage(const Image *,const PreviewType,ExceptionInfo *),
   *RadialBlurImage(const Image *,const double,ExceptionInfo *),
+  *RadialBlurImageChannel(const Image *,const ChannelType,const double,
+    ExceptionInfo *),
   *ReduceNoiseImage(const Image *,const double,ExceptionInfo *),
   *ShadeImage(const Image *,const MagickBooleanType,const double,const double,
     ExceptionInfo *),
   *SharpenImage(const Image *,const double,const double,ExceptionInfo *),
-  *SharpenImageChannel(const Image *,const ChannelType channel,const double,
+  *SharpenImageChannel(const Image *,const ChannelType,const double,
     const double,ExceptionInfo *),
   *SpreadImage(const Image *,const double,ExceptionInfo *),
   *UnsharpMaskImage(const Image *,const double,const double,const double,
     const double,ExceptionInfo *),
-  *UnsharpMaskImageChannel(const Image *,const ChannelType channel,const double,
+  *UnsharpMaskImageChannel(const Image *,const ChannelType,const double,
     const double,const double,const double,ExceptionInfo *);
 
-extern MagickExport MagickBooleanType
-  BlackThresholdImage(Image *,const char *),
-  BilevelImage(Image *,const double),
-  BilevelImageChannel(Image *,const ChannelType,const double),
-  RandomThresholdImageChannel(Image *,const ChannelType,const char *,
-    ExceptionInfo *),
-  WhiteThresholdImage(Image *,const char *);
-
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
