@@ -271,10 +271,10 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  histogram=(MagickPixelPacket *)
-    AcquireMagickMemory((MaxMap+1)*sizeof(*histogram));
-  normalize_map=(MagickPixelPacket *)
-    AcquireMagickMemory((MaxMap+1)*sizeof(*normalize_map));
+  histogram=(MagickPixelPacket *) AcquireMagickMemory((MaxMap+1)*
+    sizeof(*histogram));
+  normalize_map=(MagickPixelPacket *) AcquireMagickMemory((MaxMap+1)*
+    sizeof(*normalize_map));
   if ((histogram == (MagickPixelPacket *) NULL) ||
       (normalize_map == (MagickPixelPacket *) NULL))
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
@@ -283,34 +283,26 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
     Form histogram.
   */
   (void) ResetMagickMemory(histogram,0,(MaxMap+1)*sizeof(*histogram));
-  if (channel == DefaultChannels)
-    {
-      for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (long) image->rows; y++)
+  {
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    indexes=GetIndexes(image);
+    if (channel == DefaultChannels)
+      for (x=0; x < (long) image->columns; x++)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-        if (p == (const PixelPacket *) NULL)
-          break;
-        for (x=0; x < (long) image->columns; x++)
-        {
-          histogram[ScaleQuantumToMap(PixelIntensityToQuantum(p))].index++;
-          p++;
-        }
+        Quantum
+          intensity;
+
+        intensity=PixelIntensityToQuantum(p);
+        histogram[ScaleQuantumToMap(intensity)].red++;
+        histogram[ScaleQuantumToMap(intensity)].green++;
+        histogram[ScaleQuantumToMap(intensity)].blue++;
+        histogram[ScaleQuantumToMap(intensity)].index++;
+        p++;
       }
-      for (x=0; x <= (long) MaxMap; x++)
-      {
-        histogram[x].red=histogram[x].index;
-        histogram[x].green=histogram[x].index;
-        histogram[x].blue=histogram[x].index;
-        histogram[x].opacity=histogram[x].index;
-      }
-    }
-  else
-    for (y=0; y < (long) image->rows; y++)
-    {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (const PixelPacket *) NULL)
-        break;
-      indexes=GetIndexes(image);
+    else
       for (x=0; x < (long) image->columns; x++)
       {
         if ((channel & RedChannel) != 0)
@@ -326,12 +318,12 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
           histogram[ScaleQuantumToMap(indexes[x])].index++;
         p++;
       }
-    }
+  }
   /*
     Find the histogram boundaries by locating the black/white levels.
   */
   black.red=0.0;
-  white.red=MaxRange(QuantumRange)-1.0;
+  white.red=MaxRange(QuantumRange);
   if ((channel & RedChannel) != 0)
     {
       intensity=0.0;
@@ -352,7 +344,7 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
       white.red=(MagickRealType) x;
     }
   black.green=0.0;
-  white.green=MaxRange(QuantumRange)-1.0;
+  white.green=MaxRange(QuantumRange);
   if ((channel & GreenChannel) != 0)
     {
       intensity=0.0;
@@ -373,7 +365,7 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
       white.green=(MagickRealType) x;
     }
   black.blue=0.0;
-  white.blue=MaxRange(QuantumRange)-1.0;
+  white.blue=MaxRange(QuantumRange);
   if ((channel & BlueChannel) != 0)
     {
       intensity=0.0;
@@ -394,7 +386,7 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
       white.blue=(MagickRealType) x;
     }
   black.opacity=0.0;
-  white.opacity=MaxRange(QuantumRange)-1.0;
+  white.opacity=MaxRange(QuantumRange);
   if (((channel & OpacityChannel) != 0) && (image->matte != MagickFalse))
     {
       intensity=0.0;
@@ -415,7 +407,7 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
       white.opacity=(MagickRealType) x;
     }
   black.index=0.0;
-  white.index=MaxRange(QuantumRange)-1.0;
+  white.index=MaxRange(QuantumRange);
   if (((channel & IndexChannel) != 0) && (image->colorspace == CMYKColorspace))
     {
       intensity=0.0;
@@ -540,6 +532,12 @@ MagickExport MagickBooleanType ContrastStretchImageChannel(Image *image,
               image->colormap[i].blue=RoundToQuantum(normalize_map[
                 ScaleQuantumToMap(image->colormap[i].blue)].blue);
           }
+        if (((channel & OpacityChannel) != 0) && (image->matte != MagickFalse))
+          {
+            if (black.opacity != white.opacity)
+              image->colormap[i].opacity=RoundToQuantum(normalize_map[
+                ScaleQuantumToMap(image->colormap[i].opacity)].opacity);
+          }
       }
     }
   /*
@@ -632,19 +630,19 @@ MagickExport Image *EnhanceImage(const Image *image,ExceptionInfo *exception)
 #define Enhance(weight) \
   mean=((MagickRealType) r->red+pixel.red)/2; \
   distance=(MagickRealType) r->red-(MagickRealType) pixel.red; \
-  distance_squared=(2.0*((MagickRealType) QuantumRange+1.0)+mean)* \
-    distance*distance/QuantumRange; \
+  distance_squared=QuantumScale*(2.0*((MagickRealType) QuantumRange+1.0)+ \
+     mean)*distance*distance; \
   mean=((MagickRealType) r->green+pixel.green)/2; \
   distance=(MagickRealType) r->green-(MagickRealType) pixel.green; \
   distance_squared+=4.0*distance*distance; \
   mean=((MagickRealType) r->blue+pixel.blue)/2; \
   distance=(MagickRealType) r->blue-(MagickRealType) pixel.blue; \
-  distance_squared+=(3.0*((MagickRealType) \
-    QuantumRange+1.0)-1.0-mean)*distance*distance/QuantumRange; \
+  distance_squared+=QuantumScale*(3.0*((MagickRealType) \
+    QuantumRange+1.0)-1.0-mean)*distance*distance; \
   mean=((MagickRealType) r->opacity+pixel.opacity)/2; \
   distance=(MagickRealType) r->opacity-(MagickRealType) pixel.opacity; \
-  distance_squared+= (3.0*((MagickRealType) \
-    QuantumRange+1.0)-1.0-mean)*distance*distance/QuantumRange; \
+  distance_squared+=QuantumScale*(3.0*((MagickRealType) \
+    QuantumRange+1.0)-1.0-mean)*distance*distance; \
   if (distance_squared < ((MagickRealType) QuantumRange*QuantumRange/25.0)) \
     { \
       aggregate.red+=(weight)*r->red; \
@@ -703,7 +701,11 @@ MagickExport Image *EnhanceImage(const Image *image,ExceptionInfo *exception)
   if (enhance_image == (Image *) NULL)
     return((Image *) NULL);
   if (SetImageStorageClass(enhance_image,DirectClass) == MagickFalse)
-    return((Image *) NULL);
+    {
+      InheritException(exception,&enhance_image->exception);
+      enhance_image=DestroyImage(enhance_image);
+      return((Image *) NULL);
+    }
   /*
     Enhance image.
   */
@@ -1244,13 +1246,13 @@ MagickExport MagickBooleanType LevelImageChannel(Image *image,
   if (gamma == 1.0)
     {
       for ( ; i < (long) white; i++)
-        levels_map[i]=(MagickRealType)
-          (MaxMap*(((MagickRealType) i-black)/(white-black)));
+        levels_map[i]=(MagickRealType) (MaxMap*(((MagickRealType) i-black)/
+          (white-black)));
     }
   else
     for ( ; i < (long) white; i++)
-      levels_map[i]=(MagickRealType)
-        (MaxMap*(pow(((double) i-black)/(white-black),1.0/gamma)));
+      levels_map[i]=(MagickRealType) (MaxMap*(pow(((double) i-black)/
+        (white-black),1.0/gamma)));
   for ( ; i <= (long) MaxMap; i++)
     levels_map[i]=(MagickRealType) MaxMap;
   if (image->storage_class == PseudoClass)
@@ -1806,7 +1808,7 @@ MagickExport MagickBooleanType SigmoidalContrastImageChannel(Image *image,
         continue;
       }
     sigmoidal_map[i]=(MagickRealType) ScaleMapToQuantum((unsigned long)
-      (MaxMap*(midpoint/QuantumRange-log((1.0-(1.0/(1.0+exp(midpoint/
+      (MaxMap*(QuantumScale*midpoint-log((1.0-(1.0/(1.0+exp(midpoint/
       (double) QuantumRange*contrast))+((double) i/MaxMap)*((1.0/
       (1.0+exp(contrast*(midpoint/(double) QuantumRange-1.0))))-(1.0/
       (1.0+exp(midpoint/(double) QuantumRange*contrast))))))/
