@@ -179,6 +179,7 @@ MagickExport void ClearMagickException(ExceptionInfo *exception)
   exception->reason=(char *) NULL;
   exception->description=(char *) NULL;
   RelinquishSemaphoreInfo(exception->semaphore);
+  errno=0;
 }
 
 /*
@@ -203,7 +204,6 @@ MagickExport void ClearMagickException(ExceptionInfo *exception)
 %
 %    o exception: The exception info.
 %
-%
 */
 MagickExport void CatchException(ExceptionInfo *exception)
 {
@@ -258,7 +258,6 @@ MagickExport void CatchException(ExceptionInfo *exception)
 %
 %    o description: Specifies any description to the reason.
 %
-%
 */
 static void DefaultErrorHandler(const ExceptionType magick_unused(severity),
   const char *reason,const char *description)
@@ -300,9 +299,8 @@ static void DefaultErrorHandler(const ExceptionType magick_unused(severity),
 %
 %    o description: Specifies any description to the reason.
 %
-%
 */
-static void DefaultFatalErrorHandler(const ExceptionType severity,
+static void DefaultFatalErrorHandler(const ExceptionType magick_unused(severity),
   const char *reason,const char *description)
 {
   if (reason == (char *) NULL)
@@ -313,7 +311,7 @@ static void DefaultFatalErrorHandler(const ExceptionType severity,
   (void) fprintf(stderr,".\n");
   (void) fflush(stderr);
   DestroyMagick();
-  exit(severity);
+  exit(1);
 }
 
 /*
@@ -343,7 +341,6 @@ static void DefaultFatalErrorHandler(const ExceptionType severity,
 %
 %    o description: Specifies any description to the reason.
 %
-%
 */
 static void DefaultWarningHandler(const ExceptionType magick_unused(severity),
   const char *reason,const char *description)
@@ -417,7 +414,6 @@ MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
 %
 %    o exception: The exception info.
 %
-%
 */
 MagickExport void GetExceptionInfo(ExceptionInfo *exception)
 {
@@ -592,7 +588,6 @@ MagickExport const char *GetLocaleExceptionMessage(const ExceptionType severity,
 %
 %    o relative: The related exception info.
 %
-%
 */
 MagickExport void InheritException(ExceptionInfo *exception,
   const ExceptionInfo *relative)
@@ -644,7 +639,6 @@ MagickExport void InheritException(ExceptionInfo *exception,
 %
 %    o description: Specifies any description to the reason.
 %
-%
 */
 MagickExport void MagickError(const ExceptionType error,const char *reason,
   const char *description)
@@ -681,7 +675,6 @@ MagickExport void MagickError(const ExceptionType error,const char *reason,
 %
 %    o description: Specifies any description to the reason.
 %
-%
 */
 MagickExport void MagickFatalError(const ExceptionType error,const char *reason,
   const char *description)
@@ -716,7 +709,6 @@ MagickExport void MagickFatalError(const ExceptionType error,const char *reason,
 %
 %    o description: Describe the warning.
 %
-%
 */
 MagickExport void MagickWarning(const ExceptionType warning,const char *reason,
   const char *description)
@@ -747,7 +739,6 @@ MagickExport void MagickWarning(const ExceptionType warning,const char *reason,
 %
 %    o handler: The method to handle errors.
 %
-%
 */
 MagickExport ErrorHandler SetErrorHandler(ErrorHandler handler)
 {
@@ -770,8 +761,8 @@ MagickExport ErrorHandler SetErrorHandler(ErrorHandler handler)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SetFatalErrorHandler() sets the fatal exception handler to the specified method
-%  and returns the previous fatal exception handler.
+%  SetFatalErrorHandler() sets the fatal exception handler to the specified
+%  method and returns the previous fatal exception handler.
 %
 %  The format of the SetErrorHandler method is:
 %
@@ -781,7 +772,6 @@ MagickExport ErrorHandler SetErrorHandler(ErrorHandler handler)
 %
 %    o handler: The method to handle errors.
 %
-%
 */
 MagickExport FatalErrorHandler SetFatalErrorHandler(FatalErrorHandler handler)
 {
@@ -815,7 +805,6 @@ MagickExport FatalErrorHandler SetFatalErrorHandler(FatalErrorHandler handler)
 %
 %    o handler: The method to handle warnings.
 %
-%
 */
 MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 {
@@ -857,7 +846,6 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 %
 %    o description: The exception description.
 %
-%
 */
 MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
   const ExceptionType severity,const char *reason,const char *description)
