@@ -41,6 +41,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/color-private.h"
 #include "magick/draw.h"
 #include "magick/gem.h"
 #include "magick/image.h"
@@ -50,6 +51,8 @@
 #include "magick/pixel-private.h"
 #include "magick/quantum.h"
 #include "magick/random_.h"
+#include "magick/resize.h"
+#include "magick/transform.h"
 #include "magick/signature.h"
 
 /*
@@ -76,7 +79,6 @@
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
-%
 */
 MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
   Quantum *blue)
@@ -131,7 +133,6 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
 %
 %    o affine: A pointer the the affine transform of type AffineMatrix.
 %
-%
 */
 MagickExport double ExpandAffine(const AffineMatrix *affine)
 {
@@ -169,7 +170,6 @@ MagickExport double ExpandAffine(const AffineMatrix *affine)
 %
 %    o sigma: The standard deviation of the Gaussian, in pixels.
 %
-%
 */
 
 MagickExport unsigned long GetOptimalKernelWidth1D(const double radius,
@@ -188,7 +188,7 @@ MagickExport unsigned long GetOptimalKernelWidth1D(const double radius,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(sigma != 0.0);
   if (radius > 0.0)
-    return(Max(2*((unsigned long) radius)+1,3));
+    return(Max(2*((unsigned long) (radius+0.5))+1,3));
   for (width=5; ;)
   {
     normalize=0.0;
@@ -214,6 +214,7 @@ MagickExport unsigned long GetOptimalKernelWidth2D(const double radius,
     alpha,
     normalize,
     value;
+
   register long
     u,
     v;
@@ -221,7 +222,7 @@ MagickExport unsigned long GetOptimalKernelWidth2D(const double radius,
   assert(sigma != 0.0);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (radius > 0.0)
-    return(Max(2*((unsigned long) radius)+1,3));
+    return(Max(2*((unsigned long) (radius+0.5))+1,3));
   for (width=5; ;)
   {
     normalize=0.0;
@@ -274,7 +275,6 @@ MagickExport unsigned long  GetOptimalKernelWidth(const double radius,
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
-%
 */
 MagickExport void HSBTransform(const double hue,const double saturation,
   const double brightness,Quantum *red,Quantum *green,Quantum *blue)
@@ -378,7 +378,6 @@ MagickExport void HSBTransform(const double hue,const double saturation,
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
-%
 */
 
 static inline MagickRealType HueToRGB(MagickRealType m1,MagickRealType m2,
@@ -452,7 +451,6 @@ MagickExport void HSLTransform(const double hue,const double saturation,
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
-%
 */
 MagickExport void HWBTransform(const double hue,const double whiteness,
   const double blackness,Quantum *red,Quantum *green,Quantum *blue)
@@ -536,7 +534,6 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
 %
 %    o f, g: A pointer to an image pixel and one of it's neighbor.
 %
-%
 */
 MagickExport void Hull(const long x_offset,const long y_offset,
   const unsigned long columns,const unsigned long rows,Quantum *f,Quantum *g,
@@ -643,39 +640,69 @@ MagickExport void Hull(const long x_offset,const long y_offset,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n t e r p o l a t e C o l o r                                           %
+%   I n t e r p o l a t e P i x e l C o l o r                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  InterpolateColor() applies bi-linear interpolation between a pixel and its
-%  neighbors.
+%  InterpolatePixelColor() applies bi-linear or tri-linear interpolation
+%  between a pixel and it's neighbors.
 %
 %  The format of the InterpolateColor method is:
 %
 %      MagickPixelPacket InterpolateColor(const Image *image,
-%        const double x_offset,const double y_offset,ExceptionInfo *exception)
+%        ViewInfo *view_info,InterpolatePixelMethod method,const double x,
+%        const double y,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: The image.
 %
-%    o x_offset,y_offset: A double representing the current (x,y) position of
-%      the pixel.
+%    o image_view: The image cache view.
 %
+%    o type:  the type of pixel color interpolation.
+%
+%    o x,y: A double representing the current (x,y) position of the pixel.
+%
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
-MagickExport MagickPixelPacket InterpolateColor(const Image *image,
-  const double x_offset,const double y_offset,ExceptionInfo *exception)
+static inline MagickRealType CubicWeightingFunction(const MagickRealType x)
+{
+  MagickRealType
+    alpha,
+    gamma;
+
+  alpha=Max(x+2.0,0.0);
+  gamma=1.0*alpha*alpha*alpha;
+  alpha=Max(x+1.0,0.0);
+  gamma-=4.0*alpha*alpha*alpha;
+  alpha=Max(x+0.0,0.0);
+  gamma+=6.0*alpha*alpha*alpha;
+  alpha=Max(x-1.0,0.0);
+  gamma-=4.0*alpha*alpha*alpha;
+  return(gamma/6.0);
+}
+
+static inline long NearestNeighbor(MagickRealType x)
+{
+  if (x >= 0.0)
+    return((long) (x+0.5));
+  return((long) (x-0.5));
+}
+
+MagickExport MagickPixelPacket InterpolatePixelColor(const Image *image,
+  ViewInfo *image_view,InterpolatePixelMethod method,const double x,
+  const double y,ExceptionInfo *exception)
 {
   MagickPixelPacket
     pixel,
-    pixels[4];
+    pixels[16];
 
   MagickRealType
-    alpha[4],
+    alpha[16],
     gamma;
 
   PointInfo
@@ -692,47 +719,348 @@ MagickExport MagickPixelPacket InterpolateColor(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  assert(image_view != (ViewInfo *) NULL);
   GetMagickPixelPacket(image,&pixel);
-  p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
-  if (p == (const PixelPacket *) NULL)
-    return(pixel);
-  for (i=0; i < 4L; i++)
+  switch (method)
   {
-    indexes=GetIndexes(image);
-    alpha[i]=1.0;
-    if (image->matte != MagickFalse)
-      alpha[i]=((MagickRealType) QuantumRange-p->opacity)/QuantumRange;
-    GetMagickPixelPacket(image,pixels+i);
-    SetMagickPixelPacket(p,indexes,pixels+i);
-    pixels[i].red*=alpha[i];
-    pixels[i].green*=alpha[i];
-    pixels[i].blue*=alpha[i];
-    if (image->colorspace == CMYKColorspace)
-      pixels[i].index*=alpha[i];
-    p++;
+    case AverageInterpolatePixel:
+    {
+      p=AcquireCacheView(image_view,(long) floor(x)-1,(long) floor(y)-1,4,4,
+        exception);
+      if (p == (const PixelPacket *) NULL)
+        return(pixel);
+      indexes=GetCacheViewIndexes(image_view);
+      for (i=0; i < 16L; i++)
+      {
+        GetMagickPixelPacket(image,pixels+i);
+        SetMagickPixelPacket(p,indexes+i,pixels+i);
+        alpha[i]=1.0;
+        if (image->matte != MagickFalse)
+          {
+            alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+            pixels[i].red*=alpha[i];
+            pixels[i].green*=alpha[i];
+            pixels[i].blue*=alpha[i];
+            if (image->colorspace == CMYKColorspace)
+              pixels[i].index*=alpha[i];
+          }
+        gamma=alpha[i];
+        gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+        pixel.red+=gamma*0.0625*pixels[i].red;
+        pixel.green+=gamma*0.0625*pixels[i].green;
+        pixel.blue+=gamma*0.0625*pixels[i].blue;
+        if (image->matte != MagickFalse)
+          pixel.opacity+=0.0625*pixels[i].opacity;
+        if (image->colorspace == CMYKColorspace)
+          pixel.index+=gamma*0.0625*pixels[i].index;
+        p++;
+      }
+      break;
+    }
+    case BicubicInterpolatePixel:
+    {
+      long
+        n;
+
+      MagickRealType
+        dx,
+        dy;
+
+      register long
+        j;
+
+      p=AcquireCacheView(image_view,(long) floor(x)-1,(long) floor(y)-1,4,4,
+        exception);
+      if (p == (const PixelPacket *) NULL)
+        return(pixel);
+      indexes=GetCacheViewIndexes(image_view);
+      n=0;
+      delta.x=x-floor(x);
+      delta.y=y-floor(y);
+      for (i=(-1); i < 3L; i++)
+      {
+        dy=CubicWeightingFunction((MagickRealType) i-delta.y);
+        for (j=(-1); j < 3L; j++)
+        {
+          GetMagickPixelPacket(image,pixels+n);
+          SetMagickPixelPacket(p,indexes+n,pixels+n);
+          alpha[n]=1.0;
+          if (image->matte != MagickFalse)
+            {
+              alpha[n]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+              pixels[n].red*=alpha[n];
+              pixels[n].green*=alpha[n];
+              pixels[n].blue*=alpha[n];
+              if (image->colorspace == CMYKColorspace)
+                pixels[n].index*=alpha[n];
+            }
+          dx=CubicWeightingFunction(delta.x-(MagickRealType) j);
+          gamma=alpha[n];
+          gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+          pixel.red+=gamma*dx*dy*pixels[n].red;
+          pixel.green+=gamma*dx*dy*pixels[n].green;
+          pixel.blue+=gamma*dx*dy*pixels[n].blue;
+          if (image->matte != MagickFalse)
+            pixel.opacity+=dx*dy*pixels[n].opacity;
+          if (image->colorspace == CMYKColorspace)
+            pixel.index+=gamma*dx*dy*pixels[n].index;
+          n++;
+          p++;
+        }
+      }
+      break;
+    }
+    case BilinearInterpolatePixel:
+    default:
+    {
+      delta.x=x-floor(x);
+      delta.y=y-floor(y);
+      if ((delta.x == 0.0) && (delta.y == 0.0))
+        {
+          p=AcquireCacheView(image_view,(long) x,(long) y,1,1,exception);
+          if (p == (const PixelPacket *) NULL)
+            return(pixel);
+          indexes=GetCacheViewIndexes(image_view);
+          GetMagickPixelPacket(image,pixels);
+          SetMagickPixelPacket(p,indexes,&pixel);
+          return(pixel);
+        }
+      p=AcquireCacheView(image_view,(long) floor(x),(long) floor(y),2,2,
+        exception);
+      if (p == (const PixelPacket *) NULL)
+        return(pixel);
+      indexes=GetCacheViewIndexes(image_view);
+      for (i=0; i < 4L; i++)
+      {
+        GetMagickPixelPacket(image,pixels+i);
+        SetMagickPixelPacket(p,indexes+i,pixels+i);
+        alpha[i]=1.0;
+        if (image->matte != MagickFalse)
+          {
+            alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+            pixels[i].red*=alpha[i];
+            pixels[i].green*=alpha[i];
+            pixels[i].blue*=alpha[i];
+            if (image->colorspace == CMYKColorspace)
+              pixels[i].index*=alpha[i];
+          }
+        p++;
+      }
+      gamma=(((1.0-delta.y)*((1.0-delta.x)*alpha[0]+delta.x*alpha[1])+
+        delta.y*((1.0-delta.x)*alpha[2]+delta.x*alpha[3])));
+      gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+      pixel.red=gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].red+delta.x*
+        pixels[1].red)+delta.y*((1.0-delta.x)*pixels[2].red+delta.x*
+        pixels[3].red));
+      pixel.green=gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].green+
+        delta.x*pixels[1].green)+delta.y*((1.0-delta.x)*pixels[2].green+
+        delta.x*pixels[3].green));
+      pixel.blue=gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].blue+delta.x*
+        pixels[1].blue)+delta.y*((1.0-delta.x)*pixels[2].blue+delta.x*
+        pixels[3].blue));
+      if (image->matte != MagickFalse)
+        pixel.opacity=((1.0-delta.y)*((1.0-delta.x)*pixels[0].opacity+
+          delta.x*pixels[1].opacity)+delta.y*((1.0-delta.x)*
+          pixels[2].opacity+delta.x*pixels[3].opacity));
+      if (image->colorspace == CMYKColorspace)
+        pixel.index=gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].index+
+          delta.x*pixels[1].index)+delta.y*((1.0-delta.x)*pixels[2].index+
+          delta.x*pixels[3].index));
+      break;
+    }
+    case FilterInterpolatePixel:
+    {
+      Image
+        *crop_image,
+        *zoom_image;
+
+      RectangleInfo
+        geometry;
+
+      geometry.width=4;
+      geometry.height=4;
+      geometry.x=(long) (x-1.0);
+      geometry.y=(long) (y-1.0);
+      crop_image=CropImage(image,&geometry,exception);
+      if (crop_image == (Image *) NULL)
+        break;
+      zoom_image=ZoomImage(crop_image,1,1,exception);
+      crop_image=DestroyImage(crop_image);
+      if (zoom_image == (Image *) NULL)
+        break;
+      p=AcquireImagePixels(zoom_image,0,0,1,1,exception);
+      if (p == (const PixelPacket *) NULL)
+        {
+          zoom_image=DestroyImage(zoom_image);
+          return(pixel);
+        }
+      indexes=GetIndexes(zoom_image);
+      GetMagickPixelPacket(image,pixels);
+      SetMagickPixelPacket(p,indexes,&pixel);
+      zoom_image=DestroyImage(zoom_image);
+      break;
+    }
+    case IntegerInterpolatePixel:
+    {
+      p=AcquireCacheView(image_view,(long) floor(x),(long) floor(y),1,1,
+        exception);
+      if (p == (const PixelPacket *) NULL)
+        return(pixel);
+      indexes=GetCacheViewIndexes(image_view);
+      GetMagickPixelPacket(image,pixels);
+      SetMagickPixelPacket(p,indexes,&pixel);
+      break;
+    }
+    case MeshInterpolatePixel:
+    {
+      PointInfo
+        luminance;
+
+      p=AcquireCacheView(image_view,(long) floor(x),(long) floor(y),2,2,
+        exception);
+      if (p == (const PixelPacket *) NULL)
+        return(pixel);
+      indexes=GetCacheViewIndexes(image_view);
+      for (i=0; i < 4L; i++)
+      {
+        GetMagickPixelPacket(image,pixels+i);
+        SetMagickPixelPacket(p,indexes+i,pixels+i);
+        alpha[i]=1.0;
+        if (image->matte != MagickFalse)
+          {
+            alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+            pixels[i].red*=alpha[i];
+            pixels[i].green*=alpha[i];
+            pixels[i].blue*=alpha[i];
+            if (image->colorspace == CMYKColorspace)
+              pixels[i].index*=alpha[i];
+          }
+        p++;
+      }
+      delta.x=x-floor(x);
+      delta.y=y-floor(y);
+      luminance.x=MagickPixelLuminance(pixels+0)-MagickPixelLuminance(pixels+3);
+      luminance.y=MagickPixelLuminance(pixels+2)-MagickPixelLuminance(pixels+1);
+      if (AbsoluteValue(luminance.x) < AbsoluteValue(luminance.y))
+        {
+          if (delta.x <= delta.y)
+            {
+              gamma=((1.0-delta.x)*(1.0-delta.y)*alpha[0]+(1.0-delta.x)*
+                delta.y*alpha[2]+delta.x*delta.y*alpha[3]+(alpha[0]+
+                alpha[3])/2.0*delta.x*(1.0-delta.y));
+              gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+              pixel.red=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].red+(1.0-
+                delta.x)*delta.y*pixels[2].red+delta.x*delta.y*pixels[3].red+
+                (pixels[0].red+pixels[3].red)/2.0*delta.x*(1.0-delta.y);
+              pixel.green=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].green+
+                (1.0-delta.x)*delta.y*pixels[2].green+delta.x*delta.y*
+                pixels[3].green+(pixels[0].green+pixels[3].green)/2.0*delta.x*
+                (1.0-delta.y);
+              pixel.blue=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].blue+(1.0-
+                delta.x)*delta.y*pixels[2].blue+delta.x*delta.y*pixels[3].blue+
+                (pixels[0].blue+pixels[3].blue)/2.0*delta.x*(1.0-delta.y);
+              if (image->matte != MagickFalse)
+                pixel.opacity=((1.0-delta.x)*(1.0-delta.y)*pixels[0].opacity+
+                  (1.0-delta.x)*delta.y*pixels[2].opacity+delta.x*delta.y*
+                  pixels[3].opacity+(pixels[0].opacity+pixels[3].opacity)/2.0*
+                  delta.x*(1.0-delta.y));
+              if (image->colorspace == CMYKColorspace)
+                pixel.index=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].index+
+                  (1.0-delta.x)*delta.y*pixels[2].index+delta.x*delta.y*
+                  pixels[3].index+(pixels[0].index+pixels[3].index)/2.0*
+                  delta.x*(1.0-delta.y);
+              break;
+            }
+          gamma=((1.0-delta.x)*(1.0-delta.y)*alpha[0]+delta.x*delta.y*
+            alpha[3]+delta.x*(1.0-delta.y)*alpha[1]+(alpha[0]+alpha[3])/2.0*
+            (1.0-delta.x)*delta.y);
+          gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+          pixel.red=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].red+delta.x*
+            delta.y*pixels[3].red+delta.x*(1.0-delta.y)*pixels[1].red+
+            (pixels[0].red+pixels[3].red)/2.0*(1.0-delta.x)*delta.y;
+          pixel.green=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].green+delta.x*
+            delta.y*pixels[3].green+delta.x*(1.0-delta.y)*pixels[1].green+
+            (pixels[0].green+pixels[3].green)/2.0*(1.0-delta.x)*delta.y;
+          pixel.blue=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].blue+
+            delta.x*delta.y*pixels[3].blue+delta.x*(1.0-delta.y)*
+            pixels[1].blue+(pixels[0].blue+pixels[3].blue)/2.0*(1.0-delta.x)*
+            delta.y;
+          if (image->matte != MagickFalse)
+            pixel.opacity=(1.0-delta.x)*(1.0-delta.y)*pixels[0].opacity+
+              delta.x*delta.y*pixels[3].opacity+delta.x*(1.0-delta.y)*
+              pixels[1].opacity+(pixels[0].opacity+pixels[3].opacity)/2.0*
+              (1.0-delta.x)*delta.y;
+          if (image->colorspace == CMYKColorspace)
+            pixel.index=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].index+
+              delta.x*delta.y*pixels[3].index+delta.x*(1.0-delta.y)*
+              pixels[1].index+(pixels[0].index+pixels[3].index)/2.0*(1.0-
+              delta.x)*delta.y;
+          break;
+        }
+      if (delta.x <= (1.0-delta.y))
+        {
+          gamma=(1.0-delta.x)*(1.0-delta.y)*alpha[0]+(1.0-delta.x)*delta.y*
+            alpha[2]+delta.x*(1.0-delta.y)*alpha[1]+(alpha[1]+alpha[2])/2.0*
+            delta.x*delta.y;
+          gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+          pixel.red=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].red+(1.0-
+            delta.x)*delta.y*pixels[2].red+delta.x*(1.0-delta.y)*pixels[1].red+
+            (pixels[1].red+pixels[2].red)/2.0*delta.x*delta.y;
+          pixel.green=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].green+(1.0-
+            delta.x)*delta.y*pixels[2].green+delta.x*(1.0-delta.y)*
+            pixels[1].green+(pixels[1].green+pixels[2].green)/2.0*delta.x*
+            delta.y;
+          pixel.blue=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].blue+(1.0-
+            delta.x)*delta.y*pixels[2].blue+delta.x*(1.0-delta.y)*
+            pixels[1].blue+(pixels[1].blue+pixels[2].blue)/2.0*delta.x*delta.y;
+          if (image->matte != MagickFalse)
+            pixel.opacity=(1.0-delta.x)*(1.0-delta.y)*pixels[0].opacity+(1.0-
+              delta.x)*delta.y*pixels[2].opacity+delta.x*(1.0-delta.y)*
+              pixels[1].opacity+(pixels[1].opacity+pixels[2].opacity)/2.0*
+              delta.x*delta.y;
+          if (image->colorspace == CMYKColorspace)
+            pixel.index=gamma*(1.0-delta.x)*(1.0-delta.y)*pixels[0].index+(1.0-
+              delta.x)*delta.y*pixels[2].index+delta.x*(1.0-delta.y)*
+              pixels[1].index+(pixels[1].index+pixels[2].index)/2.0*delta.x*
+              delta.y;
+          break;
+        }
+      gamma=((1.0-delta.x)*delta.y*alpha[2]+delta.x*delta.y*alpha[3]+
+        delta.x*(1.0-delta.y)*alpha[1]+(alpha[1]+alpha[2])/2.0*(1.0-delta.x)*
+        (1.0-delta.y));
+      gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+      pixel.red=gamma*(1.0-delta.x)*delta.y*pixels[2].red+delta.x*delta.y*
+        pixels[3].red+delta.x*(1.0-delta.y)*pixels[1].red+(pixels[1].red+
+        pixels[2].red)/2.0*(1.0-delta.x)*(1.0-delta.y);
+      pixel.green=gamma*(1.0-delta.x)*delta.y*pixels[2].green+delta.x*delta.y*
+        pixels[3].green+delta.x*(1.0-delta.y)*pixels[1].green+(pixels[1].green+
+        pixels[2].green)/2.0*(1.0-delta.x)*(1.0-delta.y);
+      pixel.blue=gamma*(1.0-delta.x)*delta.y*pixels[2].blue+delta.x*delta.y*
+        pixels[3].blue+delta.x*(1.0-delta.y)*pixels[1].blue+(pixels[1].blue+
+        pixels[2].blue)/2.0*(1.0-delta.x)*(1.0-delta.y);
+      if (image->matte != MagickFalse)
+        pixel.opacity=(1.0-delta.x)*delta.y*pixels[2].opacity+delta.x*delta.y*
+          pixels[3].opacity+delta.x*(1.0-delta.y)*pixels[1].opacity+
+          (pixels[1].opacity+pixels[2].opacity)/2.0*(1.0-delta.x)*(1.0-
+          delta.y);
+      if (image->colorspace == CMYKColorspace)
+        pixel.index=gamma*(1.0-delta.x)*delta.y*pixels[2].index+delta.x*delta.y*
+          pixels[3].index+delta.x*(1.0-delta.y)*pixels[1].index+
+          (pixels[1].index+pixels[2].index)/2.0*(1.0-delta.x)*(1.0-delta.y);
+      break;
+    }
+    case NearestNeighborInterpolatePixel:
+    {
+      p=AcquireCacheView(image_view,NearestNeighbor(x),NearestNeighbor(y),1,1,
+        exception);
+      if (p == (const PixelPacket *) NULL)
+        return(pixel);
+      indexes=GetCacheViewIndexes(image_view);
+      GetMagickPixelPacket(image,pixels);
+      SetMagickPixelPacket(p,indexes,&pixel);
+      break;
+    }
   }
-  delta.x=x_offset-floor(x_offset);
-  delta.y=y_offset-floor(y_offset);
-  gamma=(((1.0-delta.y)*((1.0-delta.x)*alpha[0]+delta.x*alpha[1])+
-    delta.y*((1.0-delta.x)*alpha[2]+delta.x*alpha[3])));
-  gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
-  pixel.red=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].red+delta.x*
-    pixels[1].red)+delta.y*((1.0-delta.x)*pixels[2].red+delta.x*
-    pixels[3].red)));
-  pixel.green=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].green+delta.x*
-    pixels[1].green)+delta.y*((1.0-delta.x)*pixels[2].green+delta.x*
-    pixels[3].green)));
-  pixel.blue=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].blue+delta.x*
-    pixels[1].blue)+delta.y*((1.0-delta.x)*pixels[2].blue+delta.x*
-    pixels[3].blue)));
-  if (image->matte != MagickFalse)
-    pixel.opacity=((1.0-delta.y)*((1.0-delta.x)*pixels[0].opacity+delta.x*
-      pixels[1].opacity)+delta.y*((1.0-delta.x)*pixels[2].opacity+delta.x*
-      pixels[3].opacity));
-  if (image->colorspace == CMYKColorspace)
-    pixel.index=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].index+delta.x*
-      pixels[1].index)+delta.y*((1.0-delta.x)*pixels[2].index+delta.x*
-      pixels[3].index)));
   return(pixel);
 }
 
@@ -762,7 +1090,6 @@ MagickExport MagickPixelPacket InterpolateColor(const Image *image,
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
-%
 */
 MagickExport void ModulateHSB(const double percent_hue,
   const double percent_saturation,const double percent_brightness,
@@ -816,7 +1143,6 @@ MagickExport void ModulateHSB(const double percent_hue,
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
-%
 */
 MagickExport void ModulateHSL(const double percent_hue,
   const double percent_saturation,const double percent_luminosity,
@@ -870,7 +1196,6 @@ MagickExport void ModulateHSL(const double percent_hue,
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
-%
 */
 MagickExport void ModulateHWB(const double percent_hue,
   const double percent_whiteness,const double percent_blackness,
@@ -925,7 +1250,6 @@ MagickExport void ModulateHWB(const double percent_hue,
 %    o hue, saturation, brightness: A pointer to a double value representing a
 %      component of the HSB color space.
 %
-%
 */
 MagickExport void TransformHSB(const Quantum red,const Quantum green,
   const Quantum blue,double *hue,double *saturation,double *brightness)
@@ -995,7 +1319,6 @@ MagickExport void TransformHSB(const Quantum red,const Quantum green,
 %    o hue, saturation, luminosity: A pointer to a double value representing a
 %      component of the HSL color space.
 %
-%
 */
 MagickExport void TransformHSL(const Quantum red,const Quantum green,
   const Quantum blue,double *hue,double *saturation,double *luminosity)
@@ -1014,9 +1337,9 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   assert(hue != (double *) NULL);
   assert(saturation != (double *) NULL);
   assert(luminosity != (double *) NULL);
-  r=(MagickRealType) red/QuantumRange;
-  g=(MagickRealType) green/QuantumRange;
-  b=(MagickRealType) blue/QuantumRange;
+  r=QuantumScale*red;
+  g=QuantumScale*green;
+  b=QuantumScale*blue;
   max=Max(r,Max(g,b));
   min=Min(r,Min(g,b));
   *hue=0.0;
@@ -1064,7 +1387,6 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
 %    o hue, whiteness, blackness: A pointer to a double value representing a
 %      component of the HWB color space.
 %
-%
 */
 MagickExport void TransformHWB(const Quantum red,const Quantum green,
   const Quantum blue,double *hue,double *whiteness,double *blackness)
