@@ -257,7 +257,6 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
 %      The string is either a Postscript page name (e.g. A4) or a postscript
 %      page geometry (e.g. 612x792+36+36).
 %
-%
 */
 MagickExport char *GetPageGeometry(const char *page_geometry)
 {
@@ -273,65 +272,72 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
       { "10x13",  "720x936" },
       { "10x14",  "720x1008" },
       { "11x17",  "792x1224" },
-      { "A0",  "2384x3370" },
-      { "A1",  "1684x2384" },
-      { "A10", "73x105" },
-      { "A2",  "1191x1684" },
-      { "A3",  "842x1191" },
-      { "A4",  "595x842" },
-      { "A4SMALL", "595x842" },
-      { "A5",  "420x595" },
-      { "A6",  "297x420" },
-      { "A7",  "210x297" },
-      { "A8",  "148x210" },
-      { "A9",  "105x148" },
-      { "ARCHA", "648x864" },
-      { "ARCHB", "864x1296" },
-      { "ARCHC", "1296x1728" },
-      { "ARCHD", "1728x2592" },
-      { "ARCHE", "2592x3456" },
-      { "B0",  "2920x4127" },
-      { "B1",  "2064x2920" },
-      { "B10", "91x127" },
-      { "B2",  "1460x2064" },
-      { "B3",  "1032x1460" },
-      { "B4",  "729x1032" },
-      { "B5",  "516x729" },
-      { "B6",  "363x516" },
-      { "B7",  "258x363" },
-      { "B8",  "181x258" },
-      { "B9",  "127x181" },
-      { "C0",  "2599x3676" },
-      { "C1",  "1837x2599" },
-      { "C2",  "1298x1837" },
-      { "C3",  "918x1296" },
-      { "C4",  "649x918" },
-      { "C5",  "459x649" },
-      { "C6",  "323x459" },
-      { "C7",  "230x323" },
-      { "EXECUTIVE", "540x720" },
-      { "FLSA", "612x936" },
-      { "FLSE", "612x936" },
-      { "FOLIO",  "612x936" },
-      { "HALFLETTER", "396x612" },
-      { "ISOB0", "2835x4008" },
-      { "ISOB1", "2004x2835" },
-      { "ISOB10", "88x125" },
-      { "ISOB2", "1417x2004" },
-      { "ISOB3", "1001x1417" },
-      { "ISOB4", "709x1001" },
-      { "ISOB5", "499x709" },
-      { "ISOB6", "354x499" },
-      { "ISOB7", "249x354" },
-      { "ISOB8", "176x249" },
-      { "ISOB9", "125x176" },
-      { "LEDGER",  "1224x792" },
-      { "LEGAL",  "612x1008" },
-      { "LETTER", "612x792" },
-      { "LETTERSMALL",  "612x792" },
-      { "QUARTO",  "610x780" },
-      { "STATEMENT",  "396x612" },
-      { "TABLOID",  "792x1224" },
+      { "a0",  "2384x3370" },
+      { "a1",  "1684x2384" },
+      { "a10", "73x105" },
+      { "a2",  "1191x1684" },
+      { "a3",  "842x1191" },
+      { "a4",  "595x842" },
+      { "a4smaLL", "595x842" },
+      { "a5",  "420x595" },
+      { "a6",  "297x420" },
+      { "a7",  "210x297" },
+      { "a8",  "148x210" },
+      { "a9",  "105x148" },
+      { "archa", "648x864" },
+      { "archb", "864x1296" },
+      { "archC", "1296x1728" },
+      { "archd", "1728x2592" },
+      { "arche", "2592x3456" },
+      { "b0",  "2920x4127" },
+      { "b1",  "2064x2920" },
+      { "b10", "91x127" },
+      { "b2",  "1460x2064" },
+      { "b3",  "1032x1460" },
+      { "b4",  "729x1032" },
+      { "b5",  "516x729" },
+      { "b6",  "363x516" },
+      { "b7",  "258x363" },
+      { "b8",  "181x258" },
+      { "b9",  "127x181" },
+      { "c0",  "2599x3676" },
+      { "c1",  "1837x2599" },
+      { "c2",  "1298x1837" },
+      { "c3",  "918x1296" },
+      { "c4",  "649x918" },
+      { "c5",  "459x649" },
+      { "c6",  "323x459" },
+      { "c7",  "230x323" },
+      { "executive", "540x720" },
+      { "flsa", "612x936" },
+      { "flse", "612x936" },
+      { "folio",  "612x936" },
+      { "halfletter", "396x612" },
+      { "isob0", "2835x4008" },
+      { "isob1", "2004x2835" },
+      { "isob10", "88x125" },
+      { "isob2", "1417x2004" },
+      { "isob3", "1001x1417" },
+      { "isob4", "709x1001" },
+      { "isob5", "499x709" },
+      { "isob6", "354x499" },
+      { "isob7", "249x354" },
+      { "isob8", "176x249" },
+      { "isob9", "125x176" },
+      { "jisb0", "1030x1456" },
+      { "jisb1", "728x1030" },
+      { "jisb2", "515x728" },
+      { "jisb3", "364x515" },
+      { "jisb4", "257x364" },
+      { "jisb5", "182x257" },
+      { "jisb6", "128x182" },
+      { "ledger",  "1224x792" },
+      { "legal",  "612x1008" },
+      { "letter", "612x792" },
+      { "lettersmaLL",  "612x792" },
+      { "quarto",  "610x780" },
+      { "statement",  "396x612" },
+      { "tabloid",  "792x1224" },
       { (char *) NULL, (char *) NULL }
     };
 
@@ -402,7 +408,7 @@ MagickExport MagickBooleanType IsGeometry(const char *geometry)
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(geometry,&geometry_info);
-  return((MagickBooleanType) (flags != NoValue));
+  return(flags != NoValue ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -494,6 +500,69 @@ MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P a r s e A f f i n e G e o m e t r y                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ParseAffineGeometry() returns an affine matrix as defined by the geometry
+%  string.
+%
+%  The format of the ParseAffineGeometry method is:
+%
+%      MagickStatusType ParseAffineGeometry(const char *geometry,
+%        AffineMatrix *affine_matrix)
+%
+%  A description of each parameter follows:
+%
+%    o geometry:  The geometry (e.g. 1.0,0.0,0.0,1.0,3.2,1.2).
+%
+%    o affine_matrix: The affine matrix as defined by the geometry string.
+%
+*/
+MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
+  AffineMatrix *affine_matrix)
+{
+  char
+    *p;
+
+  double
+    value;
+
+  int
+    flags;
+
+  register long
+    i;
+
+  GetAffineMatrix(affine_matrix);
+  flags=NoValue;
+  p=(char *) geometry;
+  for (i=0; i < 6; i++)
+  {
+    value=strtod(p,&p);
+    switch (i)
+    {
+      case 0: affine_matrix->sx=value; break;
+      case 1: affine_matrix->rx=value; break;
+      case 2: affine_matrix->ry=value; break;
+      case 3: affine_matrix->sy=value; break;
+      case 4: affine_matrix->tx=value; flags|=XValue; break;
+      case 5: affine_matrix->ty=value; flags|=YValue; break;
+    }
+    if (*p !=',')
+      break;
+    p++;
+  }
+  return(flags);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P a r s e G e o m e t r y                                                 %
 %                                                                             %
 %                                                                             %
@@ -604,6 +673,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       case 'X':
       case ',':
       case '/':
+      case ':':
       {
         p++;
         break;
@@ -729,7 +799,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       /*
         Normalize sampling factor (e.g. 4:2:2 => 2x1).
       */
-      geometry_info->rho*=geometry_info->sigma;
+      geometry_info->rho/=geometry_info->sigma;
       geometry_info->sigma=1.0;
       if (geometry_info->xi == 0.0)
         geometry_info->sigma=2.0;
@@ -775,6 +845,10 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
     width;
 
   SetGeometry(image,region_info);
+  if (image->page.width != 0)
+    region_info->width=image->page.width;
+  if (image->page.height != 0)
+    region_info->height=image->page.height;
   flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
@@ -1066,6 +1140,10 @@ MagickExport MagickStatusType ParsePageGeometry(Image *image,
     flags;
 
   SetGeometry(image,region_info);
+  if (image->page.width != 0)
+    region_info->width=image->page.width;
+  if (image->page.height != 0)
+    region_info->height=image->page.height;
   flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
@@ -1103,7 +1181,6 @@ MagickExport MagickStatusType ParsePageGeometry(Image *image,
 %
 %    o region_info: The region as defined by the geometry string.
 %
-%
 */
 MagickExport MagickStatusType ParseSizeGeometry(Image *image,
   const char *geometry,RectangleInfo *region_info)
@@ -1154,11 +1231,7 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
   assert(geometry != (RectangleInfo *) NULL);
   (void) ResetMagickMemory(geometry,0,sizeof(*geometry));
   geometry->width=image->columns;
-  if (image->page.width != 0)
-    geometry->width=image->page.width;
   geometry->height=image->rows;
-  if (image->page.height != 0)
-    geometry->height=image->page.height;
 }
 
 /*
