@@ -72,6 +72,7 @@
 #include "magick/monitor.h"
 #include "magick/montage.h"
 #include "magick/option.h"
+#include "magick/pixel-private.h"
 #include "magick/prepress.h"
 #include "magick/profile.h"
 #include "magick/quantize.h"
@@ -151,6 +152,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   Image
     *p;
 
+  ImageType
+    type;
+
   long
     y;
 
@@ -171,7 +175,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   elapsed_time=GetElapsedTime(&image->timer);
   user_time=GetUserTime(&image->timer);
   GetTimerInfo(&image->timer);
-  if ((verbose == MagickFalse) || (image->cache == (void *) NULL))
+  if (verbose == MagickFalse)
     {
       /*
         Display summary info about the image.
@@ -200,7 +204,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           (void) fprintf(file,"DirectClass ");
           if (image->total_colors != 0)
             {
-              FormatSize(image->total_colors,format);
+              (void) FormatMagickSize(image->total_colors,format);
               (void) fprintf(file,"%s ",format);
             }
         }
@@ -208,10 +212,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         if (image->total_colors <= image->colors)
           (void) fprintf(file,"PseudoClass %luc ",image->colors);
         else
-          {
-            (void) fprintf(file,"PseudoClass %lu=>%luc ",image->total_colors,
-              image->colors);
-          }
+          (void) fprintf(file,"PseudoClass %lu=>%luc ",image->total_colors,
+            image->colors);
+      (void) fprintf(file,"%lu-bit ",image->depth);
       if (image->error.mean_error_per_pixel != 0.0)
         (void) fprintf(file,"%ld/%f/%fdb ",
           (long) (image->error.mean_error_per_pixel+0.5),
@@ -219,7 +222,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           image->error.normalized_maximum_error);
       if (GetBlobSize(image) != 0)
         {
-          FormatSize(GetBlobSize(image),format);
+          (void) FormatMagickSize(GetBlobSize(image),format);
           (void) fprintf(file,"%s ",format);
         }
       if (elapsed_time > 0.02)
@@ -232,12 +235,20 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   /*
     Display verbose info about the image.
   */
+  type=GetImageType(image,&image->exception);
+  (void) SetImageType(image,type);
   (void) SignatureImage(image);
   image->total_colors=GetNumberColors(image,(FILE *) NULL,&image->exception);
     (void) fprintf(file,"Image: %s\n",image->filename);
   if (*image->magick_filename != '\0')
     if (LocaleCompare(image->magick_filename,image->filename) != 0)
-      (void) fprintf(file,"  Base filename: %s\n",image->magick_filename);
+      {
+        char
+          filename[MaxTextExtent];
+
+        GetPathComponent(image->magick_filename,TailPath,filename);
+        (void) fprintf(file,"  Base filename: %s\n",filename);
+      }
   magick_info=GetMagickInfo(image->magick,&image->exception);
   if ((magick_info == (const MagickInfo *) NULL) ||
       (*GetMagickDescription(magick_info) == '\0'))
@@ -253,10 +264,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         (image->magick_rows != image->rows))
       (void) fprintf(file,"  Base geometry: %lux%lu\n",image->magick_columns,
         image->magick_rows);
-  (void) fprintf(file,"  Type: %s\n",MagickOptionToMnemonic(
-    MagickImageOptions,(long) GetImageType(image,&image->exception)));
-  (void) fprintf(file,"  Endianess: %s\n",
-    MagickOptionToMnemonic(MagickEndianOptions,(long) image->endian));
+  (void) fprintf(file,"  Type: %s\n",MagickOptionToMnemonic(MagickImageOptions,
+    (long) type));
+  (void) fprintf(file,"  Endianess: %s\n",MagickOptionToMnemonic(
+    MagickEndianOptions,(long) image->endian));
   /*
     Detail channel depth and extrema.
   */
@@ -272,38 +283,38 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
     case RGBColorspace:
     default:
     {
-      (void) fprintf(file,"    Red: %lu-bits\n",
+      (void) fprintf(file,"    Red: %lu-bit\n",
         channel_statistics[RedChannel].depth);
-      (void) fprintf(file,"    Green: %lu-bits\n",
+      (void) fprintf(file,"    Green: %lu-bit\n",
         channel_statistics[GreenChannel].depth);
-      (void) fprintf(file,"    Blue: %lu-bits\n",
+      (void) fprintf(file,"    Blue: %lu-bit\n",
         channel_statistics[BlueChannel].depth);
       if (image->matte != MagickFalse)
-        (void) fprintf(file,"    Alpha: %lu-bits\n",
+        (void) fprintf(file,"    Alpha: %lu-bit\n",
           channel_statistics[OpacityChannel].depth);
       break;
     }
     case CMYKColorspace:
     {
-      (void) fprintf(file,"    Cyan: %lu-bits\n",
+      (void) fprintf(file,"    Cyan: %lu-bit\n",
         channel_statistics[CyanChannel].depth);
-      (void) fprintf(file,"    Magenta: %lu-bits\n",
+      (void) fprintf(file,"    Magenta: %lu-bit\n",
         channel_statistics[MagentaChannel].depth);
-      (void) fprintf(file,"    Yellow: %lu-bits\n",
+      (void) fprintf(file,"    Yellow: %lu-bit\n",
         channel_statistics[YellowChannel].depth);
-      (void) fprintf(file,"    Black: %lu-bits\n",
+      (void) fprintf(file,"    Black: %lu-bit\n",
         channel_statistics[BlackChannel].depth);
       if (image->matte != MagickFalse)
-        (void) fprintf(file,"    Alpha: %lu-bits\n",
+        (void) fprintf(file,"    Alpha: %lu-bit\n",
           channel_statistics[OpacityChannel].depth);
       break;
     }
     case GRAYColorspace:
     {
-      (void) fprintf(file,"    Gray: %lu-bits\n",
+      (void) fprintf(file,"    Gray: %lu-bit\n",
         channel_statistics[GrayChannel].depth);
       if (image->matte != MagickFalse)
-        (void) fprintf(file,"    Alpha: %lu-bits\n",
+        (void) fprintf(file,"    Alpha: %lu-bit\n",
           channel_statistics[OpacityChannel].depth);
       break;
     }
@@ -426,19 +437,22 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
       GetImageTotalInkDensity(image)/(double) QuantumRange);
   x=0;
   p=NewImageList();
-  if (((image->matte != MagickFalse) &&
-      (LocaleCompare(image->magick,"GIF") != 0)) ||
-      (image->taint != MagickFalse))
+  if (image->matte != MagickFalse)
     {
+      IndexPacket
+        *indexes;
+
       register const PixelPacket
         *p;
 
       p=(PixelPacket *) NULL;
+      indexes=(IndexPacket *) NULL;
       for (y=0; y < (long) image->rows; y++)
       {
         p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
+        indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
           if (p->opacity == TransparentOpacity)
@@ -457,64 +471,50 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             pixel;
 
           GetMagickPixelPacket(image,&pixel);
-          pixel.colorspace=RGBColorspace;
-          pixel.red=(MagickRealType) p->red;
-          pixel.green=(MagickRealType) p->green;
-          pixel.blue=(MagickRealType) p->blue;
-          pixel.opacity=(MagickRealType) p->opacity;
-          GetColorTuple(&pixel,MagickFalse,tuple);
-          (void) fprintf(file,"  Alpha: %s\t",tuple);
-          GetColorTuple(&pixel,MagickTrue,tuple);
+          SetMagickPixelPacket(p,indexes+x,&pixel);
+          (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickFalse,
+            tuple,&image->exception);
+          (void) fprintf(file,"  Alpha: %-45.45s  ",tuple);
+          (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickTrue,
+            tuple,&image->exception);
           (void) fprintf(file,"  %s\n",tuple);
         }
     }
-  if (image->storage_class == DirectClass)
-    (void) fprintf(file,"  Colors: %lu\n",image->total_colors);
-  else
-    if (image->total_colors <= image->colors)
-      (void) fprintf(file,"  Colors: %lu\n",image->colors);
-    else
-      (void) fprintf(file,"  Colors: %lu=>%lu\n",image->total_colors,
-        image->colors);
-  if ((image->storage_class == PseudoClass) && (colorspace != GRAYColorspace))
-    {
-      char
-        name[MaxTextExtent];
-
-      register PixelPacket
-        *p;
-
-      /*
-        Display image colormap.
-      */
-      *name='\0';
-      p=image->colormap;
-      for (i=0; i < (long) image->colors; i++)
-      {
-        char
-          tuple[MaxTextExtent];
-
-        MagickPixelPacket
-          pixel;
-
-        GetMagickPixelPacket(image,&pixel);
-        pixel.colorspace=RGBColorspace;
-        pixel.red=(MagickRealType) p->red;
-        pixel.green=(MagickRealType) p->green;
-        pixel.blue=(MagickRealType) p->blue;
-        pixel.opacity=(MagickRealType) p->opacity;
-        GetColorTuple(&pixel,MagickFalse,tuple);
-        (void) fprintf(file,"  %8ld: %s\t",i,tuple);
-        (void) QueryColorname(image,p,SVGCompliance,name,&image->exception);
-        (void) fprintf(file,"%s\n",name);
-        p++;
-      }
-    }
-  if (image->total_colors < 1024)
+  (void) fprintf(file,"  Colors: %lu\n",image->total_colors);
+  if (image->total_colors <= 1024)
     {
       (void) fprintf(file,"  Histogram:\n");
       (void) GetNumberColors(image,file,&image->exception);
     }
+  if (image->storage_class == PseudoClass)
+    {
+      (void) fprintf(file,"  Colormap: %lu\n",image->colors);
+      if (image->colors <= 1024)
+        {
+          char
+            color[MaxTextExtent],
+            hex[MaxTextExtent];
+
+          MagickPixelPacket
+            pixel;
+
+          register PixelPacket
+            *p;
+
+          GetMagickPixelPacket(image,&pixel);
+          p=image->colormap;
+          for (i=0; i < (long) image->colors; i++)
+          {
+            SetMagickPixelPacket(p,(IndexPacket *) NULL,&pixel);
+            (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickFalse,
+              color,&image->exception);
+            (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickTrue,
+              hex,&image->exception);
+            (void) fprintf(file,"  %8ld: %-45.45s %s\n",i,color,hex);
+            p++;
+          }
+        }
+    }
   if (image->error.mean_error_per_pixel != 0.0)
     (void) fprintf(file,"  Mean error per pixel: %g\n",
       image->error.mean_error_per_pixel);
@@ -524,8 +524,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   if (image->error.normalized_maximum_error != 0.0)
     (void) fprintf(file,"  Normalized maximum error: %gn",
       image->error.normalized_maximum_error);
-  (void) fprintf(file,"  Rendering intent: %s\n",
-    MagickOptionToMnemonic(MagickIntentOptions,(long) image->rendering_intent));
+  (void) fprintf(file,"  Rendering intent: %s\n",MagickOptionToMnemonic(
+    MagickIntentOptions,(long) image->rendering_intent));
   if (image->gamma != 0.0)
     (void) fprintf(file,"  Gamma: %g\n",image->gamma);
   if ((image->chromaticity.red_primary.x != 0.0) ||
@@ -556,10 +556,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
       image->y_resolution);
   (void) fprintf(file,"  Units: %s\n",
     MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-  FormatSize(GetBlobSize(image),format);
+  (void) FormatMagickSize(GetBlobSize(image),format);
   (void) fprintf(file,"  Filesize: %s\n",format);
-  (void) fprintf(file,"  Interlace: %s\n",
-    MagickOptionToMnemonic(MagickInterlaceOptions,(long) image->interlace));
+  (void) fprintf(file,"  Interlace: %s\n",MagickOptionToMnemonic(
+    MagickInterlaceOptions,(long) image->interlace));
   (void) QueryColorname(image,&image->background_color,SVGCompliance,color,
     &image->exception);
   (void) fprintf(file,"  Background color: %s\n",color);
@@ -569,12 +569,15 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   (void) QueryColorname(image,&image->matte_color,SVGCompliance,color,
     &image->exception);
   (void) fprintf(file,"  Matte color: %s\n",color);
+  (void) QueryColorname(image,&image->transparent_color,SVGCompliance,color,
+    &image->exception);
+  (void) fprintf(file,"  Transparent color: %s\n",color);
   if ((image->page.width != 0) || (image->page.height != 0) ||
       (image->page.x != 0) || (image->page.y != 0))
     (void) fprintf(file,"  Page geometry: %lux%lu%+ld%+ld\n",image->page.width,
       image->page.height,image->page.x,image->page.y);
-  (void) fprintf(file,"  Dispose: %s\n",
-    MagickOptionToMnemonic(MagickDisposeOptions,(long) image->dispose));
+  (void) fprintf(file,"  Dispose: %s\n",MagickOptionToMnemonic(
+    MagickDisposeOptions,(long) image->dispose));
   if (image->delay != 0)
     (void) fprintf(file,"  Delay: %lux%lu\n",image->delay,
       image->ticks_per_second);
@@ -616,7 +619,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
       /*
         Display visual image directory.
       */
-      image_info=CloneImageInfo((ImageInfo *) NULL);
+      image_info=AcquireImageInfo();
       (void) CloneString(&image_info->size,"64x64");
       (void) fprintf(file,"  Directory:\n");
       for (p=image->directory; *p != '\0'; p++)
@@ -863,8 +866,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
       (void) fprintf(file,"  User time: %0.3fu\n",user_time);
       (void) fprintf(file,"  Elapsed time: %ld:%02ld\n",
       (long) (elapsed_time/60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
-      FormatSize((MagickSizeType) ((double) image->columns*image->rows/
-        elapsed_time),format);
+      (void) FormatMagickSize((MagickSizeType) ((double) image->columns*
+        image->rows/elapsed_time),format);
       (void) fprintf(file,"  Pixels per second: %s\n",format);
     }
   (void) fprintf(file,"  Version: %s\n",
