@@ -54,11 +54,13 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
 
 /*
   Define declarations.
 */
 #define LocaleFilename  "locale.xml"
+#define MaxRecursionDepth  200
 
 /*
   Static declarations.
@@ -108,7 +110,6 @@ static MagickBooleanType
 %
 %      DestroyLocaleList(void)
 %
-%
 */
 MagickExport void DestroyLocaleList(void)
 {
@@ -142,7 +143,6 @@ MagickExport void DestroyLocaleList(void)
 %
 %    o image: The image.
 %
-%
 */
 
 static void *DestroyOptions(void *message)
@@ -182,14 +182,10 @@ MagickExport LinkedListInfo *DestroyLocaleOptions(LinkedListInfo *messages)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
   ExceptionInfo *exception)
 {
-  register const LocaleInfo
-    *p;
-
   assert(exception != (ExceptionInfo *) NULL);
   if ((locale_list == (SplayTreeInfo *) NULL) ||
       (instantiate_locale == MagickFalse))
@@ -203,20 +199,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
       ResetSplayTreeIterator(locale_list);
       return((const LocaleInfo *) GetNextValueInSplayTree(locale_list));
     }
-  /*
-    Search for named tag.
-  */
-  AcquireSemaphoreInfo(&locale_semaphore);
-  ResetSplayTreeIterator(locale_list);
-  p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
-  while (p != (const LocaleInfo *) NULL)
-  {
-    if (LocaleCompare(tag,p->tag) == 0)
-      break;
-    p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
-  }
-  RelinquishSemaphoreInfo(locale_semaphore);
-  return(p);
+  return((const LocaleInfo *) GetValueFromSplayTree(locale_list,tag));
 }
 
 /*
@@ -431,7 +414,6 @@ MagickExport char **GetLocaleList(const char *pattern,
 %
 %    o tag: Return a message that matches this tag in the current locale.
 %
-%
 */
 MagickExport const char *GetLocaleMessage(const char *tag)
 {
@@ -480,7 +462,6 @@ MagickExport const char *GetLocaleMessage(const char *tag)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
   ExceptionInfo *exception)
@@ -628,7 +609,7 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
         }
       RelinquishSemaphoreInfo(locale_semaphore);
     }
-  return((MagickBooleanType) (locale_list != (SplayTreeInfo *) NULL));
+  return(locale_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -654,7 +635,6 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
   ExceptionInfo *exception)
@@ -732,32 +712,8 @@ MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
-static void ChopLocaleComponents(char *path,const unsigned long components)
-{
-  long
-    count;
-
-  register char
-    *p;
-
-  if (*path == '\0')
-    return;
-  p=path+strlen(path)-1;
-  if (*p == '/')
-    *p='\0';
-  for (count=0; (count < (long) components) && (p > path); p--)
-    if (*p == '/')
-      {
-        *p='\0';
-        count++;
-      }
-  if (count < (long) components)
-    *path='\0';
-}
-
 static void *DestroyLocaleNode(void *locale_info)
 {
   register LocaleInfo
@@ -773,28 +729,106 @@ static void *DestroyLocaleNode(void *locale_info)
   return(RelinquishMagickMemory(p));
 }
 
+static MagickBooleanType TraverseLocaleMap(const char *filename,
+  XMLTreeInfo **nodes,XMLTreeInfo *node,const unsigned long depth,
+  ExceptionInfo *exception)
+{
+  MagickBooleanType
+    status;
+
+  status=MagickTrue;
+  if (depth >= MaxRecursionDepth)
+    (void) ThrowMagickException(exception,GetMagickModule(),
+      ConfigureError,"IncludeElementNestedTooDeeply","`%s'",filename);
+  else
+    if (node != (XMLTreeInfo *) NULL)
+      {
+        XMLTreeInfo
+          *child,
+          *sibling;
+
+        nodes[depth]=node;
+        if (strcmp(GetXMLTreeTag(node),"message") == 0)
+          {
+            char
+              message[MaxTextExtent],
+              tag[MaxTextExtent];
+
+            const char
+              *attribute,
+              *content;
+
+            LocaleInfo
+              *locale_info;
+
+            register long
+              i;
+
+            *tag='\0';
+            for (i=1; i < (long) depth; i++)
+            {
+              (void) ConcatenateMagickString(tag,GetXMLTreeTag(nodes[i]),
+                MaxTextExtent);
+              (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+            }
+            attribute=GetXMLTreeAttribute(node,"name");
+            if (attribute != (const char *) NULL)
+              {
+                (void) ConcatenateMagickString(tag,attribute,MaxTextExtent);
+                (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+              }
+            locale_info=(LocaleInfo *) AcquireMagickMemory(
+              sizeof(*locale_info));
+            if (locale_info == (LocaleInfo *) NULL)
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",filename);
+            (void) ResetMagickMemory(locale_info,0,sizeof(*locale_info));
+            locale_info->path=ConstantString(filename);
+            locale_info->signature=MagickSignature;
+            locale_info->tag=ConstantString(tag);
+            *message='\0';
+            content=GetXMLTreeContent(node);
+            if (content != (const char *) NULL)
+              {
+                (void) CopyMagickString(message,content,MaxTextExtent);
+                StripString(message);
+              }
+            locale_info->message=ConstantString(message);
+            status=AddValueToSplayTree(locale_list,ConstantString(
+              locale_info->tag),locale_info);
+            if (status == MagickFalse)
+              (void) ThrowMagickException(exception,GetMagickModule(),
+                ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
+          }
+        child=GetXMLTreeChild(node,(const char *) NULL);
+        if (child != (XMLTreeInfo *) NULL)
+          status=TraverseLocaleMap(filename,nodes,child,depth+1,exception);
+        sibling=GetXMLTreeOrdered(node);
+        if (sibling != (XMLTreeInfo *) NULL)
+          status=TraverseLocaleMap(filename,nodes,sibling,depth,exception);
+      }
+  return(status);
+}
+
 static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
   const char *locale,const unsigned long depth,ExceptionInfo *exception)
 {
-  char
-    keyword[MaxTextExtent],
-    message[MaxTextExtent],
-    *q,
-    tag[MaxTextExtent],
-    *token;
+  const char
+    *attribute;
 
-  LocaleInfo
-    *locale_info = (LocaleInfo *) NULL;
-
-  MagickStatusType
+  MagickBooleanType
     status;
 
-  register char
-    *p;
+  XMLTreeInfo
+    *include,
+    **nodes,
+    *locale_map;
 
   /*
-    Read the locale configure file.
+    Load the locale map file.
   */
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+    "Loading locale map \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (locale_list == (SplayTreeInfo *) NULL)
@@ -808,193 +842,58 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
-  status=MagickTrue;
-  *tag='\0';
-  *message='\0';
-  *keyword='\0';
-  token=AcquireString((char *) xml);
-  for (q=(char *) xml; *q != '\0'; )
+  locale_map=NewXMLTree(xml,exception);
+  if (locale_map == (XMLTreeInfo *) NULL)
+    return(MagickFalse);
+  include=GetXMLTreeChild(locale_map,"include");
+  while (include != (XMLTreeInfo *) NULL)
   {
     /*
-      Interpret XML.
+      Process include element.
     */
-    GetMagickToken(q,&q,token);
-    if (*token == '\0')
-      break;
-    (void) CopyMagickString(keyword,token,MaxTextExtent);
-    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+    attribute=GetXMLTreeAttribute(include,"locale");
+    if ((attribute != (const char *) NULL) &&
+        (LocaleCompare(locale,attribute) != 0))
       {
-        /*
-          Doctype element.
-        */
-        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-        {
-          GetMagickToken(q,&q,token);
-          while (isspace((int) ((unsigned char) *q)) != 0)
-            q++;
-        }
+        include=GetNextXMLTreeTag(include);
         continue;
       }
-    if (LocaleNCompare(keyword,"<!--",4) == 0)
+    attribute=GetXMLTreeAttribute(include,"file");
+    if (attribute != (const char *) NULL)
       {
-        /*
-          Comment element.
-        */
-        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-        {
-          GetMagickToken(q,&q,token);
-          while (isspace((int) ((unsigned char) *q)) != 0)
-            q++;
-        }
-        continue;
-      }
-    if (LocaleCompare(keyword,"<include") == 0)
-      {
-        /*
-          Include element.
-        */
-        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
-        {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
-          if (*token != '=')
-            continue;
-          GetMagickToken(q,&q,token);
-          if (LocaleCompare(keyword,"locale") == 0)
-            {
-              if (LocaleCompare(locale,token) != 0)
-                break;
-              continue;
-            }
-          if (LocaleCompare(keyword,"file") == 0)
-            {
-              if (depth > 200)
-                (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
-              else
-                {
-                  char
-                    path[MaxTextExtent],
-                    *xml;
-
-                  *path='\0';
-                  GetPathComponent(filename,HeadPath,path);
-                  if (*path != '\0')
-                    (void) ConcatenateMagickString(path,DirectorySeparator,
-                      MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path,~0,exception);
-                  status|=LoadLocaleList(xml,path,locale,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
-                }
-            }
-        }
-        continue;
-      }
-    if (LocaleCompare(keyword,"<locale") == 0)
-      {
-        /*
-          Locale element.
-        */
-        while ((*token != '>') && (*q != '\0'))
-        {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
-          if (*token != '=')
-            continue;
-          GetMagickToken(q,&q,token);
-        }
-        continue;
-      }
-    if (LocaleCompare(keyword,"</locale>") == 0)
-      {
-        ChopLocaleComponents(tag,1);
-        (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
-        continue;
-      }
-    if (LocaleCompare(keyword,"<localemap>") == 0)
-      continue;
-    if (LocaleCompare(keyword,"</localemap>") == 0)
-      continue;
-    if (LocaleCompare(keyword,"<message") == 0)
-      {
-        /*
-          Message element.
-        */
-        while ((*token != '>') && (*q != '\0'))
-        {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
-          if (*token != '=')
-            continue;
-          GetMagickToken(q,&q,token);
-          if (LocaleCompare(keyword,"name") == 0)
-            {
-              (void) ConcatenateMagickString(tag,token,MaxTextExtent);
-              (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
-            }
-        }
-        for (p=q; (*q != '<') && (*q != '\0'); q++);
-        while (isspace((int) ((unsigned char) *p)) != 0)
-          p++;
-        q--;
-        while ((isspace((int) ((unsigned char) *q)) != 0) && (q > p))
-          q--;
-        (void) CopyMagickString(message,p,(size_t) (q-p+2));
-        locale_info=(LocaleInfo *) AcquireMagickMemory(sizeof(*locale_info));
-        if (locale_info == (LocaleInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
-        (void) ResetMagickMemory(locale_info,0,sizeof(*locale_info));
-        locale_info->path=ConstantString(filename);
-        locale_info->tag=ConstantString(tag);
-        locale_info->message=ConstantString(message);
-        locale_info->signature=MagickSignature;
-        status=AddValueToSplayTree(locale_list,ConstantString(locale_info->tag),
-          locale_info);
-        if (status == MagickFalse)
+        if (depth > MaxRecursionDepth)
           (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed","`%s'",
-            locale_info->tag);
-        (void) ConcatenateMagickString(tag,message,MaxTextExtent);
-        (void) ConcatenateMagickString(tag,"\n",MaxTextExtent);
-        q++;
-        continue;
-      }
-    if (LocaleCompare(keyword,"</message>") == 0)
-      {
-        ChopLocaleComponents(tag,2);
-        (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
-        continue;
-      }
-    if (*keyword == '<')
-      {
-        /*
-          Subpath element.
-        */
-        if (*(keyword+1) == '?')
-          continue;
-        if (*(keyword+1) == '/')
+            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",filename);
+        else
           {
-            ChopLocaleComponents(tag,1);
-            if (*tag != '\0')
-              (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
-            continue;
+            char
+              path[MaxTextExtent],
+              *xml;
+
+            GetPathComponent(filename,HeadPath,path);
+            if (*path != '\0')
+              (void) ConcatenateMagickString(path,DirectorySeparator,
+                MaxTextExtent);
+            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
+            xml=FileToString(path,~0,exception);
+            status=LoadLocaleList(xml,path,locale,depth+1,exception);
+            xml=(char *) RelinquishMagickMemory(xml);
           }
-        token[strlen(token)-1]='\0';
-        (void) CopyMagickString(token,token+1,MaxTextExtent);
-        (void) ConcatenateMagickString(tag,token,MaxTextExtent);
-        (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
-        continue;
       }
-    GetMagickToken(q,(char **) NULL,token);
-    if (*token != '=')
-      continue;
+    include=GetNextXMLTreeTag(include);
   }
-  token=(char *) RelinquishMagickMemory(token);
-  if (locale_list == (SplayTreeInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  nodes=(XMLTreeInfo **) AcquireMagickMemory(MaxRecursionDepth*sizeof(*nodes));
+  if (nodes == (XMLTreeInfo **) NULL)
+    {
+      locale_map=DestroyXMLTree(locale_map);
+      ThrowFileException(exception,ResourceLimitError,
+        "MemoryAllocationFailed",filename);
+      return(MagickFalse);
+    }
+  status=TraverseLocaleMap(filename,nodes,locale_map,0,exception);
+  nodes=(XMLTreeInfo **) RelinquishMagickMemory(nodes);
+  locale_map=DestroyXMLTree(locale_map);
+  return(status);
 }
 
 /*
@@ -1024,7 +923,6 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadLocaleLists(const char *filename,
   const char *locale,ExceptionInfo *exception)
