@@ -55,6 +55,7 @@
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/version.h"
+#include "magick/xml-tree.h"
 #if defined(HAVE_PTHREAD)
 #include <pthread.h>
 #endif
@@ -121,23 +122,16 @@ struct _LogInfo
   FILE
     *file;
 
-  MagickBooleanType
-    append;
-
   unsigned long
-    generation,
-    count;
+    generation;
 
   MagickBooleanType
+    append,
     stealth;
 
   TimerInfo
     timer;
 
-  struct _LogInfo
-    *previous,
-    *next;  /* deprecated, use GetLogInfoList() */
-
   unsigned long
     signature;
 };
@@ -165,8 +159,6 @@ static const char
     "  <log events=\"None\" />"
     "  <log output=\"console\" />"
     "  <log filename=\"Magick-%d.log\" />"
-    "  <log generations=\"3\" />"
-    "  <log limit=\"2000\" />"
     "  <log format=\"%t %r %u %v %d %c[%p]: %m/%f/%l/%d\n  %e\" />"
     "</logmap>";
 
@@ -200,6 +192,48 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C l o s e M a g i c k L o g                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  CloseMagickLog() closes the Magick log.
+%
+%  The format of the CloseMagickLog method is:
+%
+%      CloseMagickLog(void)
+%
+*/
+MagickExport void CloseMagickLog(void)
+{
+  ExceptionInfo
+    *exception;
+
+  LogInfo
+    *log_info;
+
+  if (IsEventLogging() == MagickFalse)
+    return;
+  exception=AcquireExceptionInfo();
+  log_info=(LogInfo *) GetLogInfo("*",exception);
+  exception=DestroyExceptionInfo(exception);
+  AcquireSemaphoreInfo(&log_semaphore);
+  if (log_info->file != (FILE *) NULL)
+    {
+      if (log_info->append == MagickFalse)
+        (void) fprintf(log_info->file,"</log>\n");
+      (void) fclose(log_info->file);
+      log_info->file=(FILE *) NULL;
+    }
+  RelinquishSemaphoreInfo(log_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   D e s t r o y L o g L i s t                                               %
 %                                                                             %
 %                                                                             %
@@ -212,7 +246,6 @@ static MagickBooleanType
 %
 %      DestroyLogList(void)
 %
-%
 */
 
 static void *DestroyLogElement(void *log_info)
@@ -226,6 +259,7 @@ static void *DestroyLogElement(void *log_info)
       if (p->append == MagickFalse)
         (void) fprintf(p->file,"</log>\n");
       (void) fclose(p->file);
+      p->file=(FILE *) NULL;
     }
   if (p->filename != (char *) NULL)
     p->filename=(char *) RelinquishMagickMemory(p->filename);
@@ -271,7 +305,6 @@ MagickExport void DestroyLogList(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const LogInfo *GetLogInfo(const char *name,
   ExceptionInfo *exception)
@@ -573,7 +606,6 @@ static MagickBooleanType InitializeLogList(ExceptionInfo *exception)
 %
 %      MagickBooleanType IsEventLogging(void)
 %
-%
 */
 MagickExport MagickBooleanType IsEventLogging(void)
 {
@@ -589,7 +621,7 @@ MagickExport MagickBooleanType IsEventLogging(void)
   GetExceptionInfo(&exception);
   log_info=GetLogInfo("*",&exception);
   (void) DestroyExceptionInfo(&exception);
-  return((MagickBooleanType) (log_info->event_mask != NoEvents));
+  return(log_info->event_mask != NoEvents ? MagickTrue : MagickFalse);
 }
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -614,10 +646,14 @@ MagickExport MagickBooleanType IsEventLogging(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 {
+#define MegabytesToBytes(value) ((MagickSizeType) (value)*1024*1024)
+
+  char
+    limit[MaxTextExtent];
+
   const char
     *path;
 
@@ -649,7 +685,7 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
       {
         if (log_info[i]->path != (char *) NULL)
           (void) fprintf(file,"\nPath: %s\n\n",log_info[i]->path);
-        (void) fprintf(file,"Filename         Generations  Limit  Format\n");
+        (void) fprintf(file,"Filename       Generations     Limit  Format\n");
         (void) fprintf(file,"-------------------------------------------------"
           "------------------------------\n");
       }
@@ -660,8 +696,9 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
         for (j=(long) strlen(log_info[i]->filename); j <= 16; j++)
           (void) fprintf(file," ");
       }
-    (void) fprintf(file,"%11lu",log_info[i]->generations);
-    (void) fprintf(file,"%7lu  ",log_info[i]->limit);
+    (void) fprintf(file,"%9lu  ",log_info[i]->generations);
+    FormatMagickSize(MegabytesToBytes(log_info[i]->limit),limit);
+    (void) fprintf(file,"%8s  ",limit);
     if (log_info[i]->format != (char *) NULL)
       (void) fprintf(file,"%s",log_info[i]->format);
     (void) fprintf(file,"\n");
@@ -703,7 +740,6 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 %
 %    o format: The output format.
 %
-%
 */
 
 static unsigned long GetLogThreadId(void)
@@ -717,9 +753,9 @@ static unsigned long GetLogThreadId(void)
   return(0);
 }
 
-static char *TranslateEvent(const LogEventType type,const char *module,
-  const char *function,const unsigned long line,const char *domain,
-  const char *event)
+static char *TranslateEvent(const LogEventType magick_unused(type),
+  const char *module,const char *function,const unsigned long line,
+  const char *domain,const char *event)
 {
   char
     *text;
@@ -809,8 +845,7 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         %d   domain
         %e   event
         %f   function
-				%g   generation
-        %i   thread id
+        %g   generation
         %l   line
         %m   module
         %n   log name
@@ -869,24 +904,18 @@ static char *TranslateEvent(const LogEventType type,const char *module,
       }
       case 'g':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          log_info->generation % log_info->generations);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'i':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",GetLogThreadId());
+        if (log_info->generations == 0)
+          (void) CopyMagickString(q,"0",MaxTextExtent);
+        else
+          (void) FormatMagickString(q,MaxTextExtent,"%lu",
+            log_info->generation % log_info->generations);
         while (*q != '\0')
           q++;
         break;
       }
       case 'l':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",line);
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,MaxTextExtent,"%lu",line);
         break;
       }
       case 'm':
@@ -912,32 +941,23 @@ static char *TranslateEvent(const LogEventType type,const char *module,
       }
       case 'p':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%ld",(long) getpid());
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,MaxTextExtent,"%ld",(long) getpid());
         break;
       }
       case 'r':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%ld:%02ld",
+        q+=FormatMagickString(q,MaxTextExtent,"%ld:%02ld",
           (long) (elapsed_time/60.0),(long) ceil(fmod(elapsed_time,60.0)));
-        while (*q != '\0')
-          q++;
         break;
       }
       case 't':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%02d:%02d:%02d",
-          time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickTime(seconds,MaxTextExtent,q);
         break;
       }
       case 'u':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%0.3fu",user_time);
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,MaxTextExtent,"%0.3fu",user_time);
         break;
       }
       case 'v':
@@ -946,8 +966,14 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         q+=strlen(MagickLibVersionText);
         break;
       }
+      case '%':
+      {
+        *q++=(*p);
+        break;
+      }
       default:
       {
+        *q++='%';
         *q++=(*p);
         break;
       }
@@ -997,7 +1023,6 @@ static char *TranslateFilename(const LogInfo *log_info)
 
         %c   client name
         %n   log name
-        %i   thread id
         %p   process id
         %v   version
         %%   percent sign
@@ -1018,15 +1043,11 @@ static char *TranslateFilename(const LogInfo *log_info)
       }
       case 'g':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          log_info->generation % log_info->generations);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'i':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",GetLogThreadId());
+        if (log_info->generations == 0)
+          (void) CopyMagickString(q,"0",MaxTextExtent);
+        else
+          (void) FormatMagickString(q,MaxTextExtent,"%lu",
+            log_info->generation % log_info->generations);
         while (*q != '\0')
           q++;
         break;
@@ -1039,9 +1060,7 @@ static char *TranslateFilename(const LogInfo *log_info)
       }
       case 'p':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%ld",(long) getpid());
-        while (*q != '\0')
-          q++;
+        q+=FormatMagickString(q,MaxTextExtent,"%ld",(long) getpid());
         break;
       }
       case 'v':
@@ -1050,8 +1069,14 @@ static char *TranslateFilename(const LogInfo *log_info)
         q+=strlen(MagickLibVersionText);
         break;
       }
+      case '%':
+      {
+        *q++=(*p);
+        break;
+      }
       default:
       {
+        *q++='%';
         *q++=(*p);
         break;
       }
@@ -1126,13 +1151,17 @@ MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
     }
   if ((log_info->handler_mask & FileHandler) != 0)
     {
-      log_info->count++;
-      if (log_info->count == log_info->limit)
+      struct stat
+        file_info;
+
+      file_info.st_size=0;
+      if (log_info->file != (FILE *) NULL)
+        (void) fstat(fileno(log_info->file),&file_info);
+      if (file_info.st_size > (off_t) (1024*1024*log_info->limit))
         {
           (void) fprintf(log_info->file,"</log>\n");
           (void) fclose(log_info->file);
           log_info->file=(FILE *) NULL;
-          log_info->count=0;
         }
       if (log_info->file == (FILE *) NULL)
         {
@@ -1225,7 +1254,6 @@ MagickBooleanType LogMagickEvent(const LogEventType type,const char *module,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadLogList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
@@ -1390,6 +1418,11 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"generations") == 0)
           {
+            if (LocaleCompare(token,"unlimited") == 0)
+              {
+                log_info->generations=(~0UL);
+                break;
+              }
             log_info->generations=(unsigned long) atol(token);
             break;
           }
@@ -1400,6 +1433,11 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"limit") == 0)
           {
+            if (LocaleCompare(token,"unlimited") == 0)
+              {
+                log_info->limit=(~0UL);
+                break;
+              }
             log_info->limit=(unsigned long) atol(token);
             break;
           }
@@ -1451,7 +1489,6 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadLogLists(const char *filename,
   ExceptionInfo *exception)
@@ -1506,7 +1543,6 @@ static MagickBooleanType LoadLogLists(const char *filename,
 %
 %    o handlers: one or more handlers separated by commas.
 %
-%
 */
 static LogHandlerType ParseLogHandlers(const char *handlers)
 {
@@ -1566,7 +1602,6 @@ static LogHandlerType ParseLogHandlers(const char *handlers)
 %
 %    o events: log these events.
 %
-%
 */
 MagickExport LogEventType SetLogEventMask(const char *events)
 {
@@ -1601,7 +1636,6 @@ MagickExport LogEventType SetLogEventMask(const char *events)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  SetLogFormat() sets the format for the "human readable" log record.
@@ -1614,7 +1648,6 @@ MagickExport LogEventType SetLogEventMask(const char *events)
 %
 %    o format: The log record format.
 %
-%
 */
 MagickExport void SetLogFormat(const char *format)
 {
