@@ -16,7 +16,7 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
 
 /*
   Define declarations.
@@ -61,7 +62,7 @@
   Static declarations.
 */
 static const char
-  *MagicMap = (char *)
+  *MagicMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<magicmap>"
     "  <magic name=\"AVI\" offset=\"0\" target=\"RIFF\" />"
@@ -80,6 +81,8 @@ static const char
     "  <magic name=\"DCX\" offset=\"0\" target=\"\\261\\150\\336\\72\" />"
     "  <magic name=\"DIB\" offset=\"0\" target=\"\\050\\000\" />"
     "  <magic name=\"DOT\" offset=\"0\" target=\"digraph\" />"
+    "  <magic name=\"DPX\" offset=\"0\" target=\"SDPX\" />"
+    "  <magic name=\"DPX\" offset=\"0\" target=\"XPDS\" />"
     "  <magic name=\"EMF\" offset=\"40\" target=\"\\040\\105\\115\\106\\000\\000\\001\\000\" />"
     "  <magic name=\"EPT\" offset=\"0\" target=\"\\305\\320\\323\\306\" />"
     "  <magic name=\"FAX\" offset=\"0\" target=\"DFAX\" />"
@@ -187,7 +190,6 @@ static MagickBooleanType
 %
 %      DestroyMagicList(void)
 %
-%
 */
 
 static void *DestroyMagicElement(void *magic_info)
@@ -210,7 +212,7 @@ static void *DestroyMagicElement(void *magic_info)
 
 MagickExport void DestroyMagicList(void)
 {
-  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
+  AcquireSemaphoreInfo(&magic_semaphore);
   if (magic_list != (LinkedListInfo *) NULL)
     magic_list=DestroyLinkedList(magic_list,DestroyMagicElement);
   instantiate_magic=MagickFalse;
@@ -246,7 +248,6 @@ MagickExport void DestroyMagicList(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   const size_t length,ExceptionInfo *exception)
@@ -269,7 +270,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   /*
     Search for requested magic.
   */
-  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
+  AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   while (p != (const MagicInfo *) NULL)
@@ -307,7 +308,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
-%    o number_aliases:  This integer returns the number of magics in the
+%    o number_aliases:  This integer returns the number of aliases in the
 %      list.
 %
 %    o exception: Return any errors or warnings in this structure.
@@ -354,7 +355,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_aliases != (unsigned long *) NULL);
   *number_aliases=0;
-  p=GetMagicInfo((unsigned char *) "*",0,exception);
+  p=GetMagicInfo((const unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((const MagicInfo **) NULL);
   aliases=(const MagicInfo **) AcquireMagickMemory((size_t)
@@ -364,7 +365,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   /*
     Generate magic list.
   */
-  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
+  AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
@@ -417,12 +418,12 @@ extern "C" {
 
 static int MagicCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     *p,
     *q;
 
-  p=(char *) x;
-  q=(char *) y;
+  p=(const char *) x;
+  q=(const char *) y;
   return(LocaleCompare(p,q));
 }
 
@@ -449,21 +450,21 @@ MagickExport char **GetMagicList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_aliases != (unsigned long *) NULL);
   *number_aliases=0;
-  p=GetMagicInfo((unsigned char *) "*",0,exception);
+  p=GetMagicInfo((const unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((char **) NULL);
   aliases=(char **) AcquireMagickMemory((size_t)
     (GetNumberOfElementsInLinkedList(magic_list)+1)*sizeof(*aliases));
   if (aliases == (char **) NULL)
     return((char **) NULL);
-  magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
+  AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
   p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   for (i=0; p != (const MagicInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      aliases[i++]=ConstantString(AcquireString(p->name));
+      aliases[i++]=ConstantString(p->name);
     p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
   RelinquishSemaphoreInfo(magic_semaphore);
@@ -530,7 +531,7 @@ static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
   if ((magic_list == (LinkedListInfo *) NULL) &&
       (instantiate_magic == MagickFalse))
     {
-      magic_semaphore=AcquireSemaphoreInfo(magic_semaphore);
+      AcquireSemaphoreInfo(&magic_semaphore);
       if ((magic_list == (LinkedListInfo *) NULL) &&
           (instantiate_magic == MagickFalse))
         {
@@ -565,7 +566,6 @@ static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListMagicInfo(FILE *file,
   ExceptionInfo *exception)
@@ -648,28 +648,30 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
-  char
-    keyword[MaxTextExtent],
-    *q,
-    *token;
+  const char
+    *attribute;
 
   MagicInfo
     *magic_info = (MagicInfo *) NULL;
 
-  MagickStatusType
+  MagickBooleanType
     status;
 
+  XMLTreeInfo
+    *magic,
+    *magic_map,
+    *include;
+
   /*
     Load the magic map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading magic file \"%s\" ...",filename);
-  if (xml == (char *) NULL)
+    "Loading magic map \"%s\" ...",filename);
+  if (xml == (const char *) NULL)
     return(MagickFalse);
   if (magic_list == (LinkedListInfo *) NULL)
     {
@@ -681,186 +683,121 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
+  magic_map=NewXMLTree(xml,exception);
+  if (magic_map == (XMLTreeInfo *) NULL)
+    return(MagickFalse);
   status=MagickTrue;
-  token=AcquireString((char *) xml);
-  for (q=(char *) xml; *q != '\0'; )
+  include=GetXMLTreeChild(magic_map,"include");
+  while (include != (XMLTreeInfo *) NULL)
   {
     /*
-      Interpret XML.
+      Process include element.
     */
-    GetMagickToken(q,&q,token);
-    if (*token == '\0')
-      break;
-    (void) CopyMagickString(keyword,token,MaxTextExtent);
-    if (LocaleNCompare(keyword,"<!--",4) == 0)
+    attribute=GetXMLTreeAttribute(include,"file");
+    if (attribute != (const char *) NULL)
       {
-        /*
-          Comment element.
-        */
-        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
-        continue;
+        if (depth > 200)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",filename);
+        else
+          {
+            char
+              path[MaxTextExtent],
+              *xml;
+
+            GetPathComponent(filename,HeadPath,path);
+            if (*path != '\0')
+              (void) ConcatenateMagickString(path,DirectorySeparator,
+                MaxTextExtent);
+            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
+            xml=FileToString(path,~0,exception);
+            status=LoadMagicList(xml,path,depth+1,exception);
+            xml=(char *) RelinquishMagickMemory(xml);
+          }
       }
-    if (LocaleCompare(keyword,"<include") == 0)
+    include=GetNextXMLTreeTag(include);
+  }
+  magic=GetXMLTreeChild(magic_map,"magic");
+  while (magic != (XMLTreeInfo *) NULL)
+  {
+    const char
+      *attribute;
+
+    /*
+      Process magic element.
+    */
+    magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(*magic_info));
+    if (magic_info == (MagicInfo *) NULL)
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",filename);
+    (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
+    magic_info->path=ConstantString(filename);
+    magic_info->signature=MagickSignature;
+    attribute=GetXMLTreeAttribute(magic,"name");
+    if (attribute != (const char *) NULL)
+      magic_info->name=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(magic,"offset");
+    if (attribute != (const char *) NULL)
+      magic_info->offset=(MagickOffsetType) atol(attribute);
+    attribute=GetXMLTreeAttribute(magic,"stealth");
+    if (attribute != (const char *) NULL)
+      magic_info->stealth=IsMagickTrue(attribute);
+    attribute=GetXMLTreeAttribute(magic,"target");
+    if (attribute != (const char *) NULL)
       {
-        /*
-          Include element.
-        */
-        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
+        const char
+          *p;
+
+        register unsigned char
+          *q;
+
+        magic_info->target=ConstantString(attribute);
+        magic_info->magic=(unsigned char *) ConstantString(attribute);
+        q=magic_info->magic;
+        for (p=magic_info->target; *p != '\0'; )
         {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
-          if (*token != '=')
-            continue;
-          GetMagickToken(q,&q,token);
-          if (LocaleCompare(keyword,"file") == 0)
+          if (*p == '\\')
             {
-              if (depth > 200)
-                (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
-              else
+              p++;
+              if (isdigit((int) ((unsigned char) *p)) != 0)
                 {
                   char
-                    path[MaxTextExtent],
-                    *xml;
+                    *end;
 
-                  GetPathComponent(filename,HeadPath,path);
-                  if (*path != '\0')
-                    (void) ConcatenateMagickString(path,DirectorySeparator,
-                      MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
-                  status|=LoadMagicList(xml,path,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
-                }
-            }
-        }
-        continue;
-      }
-    if (LocaleCompare(keyword,"<magic") == 0)
-      {
-       /*
-          Allocate memory for the magic list.
-        */
-        magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(*magic_info));
-        if (magic_info == (MagicInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
-        (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
-        magic_info->path=ConstantString(AcquireString(filename));
-        magic_info->signature=MagickSignature;
-        continue;
-      }
-    if (magic_info == (MagicInfo *) NULL)
-      continue;
-    if (LocaleCompare(keyword,"/>") == 0)
-      {
-        status=AppendValueToLinkedList(magic_list,magic_info);
-        if (status == MagickFalse)
-          (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed","`%s'",
-            magic_info->name);
-        magic_info=(MagicInfo *) NULL;
-      }
-    GetMagickToken(q,(char **) NULL,token);
-    if (*token != '=')
-      continue;
-    GetMagickToken(q,&q,token);
-    GetMagickToken(q,&q,token);
-    switch (*keyword)
-    {
-      case 'N':
-      case 'n':
-      {
-        if (LocaleCompare((char *) keyword,"name") == 0)
-          {
-            magic_info->name=ConstantString(AcquireString(token));
-            break;
-          }
-        break;
-      }
-      case 'O':
-      case 'o':
-      {
-        if (LocaleCompare((char *) keyword,"offset") == 0)
-          {
-            magic_info->offset=(MagickOffsetType) atol(token);
-            break;
-          }
-        break;
-      }
-      case 'S':
-      case 's':
-      {
-        if (LocaleCompare((char *) keyword,"stealth") == 0)
-          {
-            magic_info->stealth=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
-            break;
-          }
-        break;
-      }
-      case 'T':
-      case 't':
-      {
-        if (LocaleCompare((char *) keyword,"target") == 0)
-          {
-            const char
-              *p;
-
-            register unsigned char
-              *q;
-
-            magic_info->target=AcquireString(token);
-            magic_info->magic=(unsigned char *) AcquireString(token);
-            q=magic_info->magic;
-            for (p=magic_info->target; *p != '\0'; )
-            {
-              if (*p == '\\')
-                {
-                  p++;
-                  if (isdigit((int) ((unsigned char) *p)) != 0)
-                    {
-                      char
-                        *end;
-
-                      *q++=(unsigned char) strtol(p,&end,8);
-                      p+=(end-p);
-                      magic_info->length++;
-                      continue;
-                    }
-                  switch (*p)
-                  {
-                    case 'b': *q='\b'; break;
-                    case 'f': *q='\f'; break;
-                    case 'n': *q='\n'; break;
-                    case 'r': *q='\r'; break;
-                    case 't': *q='\t'; break;
-                    case 'v': *q='\v'; break;
-                    case 'a': *q='a'; break;
-                    case '?': *q='\?'; break;
-                    default: *q=(unsigned char) (*p); break;
-                  }
-                  p++;
-                  q++;
+                  *q++=(unsigned char) strtol(p,&end,8);
+                  p+=(end-p);
                   magic_info->length++;
                   continue;
                 }
-              *q++=(unsigned char) (*p++);
+              switch (*p)
+              {
+                case 'b': *q='\b'; break;
+                case 'f': *q='\f'; break;
+                case 'n': *q='\n'; break;
+                case 'r': *q='\r'; break;
+                case 't': *q='\t'; break;
+                case 'v': *q='\v'; break;
+                case 'a': *q='a'; break;
+                case '?': *q='\?'; break;
+                default: *q=(unsigned char) (*p); break;
+              }
+              p++;
+              q++;
               magic_info->length++;
+              continue;
             }
-            break;
-          }
-        break;
+          *q++=(unsigned char) (*p++);
+          magic_info->length++;
+        }
       }
-      default:
-        break;
-    }
+    status=AppendValueToLinkedList(magic_list,magic_info);
+    if (status == MagickFalse)
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
+    magic=GetNextXMLTreeTag(magic);
   }
-  token=(char *) RelinquishMagickMemory(token);
-  if (magic_list == (LinkedListInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  magic_map=DestroyXMLTree(magic_map);
+  return(status);
 }
 
 /*
@@ -888,7 +825,6 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadMagicLists(const char *filename,
   ExceptionInfo *exception)
