@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,6 +41,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/compare.h"
@@ -50,10 +51,14 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/fx.h"
+#include "magick/gem.h"
+#include "magick/geometry.h"
+#include "magick/layer.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/montage.h"
 #include "magick/option.h"
+#include "magick/quantum.h"
 #include "magick/resource_.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
@@ -91,14 +96,17 @@ static const OptionInfo
     { "Black", (long) BlackChannel },
     { "Blue", (long) BlueChannel },
     { "Cyan", (long) CyanChannel },
-    { "Default", (long) AllChannels &~ (long) OpacityChannel },
+    { "Default", (long) DefaultChannels },
     { "Gray", (long) GrayChannel },
     { "Green", (long) GreenChannel },
+    { "Hue", (long) RedChannel },
     { "Index", (long) IndexChannel },
+    { "Luminosity", (long) BlueChannel },
     { "Magenta", (long) MagentaChannel },
     { "Matte", (long) OpacityChannel },
     { "Opacity", (long) OpacityChannel },
     { "Red", (long) RedChannel },
+    { "Saturation", (long) GreenChannel },
     { "Yellow", (long) YellowChannel },
     { (char *) NULL, (long) UndefinedChannel }
   },
@@ -119,186 +127,455 @@ static const OptionInfo
   },
   CommandOptions[] =
   {
+    { "+adjoin", 0L },
     { "-adjoin", 0L },
+    { "-adaptive-sharpen", 1L },
+    { "+adaptive-sharpen", 1L },
+    { "-adaptive-threshold", 1L },
+    { "+adaptive-threshold", 1L },
+    { "+affine", 0L },
     { "-affine", 1L },
+    { "+annotate", 0L },
     { "-annotate", 2L },
+    { "+antialias", 0L },
     { "-antialias", 0L },
+    { "+append", 0L },
     { "-append", 0L },
+    { "+authenticate", 0L },
     { "-authenticate", 1L },
+    { "+auto-orient", 0L },
+    { "-auto-orient", 0L },
+    { "+average", 0L },
     { "-average", 0L },
+    { "+backdrop", 0L },
+    { "-backdrop", 1L },
+    { "+background", 0L },
     { "-background", 1L },
+    { "+bias", 0L },
     { "-bias", 1L },
+    { "+black-threshold", 0L },
     { "-black-threshold", 1L },
+    { "+blend", 0L },
     { "-blend", 1L },
+    { "+blue-primary", 0L },
     { "-blue-primary", 1L },
+    { "+blur", 0L },
     { "-blur", 1L },
+    { "+border", 0L },
     { "-border", 1L },
+    { "+bordercolor", 0L },
     { "-bordercolor", 1L },
+    { "+borderwidth", 0L },
+    { "-borderwidth", 1L },
+    { "+box", 0L },
     { "-box", 1L },
+    { "+cache", 0L },
     { "-cache", 1L },
+    { "+channel", 0L },
     { "-channel", 1L },
-    { "-charcoal", 1L },
+    { "+charcoal", 0L },
+    { "-charcoal", 0L },
+    { "+chop", 0L },
     { "-chop", 1L },
+    { "+clip", 0L },
+    { "-clip", 1L },
+    { "+clip-mask", 0L },
+    { "-clip-mask", 0L },
+    { "+clip-path", 0L },
     { "-clip-path", 1L },
+    { "+clone", 0L },
     { "-clone", 1L },
+    { "+coalesce", 0L },
     { "-coalesce", 0L },
+    { "+colorize", 0L },
     { "-colorize", 1L },
+    { "+colormap", 0L },
+    { "-colormap", 1L },
+    { "+colors", 0L },
     { "-colors", 1L },
+    { "+colorspace", 0L },
     { "-colorspace", 1L },
+    { "+combine", 0L },
     { "-combine", 0L },
+    { "+comment", 0L },
     { "-comment", 1L },
+    { "+compose", 0L },
     { "-compose", 1L },
+    { "+composite", 0L },
     { "-composite", 0L },
+    { "+compress", 0L },
     { "-compress", 1L },
+    { "+contrast", 0L },
     { "-contrast", 0L },
+    { "+contrast-stretch", 0L },
+    { "-contrast-stretch", 1L },
+    { "+convolve", 0L },
     { "-convolve", 1L },
+    { "+crop", 0L },
     { "-crop", 1L },
+    { "+cycle", 0L },
     { "-cycle", 1L },
-    { "-deconstruct", 0L },
+    { "+debug", 0L },
     { "-debug", 1L },
-    { "-define", 1L },
+    { "+deconstruct", 0L },
+    { "-deconstruct", 0L },
     { "+define", 1L },
+    { "-define", 1L },
+    { "+delay", 0L },
     { "-delay", 1L },
+    { "+delete", 0L },
     { "-delete", 1L },
+    { "+density", 0L },
     { "-density", 1L },
+    { "+depth", 0L },
     { "-depth", 1L },
+    { "+descend", 0L },
+    { "-descend", 1L },
+    { "+despeckle", 0L },
     { "-despeckle", 0L },
+    { "+displace", 0L },
+    { "-displace", 1L },
+    { "+display", 0L },
     { "-display", 1L },
+    { "+dispose", 0L },
     { "-dispose", 1L },
+    { "+dissolve", 0L },
+    { "-dissolve", 1L },
+    { "+dither", 0L },
     { "-dither", 0L },
+    { "+draw", 0L },
     { "-draw", 1L },
+    { "+edge", 0L },
     { "-edge", 1L },
+    { "+emboss", 0L },
     { "-emboss", 1L },
+    { "+encoding", 0L },
     { "-encoding", 1L },
+    { "+endian", 0L },
     { "-endian", 1L },
+    { "+enhance", 0L },
     { "-enhance", 0L },
+    { "+equalize", 0L },
     { "-equalize", 0L },
+    { "+evaluate", 0L },
     { "-evaluate", 2L },
+    { "+extent", 0L },
+    { "-extent", 1L },
+    { "+extract", 0L },
     { "-extract", 1L },
+    { "+family", 0L },
     { "-family", 1L },
+    { "+fill", 0L },
     { "-fill", 1L },
+    { "+filter", 0L },
     { "-filter", 1L },
+    { "+flatten", 0L },
     { "-flatten", 0L },
+    { "+flip", 0L },
     { "-flip", 0L },
-    { "-flop", 0L },
+    { "+floodfill", 0L },
     { "-floodfill", 2L },
-    { "-family", 1L },
+    { "+flop", 0L },
+    { "-flop", 0L },
+    { "+font", 0L },
     { "-font", 1L },
+    { "+foreground", 0L },
+    { "-foreground", 1L },
+    { "+format", 0L },
     { "-format", 1L },
+    { "+frame", 0L },
     { "-frame", 1L },
+    { "+fuzz", 0L },
     { "-fuzz", 1L },
+    { "+fx", 0L },
     { "-fx", 1L },
+    { "+gamma", 0L },
     { "-gamma", 1L },
+    { "+gaussian", 0L },
     { "-gaussian", 1L },
+    { "+geometry", 0L },
     { "-geometry", 1L },
-    { "-green-primary", 1L },
+    { "+gravity", 0L },
     { "-gravity", 1L },
+    { "+green-primary", 0L },
+    { "-green-primary", 1L },
+    { "+help", 0L },
     { "-help", 0L },
+    { "+iconGeometry", 0L },
+    { "-iconGeometry", 1L },
+    { "+iconic", 0L },
+    { "-iconic", 1L },
+    { "+identify", 0L },
     { "-identify", 0L },
+    { "+immutable", 0L },
+    { "-immutable", 1L },
+    { "+implode", 0L },
     { "-implode", 1L },
+    { "+insert", 0L },
     { "-insert", 1L },
+    { "+intent", 0L },
     { "-intent", 1L },
+    { "+interlace", 0L },
     { "-interlace", 1L },
+    { "+interpolate", 0L },
+    { "-interpolate", 1L },
+    { "+label", 0L },
     { "-label", 1L },
+    { "+lat", 0L },
     { "-lat", 1L },
+    { "+layers", 0L },
+    { "-layers", 1L },
+    { "+level", 0L },
     { "-level", 1L },
+    { "+limit", 0L },
     { "-limit", 2L },
+    { "+linewidth", 0L },
     { "-linewidth", 1L },
+    { "+list", 0L },
     { "-list", 1L },
+    { "+log", 0L },
     { "-log", 1L },
+    { "+loop", 0L },
     { "-loop", 1L },
+    { "+magnify", 0L },
+    { "-magnify", 1L },
+    { "+map", 0L },
     { "-map", 1L },
+    { "+mask", 0L },
     { "-mask", 1L },
+    { "+matte", 0L },
     { "-matte", 0L },
+    { "+mattecolor", 0L },
     { "-mattecolor", 1L },
+    { "+median", 0L },
     { "-median", 1L },
+    { "+metric", 0L },
+    { "-metric", 1L },
+    { "+mode", 0L },
     { "-mode", 1L },
+    { "+modulate", 0L },
     { "-modulate", 1L },
+    { "+monitor", 0L },
+    { "-monitor", 1L },
+    { "+monochrome", 0L },
     { "-monochrome", 0L },
+    { "+morph", 0L },
     { "-morph", 1L },
+    { "+mosaic", 0L },
     { "-mosaic", 0L },
+    { "+motion-blur", 0L },
+    { "-motion-blur", 1L },
+    { "+name", 0L },
+    { "-name", 1L },
+    { "+negate", 0L },
     { "-negate", 0L },
-    { "-normalize", 0L },
-    { "-noise", 1L },
     { "+noise", 1L },
+    { "-noise", 1L },
+    { "+noop", 0L },
+    { "-noop", 0L },
+    { "+normalize", 0L },
+    { "-normalize", 0L },
+    { "+opaque", 0L },
     { "-opaque", 1L },
+    { "+ordered-dither", 0L },
     { "-ordered-dither", 1L },
+    { "+orient", 0L },
     { "-orient", 1L },
+    { "+page", 0L },
     { "-page", 1L },
+    { "+paint", 0L },
     { "-paint", 1L },
+    { "+pause", 0L },
+    { "-pause", 1L },
+    { "+pen", 0L },
     { "-pen", 1L },
+    { "+ping", 0L },
     { "-ping", 0L },
+    { "+pointsize", 0L },
     { "-pointsize", 1L },
+    { "+posterize", 0L },
     { "-posterize", 1L },
+    { "+preview", 0L },
     { "-preview", 1L },
-    { "-profile", 1L },
-    { "+profile", 1L },
+    { "+process", 0L },
     { "-process", 1L },
+    { "+profile", 1L },
+    { "-profile", 1L },
+    { "+quality", 0L },
     { "-quality", 1L },
+    { "+quiet", 0L },
+    { "-quiet", 0L },
+    { "+radial-blur", 0L },
     { "-radial-blur", 1L },
+    { "+raise", 0L },
     { "-raise", 1L },
+    { "+random-threshold", 0L },
     { "-random-threshold", 1L },
-    { "-region", 1L },
-    { "-raise", 1L },
+    { "+red-primary", 0L },
     { "-red-primary", 1L },
+    { "+regard-warnings", 0L },
+    { "-regard-warnings", 0L },
+    { "+region", 0L },
+    { "-region", 1L },
+    { "+remote", 0L },
+    { "-remote", 1L },
+    { "+render", 0L },
     { "-render", 0L },
+    { "+repage", 0L },
     { "-repage", 1L },
+    { "+resample", 0L },
     { "-resample", 1L },
+    { "+resize", 0L },
     { "-resize", 1L },
+    { "+roll", 0L },
     { "-roll", 1L },
+    { "+rotate", 0L },
     { "-rotate", 1L },
+    { "+sample", 0L },
     { "-sample", 1L },
+    { "+sampling-factor", 0L },
     { "-sampling-factor", 1L },
+    { "+sans", 0L },
+    { "-sans", 1L },
+    { "+sans0", 0L },
+    { "-sans0", 0L },
+    { "+sans2", 2L },
+    { "-sans2", 2L },
+    { "+scale", 0L },
     { "-scale", 1L },
+    { "+scene", 0L },
     { "-scene", 1L },
+    { "+scenes", 0L },
+    { "-scenes", 1L },
+    { "+screen", 0L },
+    { "-screen", 1L },
+    { "+seed", 0L },
     { "-seed", 1L },
+    { "+segment", 0L },
     { "-segment", 1L },
+    { "+separate", 0L },
     { "-separate", 0L },
+    { "+sepia-tone", 0L },
+    { "-sepia-tone", 1L },
     { "+set", 1L },
     { "-set", 2L },
+    { "+shade", 0L },
     { "-shade", 1L },
+    { "+shadow", 0L },
     { "-shadow", 1L },
+    { "+shared-memory", 0L },
+    { "-shared-memory", 1L },
+    { "+sharpen", 0L },
     { "-sharpen", 1L },
+    { "+shave", 0L },
     { "-shave", 1L },
+    { "+shear", 0L },
     { "-shear", 1L },
+    { "+sigmoidal-contrast", 0L },
+    { "-sigmoidal-contrast", 1L },
+    { "+silent", 0L },
+    { "-silent", 1L },
+    { "+size", 0L },
     { "-size", 1L },
+    { "+sketch", 0L },
+    { "-sketch", 1L },
+    { "+snaps", 0L },
     { "-snaps", 1L },
+    { "+solarize", 0L },
     { "-solarize", 1L },
+    { "+splice", 0L },
     { "-splice", 1L },
+    { "+spread", 0L },
     { "-spread", 1L },
+    { "+stegano", 0L },
     { "-stegano", 1L },
-    { "-stroke", 1L },
-    { "-strokewidth", 1L },
+    { "+stereo", 0L },
+    { "-stereo", 1L },
+    { "+stretch", 0L },
     { "-stretch", 1L },
+    { "+strip", 0L },
     { "-strip", 0L },
+    { "+stroke", 0L },
+    { "-stroke", 1L },
+    { "+strokewidth", 0L },
+    { "-strokewidth", 1L },
+    { "+style", 0L },
     { "-style", 1L },
+    { "+support", 0L },
     { "-support", 1L },
+    { "+swap", 0L },
     { "-swap", 1L },
+    { "+swirl", 0L },
     { "-swirl", 1L },
+    { "+text-font", 0L },
+    { "-text-font", 1L },
+    { "+texture", 0L },
     { "-texture", 1L },
+    { "+threshold", 0L },
     { "-threshold", 1L },
+    { "+thumbnail", 0L },
     { "-thumbnail", 1L },
+    { "+thumnail", 0L },
+    { "-thumnail", 1L },
+    { "+tile", 0L },
     { "-tile", 1L },
+    { "+tint", 0L },
     { "-tint", 1L },
+    { "+title", 0L },
+    { "-title", 1L },
+    { "+transform", 0L },
+    { "-transform", 0L },
+    { "+transparent", 0L },
     { "-transparent", 1L },
+    { "+transpose", 0L },
+    { "-transpose", 0L },
+    { "+transverse", 0L },
+    { "-transverse", 0L },
+    { "+treedepth", 0L },
     { "-treedepth", 1L },
+    { "+trim", 0L },
     { "-trim", 0L },
+    { "+type", 0L },
     { "-type", 1L },
+    { "+undercolor", 0L },
     { "-undercolor", 1L },
+    { "+unique-colors", 0L },
+    { "-unique-colors", 0L },
+    { "+units", 0L },
     { "-units", 1L },
+    { "+unsharp", 0L },
     { "-unsharp", 1L },
+    { "+update", 0L },
     { "-update", 1L },
+    { "+use-pixmap", 0L },
+    { "-use-pixmap", 1L },
+    { "+verbose", 0L },
     { "-verbose", 0L },
-    { "-version", 0L },
+    { "+version", 0L },
+    { "-version", 1L },
+    { "+view", 0L },
     { "-view", 1L },
+    { "+vignette", 0L },
+    { "-vignette", 1L },
+    { "+virtual-pixel", 0L },
     { "-virtual-pixel", 1L },
+    { "+visual", 0L },
+    { "-visual", 1L },
+    { "+watermark", 0L },
     { "-watermark", 1L },
+    { "+wave", 0L },
     { "-wave", 1L },
+    { "+weight", 0L },
     { "-weight", 1L },
-    { "-window", 1L },
-    { "-window_group", 1L },
+    { "+white-point", 0L },
     { "-white-point", 1L },
+    { "+white-threshold", 0L },
     { "-white-threshold", 1L },
+    { "+window", 0L },
+    { "-window", 1L },
+    { "+window-group", 0L },
+    { "-window-group", 1L },
+    { "+write", 0L },
     { "-write", 1L },
     { (char *) NULL, (long) 0L }
   },
@@ -306,7 +583,7 @@ static const OptionInfo
   {
     { "Undefined", (long) UndefinedCompositeOp },
     { "Add", (long) AddCompositeOp },
-    { "Atop", (long) SrcAtopCompositeOp },
+    { "Atop", (long) AtopCompositeOp },
     { "Blend", (long) BlendCompositeOp },
     { "Bumpmap", (long) BumpmapCompositeOp },
     { "Clear", (long) ClearCompositeOp },
@@ -342,7 +619,7 @@ static const OptionInfo
     { "Modulate", (long) ModulateCompositeOp },
     { "Multiply", (long) MultiplyCompositeOp },
     { "None", (long) NoCompositeOp },
-    { "Out", (long) SrcOutCompositeOp },
+    { "Out", (long) OutCompositeOp },
     { "Overlay", (long) OverlayCompositeOp },
     { "Over", (long) OverCompositeOp },
     { "Plus", (long) PlusCompositeOp },
@@ -387,9 +664,12 @@ static const OptionInfo
     { "HSL", (long) HSLColorspace },
     { "HWB", (long) HWBColorspace },
     { "LAB", (long) LABColorspace },
+    { "Log", (long) LogColorspace },
     { "OHTA", (long) OHTAColorspace },
     { "Rec601Luma", (long) Rec601LumaColorspace },
+    { "Rec601YCbCr", (long) Rec601YCbCrColorspace },
     { "Rec709Luma", (long) Rec709LumaColorspace },
+    { "Rec709YCbCr", (long) Rec709YCbCrColorspace },
     { "RGB", (long) RGBColorspace },
     { "sRGB", (long) sRGBColorspace },
     { "Transparent", (long) TransparentColorspace },
@@ -499,8 +779,9 @@ static const OptionInfo
     { "Grayscale", (long) GrayscaleType },
     { "GrayscaleMatte", (long) GrayscaleMatteType },
     { "Optimize", (long) OptimizeType },
-    { "PaletteMatte", (long) PaletteMatteType },
     { "Palette", (long) PaletteType },
+    { "PaletteBilevelMatte", (long) PaletteBilevelMatteType },
+    { "PaletteMatte", (long) PaletteMatteType },
     { "TrueColorMatte", (long) TrueColorMatteType },
     { "TrueColor", (long) TrueColorType },
     { (char *) NULL, (long) UndefinedType }
@@ -523,6 +804,30 @@ static const OptionInfo
     { "Partition", (long) PartitionInterlace },
     { (char *) NULL, (long) UndefinedInterlace }
   },
+  InterpolateOptions[] =
+  {
+    { "Undefined", (long) UndefinedInterpolatePixel },
+    { "Average", (long) AverageInterpolatePixel },
+    { "Bicubic", (long) BicubicInterpolatePixel },
+    { "Bilinear", (long) BilinearInterpolatePixel },
+    { "filter", (long) FilterInterpolatePixel },
+    { "Integer", (long) IntegerInterpolatePixel },
+    { "Mesh", (long) MeshInterpolatePixel },
+    { "NearestNeighbor", (long) NearestNeighborInterpolatePixel },
+    { (char *) NULL, (long) UndefinedInterpolatePixel }
+  },
+  LayersOptions[] =
+  {
+    { "Undefined", (long) UndefinedLayer },
+    { "Coalesce", (long) CoalesceLayer },
+    { "CompareAny", (long) CompareAnyLayer },
+    { "CompareClear", (long) CompareClearLayer },
+    { "CompareOverlay", (long) CompareOverlayLayer },
+    { "Dispose", (long) DisposeLayer },
+    { "Optimize", (long) OptimizeLayer },
+    { "OptimizePlus", (long) OptimizePlusLayer },
+    { (char *) NULL, (long) UndefinedLayer }
+  },
   LineCapOptions[] =
   {
     { "Undefined", (long) UndefinedCap },
@@ -540,19 +845,65 @@ static const OptionInfo
     { (char *) NULL, (long) UndefinedJoin }
   },
   ListOptions[] =
-  {
-    { "Coder", 1L },
-    { "Color", 1L },
-    { "Configure", 1L },
-    { "Delegate", 1L },
-    { "Format", 1L },
-    { "Locale", 1L },
-    { "Log", 1L },
-    { "Magic", 1L },
-    { "Module", 1L },
-    { "Resource", 1L },
-    { "Type", 1L },
-    { (char *) NULL, 0L }
+  { /* lookup value is a index to a specific List Function */
+    { "Undefined",     MagickUndefinedOptions },
+    { "Align",         MagickAlignOptions },
+    { "Boolean",       MagickBooleanOptions },
+    { "Channel",       MagickChannelOptions },
+    { "Class",         MagickClassOptions },
+    { "ClipPath",      MagickClipPathOptions },
+    { "Colorspace",    MagickColorspaceOptions },
+    { "Composite",     MagickCompositeOptions },
+    { "Command",       MagickCommandOptions },
+    { "Compression",   MagickCompressionOptions },
+    { "Decoration",    MagickDecorationOptions },
+    { "Dispose",       MagickDisposeOptions },
+    { "Endian",        MagickEndianOptions },
+    { "Evaluate",      MagickEvaluateOptions },
+    { "FillRule",      MagickFillRuleOptions },
+    { "Filter",        MagickFilterOptions },
+    { "Gravity",       MagickGravityOptions },
+    { "Image",         MagickImageOptions },
+    { "Intent",        MagickIntentOptions },
+    { "Interlace",     MagickInterlaceOptions },
+    { "Interpolate",   MagickInterpolateOptions },
+    { "Layers",        MagickLayersOptions },
+    { "LineCap",       MagickLineCapOptions },
+    { "LineJoin",      MagickLineJoinOptions },
+    { "List",          MagickListOptions },
+    { "LogEvent",      MagickLogEventOptions },
+    { "Metric",        MagickMetricOptions },
+    { "Method",        MagickMethodOptions },
+    { "Mode",          MagickModeOptions },
+    { "Mogrify",       MagickMogrifyOptions },
+    { "Noise",         MagickNoiseOptions },
+    { "Orientation",   MagickOrientationOptions },
+    { "Preview",       MagickPreviewOptions },
+    { "Primitive",     MagickPrimitiveOptions },
+    { "QuantumFormat", MagickQuantumFormatOptions },
+    { "Resolution",    MagickResolutionOptions },
+    { "Resource",      MagickResourceOptions },
+    { "Storage",       MagickStorageOptions },
+    { "Stretch",       MagickStretchOptions },
+    { "Style",         MagickStyleOptions },
+    { "VirtualPixel",  MagickVirtualPixelOptions },
+    /* Option has a specific function to List these Options or Values */
+    { "Coder",         MagickCoderOptions },
+    { "Color",         MagickColorOptions },
+    { "Configure",     MagickConfigureOptions },
+    { "Delegate",      MagickDelegateOptions },
+    { "Format",        MagickFormatOptions },
+    { "Locale",        MagickLocaleOptions },
+    { "Log",           MagickLogOptions },
+    { "Magic",         MagickMagicOptions },
+    { "Module",        MagickModuleOptions },
+    /*{ "Resource",      MagickResourceOptions }, -- duplicate */
+    { "Threshold",     MagickThresholdOptions },
+    { "Type",          MagickTypeOptions },
+    /* Future option names to reference appropraite option list
+     * EG:   "OrderedDither", MagickThresholdOptions,
+     */
+    { (char *) NULL,   MagickUndefinedOptions }
   },
   LogEventOptions[] =
   {
@@ -580,6 +931,7 @@ static const OptionInfo
   MetricOptions[] =
   {
     { "Undefined", (long) UndefinedMetric },
+    { "AE", (long) AbsoluteErrorMetric },
     { "MAE", (long) MeanAbsoluteErrorMetric },
     { "MSE", (long) MeanSquaredErrorMetric },
     { "PAE", (long) PeakAbsoluteErrorMetric },
@@ -609,10 +961,11 @@ static const OptionInfo
   {
     { "append", MagickTrue },
     { "average", MagickTrue },
-    { "crop", MagickTrue },
+    { "clip-mask", MagickTrue },
     { "coalesce", MagickTrue },
     { "combine", MagickTrue },
     { "composite", MagickTrue },
+    { "crop", MagickTrue },
     { "deconstruct", MagickTrue },
     { "delete", MagickTrue },
     { "flatten", MagickTrue },
@@ -621,7 +974,10 @@ static const OptionInfo
     { "map", MagickTrue },
     { "morph", MagickTrue },
     { "mosaic", MagickTrue },
+    { "optimize", MagickTrue },
     { "process", MagickTrue },
+    { "separate", MagickTrue },
+    { "scene", MagickTrue },
     { "swap", MagickTrue },
     { "write", MagickTrue },
     { (char *) NULL, MagickFalse }
@@ -703,6 +1059,14 @@ static const OptionInfo
     { "Text", (long) TextPrimitive },
     { (char *) NULL, (long) UndefinedPrimitive }
   },
+  QuantumFormatOptions[] =
+  {
+    { "Undefined", (long) UndefinedClass },
+    { "FloatingPoint", (long) FloatingPointQuantumFormat },
+    { "Signed", (long) SignedQuantumFormat },
+    { "Unsigned", (long) UnsignedQuantumFormat },
+    { (char *) NULL, (long) FloatingPointQuantumFormat }
+  },
   ResolutionOptions[] =
   {
     { "Undefined", (long) UndefinedResolution },
@@ -759,10 +1123,14 @@ static const OptionInfo
   VirtualPixelOptions[] =
   {
     { "Undefined", (long) UndefinedVirtualPixelMethod },
-    { "Constant", (long) ConstantVirtualPixelMethod },
+    { "Background", (long) BackgroundVirtualPixelMethod },
+    { "Constant", (long) BackgroundVirtualPixelMethod },
+    { "Dither", (long) DitherVirtualPixelMethod },
     { "Edge", (long) EdgeVirtualPixelMethod },
     { "Mirror", (long) MirrorVirtualPixelMethod },
+    { "Random", (long) RandomVirtualPixelMethod },
     { "Tile", (long) TileVirtualPixelMethod },
+    { "Transparent", (long) TransparentVirtualPixelMethod },
     { (char *) NULL, (long) UndefinedVirtualPixelMethod }
   };
 
@@ -903,7 +1271,7 @@ MagickExport MagickBooleanType DeleteImageOption(ImageInfo *image_info,
       image_info->filename);
   if (image_info->options == (void *) NULL)
     return(MagickFalse);
-  return(RemoveNodeFromSplayTree(image_info->options,option));
+  return(RemoveNodeFromSplayTree((SplayTreeInfo *) image_info->options,option));
 }
 
 /*
@@ -937,7 +1305,7 @@ MagickExport void DestroyImageOptions(ImageInfo *image_info)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   if (image_info->options != (void *) NULL)
-    image_info->options=DestroySplayTree(image_info->options);
+    image_info->options=DestroySplayTree((SplayTreeInfo *) image_info->options);
 }
 
 /*
@@ -968,6 +1336,9 @@ MagickExport void DestroyImageOptions(ImageInfo *image_info)
 MagickExport const char *GetImageOption(const ImageInfo *image_info,
   const char *key)
 {
+  const char
+    *option;
+
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
@@ -975,7 +1346,9 @@ MagickExport const char *GetImageOption(const ImageInfo *image_info,
       image_info->filename);
   if (image_info->options == (void *) NULL)
     return((const char *) NULL);
-  return((const char *) GetValueFromSplayTree(image_info->options,key));
+  option=(const char *)
+    GetValueFromSplayTree((SplayTreeInfo *) image_info->options,key);
+  return(option);
 }
 
 /*
@@ -1025,6 +1398,8 @@ static const OptionInfo *GetOptionInfo(const MagickOption value)
     case MagickImageOptions: return(ImageOptions);
     case MagickIntentOptions: return(IntentOptions);
     case MagickInterlaceOptions: return(InterlaceOptions);
+    case MagickInterpolateOptions: return(InterpolateOptions);
+    case MagickLayersOptions: return(LayersOptions);
     case MagickLineCapOptions: return(LineCapOptions);
     case MagickLineJoinOptions: return(LineJoinOptions);
     case MagickListOptions: return(ListOptions);
@@ -1037,6 +1412,7 @@ static const OptionInfo *GetOptionInfo(const MagickOption value)
     case MagickOrientationOptions: return(OrientationOptions);
     case MagickPreviewOptions: return(PreviewOptions);
     case MagickPrimitiveOptions: return(PrimitiveOptions);
+    case MagickQuantumFormatOptions: return(QuantumFormatOptions);
     case MagickResolutionOptions: return(ResolutionOptions);
     case MagickResourceOptions: return(ResourceOptions);
     case MagickStorageOptions: return(StorageOptions);
@@ -1104,7 +1480,7 @@ MagickExport char *GetNextImageOption(const ImageInfo *image_info)
       image_info->filename);
   if (image_info->options == (void *) NULL)
     return((char *) NULL);
-  return((char *) GetNextKeyInSplayTree(image_info->options));
+  return((char *) GetNextKeyInSplayTree((SplayTreeInfo *) image_info->options));
 }
 
 /*
@@ -1117,7 +1493,8 @@ MagickExport char *GetNextImageOption(const ImageInfo *image_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsMagickOption() returns MagickTrue if the option begins with a - or +.
+%  IsMagickOption() returns MagickTrue if the option begins with a - or + and
+%  the first character that follows is alphanumeric.
 %
 %  The format of the IsMagickOption method is:
 %
@@ -1129,7 +1506,6 @@ MagickExport char *GetNextImageOption(const ImageInfo *image_info)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType IsMagickOption(const char *option)
 {
@@ -1138,7 +1514,8 @@ MagickExport MagickBooleanType IsMagickOption(const char *option)
     return(MagickFalse);
   if (strlen(option) == 1)
     return(MagickFalse);
-  if (isalpha((int) (unsigned char) *(option+1)) == 0)
+  option++;
+  if (isalpha((int) ((unsigned char) *option)) == 0)
     return(MagickFalse);
   return(MagickTrue);
 }
@@ -1192,6 +1569,58 @@ MagickExport const char *MagickOptionToMnemonic(const MagickOption value,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   L i s t M a g i c k O p t i o n I n f o                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ListMagickOptionInfo() lists the contents of enumerated option type(s).
+%
+%  The format of the ParseMagickOption method is:
+%
+%      MagickBooleanType ListMagickOptionInfo(FILE *file,
+%           const MagickOption option, ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o file:   List Magick Options Available
+%
+%    o option: Index to the option table to list
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+*/
+MagickExport MagickBooleanType ListMagickOptionInfo(FILE *file,
+  const MagickOption option, ExceptionInfo *exception)
+{
+  const OptionInfo
+    *option_info;
+
+  register long
+    i;
+
+  if ( file == (FILE *)NULL )
+    file = stdout;
+
+  option_info=GetOptionInfo(option);
+  if (option_info == (const OptionInfo *) NULL)
+    return(MagickFalse);
+
+  for (i=0; option_info[i].mnemonic != (char *) NULL; i++)
+  {
+    if ( i==0 && strcmp(option_info[i].mnemonic, "Undefined") == 0 )
+      continue;
+    fprintf(file, "%s\n", option_info[i].mnemonic);
+  }
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P a r s e C h a n n e l O p t i o n                                       %
 %                                                                             %
 %                                                                             %
@@ -1314,20 +1743,18 @@ MagickExport long ParseChannelOption(const char *channels)
 %
 %  A description of each parameter follows:
 %
-%    o option: The option.
+%    o option: Index to the option table to lookup
 %
 %    o list: A option other than zero permits more than one option separated by
 %      commas.
 %
 %    o options: One or more options separated by commas.
 %
-%
 */
 MagickExport long ParseMagickOption(const MagickOption option,
   const MagickBooleanType list,const char *options)
 {
   char
-    *q,
     token[MaxTextExtent];
 
   const OptionInfo
@@ -1340,6 +1767,9 @@ MagickExport long ParseMagickOption(const MagickOption option,
     negate;
 
   register char
+    *q;
+
+  register const char
     *p;
 
   register long
@@ -1349,21 +1779,27 @@ MagickExport long ParseMagickOption(const MagickOption option,
   if (option_info == (const OptionInfo *) NULL)
     return(-1);
   option_types=0;
-  for (q=(char *) options; *q != '\0'; )
+  for (p=options; p != (char *) NULL; p=strchr(p,','))
   {
-    GetMagickToken(q,&q,token);
-    if ((*token == '-') || (*token == '+'))
-      GetMagickToken(q,&q,token+1);
-    negate=(*token == '!') ? MagickTrue : MagickFalse;
+    while (((isspace((int) ((unsigned char) *p)) != 0) || (*p == ',')) &&
+           (*p != '\0'))
+      p++;
+    if ((*p == '-') || (*p == '+'))
+      p++;
+    negate=(*p == '!') ? MagickTrue : MagickFalse;
     if (negate != MagickFalse)
-      GetMagickToken(q,&q,token);
-    p=strchr(token+1,'-');
-    if (p != (char *) NULL)
-      (void) CopyMagickString(p,p+1,MaxTextExtent);
-    p=strchr(token+1,'_');
-    if (p != (char *) NULL)
-      (void) CopyMagickString(p,p+1,MaxTextExtent);
+      p++;
+    q=token;
+    while (((isspace((int) ((unsigned char) *p)) == 0) && (*p != ',')) &&
+           (*p != '\0'))
+    {
+      if ((q-token) >= MaxTextExtent)
+        break;
+      *q++=(*p++);
+    }
+    *q='\0';
     for (i=0; option_info[i].mnemonic != (char *) NULL; i++)
+    {
       if (LocaleCompare(token,option_info[i].mnemonic) == 0)
         {
           if (*token == '!')
@@ -1372,13 +1808,31 @@ MagickExport long ParseMagickOption(const MagickOption option,
             option_types=option_types | option_info[i].type;
           break;
         }
+    }
+    if ((option_info[i].mnemonic == (char *) NULL) &&
+        ((strchr(token+1,'-') != (char *) NULL) || 
+         (strchr(token+1,'_') != (char *) NULL)))
+      {
+        q=strchr(token+1,'-');
+        if (q != (char *) NULL)
+          (void) CopyMagickString(q,q+1,MaxTextExtent-strlen(q));
+        q=strchr(token+1,'_');
+        if (q != (char *) NULL)
+          (void) CopyMagickString(q,q+1,MaxTextExtent-strlen(q));
+        for (i=0; option_info[i].mnemonic != (char *) NULL; i++)
+          if (LocaleCompare(token,option_info[i].mnemonic) == 0)
+            {
+              if (*token == '!')
+                option_types=option_types &~ option_info[i].type;
+              else
+                option_types=option_types | option_info[i].type;
+              break;
+            }
+      }
     if (option_info[i].mnemonic == (char *) NULL)
       return(-1);
     if (list == MagickFalse)
       break;
-    GetMagickToken(q,&q,token);
-    if (*token != ',')
-      break;
   }
   return(option_types);
 }
@@ -1418,7 +1872,7 @@ MagickExport MagickBooleanType RemoveImageOption(ImageInfo *image_info,
       image_info->filename);
   if (image_info->options == (void *) NULL)
     return(MagickFalse);
-  return(RemoveNodeFromSplayTree(image_info->options,option));
+  return(RemoveNodeFromSplayTree((SplayTreeInfo *) image_info->options,option));
 }
 
 /*
@@ -1454,7 +1908,7 @@ MagickExport void ResetImageOptionIterator(const ImageInfo *image_info)
       image_info->filename);
   if (image_info->options == (void *) NULL)
     return;
-  ResetSplayTreeIterator(image_info->options);
+  ResetSplayTreeIterator((SplayTreeInfo *) image_info->options);
 }
 
 /*
@@ -1498,7 +1952,48 @@ MagickExport MagickBooleanType SetImageOption(ImageInfo *image_info,
   if (image_info->options == (void *) NULL)
     image_info->options=NewSplayTree(CompareSplayTreeString,
       RelinquishMagickMemory,RelinquishMagickMemory);
-  status=AddValueToSplayTree(image_info->options,
-    ConstantString(AcquireString(option)),ConstantString(AcquireString(value)));
+  status=AddValueToSplayTree((SplayTreeInfo *) image_info->options,
+    ConstantString(option),ConstantString(value));
   return(status);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S y n c I m a g e O p t i o n                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SyncImageOption() syncs the image settings with a particular image.
+%
+%  The format of the SyncImageOption method is:
+%
+%      MagickBooleanType SyncImageOption(const ImageInfo *image_info,
+%        const char *option,const char *value)
+%
+%  A description of each parameter follows:
+%
+%    o image_info: The image info.
+%
+%    o image: The image.
+%
+*/
+MagickExport MagickBooleanType SyncImageOptions(const ImageInfo *image_info,
+  Image *image)
+{
+  const char
+    *option;
+
+  option=GetImageOption(image_info,"depth");
+  if (option != (const char *) NULL)
+    image->depth=(unsigned long) atol(option);
+  option=GetImageOption(image_info,"gravity");
+  if (option != (const char *) NULL)
+    image->gravity=(GravityType) ParseMagickOption(MagickGravityOptions,
+      MagickFalse,option);
+  return(MagickTrue);
+}
