@@ -24,7 +24,9 @@ extern "C" {
 
 typedef enum
 {
-  MagickAlignOptions,
+  /* Options that are listed/parsed from tables in "options.c" */
+  MagickUndefinedOptions = -1,
+  MagickAlignOptions = 0,
   MagickBooleanOptions,
   MagickChannelOptions,
   MagickClassOptions,
@@ -43,6 +45,8 @@ typedef enum
   MagickImageOptions,
   MagickIntentOptions,
   MagickInterlaceOptions,
+  MagickInterpolateOptions,
+  MagickLayersOptions,
   MagickLineCapOptions,
   MagickLineJoinOptions,
   MagickListOptions,
@@ -55,12 +59,26 @@ typedef enum
   MagickOrientationOptions,
   MagickPreviewOptions,
   MagickPrimitiveOptions,
+  MagickQuantumFormatOptions,
   MagickResolutionOptions,
   MagickResourceOptions,
   MagickStorageOptions,
   MagickStretchOptions,
   MagickStyleOptions,
-  MagickVirtualPixelOptions
+  MagickVirtualPixelOptions,
+  /* Special Option Lists Indexed to a Specific List Function for that Option */
+  MagickCoderOptions,
+  MagickColorOptions,
+  MagickConfigureOptions,
+  MagickDelegateOptions,
+  MagickFormatOptions,
+  MagickLocaleOptions,
+  MagickLogOptions,
+  MagickMagicOptions,
+  MagickModuleOptions,
+  /* MagickResourceOptions, -- duplicate Function context dependant */
+  MagickThresholdOptions,
+  MagickTypeOptions,
 } MagickOption;
 
 typedef struct _OptionInfo
@@ -89,6 +107,7 @@ extern MagickExport MagickBooleanType
   DefineImageOption(ImageInfo *,const char *),
   DeleteImageOption(ImageInfo *,const char *),
   IsMagickOption(const char *),
+  ListMagickOptionInfo(FILE *, const MagickOption, ExceptionInfo *),
   RemoveImageOption(ImageInfo *,const char *),
   SetImageOption(ImageInfo *,const char *,const char *),
   SyncImageOptions(const ImageInfo *,Image *);
