@@ -232,16 +232,21 @@ MagickExport char *GetNextImageProfile(const Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ProfileImage() adds or removes a ICC, IPTC, or generic profile from an
-%  image.  If the profile is NULL, it is removed from the image otherwise
-%  added.  Use a name of '*' and a profile of NULL to remove all profiles
-%  from the image.
+%  ProfileImage() associates, applies, or removes an ICM, IPTC, or generic
+%  profile with/to/from an image.  If the profile is NULL, it is removed from
+%  the image otherwise added or applied.  Use a name of '*' and a profile of
+%  NULL to remove all profiles from the image. 
+%
+%  ICC and ICM profiles are are handled as follows: If the image does not
+%  have an associated color profile, the one you provide is associated with the
+%  image and the image pixels are not transformed.  Otherwise, the colorspace
+%  transform defined by the existing and new profile are applied to the image
+%  pixels and the new profile is associated with the image.
 %
 %  The format of the ProfileImage method is:
 %
 %      MagickBooleanType ProfileImage(Image *image,const char *name,
-%        const void *datum,const unsigned long length,
-%        const MagickBooleanType clone)
+%        const void *datum,const size_t length,const MagickBooleanType clone)
 %
 %  A description of each parameter follows:
 %
@@ -270,7 +275,8 @@ static int LCMSErrorHandler(int severity,const char *context)
 #endif
 
 MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
-  const void *datum,const unsigned long length,const MagickBooleanType clone)
+  const void *datum,const size_t length,
+  const MagickBooleanType magick_unused(clone))
 {
 #define ThrowProfileException(severity,tag,context) \
 { \
@@ -335,6 +341,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             transform;
 
           DWORD
+            flags,
             source_type,
             target_type;
 
@@ -504,8 +511,13 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             case SaturationIntent: intent=INTENT_SATURATION; break;
             default: intent=INTENT_PERCEPTUAL; break;
           }
+          flags=cmsFLAGS_HIGHRESPRECALC;
+#if defined(cmsFLAGS_BLACKPOINTCOMPENSATION)
+          if (image->black_point_compensation != MagickFalse)
+            flags|=cmsFLAGS_BLACKPOINTCOMPENSATION;
+#endif
           transform=cmsCreateTransform(source_profile,source_type,
-            target_profile,target_type,intent,cmsFLAGS_HIGHRESPRECALC);
+            target_profile,target_type,intent,flags);
           (void) cmsCloseProfile(source_profile);
           (void) cmsCloseProfile(target_profile);
           if (transform == (cmsHTRANSFORM) NULL)
