@@ -16,7 +16,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,14 +40,25 @@
 */
 #include "magick/studio.h"
 #include "magick/attribute.h"
+#include "magick/blob.h"
+#include "magick/cache-view.h"
 #include "magick/color.h"
+#include "magick/color-private.h"
+#include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/gem.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
 #include "magick/memory_.h"
+#include "magick/pixel-private.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/resize.h"
 #include "magick/string_.h"
+#include "magick/utility.h"
+#include "magick/version.h"
 
 /*
   Typedef declarations.
@@ -73,6 +84,115 @@ typedef struct _FilterInfo
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A d a p t i v e R e s i z e I m a g e                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AdaptiveResizeImage() adaptively resize image with data
+%  dependent triangulation.
+%
+%  The format of the AdaptiveResizeImage method is:
+%
+%      Image *AdaptiveResizeImage(const Image *image,
+%        const unsigned long columns,const unsigned long rows,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o columns: The number of columns in the resized image.
+%
+%    o rows: The number of rows in the resized image.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+*/
+MagickExport Image *AdaptiveResizeImage(const Image *image,
+  const unsigned long columns,const unsigned long rows,ExceptionInfo *exception)
+{
+#define AdaptiveResizeImageTag  "Resize/Image"
+
+  Image
+    *resize_image;
+
+  long
+    y;
+
+  MagickPixelPacket
+    pixel;
+
+  PointInfo
+    offset;
+
+  register IndexPacket
+    *resize_indexes;
+
+  register long
+    x;
+
+  register PixelPacket
+    *q;
+
+  ViewInfo
+    *image_view,
+    *resize_view;
+
+  /*
+    Resize image.
+  */
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  if ((columns == 0) || (rows == 0))
+    return((Image *) NULL);
+  if ((columns == image->columns) && (rows == image->rows))
+    return(CloneImage(image,0,0,MagickTrue,exception));
+  resize_image=CloneImage(image,columns,rows,MagickTrue,exception);
+  if (resize_image == (Image *) NULL)
+    return((Image *) NULL);
+  if (SetImageStorageClass(resize_image,DirectClass) == MagickFalse)
+    {
+      InheritException(exception,&resize_image->exception);
+      resize_image=DestroyImage(resize_image);
+      return((Image *) NULL);
+    }
+  GetMagickPixelPacket(image,&pixel);
+  image_view=OpenCacheView(image);
+  resize_view=OpenCacheView(resize_image);
+  for (y=0; y < (long) resize_image->rows; y++)
+  {
+    q=SetCacheView(resize_view,0,y,resize_image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
+    resize_indexes=GetIndexes(resize_image);
+    offset.y=(MagickRealType) y*image->rows/resize_image->rows;
+    for (x=0; x < (long) resize_image->columns; x++)
+    {
+      offset.x=(MagickRealType) x*image->columns/resize_image->columns;
+      pixel=InterpolatePixelColor(image,image_view,MeshInterpolatePixel,
+        offset.x,offset.y,exception);
+      SetPixelPacket(&pixel,q,resize_indexes+x);
+      q++;
+    }
+    if (SyncCacheView(resize_view) == MagickFalse)
+      break;
+  }
+  resize_view=CloseCacheView(resize_view);
+  image_view=CloseCacheView(image_view);
+  return(resize_image);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   B e s s e l O r d e r O n e                                               %
 %                                                                             %
 %                                                                             %
@@ -289,7 +409,7 @@ MagickExport Image *MagnifyImage(const Image *image,ExceptionInfo *exception)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   magnify_image=ResizeImage(image,2*image->columns,2*image->rows,CubicFilter,
@@ -331,7 +451,7 @@ MagickExport Image *MinifyImage(const Image *image,ExceptionInfo *exception)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   minify_image=ResizeImage(image,image->columns/2,image->rows/2,CubicFilter,
@@ -562,7 +682,7 @@ static MagickBooleanType HorizontalFilter(const Image *source,
   Image *destination,const MagickRealType x_factor,
   const FilterInfo *filter_info,const MagickRealType blur,
   ContributionInfo *contribution,const MagickSizeType span,
-  unsigned long *quantum,ExceptionInfo *exception)
+  MagickOffsetType *quantum,ExceptionInfo *exception)
 {
 #define ResizeImageTag  "Resize/Image"
 
@@ -581,8 +701,10 @@ static MagickBooleanType HorizontalFilter(const Image *source,
     zero;
 
   MagickRealType
+    alpha,
     center,
     density,
+    gamma,
     scale,
     support;
 
@@ -607,7 +729,13 @@ static MagickBooleanType HorizontalFilter(const Image *source,
   support=scale*filter_info->support;
   destination->storage_class=source->storage_class;
   if (support > 0.5)
-    destination->storage_class=DirectClass;
+    {
+      if (SetImageStorageClass(destination,DirectClass) == MagickFalse)
+        {
+          InheritException(exception,&destination->exception);
+          return(MagickFalse);
+        }
+    }
   else
     {
       /*
@@ -650,17 +778,40 @@ static MagickBooleanType HorizontalFilter(const Image *source,
     for (y=0; y < (long) destination->rows; y++)
     {
       pixel=zero;
-      for (i=0; i < n; i++)
-      {
-        j=y*(contribution[n-1].pixel-contribution[0].pixel+1)+
-          (contribution[i].pixel-contribution[0].pixel);
-        pixel.red+=contribution[i].weight*(p+j)->red;
-        pixel.green+=contribution[i].weight*(p+j)->green;
-        pixel.blue+=contribution[i].weight*(p+j)->blue;
-      }
-      q->red=RoundToQuantum(pixel.red);
-      q->green=RoundToQuantum(pixel.green);
-      q->blue=RoundToQuantum(pixel.blue);
+      if (source->matte == MagickFalse)
+        {
+          for (i=0; i < n; i++)
+          {
+            j=y*(contribution[n-1].pixel-contribution[0].pixel+1)+
+              (contribution[i].pixel-contribution[0].pixel);
+            alpha=contribution[i].weight;
+            pixel.red+=alpha*(p+j)->red;
+            pixel.green+=alpha*(p+j)->green;
+            pixel.blue+=alpha*(p+j)->blue;
+          }
+          q->red=RoundToQuantum(pixel.red);
+          q->green=RoundToQuantum(pixel.green);
+          q->blue=RoundToQuantum(pixel.blue);
+        }
+      else
+        {
+          gamma=0.0;
+          for (i=0; i < n; i++)
+          {
+            j=y*(contribution[n-1].pixel-contribution[0].pixel+1)+
+              (contribution[i].pixel-contribution[0].pixel);
+            alpha=contribution[i].weight*QuantumScale*((MagickRealType)
+              QuantumRange-(p+j)->opacity);
+            pixel.red+=alpha*(p+j)->red;
+            pixel.green+=alpha*(p+j)->green;
+            pixel.blue+=alpha*(p+j)->blue;
+            gamma+=alpha;
+          }
+          gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+          q->red=RoundToQuantum(gamma*pixel.red);
+          q->green=RoundToQuantum(gamma*pixel.green);
+          q->blue=RoundToQuantum(gamma*pixel.blue);
+        }
       if ((source->matte != MagickFalse) && (destination->matte != MagickFalse))
         {
           for (i=0; i < n; i++)
@@ -674,13 +825,32 @@ static MagickBooleanType HorizontalFilter(const Image *source,
       if ((source->colorspace == CMYKColorspace) &&
           (destination->colorspace == CMYKColorspace))
         {
-          for (i=0; i < n; i++)
-          {
-            j=y*(contribution[n-1].pixel-contribution[0].pixel+1)+
-              (contribution[i].pixel-contribution[0].pixel);
-            pixel.index+=contribution[i].weight*(source_indexes[j]);
-          }
-          indexes[y]=(IndexPacket) RoundToQuantum(pixel.index);
+          if (source->matte == MagickFalse)
+            {
+              for (i=0; i < n; i++)
+              {
+                j=y*(contribution[n-1].pixel-contribution[0].pixel+1)+
+                  (contribution[i].pixel-contribution[0].pixel);
+                alpha=contribution[i].weight;
+                pixel.index+=alpha*source_indexes[j];
+              }
+              indexes[y]=(IndexPacket) RoundToQuantum(pixel.index);
+            }
+          else
+            {
+              gamma=0.0;
+              for (i=0; i < n; i++)
+              {
+                j=y*(contribution[n-1].pixel-contribution[0].pixel+1)+
+                  (contribution[i].pixel-contribution[0].pixel);
+                alpha=contribution[i].weight*QuantumScale*((MagickRealType)
+                  QuantumRange-(p+j)->opacity);
+                pixel.index+=alpha*source_indexes[j];
+                gamma+=alpha;
+              }
+              gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+              indexes[y]=(IndexPacket) RoundToQuantum(gamma*pixel.index);
+            }
         }
       if ((destination->storage_class == PseudoClass) &&
           (source->storage_class == PseudoClass))
@@ -704,13 +874,13 @@ static MagickBooleanType HorizontalFilter(const Image *source,
       }
     (*quantum)++;
   }
-  return((MagickBooleanType) (x == (long) destination->columns));
+  return(x == (long) destination->columns ? MagickTrue : MagickFalse);
 }
 
 static MagickBooleanType VerticalFilter(const Image *source,Image *destination,
   const MagickRealType y_factor,const FilterInfo *filter_info,
   const MagickRealType blur,ContributionInfo *contribution,
-  const MagickSizeType span,unsigned long *quantum,ExceptionInfo *exception)
+  const MagickSizeType span,MagickOffsetType *quantum,ExceptionInfo *exception)
 {
   long
     j,
@@ -727,8 +897,10 @@ static MagickBooleanType VerticalFilter(const Image *source,Image *destination,
     zero;
 
   MagickRealType
+    alpha,
     center,
     density,
+    gamma,
     scale,
     support;
 
@@ -753,7 +925,13 @@ static MagickBooleanType VerticalFilter(const Image *source,Image *destination,
   support=scale*filter_info->support;
   destination->storage_class=source->storage_class;
   if (support > 0.5)
-    destination->storage_class=DirectClass;
+    {
+      if (SetImageStorageClass(destination,DirectClass) == MagickFalse)
+        {
+          InheritException(exception,&destination->exception);
+          return(MagickFalse);
+        }
+    }
   else
     {
       /*
@@ -796,18 +974,41 @@ static MagickBooleanType VerticalFilter(const Image *source,Image *destination,
     indexes=GetIndexes(destination);
     for (x=0; x < (long) destination->columns; x++)
     {
+      gamma=0.0;
       pixel=zero;
-      for (i=0; i < n; i++)
-      {
-        j=(long) ((contribution[i].pixel-contribution[0].pixel)*
-          source->columns+x);
-        pixel.red+=contribution[i].weight*(p+j)->red;
-        pixel.green+=contribution[i].weight*(p+j)->green;
-        pixel.blue+=contribution[i].weight*(p+j)->blue;
-      }
-      q->red=RoundToQuantum(pixel.red);
-      q->green=RoundToQuantum(pixel.green);
-      q->blue=RoundToQuantum(pixel.blue);
+      if (source->matte == MagickFalse)
+        {
+          for (i=0; i < n; i++)
+          {
+            j=(long) ((contribution[i].pixel-contribution[0].pixel)*
+              source->columns+x);
+            alpha=contribution[i].weight;
+            pixel.red+=alpha*(p+j)->red;
+            pixel.green+=alpha*(p+j)->green;
+            pixel.blue+=alpha*(p+j)->blue;
+          }
+          q->red=RoundToQuantum(pixel.red);
+          q->green=RoundToQuantum(pixel.green);
+          q->blue=RoundToQuantum(pixel.blue);
+        }
+      else
+        {
+          for (i=0; i < n; i++)
+          {
+            j=(long) ((contribution[i].pixel-contribution[0].pixel)*
+              source->columns+x);
+            alpha=contribution[i].weight*QuantumScale*((MagickRealType)
+              QuantumRange-(p+j)->opacity);
+            pixel.red+=alpha*(p+j)->red;
+            pixel.green+=alpha*(p+j)->green;
+            pixel.blue+=alpha*(p+j)->blue;
+            gamma+=alpha;
+          }
+          gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+          q->red=RoundToQuantum(gamma*pixel.red);
+          q->green=RoundToQuantum(gamma*pixel.green);
+          q->blue=RoundToQuantum(gamma*pixel.blue);
+        }
       if ((source->matte != MagickFalse) && (destination->matte != MagickFalse))
         {
           for (i=0; i < n; i++)
@@ -821,13 +1022,32 @@ static MagickBooleanType VerticalFilter(const Image *source,Image *destination,
       if ((source->colorspace == CMYKColorspace) &&
           (destination->colorspace == CMYKColorspace))
         {
-          for (i=0; i < n; i++)
-          {
-            j=(long) ((contribution[i].pixel-contribution[0].pixel)*
-              source->columns+x);
-            pixel.index+=contribution[i].weight*source_indexes[j];
-          }
-          indexes[x]=(IndexPacket) RoundToQuantum(pixel.index);
+          gamma=0.0;
+          if (source->matte == MagickFalse)
+            {
+              for (i=0; i < n; i++)
+              {
+                j=(long) ((contribution[i].pixel-contribution[0].pixel)*
+                  source->columns+x);
+                alpha=contribution[i].weight;
+                pixel.index+=alpha*source_indexes[j];
+              }
+              indexes[x]=(IndexPacket) RoundToQuantum(pixel.index);
+            }
+          else
+            {
+              for (i=0; i < n; i++)
+              {
+                j=(long) ((contribution[i].pixel-contribution[0].pixel)*
+                  source->columns+x);
+                alpha=contribution[i].weight*QuantumScale*((MagickRealType)
+                  QuantumRange-(p+j)->opacity);
+                pixel.index+=alpha*source_indexes[j];
+                gamma+=alpha;
+              }
+              gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+              indexes[x]=(IndexPacket) RoundToQuantum(gamma*pixel.index);
+            }
         }
       if ((destination->storage_class == PseudoClass) &&
           (source->storage_class == PseudoClass))
@@ -851,7 +1071,7 @@ static MagickBooleanType VerticalFilter(const Image *source,Image *destination,
       }
     (*quantum)++;
   }
-  return((MagickBooleanType) (y == (long) destination->rows));
+  return(y == (long) destination->rows ? MagickTrue : MagickFalse);
 }
 
 MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
@@ -902,7 +1122,7 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
       { BlackmanSinc, 4.0f }
     };
 
-  unsigned long
+  MagickOffsetType
     quantum;
 
   /*
@@ -911,20 +1131,24 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   assert(((int) filter >= 0) && ((int) filter <= SincFilter));
   if ((columns == 0) || (rows == 0))
     ThrowImageException(ImageError,"NegativeOrZeroImageSize");
+  if ((columns == image->columns) && (rows == image->rows) &&
+      (filter == UndefinedFilter) && (blur == 1.0))
+    return(CloneImage(image,0,0,MagickTrue,exception));
   resize_image=CloneImage(image,columns,rows,MagickTrue,exception);
   if (resize_image == (Image *) NULL)
     return((Image *) NULL);
   /*
     Allocate filter contribution info.
   */
-  x_factor=(MagickRealType) resize_image->columns/image->columns;
-  y_factor=(MagickRealType) resize_image->rows/image->rows;
+  x_factor=(MagickRealType) resize_image->columns/(MagickRealType)
+    image->columns;
+  y_factor=(MagickRealType) resize_image->rows/(MagickRealType) image->rows;
   i=(long) LanczosFilter;
   if (filter != UndefinedFilter)
     i=(long) filter;
@@ -957,8 +1181,8 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
       source_image=CloneImage(image,columns,image->rows,MagickTrue,exception);
       if (source_image == (Image *) NULL)
         {
-          contribution=(ContributionInfo *)
-            RelinquishMagickMemory(contribution);
+          contribution=(ContributionInfo *) RelinquishMagickMemory(
+            contribution);
           resize_image=DestroyImage(resize_image);
           return((Image *) NULL);
         }
@@ -973,8 +1197,8 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
       source_image=CloneImage(image,image->columns,rows,MagickTrue,exception);
       if (source_image == (Image *) NULL)
         {
-          contribution=(ContributionInfo *)
-            RelinquishMagickMemory(contribution);
+          contribution=(ContributionInfo *) RelinquishMagickMemory(
+             contribution);
           resize_image=DestroyImage(resize_image);
           return((Image *) NULL);
         }
@@ -1068,23 +1292,25 @@ MagickExport Image *SampleImage(const Image *image,const unsigned long columns,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if ((columns == 0) || (rows == 0))
     ThrowImageException(ImageError,"NegativeOrZeroImageSize");
+  if ((columns == image->columns) && (rows == image->rows))
+    return(CloneImage(image,0,0,MagickTrue,exception));
   sample_image=CloneImage(image,columns,rows,MagickTrue,exception);
   if (sample_image == (Image *) NULL)
     return((Image *) NULL);
   /*
     Allocate scan line buffer and column offset buffers.
   */
-  pixels=(PixelPacket *)
-    AcquireMagickMemory((size_t) image->columns*sizeof(*pixels));
-  x_offset=(long *)
-    AcquireMagickMemory((size_t) sample_image->columns*sizeof(*x_offset));
-  y_offset=(long *)
-    AcquireMagickMemory((size_t) sample_image->rows*sizeof(*y_offset));
+  pixels=(PixelPacket *) AcquireMagickMemory((size_t) image->columns*
+    sizeof(*pixels));
+  x_offset=(long *) AcquireMagickMemory((size_t) sample_image->columns*
+    sizeof(*x_offset));
+  y_offset=(long *) AcquireMagickMemory((size_t) sample_image->rows*
+    sizeof(*y_offset));
   if ((pixels == (PixelPacket *) NULL) || (x_offset == (long *) NULL) ||
       (y_offset == (long *) NULL))
     {
@@ -1095,11 +1321,11 @@ MagickExport Image *SampleImage(const Image *image,const unsigned long columns,
     Initialize pixel offsets.
   */
   for (x=0; x < (long) sample_image->columns; x++)
-    x_offset[x]=(long)
-      (((MagickRealType) x+0.5)*image->columns/sample_image->columns);
+    x_offset[x]=(long) (((MagickRealType) x+0.5)*image->columns/
+      sample_image->columns);
   for (y=0; y < (long) sample_image->rows; y++)
-    y_offset[y]=(long)
-      (((MagickRealType) y+0.5)*image->rows/sample_image->rows);
+    y_offset[y]=(long) (((MagickRealType) y+0.5)*image->rows/
+      sample_image->rows);
   /*
     Sample each row.
   */
@@ -1205,11 +1431,9 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
     *y_vector,
     zero;
 
-  MagickRealType
-    x_scale,
-    x_span,
-    y_scale,
-    y_span;
+  PointInfo
+    scale,
+    span;
 
   register const PixelPacket
     *p;
@@ -1235,28 +1459,35 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if ((columns == 0) || (rows == 0))
     return((Image *) NULL);
+  if ((columns == image->columns) && (rows == image->rows))
+    return(CloneImage(image,0,0,MagickTrue,exception));
   scale_image=CloneImage(image,columns,rows,MagickTrue,exception);
   if (scale_image == (Image *) NULL)
     return((Image *) NULL);
-  scale_image->storage_class=DirectClass;
+  if (SetImageStorageClass(scale_image,DirectClass) == MagickFalse)
+    {
+      InheritException(exception,&scale_image->exception);
+      scale_image=DestroyImage(scale_image);
+      return((Image *) NULL);
+    }
   /*
     Allocate memory.
   */
-  x_vector=(MagickPixelPacket *)
-    AcquireMagickMemory((size_t) image->columns*sizeof(*x_vector));
+  x_vector=(MagickPixelPacket *) AcquireMagickMemory((size_t) image->columns*
+    sizeof(*x_vector));
   scanline=x_vector;
   if (image->rows != scale_image->rows)
-    scanline=(MagickPixelPacket *)
-      AcquireMagickMemory((size_t) image->columns*sizeof(*scanline));
+    scanline=(MagickPixelPacket *) AcquireMagickMemory((size_t) image->columns*
+      sizeof(*scanline));
   scale_scanline=(MagickPixelPacket *) AcquireMagickMemory((size_t)
     scale_image->columns*sizeof(*scale_scanline));
-  y_vector=(MagickPixelPacket *)
-    AcquireMagickMemory((size_t) image->columns*sizeof(*y_vector));
+  y_vector=(MagickPixelPacket *) AcquireMagickMemory((size_t) image->columns*
+    sizeof(*y_vector));
   if ((scanline == (MagickPixelPacket *) NULL) ||
       (scale_scanline == (MagickPixelPacket *) NULL) ||
       (x_vector == (MagickPixelPacket *) NULL) ||
@@ -1270,8 +1501,8 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
   */
   number_rows=0;
   next_row=MagickTrue;
-  y_span=1.0;
-  y_scale=(MagickRealType) scale_image->rows/(MagickRealType) image->rows;
+  span.y=1.0;
+  scale.y=(double) scale_image->rows/(double) image->rows;
   (void) ResetMagickMemory(y_vector,0,(size_t) image->columns*
     sizeof(*y_vector));
   GetMagickPixelPacket(image,&pixel);
@@ -1309,7 +1540,7 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
         /*
           Scale Y direction.
         */
-        while (y_scale < y_span)
+        while (scale.y < span.y)
         {
           if ((next_row != MagickFalse) && (number_rows < (long) image->rows))
             {
@@ -1335,17 +1566,16 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
             }
           for (x=0; x < (long) image->columns; x++)
           {
-            y_vector[x].red+=y_scale*x_vector[x].red;
-            y_vector[x].green+=y_scale*x_vector[x].green;
-            y_vector[x].blue+=y_scale*x_vector[x].blue;
+            y_vector[x].red+=scale.y*x_vector[x].red;
+            y_vector[x].green+=scale.y*x_vector[x].green;
+            y_vector[x].blue+=scale.y*x_vector[x].blue;
             if (scale_image->matte != MagickFalse)
-              y_vector[x].opacity+=y_scale*x_vector[x].opacity;
+              y_vector[x].opacity+=scale.y*x_vector[x].opacity;
             if (scale_indexes != (IndexPacket *) NULL)
-              y_vector[x].index+=y_scale*x_vector[x].index;
+              y_vector[x].index+=scale.y*x_vector[x].index;
           }
-          y_span-=y_scale;
-          y_scale=(MagickRealType) scale_image->rows/
-            (MagickRealType) image->rows;
+          span.y-=scale.y;
+          scale.y=(double) scale_image->rows/(double) image->rows;
           next_row=MagickTrue;
         }
         if ((next_row != MagickFalse) && (number_rows < (long) image->rows))
@@ -1374,31 +1604,30 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
         s=scanline;
         for (x=0; x < (long) image->columns; x++)
         {
-          pixel.red=y_vector[x].red+y_span*x_vector[x].red;
-          pixel.green=y_vector[x].green+y_span*x_vector[x].green;
-          pixel.blue=y_vector[x].blue+y_span*x_vector[x].blue;
+          pixel.red=y_vector[x].red+span.y*x_vector[x].red;
+          pixel.green=y_vector[x].green+span.y*x_vector[x].green;
+          pixel.blue=y_vector[x].blue+span.y*x_vector[x].blue;
           if (image->matte != MagickFalse)
-            pixel.opacity=y_vector[x].opacity+y_span*x_vector[x].opacity;
+            pixel.opacity=y_vector[x].opacity+span.y*x_vector[x].opacity;
           if (scale_indexes != (IndexPacket *) NULL)
-            pixel.index=y_vector[x].index+y_span*x_vector[x].index;
-          s->red=ClipToQuantum(pixel.red);
-          s->green=ClipToQuantum(pixel.green);
-          s->blue=ClipToQuantum(pixel.blue);
+            pixel.index=y_vector[x].index+span.y*x_vector[x].index;
+          s->red=pixel.red;
+          s->green=pixel.green;
+          s->blue=pixel.blue;
           if (scale_image->matte != MagickFalse)
-            s->opacity=ClipToQuantum(pixel.opacity);
+            s->opacity=pixel.opacity;
           if (scale_indexes != (IndexPacket *) NULL)
-            s->index=ClipToQuantum(pixel.index);
+            s->index=pixel.index;
           s++;
           y_vector[x]=zero;
         }
-        y_scale-=y_span;
-        if (y_scale <= 0)
+        scale.y-=span.y;
+        if (scale.y <= 0)
           {
-            y_scale=(MagickRealType) scale_image->rows/
-              (MagickRealType) image->rows;
+            scale.y=(double) scale_image->rows/(double) image->rows;
             next_row=MagickTrue;
           }
-        y_span=1.0;
+        span.y=1.0;
       }
     if (scale_image->columns == image->columns)
       {
@@ -1408,13 +1637,13 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
         s=scanline;
         for (x=0; x < (long) scale_image->columns; x++)
         {
-          q->red=(Quantum) (s->red+0.5);
-          q->green=(Quantum) (s->green+0.5);
-          q->blue=(Quantum) (s->blue+0.5);
+          q->red=RoundToQuantum(s->red);
+          q->green=RoundToQuantum(s->green);
+          q->blue=RoundToQuantum(s->blue);
           if (scale_image->matte != MagickFalse)
-            q->opacity=(Quantum) (s->opacity+0.5);
+            q->opacity=RoundToQuantum(s->opacity);
           if (scale_indexes != (IndexPacket *) NULL)
-            scale_indexes[x]=(IndexPacket) (s->index+0.5);
+            scale_indexes[x]=(IndexPacket) RoundToQuantum(s->index);
           q++;
           s++;
         }
@@ -1426,39 +1655,38 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
         */
         pixel=zero;
         next_column=MagickFalse;
-        x_span=1.0;
+        span.x=1.0;
         s=scanline;
         t=scale_scanline;
         for (x=0; x < (long) image->columns; x++)
         {
-          x_scale=(MagickRealType) scale_image->columns/
-            (MagickRealType) image->columns;
-          while (x_scale >= x_span)
+          scale.x=(double) scale_image->columns/(double) image->columns;
+          while (scale.x >= span.x)
           {
             if (next_column != MagickFalse)
               {
                 pixel=zero;
                 t++;
               }
-            pixel.red+=x_span*s->red;
-            pixel.green+=x_span*s->green;
-            pixel.blue+=x_span*s->blue;
+            pixel.red+=span.x*s->red;
+            pixel.green+=span.x*s->green;
+            pixel.blue+=span.x*s->blue;
             if (image->matte != MagickFalse)
-              pixel.opacity+=x_span*s->opacity;
+              pixel.opacity+=span.x*s->opacity;
             if (scale_indexes != (IndexPacket *) NULL)
-              pixel.index+=x_span*s->index;
-            t->red=ClipToQuantum(pixel.red);
-            t->green=ClipToQuantum(pixel.green);
-            t->blue=ClipToQuantum(pixel.blue);
+              pixel.index+=span.x*s->index;
+            t->red=pixel.red;
+            t->green=pixel.green;
+            t->blue=pixel.blue;
             if (scale_image->matte != MagickFalse)
-              t->opacity=ClipToQuantum(pixel.opacity);
+              t->opacity=pixel.opacity;
             if (scale_indexes != (IndexPacket *) NULL)
-              t->index=ClipToQuantum(pixel.index);
-            x_scale-=x_span;
-            x_span=1.0;
+              t->index=pixel.index;
+            scale.x-=span.x;
+            span.x=1.0;
             next_column=MagickTrue;
           }
-        if (x_scale > 0)
+        if (scale.x > 0)
           {
             if (next_column != MagickFalse)
               {
@@ -1466,38 +1694,38 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
                 next_column=MagickFalse;
                 t++;
               }
-            pixel.red+=x_scale*s->red;
-            pixel.green+=x_scale*s->green;
-            pixel.blue+=x_scale*s->blue;
+            pixel.red+=scale.x*s->red;
+            pixel.green+=scale.x*s->green;
+            pixel.blue+=scale.x*s->blue;
             if (scale_image->matte != MagickFalse)
-              pixel.opacity+=x_scale*s->opacity;
+              pixel.opacity+=scale.x*s->opacity;
             if (scale_indexes != (IndexPacket *) NULL)
-              pixel.index+=x_scale*s->index;
-            x_span-=x_scale;
+              pixel.index+=scale.x*s->index;
+            span.x-=scale.x;
           }
         s++;
       }
-      if (x_span > 0)
+      if (span.x > 0)
         {
           s--;
-          pixel.red+=x_span*s->red;
-          pixel.green+=x_span*s->green;
-          pixel.blue+=x_span*s->blue;
+          pixel.red+=span.x*s->red;
+          pixel.green+=span.x*s->green;
+          pixel.blue+=span.x*s->blue;
           if (scale_image->matte != MagickFalse)
-            pixel.opacity+=x_span*s->opacity;
+            pixel.opacity+=span.x*s->opacity;
           if (scale_indexes != (IndexPacket *) NULL)
-            pixel.index+=x_span*s->index;
+            pixel.index+=span.x*s->index;
         }
       if ((next_column == MagickFalse) &&
           ((long) (t-scale_scanline) < (long) scale_image->columns))
         {
-          t->red=ClipToQuantum(pixel.red);
-          t->green=ClipToQuantum(pixel.green);
-          t->blue=ClipToQuantum(pixel.blue);
+          t->red=pixel.red;
+          t->green=pixel.green;
+          t->blue=pixel.blue;
           if (scale_image->matte != MagickFalse)
-            t->opacity=ClipToQuantum(pixel.opacity);
+            t->opacity=pixel.opacity;
           if (scale_indexes != (IndexPacket *) NULL)
-            t->index=ClipToQuantum(pixel.index);
+            t->index=pixel.index;
         }
       /*
         Transfer scanline to scaled image.
@@ -1505,13 +1733,13 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
       t=scale_scanline;
       for (x=0; x < (long) scale_image->columns; x++)
       {
-        q->red=(Quantum) (t->red+0.5);
-        q->green=(Quantum) (t->green+0.5);
-        q->blue=(Quantum) (t->blue+0.5);
+        q->red=RoundToQuantum(t->red);
+        q->green=RoundToQuantum(t->green);
+        q->blue=RoundToQuantum(t->blue);
         if (scale_image->matte != MagickFalse)
-          q->opacity=(Quantum) (t->opacity+0.5);
+          q->opacity=RoundToQuantum(t->opacity);
         if (scale_indexes != (IndexPacket *) NULL)
-          scale_indexes[x]=(IndexPacket) (t->index+0.5);
+          scale_indexes[x]=(IndexPacket) RoundToQuantum(t->index);
         t++;
         q++;
       }
@@ -1573,6 +1801,12 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
 MagickExport Image *ThumbnailImage(const Image *image,
   const unsigned long columns,const unsigned long rows,ExceptionInfo *exception)
 {
+  char
+    value[MaxTextExtent];
+
+  const ImageAttribute
+    *attribute;
+
   Image
     *sample_image,
     *thumbnail_image;
@@ -1581,10 +1815,16 @@ MagickExport Image *ThumbnailImage(const Image *image,
     x_factor,
     y_factor;
 
+  struct stat
+    attributes;
+
+  unsigned long
+    version;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   x_factor=(MagickRealType) columns/(MagickRealType) image->columns;
@@ -1592,7 +1832,8 @@ MagickExport Image *ThumbnailImage(const Image *image,
   if ((x_factor*y_factor) > 0.1)
     {
       thumbnail_image=ZoomImage(image,columns,rows,exception);
-      (void) StripImage(thumbnail_image);
+      if (thumbnail_image != (Image *) NULL)
+        (void) StripImage(thumbnail_image);
       return(thumbnail_image);
     }
   sample_image=SampleImage(image,5*columns,5*rows,exception);
@@ -1600,7 +1841,43 @@ MagickExport Image *ThumbnailImage(const Image *image,
     return((Image *) NULL);
   thumbnail_image=ZoomImage(sample_image,columns,rows,exception);
   sample_image=DestroyImage(sample_image);
+  if (thumbnail_image == (Image *) NULL)
+    return(thumbnail_image);
+  if (thumbnail_image->matte == MagickFalse)
+    SetImageOpacity(thumbnail_image,OpaqueOpacity);
+  thumbnail_image->depth=8;
+  thumbnail_image->interlace=NoInterlace;
   (void) StripImage(thumbnail_image);
+  (void) CopyMagickString(value,image->magick_filename,MaxTextExtent);
+  if (strstr(image->magick_filename,"///") == (char *) NULL)
+    (void) FormatMagickString(value,MaxTextExtent,"file:///%s",
+      image->magick_filename);
+  (void) SetImageAttribute(thumbnail_image,"Thumb::URI",value);
+  (void) CopyMagickString(value,image->magick_filename,MaxTextExtent);
+  if (stat(image->filename,&attributes) == 0)
+    {
+      (void) FormatMagickString(value,MaxTextExtent,"%ld",attributes.st_mtime);
+      (void) SetImageAttribute(thumbnail_image,"Thumb::MTime",value);
+    }
+  (void) FormatMagickString(value,MaxTextExtent,"%ld",attributes.st_mtime);
+  (void) FormatMagickSize(GetBlobSize(image),value);
+  (void) SetImageAttribute(thumbnail_image,"Thumb::Size",value);
+  (void) FormatMagickString(value,MaxTextExtent,"image/%s",image->magick);
+  LocaleLower(value);
+  (void) SetImageAttribute(thumbnail_image,"Thumb::Mimetype",value);
+  attribute=GetImageAttribute(image,"comment");
+  if ((attribute != (const ImageAttribute *) NULL) &&
+      (attribute->value != (char *) NULL))
+    (void) SetImageAttribute(thumbnail_image,"Description",attribute->value);
+  (void) SetImageAttribute(thumbnail_image,"Software",
+    GetMagickVersion(&version));
+  (void) FormatMagickString(value,MaxTextExtent,"%lu",image->magick_columns);
+  (void) SetImageAttribute(thumbnail_image,"Thumb::Image::Width",value);
+  (void) FormatMagickString(value,MaxTextExtent,"%lu",image->magick_rows);
+  (void) SetImageAttribute(thumbnail_image,"Thumb::Image::height",value);
+  (void) FormatMagickString(value,MaxTextExtent,"%lu",
+    GetImageListLength(image));
+  (void) SetImageAttribute(thumbnail_image,"Thumb::Document::Pages",value);
   return(thumbnail_image);
 }
 
@@ -1609,7 +1886,7 @@ MagickExport Image *ThumbnailImage(const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   Z o o m I m a g e                                                         %
+%   Z o o m I m a g e                                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -1645,7 +1922,6 @@ MagickExport Image *ThumbnailImage(const Image *image,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport Image *ZoomImage(const Image *image,const unsigned long columns,
   const unsigned long rows,ExceptionInfo *exception)
@@ -1656,7 +1932,7 @@ MagickExport Image *ZoomImage(const Image *image,const unsigned long columns,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   zoom_image=ResizeImage(image,columns,rows,image->filter,image->blur,
