@@ -90,7 +90,7 @@ typedef struct _ResourceInfo
 */
 static ResourceInfo
   resource_info =
-    { 0, 0, 0, 0, 0, 384, 768, 1536, ResourceInfinity, 768 };
+    { 0, 0, 0, 0, 0, 2048, 1536, 8192, ResourceInfinity, 768 };
 
 static SemaphoreInfo
   *resource_semaphore = (SemaphoreInfo *) NULL;
@@ -141,7 +141,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     limit;
 
   status=MagickFalse;
-  FormatSize(size,resource_request);
+  (void) FormatMagickSize(size,resource_request);
   AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
@@ -151,8 +151,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       limit=MegabytesToBytes(resource_info.area_limit);
       status=(resource_info.area_limit == ResourceInfinity) || (size < limit) ?
         MagickTrue : MagickFalse;
-      FormatSize((MagickSizeType) resource_info.area,resource_current);
-      FormatSize(MegabytesToBytes(resource_info.area_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.area,
+        resource_current);
+      (void) FormatMagickSize(MegabytesToBytes(resource_info.area_limit),
+        resource_limit);
       break;
     }
     case MemoryResource:
@@ -162,8 +164,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.memory_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.memory < limit) ?
         MagickTrue : MagickFalse;
-      FormatSize((MagickSizeType) resource_info.memory,resource_current);
-      FormatSize(MegabytesToBytes(resource_info.memory_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.memory,
+        resource_current);
+      (void) FormatMagickSize(MegabytesToBytes(resource_info.memory_limit),
+        resource_limit);
       break;
     }
     case MapResource:
@@ -173,8 +177,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.map_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.map < limit) ?
         MagickTrue : MagickFalse;
-      FormatSize((MagickSizeType) resource_info.map,resource_current);
-      FormatSize(MegabytesToBytes(resource_info.map_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.map,
+        resource_current);
+      (void) FormatMagickSize(MegabytesToBytes(resource_info.map_limit),
+        resource_limit);
       break;
     }
     case DiskResource:
@@ -184,8 +190,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.disk_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.disk < limit) ?
         MagickTrue : MagickFalse;
-      FormatSize((MagickSizeType) resource_info.disk,resource_current);
-      FormatSize(GigabytesToBytes(resource_info.disk_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.disk,
+        resource_current);
+      (void) FormatMagickSize(GigabytesToBytes(resource_info.disk_limit),
+        resource_limit);
       break;
     }
     case FileResource:
@@ -195,8 +203,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.file_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.file < limit) ?
         MagickTrue : MagickFalse;
-      FormatSize((MagickSizeType) resource_info.file,resource_current);
-      FormatSize((MagickSizeType) resource_info.file_limit,resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.file,
+        resource_current);
+      (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,
+        resource_limit);
       break;
     }
     default:
@@ -277,7 +287,7 @@ MagickExport void AsynchronousDestroyMagickResources(void)
 
 static void *DestroyTemporaryResources(void *temporary_resource)
 {
-  remove((char *) temporary_resource);
+  (void) remove((char *) temporary_resource);
   return((void *) NULL);
 }
 
@@ -289,6 +299,9 @@ static MagickBooleanType GetPathTemplate(char *path)
   int
     status;
 
+  register char
+    *p;
+
   struct stat
     file_info;
 
@@ -329,6 +342,10 @@ static MagickBooleanType GetPathTemplate(char *path)
     (void) FormatMagickString(path,MaxTextExtent,"%s%smagick-XXXXXXXX",
       directory,DirectorySeparator);
   directory=(char *) RelinquishMagickMemory(directory);
+  if (*DirectorySeparator != '/')
+    for (p=path; *p != '\0'; p++)
+      if (*p == *DirectorySeparator)
+        *p='/';
   return(MagickTrue);
 }
 
@@ -354,6 +371,10 @@ MagickExport int AcquireUniqueFileResource(char *path)
   register long
     i;
 
+  static const char
+    portable_filename[65] =
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789._";
+
   unsigned char
     key[8];
 
@@ -375,8 +396,8 @@ MagickExport int AcquireUniqueFileResource(char *path)
     p=path+strlen(path)-8;
     for (i=0; i < 8; i++)
     {
-      c=(int) (key[i] & 0x1f);
-      *p++=(char) (c > 9 ? c+(int) 'a'-10 : c+(int) '0');
+      c=(int) (key[i] & 0x3f);
+      *p++=portable_filename[c];
     }
     file=open(path,O_RDWR | O_CREAT | O_EXCL | O_BINARY | O_NOFOLLOW,S_MODE);
     if ((file > 0) || (errno != EEXIST))
@@ -603,9 +624,9 @@ MagickExport void InitializeMagickResources(void)
 #if defined(PixelCacheThreshold)
   memory=PixelCacheThreshold;
 #endif
-  (void) SetMagickResourceLimit(AreaResource,memory);
-  (void) SetMagickResourceLimit(MemoryResource,2*memory);
-  (void) SetMagickResourceLimit(MapResource,4*memory);
+  (void) SetMagickResourceLimit(AreaResource,2*memory);
+  (void) SetMagickResourceLimit(MemoryResource,3*memory/2);
+  (void) SetMagickResourceLimit(MapResource,8*memory);
   limit=GetEnvironmentValue("MAGICK_AREA_LIMIT");
   if (limit != (char *) NULL)
     {
@@ -636,8 +657,8 @@ MagickExport void InitializeMagickResources(void)
 #elif defined(HAVE_GETDTABLESIZE) && defined(POSIX)
   files=getdtablesize();
 #endif
-  (void) SetMagickResourceLimit(FileResource,(unsigned long) Max(files-64,64));
-  limit=GetEnvironmentValue("MAGICK_FILES_LIMIT");
+  (void) SetMagickResourceLimit(FileResource,(unsigned long) Max(3*files/4,64));
+  limit=GetEnvironmentValue("MAGICK_FILE_LIMIT");
   if (limit != (char *) NULL)
     {
       (void) SetMagickResourceLimit(FileResource,(unsigned long) atol(limit));
@@ -672,7 +693,7 @@ MagickExport void InitializeMagickResources(void)
 %
 */
 MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
-  ExceptionInfo *exception)
+  ExceptionInfo *magick_unused(exception))
 {
   char
     area_limit[MaxTextExtent],
@@ -684,14 +705,17 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
   if (file == (const FILE *) NULL)
     file=stdout;
   AcquireSemaphoreInfo(&resource_semaphore);
-  FormatSize(MegabytesToBytes(resource_info.area_limit),area_limit);
-  FormatSize(GigabytesToBytes(resource_info.disk_limit),disk_limit);
-  FormatSize((MagickSizeType) resource_info.file_limit,file_limit);
-  FormatSize(MegabytesToBytes(resource_info.map_limit),map_limit);
-  FormatSize(MegabytesToBytes(resource_info.memory_limit),memory_limit);
+  (void) FormatMagickSize(MegabytesToBytes(resource_info.area_limit),
+    area_limit);
+  (void) FormatMagickSize(GigabytesToBytes(resource_info.disk_limit),
+    disk_limit);
+  (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,file_limit);
+  (void) FormatMagickSize(MegabytesToBytes(resource_info.map_limit),map_limit);
+  (void) FormatMagickSize(MegabytesToBytes(resource_info.memory_limit),
+    memory_limit);
   (void) fprintf(file,"File       Area     Memory        Map       Disk\n");
   (void) fprintf(file,"------------------------------------------------\n");
-  (void) fprintf(file,"%4s %10s %10s %10s %10s\n",file_limit,area_limit,
+  (void) fprintf(file,"%4s  %9s  %9s  %9s  %9s\n",file_limit,area_limit,
     memory_limit,map_limit,disk_limit);
   (void) fflush(file);
   RelinquishSemaphoreInfo(resource_semaphore);
@@ -732,43 +756,53 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     resource_limit[MaxTextExtent],
     resource_request[MaxTextExtent];
 
-  FormatSize(size,resource_request);
+  (void) FormatMagickSize(size,resource_request);
   AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
     case AreaResource:
     {
       resource_info.area=(MagickOffsetType) size;
-      FormatSize((MagickSizeType) resource_info.area,resource_current);
-      FormatSize(MegabytesToBytes(resource_info.area_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.area,
+        resource_current);
+      (void) FormatMagickSize(MegabytesToBytes(resource_info.area_limit),
+        resource_limit);
       break;
     }
     case MemoryResource:
     {
       resource_info.memory-=size;
-      FormatSize((MagickSizeType) resource_info.memory,resource_current);
-      FormatSize(MegabytesToBytes(resource_info.memory_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.memory,
+        resource_current);
+      (void) FormatMagickSize(MegabytesToBytes(resource_info.memory_limit),
+        resource_limit);
       break;
     }
     case MapResource:
     {
       resource_info.map-=size;
-      FormatSize((MagickSizeType) resource_info.map,resource_current);
-      FormatSize(MegabytesToBytes(resource_info.map_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.map,
+        resource_current);
+      (void) FormatMagickSize(MegabytesToBytes(resource_info.map_limit),
+        resource_limit);
       break;
     }
     case DiskResource:
     {
       resource_info.disk-=size;
-      FormatSize((MagickSizeType) resource_info.disk,resource_current);
-      FormatSize(GigabytesToBytes(resource_info.disk_limit),resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.disk,
+        resource_current);
+      (void) FormatMagickSize(GigabytesToBytes(resource_info.disk_limit),
+        resource_limit);
       break;
     }
     case FileResource:
     {
       resource_info.file-=size;
-      FormatSize((MagickSizeType) resource_info.file,resource_current);
-      FormatSize((MagickSizeType) resource_info.file_limit,resource_limit);
+      (void) FormatMagickSize((MagickSizeType) resource_info.file,
+        resource_current);
+      (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,
+        resource_limit);
       break;
     }
     default:
