@@ -17,7 +17,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,8 +33,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RotateImage(), XShearImage(), and YShearImage() is based on the paper
-%  "A Fast Algorithm for General Raster Rotatation" by Alan W. Paeth,
+%  The RotateImage, XShearImage, and YShearImage methods are based on the
+%  paper "A Fast Algorithm for General Raster Rotatation" by Alan W. Paeth,
 %  Graphics Interface '86 (Vancouver).  RotateImage is adapted from a similar
 %  method based on the Paeth paper written by Michael Halle of the Spatial
 %  Imaging Group, MIT Media Lab.
@@ -52,10 +52,13 @@
 #include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/gem.h"
+#include "magick/geometry.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/list.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/shear.h"
 #include "magick/transform.h"
 
@@ -63,6 +66,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %     A f f i n e T r a n s f o r m I m a g e                                 %
 %                                                                             %
 %                                                                             %
@@ -75,21 +79,21 @@
 %
 %  The format of the AffineTransformImage method is:
 %
-%      Image *AffineTransformImage(const Image *image,AffineMatrix *affine,
-%        ExceptionInfo *exception)
+%      Image *AffineTransformImage(const Image *image,
+%        AffineMatrix *affine_matrix,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: The image.
 %
-%    o affine: The affine transform.
+%    o affine_matrix: The affine matrix.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
 %
 */
 MagickExport Image *AffineTransformImage(const Image *image,
-  const AffineMatrix *affine,ExceptionInfo *exception)
+  const AffineMatrix *affine_matrix,ExceptionInfo *exception)
 {
   AffineMatrix
     transform;
@@ -113,7 +117,7 @@ MagickExport Image *AffineTransformImage(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(affine != (AffineMatrix *) NULL);
+  assert(affine_matrix != (AffineMatrix *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   extent[0].x=0.0;
@@ -127,8 +131,10 @@ MagickExport Image *AffineTransformImage(const Image *image,
   for (i=0; i < 4; i++)
   {
     point=extent[i];
-    extent[i].x=(double) (point.x*affine->sx+point.y*affine->ry+affine->tx);
-    extent[i].y=(double) (point.x*affine->rx+point.y*affine->sy+affine->ty);
+    extent[i].x=(double) (point.x*affine_matrix->sx+point.y*affine_matrix->ry+
+      affine_matrix->tx);
+    extent[i].y=(double) (point.x*affine_matrix->rx+point.y*affine_matrix->sy+
+       affine_matrix->ty);
   }
   min=extent[0];
   max=extent[0];
@@ -152,13 +158,26 @@ MagickExport Image *AffineTransformImage(const Image *image,
     return((Image *) NULL);
   affine_image->background_color.opacity=TransparentOpacity;
   SetImageBackgroundColor(affine_image);
-  transform.sx=affine->sx;
-  transform.rx=affine->rx;
-  transform.ry=affine->ry;
-  transform.sy=affine->sy;
-  transform.tx=min.x;
-  transform.ty=min.y;
+  transform=(*affine_matrix);
+  if ((affine_matrix->sx*affine_matrix->sy) > 0)
+    {
+      transform.tx=0;
+      transform.ty=0;
+    }
+  if ((affine_matrix->tx == 0.0) && (affine_matrix->ty == 0.0))
+    {
+      transform.tx=(-min.x);
+      transform.ty=(-min.y);
+    }
   (void) DrawAffineImage(affine_image,image,&transform);
+  affine_image->page.x=(long) floor(min.x+0.5);
+  affine_image->page.y=(long) floor(min.y+0.5);
+  affine_image->page.width=affine_image->columns;
+  if (affine_image->page.x > 0)
+    affine_image->page.width+=affine_image->page.x;
+  affine_image->page.height=affine_image->rows;
+  if (affine_image->page.y > 0)
+    affine_image->page.height+=affine_image->page.y;
   return(affine_image);
 }
 
@@ -207,7 +226,8 @@ static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
     max;
 
   RectangleInfo
-    geometry;
+    geometry,
+    page;
 
   register long
     i;
@@ -247,12 +267,15 @@ static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
   }
   geometry.x=(long) (min.x+0.5);
   geometry.y=(long) (min.y+0.5);
-  geometry.width=(unsigned long) (max.x-min.x+0.5);
-  geometry.height=(unsigned long) (max.y-min.y+0.5);
+  geometry.width=(unsigned long) ((long) (max.x+0.5)-(long) (min.x+0.5));
+  geometry.height=(unsigned long) ((long) (max.y+0.5)-(long) (min.y+0.5));
+  page=(*image)->page;
+  (void) ParseAbsoluteGeometry("0x0+0+0",&(*image)->page);
   crop_image=CropImage(*image,&geometry,exception);
+  (*image)->page=page;
   if (crop_image != (Image *) NULL)
     {
-      crop_image->page=(*image)->page;
+      crop_image->page=page;
       *image=DestroyImage(*image);
       *image=crop_image;
     }
@@ -530,6 +553,11 @@ static inline void MagickCompositeBlend(const PixelPacket *p,
   MagickRealType
     gamma;
 
+  if ((alpha == TransparentOpacity) && (beta == TransparentOpacity))
+    {
+      *composite=(*p);
+      return;
+    }
   gamma=RoundToUnity((1.0-QuantumScale*(QuantumRange-(1.0-area)*
     (QuantumRange-alpha)))+(1.0-QuantumScale*(QuantumRange-area*
     (QuantumRange-beta))));
@@ -546,7 +574,7 @@ static inline void MagickCompositeBlend(const PixelPacket *p,
     QuantumRange-area*(QuantumRange-beta)));
 }
 
-static inline void XShearImage(Image *image,const MagickRealType degrees,
+static void XShearImage(Image *image,const MagickRealType degrees,
   const unsigned long width,const unsigned long height,const long x_offset,
   long y_offset)
 {
@@ -710,7 +738,7 @@ static inline void XShearImage(Image *image,const MagickRealType degrees,
 %
 %
 */
-static inline void YShearImage(Image *image,const MagickRealType degrees,
+static void YShearImage(Image *image,const MagickRealType degrees,
   const unsigned long width,const unsigned long height,long x_offset,
   const long y_offset)
 {
@@ -927,7 +955,12 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   shear.y=sin((double) DegreesToRadians(angle));
   if ((shear.x == 0.0) && (shear.y == 0.0))
     return(integral_image);
-  integral_image->storage_class=DirectClass;
+  if (SetImageStorageClass(integral_image,DirectClass) == MagickFalse)
+    {
+      InheritException(exception,&integral_image->exception);
+      integral_image=DestroyImage(integral_image);
+      return(integral_image);
+    }
   if (integral_image->matte == MagickFalse)
     SetImageOpacity(integral_image,OpaqueOpacity);
   /*
@@ -940,13 +973,14 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
       width=image->rows;
       height=image->columns;
     }
-  x_offset=(long) (fabs(2.0*(double) height*shear.y)+0.5);
-  y_width=(unsigned long) (fabs((double) height*shear.x)+width+0.5);
-  y_offset=(long) (fabs((double) y_width*shear.y)+0.5);
+  y_width=width+(long) (fabs(shear.x)*height+0.5);
+  x_offset=(long) (width+((fabs(shear.y)*height+0.5)-width)/2.0+0.5);
+  y_offset=(long) (height+((fabs(shear.y)*y_width+0.5)-height)/2.0+0.5);
   /*
     Surround image with a border.
   */
   integral_image->border_color=integral_image->background_color;
+  integral_image->compose=OverCompositeOp;
   border_info.width=(unsigned long) x_offset;
   border_info.height=(unsigned long) y_offset;
   rotate_image=BorderImage(integral_image,&border_info,exception);
@@ -1049,20 +1083,27 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   shear.y=tan(DegreesToRadians(y_shear));
   if ((shear.x == 0.0) && (shear.y == 0.0))
     return(integral_image);
-  integral_image->storage_class=DirectClass;
+  if (SetImageStorageClass(integral_image,DirectClass) == MagickFalse)
+    {
+      InheritException(exception,&integral_image->exception);
+      integral_image=DestroyImage(integral_image);
+      return(integral_image);
+    }
   if (integral_image->matte == MagickFalse)
     SetImageOpacity(integral_image,OpaqueOpacity);
   /*
     Compute image size.
   */
-  x_offset=(long) (fabs((double) image->rows*shear.x)+0.5);
-  y_width=(unsigned long)
-    (fabs((double) image->rows*shear.x)+image->columns+0.5);
-  y_offset=(long) (fabs((double) y_width*shear.y)+0.5);
+  y_width=image->columns+(long) (fabs(shear.x)*image->rows+0.5);
+  x_offset=(long) (image->columns+((fabs(shear.x)*image->rows)-
+    image->columns)/2.0+0.5);
+  y_offset=(long) (image->rows+((fabs(shear.y)*y_width+0.5)-image->rows)/2.0+
+    0.5);
   /*
     Surround image with border.
   */
   integral_image->border_color=integral_image->background_color;
+  integral_image->compose=OverCompositeOp;
   border_info.width=(unsigned long) x_offset;
   border_info.height=(unsigned long) y_offset;
   shear_image=BorderImage(integral_image,&border_info,exception);
