@@ -60,7 +60,7 @@
 /*
   Define declarations.
 */
-#define MaxSplayTreeDepth  2048
+#define MaxSplayTreeDepth  1024
 
 /*
   Typedef declarations.
@@ -490,6 +490,9 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_info)
   register NodeInfo
     *node;
 
+  void
+    *key;
+
   assert(splay_info != (SplayTreeInfo *) NULL);
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
@@ -507,8 +510,9 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_info)
         node=node->left;
       splay_info->next=node->key;
     }
+  key=splay_info->root->key;
   RelinquishSemaphoreInfo(splay_info->semaphore);
-  return(splay_info->root->key);
+  return(key);
 }
 
 /*
@@ -540,6 +544,9 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_info)
   register NodeInfo
     *node;
 
+  void
+    *value;
+
   assert(splay_info != (SplayTreeInfo *) NULL);
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
@@ -557,8 +564,9 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_info)
         node=node->left;
       splay_info->next=node->key;
     }
+  value=splay_info->root->value;
   RelinquishSemaphoreInfo(splay_info->semaphore);
-  return(splay_info->root->value);
+  return(value);
 }
 
 /*
@@ -591,6 +599,9 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_info,
   int
     compare;
 
+  void
+    *value;
+
   assert(splay_info != (SplayTreeInfo *) NULL);
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
@@ -609,8 +620,9 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_info,
       RelinquishSemaphoreInfo(splay_info->semaphore);
       return((void *) NULL);
     }
+  value=splay_info->root->value;
   RelinquishSemaphoreInfo(splay_info->semaphore);
-  return(splay_info->root->value);
+  return(value);
 }
 
 /*
@@ -909,8 +921,59 @@ MagickExport MagickBooleanType RemoveNodeByValueFromSplayTree(
       }
     if (splay_info->root->value == value)
       {
+        int
+          compare;
+
+        register NodeInfo
+          *left,
+          *right;
+
+        void
+          *key;
+
+        /*
+          We found the node that matches the value; now remove it.
+        */
+        key=splay_info->root->key;
+        SplaySplayTree(splay_info,key);
+        splay_info->key=(void *) NULL;
+        if (splay_info->compare != (int (*)(const void *,const void *)) NULL)
+          compare=splay_info->compare(splay_info->root->key,key);
+        else
+          compare=(splay_info->root->key > key) ? 1 :
+            ((splay_info->root->key < key) ? -1 : 0);
+        if (compare != 0)
+          {
+            RelinquishSemaphoreInfo(splay_info->semaphore);
+            return(MagickFalse);
+          }
+        left=splay_info->root->left;
+        right=splay_info->root->right;
+        if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
+            (splay_info->root->value != (void *) NULL))
+          splay_info->root->value=splay_info->relinquish_value(
+            splay_info->root->value);
+        if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
+            (splay_info->root->key != (void *) NULL))
+          splay_info->root->key=splay_info->relinquish_key(
+            splay_info->root->key);
+        splay_info->root=(NodeInfo *) RelinquishMagickMemory(splay_info->root);
+        splay_info->nodes--;
+        if (left == (NodeInfo *) NULL)
+          {
+            splay_info->root=right;
+            RelinquishSemaphoreInfo(splay_info->semaphore);
+            return(MagickTrue);
+          }
+        splay_info->root=left;
+        if (right != (NodeInfo *) NULL)
+          {
+            while (left->right != (NodeInfo *) NULL)
+              left=left->right;
+            left->right=right;
+          }
         RelinquishSemaphoreInfo(splay_info->semaphore);
-        return(RemoveNodeFromSplayTree(splay_info,splay_info->root->key));
+        return(MagickTrue);
       }
   }
   RelinquishSemaphoreInfo(splay_info->semaphore);
@@ -1029,7 +1092,9 @@ MagickExport void ResetSplayTreeIterator(SplayTreeInfo *splay_info)
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  AcquireSemaphoreInfo(&splay_info->semaphore);
   splay_info->next=GetFirstSplayTreeNode(splay_info);
+  RelinquishSemaphoreInfo(splay_info->semaphore);
 }
 
 /*
