@@ -495,9 +495,9 @@ MagickExport MagickBooleanType ConcatenateString(char **destination,
 %  ConcatenateStringInfo() concatenates the source string to the destination
 %  string.
 %
-%  The format of the ConcatentateStringInfo method is:
+%  The format of the ConcatenateStringInfo method is:
 %
-%      void ConcatentateStringInfo(StringInfo *string_info,
+%      void ConcatenateStringInfo(StringInfo *string_info,
 %        const StringInfo *source)
 %
 %  A description of each parameter follows:
@@ -602,8 +602,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
       (void) MagickSeek(file,0,SEEK_SET);
       for (i=0; i < length; i+=count)
       {
-        count=read(file,string+i,(size_t) Min(length-i,(size_t)
-          MagickMaxBufferSize));
+        count=read(file,string+i,(size_t) Min(length-i,(size_t) SSIZE_MAX));
         if (count <= 0)
           {
             count=0;
@@ -750,7 +749,7 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DestroyString() zeros memory associated with a string.
+%  DestroyString() destroys memory associated with a string.
 %
 %  The format of the DestroyString method is:
 %
@@ -780,7 +779,7 @@ MagickExport char *DestroyString(char *string)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DestroyStringInfo() zeros memory associated with the StringInfo structure.
+%  DestroyStringInfo() destroys memory associated with the StringInfo structure.
 %
 %  The format of the DestroyStringInfo method is:
 %
@@ -997,6 +996,49 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%  F o r m a t M a g i c k S i z e                                            %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  FormatMagickSize() converts a size to a human readable format, for example,
+%  14kb, 234mb, 2.7gb, or 3.0tb.  Scaling is done by repetitively dividing by
+%  1024.
+%
+%  The format of the FormatMagickSize method is:
+%
+%      long FormatMagickSize(const MagickSizeType size,char *format)
+%
+%  A description of each parameter follows:
+%
+%    o size:  convert this size to a human readable format.
+%
+%    o format:  human readable format.
+%
+*/
+MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
+{
+  double
+    length;
+
+  register long
+    i;
+
+  static char
+    *units[7] = { "b", "kb", "mb", "gb", "tb", "pb", "eb" };
+
+  length=(double) ((MagickOffsetType) (size/2))*2.0;
+  for (i=0; (length > 1024.0) && (i < 6); i++)
+    length/=1024.0;
+  return(FormatMagickString(format,MaxTextExtent,"%g%s",length,units[i]));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  F o r m a t M a g i c k S t r i n g                                        %
 %                                                                             %
 %                                                                             %
@@ -1856,6 +1898,69 @@ MagickExport double StringToDouble(const char *string,const double interval)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S t r i n g T o k e n                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  StringToken() extracts a token a from the string.
+%
+%  The format of the StringToken method is:
+%
+%      char *StringToken(const char *delimiters,char **string)
+%
+%  A description of each parameter follows:
+%
+%    o delimiters: one or more delimiters.
+%
+%    o string: return the first token in the string.  If none is found, return
+%      NULL.
+%
+*/
+MagickExport char *StringToken(const char *delimiters,char **string)
+{
+  char
+    *q;
+
+  register char
+    *p;
+
+  register const char
+    *r;
+
+  register int
+    c,
+    d;
+
+  p=(*string);
+  if (p == (char *) NULL)
+    return((char *) NULL);
+  for (q=p; ; )
+  {
+    c=(*p++);
+    r=delimiters;
+    do
+    {
+      d=(*r++);
+      if (c == d)
+        {
+          if (c == '\0')
+            p=(char *) NULL;
+          else
+            p[-1]='\0';
+          *string=p;
+          return(q);
+        }
+    } while (d != '\0');
+  }
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  S t r i n g T o L i s t                                                    %
 %                                                                             %
 %                                                                             %
@@ -2115,8 +2220,7 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
   result_length=0;
   destination_offset=0;
   replace_length=strlen(replace);
-  search_length=strlen(search);
-  while (match != (char *) NULL)
+  for (search_length=strlen(search); match != (char *) NULL; )
   {
     /*
       Copy portion before match.
