@@ -49,6 +49,97 @@
 #include "magick/token.h"
 #include "magick/utility.h"
 
+/*
+  Typedef declaractions.
+*/
+struct _TokenInfo
+{
+  int
+    state;
+
+  MagickStatusType
+    flag;
+
+  long
+    offset;
+
+  char
+    quote;
+
+  unsigned long
+    signature;
+};
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   A c q u i r e T o k e n I n f o                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireTokenInfo() allocates the TokenInfo structure.
+%
+%  The format of the AcquireTokenInfo method is:
+%
+%      TokenInfo *AcquireTokenInfo()
+%
+*/
+MagickExport TokenInfo *AcquireTokenInfo(void)
+{
+  TokenInfo
+    *token_info;
+
+  token_info=(TokenInfo *) AcquireMagickMemory(sizeof(*token_info));
+  if (token_info == (TokenInfo *) NULL)
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
+  return(token_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   D e s t r o y T o k e n I n f o                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  DestroyTokenInfo() deallocates memory associated with an TokenInfo
+%  structure.
+%
+%  The format of the DestroyTokenInfo method is:
+%
+%      TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
+%
+%  A description of each parameter follows:
+%
+%    o token_info: Specifies a pointer to an TokenInfo structure.
+%
+*/
+MagickExport TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(token_info != (TokenInfo *) NULL);
+  assert(token_info->signature == MagickSignature);
+  token_info->signature=(~MagickSignature);
+  token_info=(TokenInfo *) RelinquishMagickMemory(token_info);
+  return(token_info);
+}
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -251,7 +342,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       /*
         Determine if pattern is a scene, i.e. img0001.pcd[2].
       */
-      image_info=CloneImageInfo((ImageInfo *) NULL);
+      image_info=AcquireImageInfo();
       (void) CopyMagickString(image_info->filename,pattern,MaxTextExtent);
       GetExceptionInfo(&exception);
       (void) SetImageInfo(image_info,MagickTrue,&exception);
@@ -450,13 +541,14 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
   }
   while (*pattern == '*')
     pattern++;
-  return((MagickBooleanType) ((*expression == '\0') && (*pattern == '\0')));
+  return((*expression == '\0') && (*pattern == '\0') ? MagickTrue : MagickFalse);
 }
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 +     I s G l o b                                                             %
 %                                                                             %
 %                                                                             %
@@ -464,7 +556,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  IsGlob() returns MagickTrue if the path specification contains a globbing
-%  patten.
+%  pattern.
 %
 %  The format of the IsGlob method is:
 %
@@ -474,7 +566,6 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %
 %    o path: The path.
 %
-%
 */
 MagickExport MagickBooleanType IsGlob(const char *path)
 {
