@@ -49,9 +49,6 @@
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#if defined(__WINDOWS__)
-# include "magick/nt-feature.h"
-#endif
 #include "magick/option.h"
 #include "magick/semaphore.h"
 #include "magick/splay-tree.h"
@@ -59,6 +56,13 @@
 #include "magick/type.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
+#if defined(HasFONTCONFIG)
+# include "fontconfig/fontconfig.h"
+#endif
+#if defined(__WINDOWS__)
+# include "magick/nt-feature.h"
+#endif
 
 /*
   Define declarations.
@@ -72,7 +76,8 @@ static const char
   *TypeMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<typemap>"
-    "  <type stealth=\"True\" />"
+    "  <type stealth=\"True\" name=\"fixed\" family=\"helvetica\"/>"
+    "  <type stealth=\"True\" name=\"helvetica\" family=\"helvetica\"/>"
     "</typemap>";
 
 /*
@@ -111,7 +116,6 @@ static MagickBooleanType
 %
 %      DestroyTypeList(void)
 %
-%
 */
 MagickExport void DestroyTypeList(void)
 {
@@ -147,14 +151,10 @@ MagickExport void DestroyTypeList(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const TypeInfo *GetTypeInfo(const char *name,
   ExceptionInfo *exception)
 {
-  register const TypeInfo
-    *p;
-
   assert(exception != (ExceptionInfo *) NULL);
   if ((type_list == (SplayTreeInfo *) NULL) ||
       (instantiate_type == MagickFalse))
@@ -168,20 +168,7 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
       ResetSplayTreeIterator(type_list);
       return((const TypeInfo *) GetNextValueInSplayTree(type_list));
     }
-  /*
-    Search for requested type.
-  */
-  AcquireSemaphoreInfo(&type_semaphore);
-  ResetSplayTreeIterator(type_list);
-  p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
-  while (p != (const TypeInfo *) NULL)
-  {
-    if (LocaleCompare(name,p->name) == 0)
-      break;
-    p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
-  }
-  RelinquishSemaphoreInfo(type_semaphore);
-  return(p);
+  return((const TypeInfo *) GetValueFromSplayTree(type_list,name));
 }
 
 /*
@@ -218,7 +205,6 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   const StyleType style,const StretchType stretch,const unsigned long weight,
@@ -613,6 +599,140 @@ MagickExport char **GetTypeList(const char *pattern,unsigned long *number_fonts,
 %    o exception: Return any errors or warnings in this structure.
 %
 */
+
+#if defined(HasFONTCONFIG)
+MagickExport MagickBooleanType LoadFontConfigFonts(SplayTreeInfo *type_list,
+  ExceptionInfo *exception)
+{
+  char
+    extension[MaxTextExtent],
+    name[MaxTextExtent];
+
+  FcChar8
+    *family,
+    *file,
+    *style;
+   
+  FcFontSet
+    *font_set;
+
+  FcObjectSet
+    *object_set;
+
+  FcPattern
+    *pattern;
+ 
+  FcResult
+    status;
+
+  int
+    slant,
+    width,
+    weight;
+ 
+  register long
+    i;
+
+  TypeInfo
+    *type_info;
+  
+  /*
+    Load system fonts.
+  */
+  (void) FcInit();
+  object_set=FcObjectSetBuild(FC_FAMILY,FC_STYLE,FC_SLANT,FC_WIDTH,FC_WEIGHT,
+    FC_FILE,(char *) NULL);
+  pattern=FcPatternCreate();
+  font_set=FcFontList(0,pattern,object_set);
+  FcPatternDestroy(pattern);
+  FcObjectSetDestroy(object_set);
+  if (font_set == (FcFontSet *) NULL)
+    {
+      FcFini();
+      return(MagickFalse);
+    }
+  for (i=0; i < (long) font_set->nfont; i++)
+  {
+    status=FcPatternGetString(font_set->fonts[i],FC_FAMILY,0,&family);
+    status=FcPatternGetString(font_set->fonts[i],FC_STYLE,0,&style);
+    status=FcPatternGetInteger(font_set->fonts[i],FC_SLANT,0,&slant);
+    status=FcPatternGetInteger(font_set->fonts[i],FC_WIDTH,0,&width);
+    status=FcPatternGetInteger(font_set->fonts[i],FC_WEIGHT,0,&weight);
+    status=FcPatternGetString(font_set->fonts[i],FC_FILE,0,&file);
+    if (status != FcResultMatch)
+      continue;
+    *extension='\0';
+    GetPathComponent((const char *) file,ExtensionPath,extension);
+    if ((*extension != '\0') && (LocaleCompare(extension,"gz") == 0))
+      continue;
+    type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
+    if (type_info == (TypeInfo *) NULL)
+      continue;
+    (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
+    type_info->path=ConstantString("System Fonts");
+    type_info->signature=MagickSignature;
+    (void) CopyMagickString(name,(const char *) family,MaxTextExtent);
+    (void) ConcatenateMagickString(name," ",MaxTextExtent);
+    (void) ConcatenateMagickString(name,(const char *) style,MaxTextExtent);
+    type_info->name=ConstantString(name);
+    (void) SubstituteString(&type_info->name," ","-");
+    (void) SubstituteString(&type_info->name,"-L-","-");
+    (void) SubstituteString(&type_info->name,"semicondensed","SemiCondensed");
+    type_info->family=ConstantString((const char *) family);
+    (void) SubstituteString(&type_info->family," L","");
+    type_info->style=NormalStyle;
+    if (slant == FC_SLANT_ITALIC)
+      type_info->style=ItalicStyle;
+    if (slant == FC_SLANT_OBLIQUE)
+      type_info->style=ObliqueStyle;
+    type_info->stretch=NormalStretch;
+    if (width >= FC_WIDTH_ULTRACONDENSED)
+      type_info->stretch=UltraCondensedStretch;
+    if (width >= FC_WIDTH_EXTRACONDENSED)
+      type_info->stretch=ExtraCondensedStretch;
+    if (width >= FC_WIDTH_CONDENSED)
+      type_info->stretch=CondensedStretch;
+    if (width >= FC_WIDTH_SEMICONDENSED)
+      type_info->stretch=SemiCondensedStretch;
+    if (width >= FC_WIDTH_NORMAL)
+      type_info->stretch=NormalStretch;
+    if (width >= FC_WIDTH_SEMIEXPANDED)
+      type_info->stretch=SemiExpandedStretch;
+    if (width >= FC_WIDTH_EXPANDED)
+      type_info->stretch=ExpandedStretch;
+    if (width >= FC_WIDTH_EXTRAEXPANDED)
+      type_info->stretch=ExtraExpandedStretch;
+    if (width >= FC_WIDTH_ULTRAEXPANDED)
+      type_info->stretch=UltraExpandedStretch;
+    type_info->weight=400;
+    if (weight >= FC_WEIGHT_THIN)
+      type_info->weight=100;
+    if (weight >= FC_WEIGHT_EXTRALIGHT)
+      type_info->weight=200;
+    if (weight >= FC_WEIGHT_LIGHT)
+      type_info->weight=300;
+    if (weight >= FC_WEIGHT_NORMAL)
+      type_info->weight=400;
+    if (weight >= FC_WEIGHT_MEDIUM)
+      type_info->weight=500;
+    if (weight >= FC_WEIGHT_DEMIBOLD)
+      type_info->weight=600;
+    if (weight >= FC_WEIGHT_BOLD)
+      type_info->weight=700;
+    if (weight >= FC_WEIGHT_EXTRABOLD)
+      type_info->weight=800;
+    if (weight >= FC_WEIGHT_BLACK)
+      type_info->weight=900;
+    type_info->glyphs=ConstantString((const char *) file);
+    (void) AddValueToSplayTree(type_list,ConstantString(type_info->name),
+      type_info);
+  }
+  FcFontSetDestroy(font_set);
+  FcFini();
+  return(MagickTrue);
+}
+#endif
+
 static MagickBooleanType InitializeTypeList(ExceptionInfo *exception)
 {
   if ((type_list == (SplayTreeInfo *) NULL) &&
@@ -625,12 +745,15 @@ static MagickBooleanType InitializeTypeList(ExceptionInfo *exception)
           (void) LoadTypeLists(MagickTypeFilename,exception);
 #if defined(__WINDOWS__)
           (void) NTLoadTypeLists(type_list,exception);
+#endif
+#if defined(HasFONTCONFIG)
+          (void) LoadFontConfigFonts(type_list,exception);
 #endif
           instantiate_type=MagickTrue;
         }
       RelinquishSemaphoreInfo(type_semaphore);
     }
-  return((MagickBooleanType) (type_list != (SplayTreeInfo *) NULL));
+  return(type_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -656,7 +779,6 @@ static MagickBooleanType InitializeTypeList(ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
 {
@@ -696,7 +818,7 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
       {
         if (type_info[i]->path != (char *) NULL)
           (void) fprintf(file,"\nPath: %s\n\n",type_info[i]->path);
-        (void) fprintf(file,"%-32.32s %-23.23s %-7.7s %-8s %-3s\n",
+        (void) fprintf(file,"%-32.32s %-22.22s %-7.7s %-8.8s  %-3s\n",
           "Name","Family","Style","Stretch","Weight");
         (void) fprintf(file,"--------------------------------------------------"
           "------------------------------\n");
@@ -711,7 +833,7 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
     style=MagickOptionToMnemonic(MagickStyleOptions,type_info[i]->style);
     stretch=MagickOptionToMnemonic(MagickStretchOptions,type_info[i]->stretch);
     (void) FormatMagickString(weight,MaxTextExtent,"%lu",type_info[i]->weight);
-    (void) fprintf(file,"%-32.32s %-23.23s %-7.7s %-9s %-3s\n",name,family,
+    (void) fprintf(file,"%-32.32s %-22.22s %-7.7s %-9.9s %6s\n",name,family,
       style,stretch,weight);
   }
   (void) fflush(file);
@@ -748,7 +870,6 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 static void *DestroyTypeNode(void *type_info)
@@ -782,24 +903,27 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
   char
-#if defined(__WINDOWS__)
-    font_path[MaxTextExtent],
-#endif
-    keyword[MaxTextExtent],
-    *q,
-    *token;
+    font_path[MaxTextExtent];
 
-  MagickStatusType
-    status;
+  const char
+    *attribute;
 
   TypeInfo
     *type_info = (TypeInfo *) NULL;
 
+  MagickBooleanType
+    status;
+
+  XMLTreeInfo
+    *type,
+    *type_map,
+    *include;
+
   /*
     Load the type map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading type file \"%s\" ...",filename);
+    "Loading type map \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (type_list == (SplayTreeInfo *) NULL)
@@ -813,243 +937,139 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
+  type_map=NewXMLTree(xml,exception);
+  if (type_map == (XMLTreeInfo *) NULL)
+    return(MagickFalse);
   status=MagickTrue;
-  token=AcquireString(xml);
-#if defined(__WINDOWS__)
-  /*
-    Determine the Ghostscript font path.
-  */
+  include=GetXMLTreeChild(type_map,"include");
+  while (include != (XMLTreeInfo *) NULL)
+  {
+    /*
+      Process include element.
+    */
+    attribute=GetXMLTreeAttribute(include,"file");
+    if (attribute != (const char *) NULL)
+      {
+        if (depth > 200)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",attribute);
+        else
+          {
+            char
+              path[MaxTextExtent],
+              *xml;
+
+            ExceptionInfo
+              *sans_exception;
+
+            GetPathComponent(filename,HeadPath,path);
+            if (*path != '\0')
+              (void) ConcatenateMagickString(path,DirectorySeparator,
+                MaxTextExtent);
+            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
+            sans_exception=AcquireExceptionInfo();
+            xml=FileToString(path,~0,sans_exception);
+            sans_exception=DestroyExceptionInfo(sans_exception);
+            status=LoadTypeList(xml,path,depth+1,exception);
+            xml=(char *) RelinquishMagickMemory(xml);
+          }
+      }
+    include=GetNextXMLTreeTag(include);
+  }
   *font_path='\0';
+#if defined(__WINDOWS__)
   if (NTGhostscriptFonts(font_path,MaxTextExtent-2))
     (void) ConcatenateMagickString(font_path,DirectorySeparator,MaxTextExtent);
 #endif
-  for (q=(char *) xml; *q != '\0'; )
+  type=GetXMLTreeChild(type_map,"type");
+  while (type != (XMLTreeInfo *) NULL)
   {
     /*
-      Interpret XML.
+      Process type element.
     */
-    GetMagickToken(q,&q,token);
-    if (*token == '\0')
-      break;
-    (void) CopyMagickString(keyword,token,MaxTextExtent);
-    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
-      {
-        /*
-          Doctype element.
-        */
-        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
-        continue;
-      }
-    if (LocaleNCompare(keyword,"<!--",4) == 0)
-      {
-        /*
-          Comment element.
-        */
-        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
-        continue;
-      }
-    if (LocaleCompare(keyword,"<include") == 0)
-      {
-        /*
-          Include element.
-        */
-        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
-        {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
-          if (*token != '=')
-            continue;
-          GetMagickToken(q,&q,token);
-          if (LocaleCompare(keyword,"file") == 0)
-            {
-              if (depth > 200)
-                (void) ThrowMagickException(exception,GetMagickModule(),
-                  ConfigureError,"IncludeNodeNestedTooDeeply","`%s'",token);
-              else
-                {
-                  char
-                    path[MaxTextExtent],
-                    *xml;
-
-                  *path='\0';
-                  GetPathComponent(filename,HeadPath,path);
-                  if (*path != '\0')
-                    (void) ConcatenateMagickString(path,DirectorySeparator,
-                      MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path,~0,exception);
-                  status|=LoadTypeList(xml,path,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
-                }
-            }
-        }
-        continue;
-      }
-    if (LocaleCompare(keyword,"<type") == 0)
-      {
-        /*
-          Allocate memory for the type list.
-        */
-        type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
-        if (type_info == (TypeInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
-        (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
-        type_info->path=ConstantString(filename);
-        type_info->signature=MagickSignature;
-        continue;
-      }
+    type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
     if (type_info == (TypeInfo *) NULL)
-      continue;
-    if (LocaleCompare(keyword,"/>") == 0)
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",filename);
+    (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
+    type_info->path=ConstantString(filename);
+    type_info->signature=MagickSignature;
+    attribute=GetXMLTreeAttribute(type,"encoding");
+    if (attribute != (const char *) NULL)
+      type_info->encoding=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(type,"face");
+    if (attribute != (const char *) NULL)
+      type_info->face=(unsigned long) atol(attribute);
+    attribute=GetXMLTreeAttribute(type,"family");
+    if (attribute != (const char *) NULL)
+      type_info->family=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(type,"format");
+    if (attribute != (const char *) NULL)
+      type_info->format=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(type,"foundry");
+    if (attribute != (const char *) NULL)
+      type_info->foundry=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(type,"fullname");
+    if (attribute != (const char *) NULL)
+      type_info->description=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(type,"glyphs");
+    if (attribute != (const char *) NULL)
       {
-        status=AddValueToSplayTree(type_list,ConstantString(type_info->name),
-          type_info);
-        if (status == MagickFalse)
-          (void) ThrowMagickException(exception,GetMagickModule(),
-            ResourceLimitError,"MemoryAllocationFailed","`%s'",type_info->name);
-        type_info=(TypeInfo *) NULL;
-      }
-    GetMagickToken(q,(char **) NULL,token);
-    if (*token != '=')
-      continue;
-    GetMagickToken(q,&q,token);
-    GetMagickToken(q,&q,token);
-    switch (*keyword)
-    {
-      case 'E':
-      case 'e':
-      {
-        if (LocaleCompare((char *) keyword,"encoding") == 0)
-          {
-            type_info->encoding=ConstantString(token);
-            break;
-          }
-        break;
-      }
-      case 'F':
-      case 'f':
-      {
-        if (LocaleCompare((char *) keyword,"face") == 0)
-          {
-            type_info->face=(unsigned long) atol(token);
-            break;
-          }
-        if (LocaleCompare((char *) keyword,"family") == 0)
-          {
-            type_info->family=ConstantString(token);
-            break;
-          }
-        if (LocaleCompare((char *) keyword,"format") == 0)
-          {
-            type_info->format=ConstantString(token);
-            break;
-          }
-        if (LocaleCompare((char *) keyword,"foundry") == 0)
-          {
-            type_info->foundry=ConstantString(token);
-            break;
-          }
-        if (LocaleCompare((char *) keyword,"fullname") == 0)
-          {
-            type_info->description=ConstantString(token);
-            break;
-          }
-        break;
-      }
-      case 'G':
-      case 'g':
-      {
-        if (LocaleCompare((char *) keyword,"glyphs") == 0)
-          {
-            char
-              *glyphs;
+        char
+          *path;
 
-            glyphs=AcquireString(token);
+        path=ConstantString(attribute);
 #if defined(__WINDOWS__)
-            if (strchr(glyphs,'@') != (char *) NULL)
-              SubstituteString(&glyphs,"@ghostscript_font_path@",font_path);
+        if (strchr(path,'@') != (char *) NULL)
+          (void) SubstituteString(&path,"@ghostscript_font_path@",font_path);
 #endif
-            type_info->glyphs=glyphs;
-            break;
-          }
-        break;
+        type_info->glyphs=path;
       }
-      case 'M':
-      case 'm':
+    attribute=GetXMLTreeAttribute(type,"metrics");
+    if (attribute != (const char *) NULL)
       {
-        if (LocaleCompare((char *) keyword,"metrics") == 0)
-          {
-            char
-              *metrics;
+        char
+          *path;
 
-            metrics=AcquireString(token);
+        path=ConstantString(attribute);
 #if defined(__WINDOWS__)
-            if (strchr(metrics,'@') != (char *) NULL)
-              SubstituteString(&metrics,"@ghostscript_font_path@",font_path);
+        if (strchr(path,'@') != (char *) NULL)
+          (void) SubstituteString(&path,"@ghostscript_font_path@",font_path);
 #endif
-            type_info->metrics=metrics;
-            break;
-          }
-        break;
+        type_info->metrics=path;
       }
-      case 'N':
-      case 'n':
+    attribute=GetXMLTreeAttribute(type,"name");
+    if (attribute != (const char *) NULL)
+      type_info->name=ConstantString(attribute);
+    attribute=GetXMLTreeAttribute(type,"stealth");
+    if (attribute != (const char *) NULL)
+      type_info->stealth=IsMagickTrue(attribute);
+    attribute=GetXMLTreeAttribute(type,"stretch");
+    if (attribute != (const char *) NULL)
+      type_info->stretch=(StretchType) ParseMagickOption(MagickStretchOptions,
+        MagickFalse,attribute);
+    attribute=GetXMLTreeAttribute(type,"style");
+    if (attribute != (const char *) NULL)
+      type_info->style=(StyleType) ParseMagickOption(MagickStyleOptions,
+        MagickFalse,attribute);
+    attribute=GetXMLTreeAttribute(type,"weight");
+    if (attribute != (const char *) NULL)
       {
-        if (LocaleCompare((char *) keyword,"name") == 0)
-          {
-            type_info->name=ConstantString(token);
-            break;
-          }
-        break;
+        type_info->weight=(unsigned long) atol(attribute);
+        if (LocaleCompare(attribute,"bold") == 0)
+          type_info->weight=700;
+        if (LocaleCompare(attribute,"normal") == 0)
+          type_info->weight=400;
       }
-      case 'S':
-      case 's':
-      {
-        if (LocaleCompare((char *) keyword,"stealth") == 0)
-          {
-            type_info->stealth=LocaleCompare(token,"True") == 0 ? MagickTrue :
-              MagickFalse;
-            break;
-          }
-        if (LocaleCompare((char *) keyword,"stretch") == 0)
-          {
-            type_info->stretch=(StretchType)
-              ParseMagickOption(MagickStretchOptions,MagickFalse,token);
-            break;
-          }
-        if (LocaleCompare((char *) keyword,"style") == 0)
-          {
-            type_info->style=(StyleType)
-              ParseMagickOption(MagickStyleOptions,MagickFalse,token);
-            break;
-          }
-        break;
-      }
-      case 'W':
-      case 'w':
-      {
-        if (LocaleCompare((char *) keyword,"weight") == 0)
-          {
-            type_info->weight=(unsigned long) atol(token);
-            if (LocaleCompare(token,"bold") == 0)
-              type_info->weight=700;
-            if (LocaleCompare(token,"normal") == 0)
-              type_info->weight=400;
-            break;
-          }
-        break;
-      }
-      default:
-        break;
-    }
+    status=AddValueToSplayTree(type_list,ConstantString(type_info->name),
+      type_info);
+    if (status == MagickFalse)
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
+    type=GetNextXMLTreeTag(type);
   }
-  token=(char *) RelinquishMagickMemory(token);
-  if (type_list == (SplayTreeInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  type_map=DestroyXMLTree(type_map);
+  return(status);
 }
 
 /*
@@ -1077,7 +1097,6 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadTypeLists(const char *filename,
   ExceptionInfo *exception)
@@ -1086,7 +1105,8 @@ static MagickBooleanType LoadTypeLists(const char *filename,
   return(LoadTypeList(TypeMap,"built-in",0,exception));
 #else
   char
-    *font_path;
+    *font_path,
+    path[MaxTextExtent];
 
   const StringInfo
     *option;
@@ -1098,10 +1118,12 @@ static MagickBooleanType LoadTypeLists(const char *filename,
     status;
 
   status=MagickFalse;
+  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
+    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadTypeList((const char *) option->datum,option->path,0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
@@ -1110,7 +1132,6 @@ static MagickBooleanType LoadTypeLists(const char *filename,
   if (font_path != (char *) NULL)
     {
       char
-        path[MaxTextExtent],
         *option;
 
       /*
