@@ -152,7 +152,7 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
   assert(source != (const char *) NULL);
   assert(destination != (char *) NULL);
 #if defined(HAVE_SYMLINK)
-  AcquireUniqueFilename(destination);
+  (void) AcquireUniqueFilename(destination);
   (void) RelinquishUniqueFileResource(destination);
   if (*source == *DirectorySeparator)
     {
@@ -660,7 +660,6 @@ MagickExport void ExpandFilename(char *path)
 %    o argv: Specifies a pointer to a text array containing the command line
 %      arguments.
 %
-%
 */
 MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
 {
@@ -730,7 +729,9 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
     if (IsGlob(filename) == MagickFalse)
       continue;
     GetPathComponent(option,MagickPath,magick);
-    if (LocaleCompare(magick,"VID") == 0)
+    if ((LocaleCompare(magick,"CAPTION") == 0) ||
+        (LocaleCompare(magick,"LABEL") == 0) ||
+        (LocaleCompare(magick,"VID") == 0))
       continue;
     GetPathComponent(option,HeadPath,path);
     GetPathComponent(option,SubimagePath,subimage);
@@ -789,6 +790,22 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
     filelist=(char **) RelinquishMagickMemory(filelist);
   }
   vector[count]=(char *) NULL;
+  if (IsEventLogging() != MagickFalse)
+    {
+      char
+        *command_line;
+
+      command_line=AcquireString(vector[0]);
+      for (i=1; i < count; i++)
+      {
+        (void) ConcatenateString(&command_line,"{");
+        (void) ConcatenateString(&command_line,vector[i]);
+        (void) ConcatenateString(&command_line,"}");
+      }
+      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+        "Command line: %s",command_line);
+      command_line=(char *) RelinquishMagickMemory(command_line);
+    }
   *argc=(int) count;
   *argv=vector;
   return(MagickTrue);
@@ -799,84 +816,6 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%  F o r m a t S i z e                                                        %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  FormatSize() converts a size to a human readable format, for example,
-%  14kb, 234mb, 2.7gb, or 3.0tb.  Scaling is done by repetitively dividing by
-%  1024.
-%
-%  The format of the FormatSize method is:
-%
-%      void FormatSize(const MagickSizeType size,char *format)
-%
-%  A description of each parameter follows:
-%
-%    o size:  convert this size to a human readable format.
-%
-%    o format:  human readable format.
-%
-%
-*/
-MagickExport void FormatSize(const MagickSizeType size,char *format)
-{
-  double
-    length;
-
-  register long
-    i;
-
-  length=(double) size;
-  for (i=0; length > 1024.0; i++)
-    length/=1024.0;
-  switch (i)
-  {
-    case 0:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%g",length);
-      break;
-    }
-    case 1:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.0gkb",length);
-      break;
-    }
-    case 2:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.1gmb",length);
-      break;
-    }
-    case 3:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.2ggb",length);
-      break;
-    }
-    case 4:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.3gtb",length);
-      break;
-    }
-    case 5:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.4gpb",length);
-      break;
-    }
-    default:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.5feb",length);
-      break;
-    }
-  }
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   G e t E x e c u t i o n P a t h                                           %
 %                                                                             %
 %                                                                             %
@@ -969,21 +908,21 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     }
   (void) CopyMagickString(component,path,MaxTextExtent);
   *magick='\0';
-  if ((IsDirectory(path) < 0) && (IsAccessible(path) == MagickFalse))
-    for (p=component; *p != '\0'; p++)
-      if (*p == ':')
-        {
-          /*
-            Look for image format specification (e.g. ps3:image).
-          */
-          (void) CopyMagickString(magick,component,(size_t) (p-component+1));
-          if (IsMagickConflict(magick) != MagickFalse)
-            *magick='\0';
-          else
-            for (q=component; *q != '\0'; q++)
-              *q=(*++p);
-          break;
-        }
+  for (p=component; *p != '\0'; p++)
+    if ((*p == ':') && (IsDirectory(path) < 0) &&
+        (IsAccessible(path) == MagickFalse))
+      {
+        /*
+          Look for image format specification (e.g. ps3:image).
+        */
+        (void) CopyMagickString(magick,component,(size_t) (p-component+1));
+        if (IsMagickConflict(magick) != MagickFalse)
+          *magick='\0';
+        else
+          for (q=component; *q != '\0'; q++)
+            *q=(*++p);
+        break;
+      }
   *subimage='\0';
   p=component;
   if (*p != '\0')
@@ -1086,6 +1025,89 @@ MagickExport void GetPathComponent(const char *path,PathType type,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%  G e t P a t h C o m p o n e n t s                                          %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetPathComponents() returns a list of path components.
+%
+%  The format of the GetPathComponents method is:
+%
+%      char **GetPathComponents(const char *path,
+%        unsigned long *number_componenets)
+%
+%  A description of each parameter follows:
+%
+%    o path:  Specifies the string to segment into a list.
+%
+%    o number_components:  return the number of components in the list
+%
+*/
+MagickExport char **GetPathComponents(const char *path,
+  unsigned long *number_components)
+{
+  char
+    **components;
+
+  register char
+    *q;
+
+  register const char
+    *p;
+
+  register long
+    i;
+
+  if (path == (char *) NULL)
+    return((char **) NULL);
+  *number_components=1;
+  for (p=path; *p != '\0'; p++)
+    if (IsBasenameSeparator(*p))
+      (*number_components)++;
+  components=(char **)
+    AcquireMagickMemory((size_t) (*number_components+1)*sizeof(*components));
+  if (components == (char **) NULL)
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "memory allocation failed `%s'",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
+  p=path;
+  for (i=0; i < (long) *number_components; i++)
+  {
+    for (q=(char *) p; *q != '\0'; q++)
+      if (IsBasenameSeparator(*q))
+        break;
+    components[i]=(char *)
+      AcquireMagickMemory(((q-p)+MaxTextExtent)*sizeof(*components));
+    if (components[i] == (char *) NULL)
+      {
+        char
+          *message;
+
+        message=GetExceptionMessage(errno);
+        ThrowMagickFatalException(ResourceLimitFatalError,
+          "MemoryAllocationFailed",message);
+        message=(char *) RelinquishMagickMemory(message);
+      }
+    (void) CopyMagickString(components[i],p,(size_t) (q-p+1));
+    p=q+1;
+  }
+  components[i]=(char *) NULL;
+  return(components);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  I s A c c e s s i b l e                                                    %
 %                                                                             %
 %                                                                             %
@@ -1178,6 +1200,47 @@ static int IsDirectory(const char *path)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s M a g i c k T r u e                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsMagickTrue() returns MagickTrue if the value is "true", "on", "yes" or
+%  "1".
+%
+%  The format of the IsMagickTrue method is:
+%
+%      MagickBooleanType IsMagickTrue(const char *value)
+%
+%  A description of each parameter follows:
+%
+%    o option: either MagickTrue or MagickFalse depending on the value
+%      parameter.
+%
+%    o value: Specifies a pointer to a character array.
+%
+*/
+MagickExport MagickBooleanType IsMagickTrue(const char *value)
+{
+  if (value == (char *) NULL)
+    return(MagickFalse);
+  if (LocaleCompare(value,"true") == 0)
+    return(MagickTrue);
+  if (LocaleCompare(value,"on") == 0)
+    return(MagickTrue);
+  if (LocaleCompare(value,"yes") == 0)
+    return(MagickTrue);
+  if (LocaleCompare(value,"1") == 0)
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   L i s t F i l e s                                                         %
 %                                                                             %
 %                                                                             %
