@@ -56,6 +56,7 @@
 #include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "magick/xml-tree.h"
+#include "magick/utility.h"
 
 /*
   Define declarations.
@@ -140,7 +141,7 @@ static char
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  AddChildToXMLTree() adds a child tag at an offset relative to the start of
-%  the parent tag's character content.  This method returns the child tag.
+%  the parent tag's character content.  Return the child tag.
 %
 %  The format of the AddChildToXMLTree method is:
 %
@@ -160,9 +161,7 @@ MagickExport XMLTreeInfo *AddChildToXMLTree(XMLTreeInfo *xml_info,
   const char *tag,const size_t offset)
 {
   XMLTreeInfo
-    *child,
-    *head,
-    *node;
+    *child;
 
   if (xml_info == (XMLTreeInfo *) NULL)
     return((XMLTreeInfo *) NULL);
@@ -172,55 +171,274 @@ MagickExport XMLTreeInfo *AddChildToXMLTree(XMLTreeInfo *xml_info,
   (void) ResetMagickMemory(child,0,sizeof(*child));
   child->tag=ConstantString(tag);
   child->attributes=sentinel;
-  child->offset=offset;
-  child->parent=xml_info;
   child->content=ConstantString("");
   child->debug=IsEventLogging();
   child->signature=MagickSignature;
-  if (xml_info->child == (XMLTreeInfo *) NULL)
-    xml_info->child=child;
-  else
+  return(InsertTagIntoXMLTree(xml_info,child,offset));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   A d d P a t h T o X M L T r e e                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AddPathToXMLTree() adds a child tag at an offset relative to the start of
+%  the parent tag's character content.  This method returns the child tag.
+%
+%  The format of the AddPathToXMLTree method is:
+%
+%      XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,const char *path,
+%        const size_t offset)
+%
+%  A description of each parameter follows:
+%
+%    o xml_info: The xml info.
+%
+%    o path: The path.
+%
+%    o offset: The tag offset.
+%
+*/
+MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
+  const char *path,const size_t offset)
+{
+  char
+    **components,
+    subnode[MaxTextExtent],
+    tag[MaxTextExtent];
+
+  long
+    j;
+
+  register long
+    i;
+
+  XMLTreeInfo
+    *child,
+    *node;
+
+  unsigned long
+    number_components;
+
+  assert(xml_info != (XMLTreeInfo *) NULL);
+  assert((xml_info->signature == MagickSignature) ||
+         (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  node=xml_info;
+  components=GetPathComponents(path,&number_components);
+  if (components == (char **) NULL)
+    return((XMLTreeInfo *) NULL);
+  for (i=0; i < (long) number_components; i++)
+  {
+    GetPathComponent(components[i],SubimagePath,subnode);
+    GetPathComponent(components[i],CanonicalPath,tag);
+    child=GetXMLTreeChild(node,tag);
+    if (child == (XMLTreeInfo *) NULL)
+      child=AddChildToXMLTree(node,tag,offset);
+    node=child;
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    for (j=atol(subnode)-1; j > 0; j--)
     {
-      head=xml_info->child;
-      if (head->offset > offset)
-        {
-          child->ordered=head;
-          xml_info->child=child;
-        }
-      else
-        {
-          node=head;
-          while ((node->ordered != (XMLTreeInfo *) NULL) &&
-                 (node->ordered->offset <= offset))
-            node=node->ordered;
-          child->ordered=node->ordered;
-          node->ordered=child;
-        }
-      node=head;
-      while ((node->sibling != (XMLTreeInfo *) NULL) &&
-             (strcmp(node->tag,tag) != 0))
-        node=node->sibling;
-      if ((strcmp(node->tag,tag) == 0) && (node->offset <= offset))
-        {
-          while ((node->next != (XMLTreeInfo *) NULL) &&
-                 (node->next->offset <= offset))
-            node=node->next;
-          child->next=node->next;
-          node->next=child;
-        }
-      else
-        {
-          if (node->offset > offset)
-            child->next=node;
-          node=head;
-          while ((node->sibling != (XMLTreeInfo *) NULL) &&
-                 (node->sibling->offset <= offset))
-            node=node->sibling;
-          child->sibling=node->sibling;
-          node->sibling=child;
-        }
+      node=GetXMLTreeOrdered(node);
+      if (node == (XMLTreeInfo *) NULL)
+        break;
     }
-  return(child);
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  }
+  for ( ; i < (long) number_components; i++)
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  components=(char **) RelinquishMagickMemory(components);
+  return(node);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   C a n o n i c a l X M L C o n t e n t                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  CanonicalXMLContent() converts text to canonical XML content by converting
+%  to UTF-8, substituting predefined entities, wrapping as CDATA, or encoding
+%  as base-64 as required.
+%
+%  The format of the CanonicalXMLContent method is:
+%
+%      char *CanonicalXMLContent(const char *content)
+%
+%  A description of each parameter follows:
+%
+%    o content: The content.
+%
+*/
+
+static unsigned char *ConvertLatin1ToUTF8(const unsigned char *content)
+{
+  register const unsigned char
+    *p;
+
+  register unsigned char
+    *q;
+
+  size_t
+    length;
+
+  unsigned char
+    *utf8;
+
+  unsigned int
+    c;
+
+  length=0;
+  for (p=content; *p != '\0'; p++)
+    length+=(*p & 0x80) != 0 ? 2 : 1;
+  utf8=(unsigned char *) AcquireMagickMemory((length+1)*sizeof(*utf8));
+  if (utf8 == (unsigned char *) NULL)
+    return((unsigned char *) NULL);
+  q=utf8;
+  for (p=content; *p != '\0'; p++)
+  {
+    c=(*p);
+    if ((c & 0x80) == 0)
+      *q++=c;
+    else
+      {
+        *q++=0xc0 | ((c >> 6) & 0x3f);
+        *q++=0x80 | (c & 0x3f);
+      }
+  }
+  *q='\0';
+  return(utf8);
+}
+
+MagickExport char *CanonicalXMLContent(const char *content,
+  const MagickBooleanType pedantic)
+{
+  char
+    *base64,
+    *canonical_content;
+
+  register const unsigned char
+    *p;
+
+  register long
+    i;
+  size_t
+    length;
+
+  unsigned char
+    *utf8;
+
+  unsigned long
+    extent;
+
+  utf8=ConvertLatin1ToUTF8((const unsigned char *) content);
+  if (utf8 == (unsigned char *) NULL)
+    return((char *) NULL);
+  for (p=utf8; *p != '\0'; p++)
+    if ((*p < 0x20) && (*p != 0x09) && (*p != 0x0a) && (*p != 0x0d))
+      break;
+  if (*p != '\0')
+    {
+      /*
+        String is binary, base64-encode it.
+      */
+      base64=Base64Encode(utf8,strlen((char *) utf8),&length);
+      utf8=(unsigned char *) RelinquishMagickMemory(utf8);
+      if (base64 == (char *) NULL)
+        return((char *) NULL);
+      canonical_content=AcquireString("<base64>");
+      (void) ConcatenateString(&canonical_content,base64);
+      base64=(char *) RelinquishMagickMemory(base64);
+      (void) ConcatenateString(&canonical_content,"</base64>");
+      return(canonical_content);
+    }
+  /*
+    Substitute predefined entities.
+  */
+  i=0;
+  extent=MaxTextExtent;
+  canonical_content=AcquireString((char *) NULL);
+  for (p=utf8; *p != '\0'; p++)
+  {
+    if ((i+MaxTextExtent) > (long) extent)
+      {
+        extent=i+MaxTextExtent;
+        canonical_content=(char *) ResizeMagickMemory(canonical_content,
+          extent*sizeof(*canonical_content));
+        if (canonical_content == (char *) NULL)
+          return(canonical_content);
+      }
+    switch (*p)
+    {
+      case '&':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&amp;");
+        break;
+      }
+      case '<':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&lt;");
+        break;
+      }
+      case '>':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&gt;");
+        break;
+      }
+      case '"':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&quot;");
+        break;
+      }
+      case '\n':
+      {
+        if (pedantic == MagickFalse)
+          {
+            canonical_content[i++]=(*p);
+            break;
+          }
+        i+=FormatMagickString(canonical_content+i,extent,"&#xA;");
+        break;
+      }
+      case '\t':
+      {
+        if (pedantic == MagickFalse)
+          {
+            canonical_content[i++]=(*p);
+            break;
+          }
+        i+=FormatMagickString(canonical_content+i,extent,"&#x9;");
+        break;
+      }
+      case '\r':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&#xD;");
+        break;
+      }
+      default:
+      {
+        canonical_content[i++]=(*p);
+        break;
+      }
+    }
+  }
+  canonical_content[i]='\0';
+  utf8=(unsigned char *) RelinquishMagickMemory(utf8);
+  return(canonical_content);
 }
 
 /*
@@ -248,9 +466,6 @@ MagickExport XMLTreeInfo *AddChildToXMLTree(XMLTreeInfo *xml_info,
 
 static char **DestroyXMLTreeAttributes(char **attributes)
 {
-  register char
-    *p;
-
   register long
     i;
 
@@ -259,17 +474,16 @@ static char **DestroyXMLTreeAttributes(char **attributes)
   */
   if ((attributes == (char **) NULL) || (attributes == sentinel))
     return((char **) NULL);
-  for (i=0; attributes[i] != (char *) NULL; i+=2);
-  p=attributes[i+1];
-  for (i=0; p[i] != (char) NULL; i++)
+  for (i=0; attributes[i] != (char *) NULL; i+=2)
   {
     /*
       Destroy attribute tag and value.
     */
-    attributes[2*i]=(char *) RelinquishMagickMemory(attributes[2*i]);
-    attributes[2*i+1]=(char *) RelinquishMagickMemory(attributes[2*i+1]);
+    if (attributes[i] != (char *) NULL)
+      attributes[i]=(char *) RelinquishMagickMemory(attributes[i]);
+    if (attributes[i+1] != (char *) NULL)
+      attributes[i+1]=(char *) RelinquishMagickMemory(attributes[i+1]);
   }
-  p=(char *) RelinquishMagickMemory(p);
   attributes=(char **) RelinquishMagickMemory(attributes);
   return((char **) NULL);
 }
@@ -466,8 +680,9 @@ MagickExport XMLTreeInfo *GetXMLTreeChild(XMLTreeInfo *xml_info,const char *tag)
          (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   child=xml_info->child;
-  while ((child != (XMLTreeInfo *) NULL) && (strcmp(child->tag,tag) != 0))
-    child=child->sibling;
+  if (tag != (const char *) NULL)
+    while ((child != (XMLTreeInfo *) NULL) && (strcmp(child->tag,tag) != 0))
+      child=child->sibling;
   return(child);
 }
 
@@ -508,6 +723,112 @@ MagickExport const char *GetXMLTreeContent(XMLTreeInfo *xml_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t X M L T r e e O r d e r e d                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetXMLTreeOrdered() returns the next ordered node if found, otherwise NULL.
+%
+%  The format of the GetXMLTreeOrdered method is:
+%
+%      XMLTreeInfo *GetXMLTreeOrdered(XMLTreeInfo *xml_info)
+%
+%  A description of each parameter follows:
+%
+%    o xml_info: The xml info.
+%
+*/
+MagickExport XMLTreeInfo *GetXMLTreeOrdered(XMLTreeInfo *xml_info)
+{
+  assert(xml_info != (XMLTreeInfo *) NULL);
+  assert((xml_info->signature == MagickSignature) ||
+         (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  return(xml_info->ordered);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t X M L T r e e P a t h                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetXMLTreePath() traverses the XML-tree as defined by the specified path
+%  and returns the node if found, otherwise NULL.
+%
+%  The format of the GetXMLTreePath method is:
+%
+%      XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
+%
+%  A description of each parameter follows:
+%
+%    o xml_info: The xml info.
+%
+%    o path: The path (e.g. property/elapsed-time).
+%
+*/
+MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
+{
+  char
+    **components,
+    subnode[MaxTextExtent],
+    tag[MaxTextExtent];
+
+  long
+    j;
+
+  register long
+    i;
+
+  XMLTreeInfo
+    *node;
+
+  unsigned long
+    number_components;
+
+  assert(xml_info != (XMLTreeInfo *) NULL);
+  assert((xml_info->signature == MagickSignature) ||
+         (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  node=xml_info;
+  components=GetPathComponents(path,&number_components);
+  if (components == (char **) NULL)
+    return((XMLTreeInfo *) NULL);
+  for (i=0; i < (long) number_components; i++)
+  {
+    GetPathComponent(components[i],SubimagePath,subnode);
+    GetPathComponent(components[i],CanonicalPath,tag);
+    node=GetXMLTreeChild(node,tag);
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    for (j=atol(subnode)-1; j > 0; j--)
+    {
+      node=GetXMLTreeOrdered(node);
+      if (node == (XMLTreeInfo *) NULL)
+        break;
+    }
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  }
+  for ( ; i < (long) number_components; i++)
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  components=(char **) RelinquishMagickMemory(components);
+  return(node);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t X M L T r e e P r o c e s s i n g I n s t r u c t i o n s           %
 %                                                                             %
 %                                                                             %
@@ -563,11 +884,11 @@ MagickExport const char **GetXMLTreeProcessingInstructions(
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetXMLTreeSibling() returns the tag sibling if found, otherwise NULL.
+%  GetXMLTreeSibling() returns the node sibling if found, otherwise NULL.
 %
 %  The format of the GetXMLTreeSibling method is:
 %
-%      XMLTreeInfo *GetXMLTreeSibling(XMLTreeInfo *xml_info,const char *tag)
+%      XMLTreeInfo *GetXMLTreeSibling(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
@@ -619,6 +940,101 @@ MagickExport const char *GetXMLTreeTag(XMLTreeInfo *xml_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I n s e r t I n t o T a g X M L T r e e                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  InsertTagIntoXMLTree() inserts a tag at an offset relative to the start of
+%  the parent tag's character content.  This method returns the child tag.
+%
+%  The format of the InsertTagIntoXMLTree method is:
+%
+%      XMLTreeInfo *InsertTagIntoXMLTree(XMLTreeInfo *xml_info,
+%        XMLTreeInfo *child,const size_t offset)
+%
+%  A description of each parameter follows:
+%
+%    o xml_info: The xml info.
+%
+%    o child: The child tag.
+%
+%    o offset: The tag offset.
+%
+*/
+MagickExport XMLTreeInfo *InsertTagIntoXMLTree(XMLTreeInfo *xml_info,
+  XMLTreeInfo *child,const size_t offset)
+{
+  XMLTreeInfo
+    *head,
+    *node,
+    *previous;
+
+  child->ordered=(XMLTreeInfo *) NULL;
+  child->sibling=(XMLTreeInfo *) NULL;
+  child->next=(XMLTreeInfo *) NULL;
+  child->offset=offset;
+  child->parent=xml_info;
+  if (xml_info->child == (XMLTreeInfo *) NULL)
+    {
+      xml_info->child=child;
+      return(child);
+    }
+  head=xml_info->child;
+  if (head->offset > offset)
+    {
+      child->ordered=head;
+      xml_info->child=child;
+    }
+  else
+    {
+      node=head;
+      while ((node->ordered != (XMLTreeInfo *) NULL) &&
+             (node->ordered->offset <= offset))
+        node=node->ordered;
+      child->ordered=node->ordered;
+      node->ordered=child;
+    }
+  previous=(XMLTreeInfo *) NULL;
+  node=head;
+  while ((node != (XMLTreeInfo *) NULL) && (strcmp(node->tag,child->tag) != 0))
+  {
+    previous=node;
+    node=node->sibling;
+  }
+  if ((node != (XMLTreeInfo *) NULL) && (node->offset <= offset))
+    {
+      while ((node->next != (XMLTreeInfo *) NULL) &&
+             (node->next->offset <= offset))
+        node=node->next;
+      child->next=node->next;
+      node->next=child;
+    }
+  else
+    {
+      if ((previous != (XMLTreeInfo *) NULL) && (node != (XMLTreeInfo *) NULL))
+        previous->sibling=node->sibling;
+      child->next=node;
+      previous=(XMLTreeInfo *) NULL;
+      node=head;
+      while ((node != (XMLTreeInfo *) NULL) && (node->offset <= offset))
+      {
+        previous=node;
+        node=node->sibling;
+      }
+      child->sibling=node;
+      if (previous != (XMLTreeInfo *) NULL)
+        previous->sibling=child;
+    }
+  return(child);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   N e w X M L T r e e                                                       %
 %                                                                             %
 %                                                                             %
@@ -640,7 +1056,7 @@ MagickExport const char *GetXMLTreeTag(XMLTreeInfo *xml_info)
 %
 */
 
-static char *ConvertUTF16ToUTF8(const char *xml,size_t *length)
+static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
 {
   char
     *utf8;
@@ -663,28 +1079,29 @@ static char *ConvertUTF16ToUTF8(const char *xml,size_t *length)
   utf8=(char *) AcquireMagickMemory((*length)*sizeof(*utf8));
   if (utf8 == (char *) NULL)
     return((char *) NULL);
-  encoding=(*xml == '\xFE') ? 1 : (*xml == '\xFF') ? 0 : -1;
+  encoding=(*content == '\xFE') ? 1 : (*content == '\xFF') ? 0 : -1;
   if (encoding == -1)
     {
       /*
         Already UTF-8.
       */
-      (void) CopyMagickMemory(utf8,xml,*length);
+      (void) CopyMagickMemory(utf8,content,*length);
       return(utf8);
     }
   j=0;
   max_length=(*length);
   for (i=2; i < (long) (*length-1); i+=2)
   {
-    c=(encoding != 0) ? ((xml[i] & 0xff) << 8) | (xml[i+1] & 0xff) :
-      ((xml[i+1] & 0xff) << 8) | (xml[i] & 0xff);
+    c=(encoding != 0) ? ((content[i] & 0xff) << 8) | (content[i+1] & 0xff) :
+      ((content[i+1] & 0xff) << 8) | (content[i] & 0xff);
     if ((c >= 0xd800) && (c <= 0xdfff) && ((i+=2) < (long) (*length-1)))
       {
-        byte=(encoding != 0) ? ((xml[i] & 0xff) << 8) | (xml[i+1] & 0xff) :
-          ((xml[i+1] & 0xff) << 8) | (xml[i] & 0xff);
+        byte=(encoding != 0) ? ((content[i] & 0xff) << 8) |
+          (content[i+1] & 0xff) : ((content[i+1] & 0xff) << 8) |
+          (content[i] & 0xff);
         c=(((c & 0x3ff) << 10) | (byte & 0x3ff))+0x10000;
       }
-    if ((j+MaxTextExtent) > (long) max_length)
+    if ((size_t) (j+MaxTextExtent) > max_length)
       {
         max_length=(size_t) j+MaxTextExtent;
         utf8=(char *) ResizeMagickMemory(utf8,max_length*sizeof(*utf8));
@@ -841,7 +1258,8 @@ static char *ParseEntities(char *xml,char **entities,char state)
                       xml=(char *) AcquireMagickMemory((size_t) j*sizeof(*xml));
                       if (xml != (char *) NULL)
                         {
-                          (void) CopyMagickString(xml,p,j*sizeof(*xml));
+                          (void) CopyMagickString(xml,p,(size_t) j*
+                            sizeof(*xml));
                           p=xml;
                         }
                     }
@@ -852,7 +1270,7 @@ static char *ParseEntities(char *xml,char **entities,char state)
                   entity=strchr(xml,';');
                 }
               (void) CopyMagickMemory(xml+length,entity+1,strlen(entity));
-              (void) CopyMagickString(xml,entities[i],length);
+              (void) strncpy(xml,entities[i],length);
             }
         }
       else
@@ -870,7 +1288,7 @@ static char *ParseEntities(char *xml,char **entities,char state)
       for (xml=p; *xml != '\0'; xml++)
       {
         i=(long) strspn(xml," ");
-        if (i != '\0')
+        if (i != 0)
           (void) CopyMagickMemory(xml,xml+i,strlen(xml+i)+1);
         while ((*xml != '\0') && (*xml != ' '))
           xml++;
@@ -921,8 +1339,8 @@ void ParseCharacterContent(XMLTreeRoot *root,char *xml,const size_t length,
   }
 }
 
-static XMLTreeInfo *ParseCloseTag(XMLTreeRoot *root,char *tag,char *xml,
-  ExceptionInfo *exception)
+static XMLTreeInfo *ParseCloseTag(XMLTreeRoot *root,char *tag,
+  char *magick_unused(xml),ExceptionInfo *exception)
 {
   if ((root->node == (XMLTreeInfo *) NULL) ||
       (root->node->tag == (char *) NULL) || (strcmp(tag,root->node->tag) != 0))
@@ -1129,6 +1547,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
               entities[i+1]=(char *) RelinquishMagickMemory(entities[i+1]);
             (void) ThrowMagickException(exception,GetMagickModule(),
               OptionWarning,"ParseError","circular entity declaration &%s",n);
+            predefined_entitites=(char **)
+              RelinquishMagickMemory(predefined_entitites);
             return(MagickFalse);
           }
         }
@@ -1143,6 +1563,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
               {
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   OptionWarning,"ParseError","unclosed <!ATTLIST");
+                predefined_entitites=(char **)
+                  RelinquishMagickMemory(predefined_entitites);
                 return(MagickFalse);
               }
             xml=t+strcspn(t,XMLWhitespace ">");
@@ -1162,6 +1584,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                 {
                   (void) ThrowMagickException(exception,GetMagickModule(),
                     OptionWarning,"ParseError","malformed <!ATTLIST");
+                  predefined_entitites=(char **)
+                    RelinquishMagickMemory(predefined_entitites);
                   return(MagickFalse);
                 }
               xml+=strspn(xml+1,XMLWhitespace)+1;
@@ -1174,9 +1598,11 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                 {
                   (void) ThrowMagickException(exception,GetMagickModule(),
                     OptionWarning,"ParseError","malformed <!ATTLIST");
+                  predefined_entitites=(char **)
+                    RelinquishMagickMemory(predefined_entitites);
                   return(MagickFalse);
                 }
-              xml+=strspn(xml,XMLWhitespace")");
+              xml+=strspn(xml,XMLWhitespace ")");
               if (strncmp(xml,"#FIXED",6) == 0)
                 xml+=strspn(xml+6,XMLWhitespace)+6;
               if (*xml == '#')
@@ -1188,12 +1614,14 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                 }
               else
                 if (((*xml == '"') || (*xml == '\''))  &&
-                    (xml=strchr(v=xml+1,*xml)))
+                    ((xml=strchr(v=xml+1,*xml)) != (char *) NULL))
                   *xml='\0';
                 else
                   {
                     (void) ThrowMagickException(exception,GetMagickModule(),
                       OptionWarning,"ParseError","malformed <!ATTLIST");
+                    predefined_entitites=(char **)
+                      RelinquishMagickMemory(predefined_entitites);
                     return(MagickFalse);
                   }
               if (root->attributes[i] == (char **) NULL)
@@ -1212,8 +1640,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                     ThrowMagickFatalException(ResourceLimitFatalError,
                       "UnableToAcquireString",xml);
                   root->attributes[i]=(char **) AcquireMagickMemory(2*
-                    sizeof(**root->attributes));
-                  if ( root->attributes[i] == (char **) NULL)
+                    sizeof(*root->attributes));
+                  if (root->attributes[i] == (char **) NULL)
                     ThrowMagickFatalException(ResourceLimitFatalError,
                       "UnableToAcquireString",xml);
                   root->attributes[i][0]=ConstantString(t);
@@ -1221,15 +1649,12 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                   root->attributes[i+1]=(char **) NULL;
                 }
               for (j=1; root->attributes[i][j] != (char *) NULL; j+=3);
-              {
-                root->attributes[i]=(char **) ResizeMagickMemory(
-                  root->attributes[i],(size_t) (j+4)*
-                  sizeof(**root->attributes));
-                  if (root->attributes[i] == (char **) NULL)
-                    ThrowMagickFatalException(ResourceLimitFatalError,
-                      "UnableToAcquireString",xml);
-              }
-              root->attributes[i][j+3]=(char *)  NULL;
+              root->attributes[i]=(char **) ResizeMagickMemory(
+                root->attributes[i],(size_t) (j+4)*sizeof(*root->attributes));
+              if (root->attributes[i] == (char **) NULL)
+                ThrowMagickFatalException(ResourceLimitFatalError,
+                  "UnableToAcquireString",xml);
+              root->attributes[i][j+3]=(char *) NULL;
               root->attributes[i][j+2]=ConstantString(c);
               root->attributes[i][j+1]=(char *) NULL;
               if (v != (char *) NULL)
@@ -1396,22 +1821,6 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
               message=(char *) RelinquishMagickMemory(message);
               return(&root->root);
             }
-          if (l == 0)
-            attributes[l+3]=(char *) AcquireMagickMemory(2*
-              sizeof(**attributes));
-          else
-            attributes[l+3]=(char *) ResizeMagickMemory(attributes[l+1],
-              (size_t) ((l/2)+2)*sizeof(**attributes));
-          if (attributes[l+3] == (char *) NULL)
-            {
-              char
-                message[MaxTextExtent];
-
-              (void) ThrowMagickException(exception,GetMagickModule(),
-                ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
-              return(&root->root);
-            }
-          (void) CopyMagickString(attributes[l+3]+(l/2)," ",2);
           attributes[l+2]=(char *) NULL;
           attributes[l+1]=(char *) NULL;
           attributes[l]=p;
@@ -1698,25 +2107,25 @@ MagickExport XMLTreeInfo *NewXMLTreeTag(const char *tag)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   R e m o v e T a g F r o m X M L T r e e                                   %
+%   P r u n e T a g F r o m X M L T r e e                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RemoveTagFromXMLTree() removes a tag from the xml-tree along with all its
+%  PruneTagFromXMLTree() prunes a tag from the xml-tree along with all its
 %  subtags.
 %
-%  The format of the RemoveTagFromXMLTree method is:
+%  The format of the PruneTagFromXMLTree method is:
 %
-%      void RemoveTagFromXMLTree(XMLTreeInfo *xml_info)
+%      XMLTreeInfo *PruneTagFromXMLTree(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
 %    o xml_info: The xml info.
 %
 */
-MagickExport void RemoveTagFromXMLTree(XMLTreeInfo *xml_info)
+MagickExport XMLTreeInfo *PruneTagFromXMLTree(XMLTreeInfo *xml_info)
 {
   XMLTreeInfo
     *node;
@@ -1756,7 +2165,9 @@ MagickExport void RemoveTagFromXMLTree(XMLTreeInfo *xml_info)
         }
     }
   xml_info->ordered=(XMLTreeInfo *) NULL;
-  xml_info=DestroyXMLTree(xml_info);
+  xml_info->sibling=(XMLTreeInfo *) NULL;
+  xml_info->next=(XMLTreeInfo *) NULL;
+  return(xml_info);
 }
 
 /*
@@ -1775,7 +2186,7 @@ MagickExport void RemoveTagFromXMLTree(XMLTreeInfo *xml_info)
 %
 %  The format of the SetXMLTreeAttribute method is:
 %
-%      void SetXMLTreeAttribute(XMLTreeInfo *xml_info,const char *tag,
+%      XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,const char *tag,
 %        const char *value)
 %
 %  A description of each parameter follows:
@@ -1787,8 +2198,8 @@ MagickExport void RemoveTagFromXMLTree(XMLTreeInfo *xml_info)
 %    o value:  The attribute value.
 %
 */
-MagickExport void SetXMLTreeAttribute(XMLTreeInfo *xml_info,const char *tag,
-  const char *value)
+MagickExport XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
+  const char *tag,const char *value)
 {
   long
     j;
@@ -1813,46 +2224,39 @@ MagickExport void SetXMLTreeAttribute(XMLTreeInfo *xml_info,const char *tag,
         Add new attribute tag.
       */
       if (value == (const char *) NULL)
-        return;
+        return(xml_info);
       if (xml_info->attributes != sentinel)
-        {
-          xml_info->attributes=(char **) ResizeMagickMemory(
-            xml_info->attributes,(size_t) (i+4)*sizeof(*xml_info->attributes));
-          if (xml_info->attributes == (char **) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAcquireString",tag);
-        }
+        xml_info->attributes=(char **) ResizeMagickMemory(
+          xml_info->attributes,(size_t) (i+4)*sizeof(*xml_info->attributes));
       else
         {
           xml_info->attributes=(char **) AcquireMagickMemory(4*sizeof(
             *xml_info->attributes));
-          if (xml_info->attributes == (char **) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAcquireString",tag);
-          xml_info->attributes[1]=ConstantString("");
+          if (xml_info->attributes != (char **) NULL)
+            xml_info->attributes[1]=ConstantString("");
         }
+      if (xml_info->attributes == (char **) NULL)
+        ThrowMagickFatalException(ResourceLimitFatalError,
+          "UnableToAcquireString",tag);
       xml_info->attributes[i]=ConstantString(tag);
       xml_info->attributes[i+2]=(char *) NULL;
       length=strlen(xml_info->attributes[i+1]);
-      xml_info->attributes[i+3]=(char *) ResizeMagickMemory(
-        xml_info->attributes[i+1],(length+2)*sizeof(*xml_info->attributes));
-      if (xml_info->attributes[i+3] == (char *) NULL)
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "UnableToAcquireString",tag);
     }
   /*
     Add new value to an existing attribute.
   */
-  for (j=i; xml_info->attributes[j]; j+=2);
-  xml_info->attributes[i+1]=(char *) RelinquishMagickMemory(
-    xml_info->attributes[i+1]);
+  for (j=i; xml_info->attributes[j] != (char *) NULL; j+=2);
+  if (xml_info->attributes[i+1] != (char *) NULL)
+    xml_info->attributes[i+1]=(char *) RelinquishMagickMemory(
+      xml_info->attributes[i+1]);
   if (value != (const char *) NULL)
     {
       xml_info->attributes[i+1]=ConstantString(value);
-      return;
+      return(xml_info);
     }
-  xml_info->attributes[i]=(char *) RelinquishMagickMemory(
-    xml_info->attributes[i]);
+  if (xml_info->attributes[i] != (char *) NULL)
+    xml_info->attributes[i]=(char *) RelinquishMagickMemory(
+      xml_info->attributes[i]);
   (void) CopyMagickMemory(xml_info->attributes+i,xml_info->attributes+i+2,
     (size_t) (j-i+2)*sizeof(*xml_info->attributes));
   xml_info->attributes=(char **) ResizeMagickMemory(xml_info->attributes,
@@ -1863,6 +2267,7 @@ MagickExport void SetXMLTreeAttribute(XMLTreeInfo *xml_info,const char *tag,
   (void) CopyMagickMemory(xml_info->attributes[j+1]+(i/2),
     xml_info->attributes[j+1]+(i/2)+1,(size_t) (j/2)-(i/2)*
     sizeof(*xml_info->attributes));
+  return(xml_info);
 }
 
 /*
@@ -1903,7 +2308,6 @@ MagickExport XMLTreeInfo *SetXMLTreeContent(XMLTreeInfo *xml_info,
   xml_info->content=(char *) ConstantString(content);
   return(xml_info);
 }
-
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1928,89 +2332,29 @@ MagickExport XMLTreeInfo *SetXMLTreeContent(XMLTreeInfo *xml_info,
 %
 */
 
-static char *EncodePredefinedEntities(const char *source,size_t offset,
+static char *EncodePredefinedEntities(const char *source,ssize_t offset,
   char **destination,size_t *length,size_t *max_length,
   MagickBooleanType pedantic)
 {
-  const char
-    *p;
+  char
+    *content;
 
-  for (p=source+offset; p != source; source++)
-  {
-    if ((*length+MaxTextExtent) > *max_length)
-      {
-        *max_length=(*length)+MaxTextExtent;
-        *destination=(char *) ResizeMagickMemory(*destination,*max_length*
-          sizeof(**destination));
-        if (*destination == (char *) NULL)
-          return(*destination);
-      }
-    switch (*source)
+  if (offset == -1)
+    content=CanonicalXMLContent(source,pedantic);
+  else
+    content=CanonicalXMLContent(source+offset,pedantic);
+  if (content == (char *) NULL)
+    return(*destination);
+  if ((*length+strlen(content)+MaxTextExtent) > *max_length)
     {
-      case '\0':
+      *max_length=(*length)+strlen(content)+MaxTextExtent;
+      *destination=(char *) ResizeMagickMemory(*destination,*max_length*
+        sizeof(**destination));
+      if (*destination == (char *) NULL)
         return(*destination);
-      case '&':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&amp;");
-        break;
-      }
-      case '<':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&lt;");
-        break;
-      }
-      case '>':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&gt;");
-        break;
-      }
-      case '"':
-      {
-        if (pedantic == MagickFalse)
-          {
-            (*destination)[*length]=(*source);
-            (*length)++;
-            break;
-          }
-        *length+=FormatMagickString(*destination+(*length),*max_length,
-          "&quot;");
-        break;
-      }
-      case '\n':
-      {
-        if (pedantic == MagickFalse)
-          {
-            (*destination)[*length]=(*source);
-            (*length)++;
-            break;
-          }
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&#xA;");
-        break;
-      }
-      case '\t':
-      {
-        if (pedantic == MagickFalse)
-          {
-            (*destination)[*length]=(*source);
-            (*length)++;
-            break;
-          }
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&#x9;");
-        break;
-      }
-      case '\r':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&#xD;");
-        break;
-      }
-      default:
-      {
-        (*destination)[*length]=(*source);
-        (*length)++;
-        break;
-      }
     }
-  }
+  *length+=FormatMagickString(*destination+(*length),*max_length,"%s",content);
+  content=(char *) RelinquishMagickMemory(content);
   return(*destination);
 }
 
@@ -2114,8 +2458,8 @@ static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
   while ((content[offset] != '\0') && (offset < xml_info->offset))
     offset++;
   if (xml_info->ordered != (XMLTreeInfo *) NULL)
-    content=XMLTreeTagToXML(xml_info->ordered,source,length,max_length,(size_t)
-      offset,attributes);
+    content=XMLTreeTagToXML(xml_info->ordered,source,length,max_length,offset,
+      attributes);
   else
     content=EncodePredefinedEntities(content+offset,-1,source,length,max_length,
       MagickFalse);
