@@ -128,6 +128,7 @@ static void AnimateUsage(void)
       "-help                print program options",
       "-identify            identify the format and characteristics of the image",
       "-interlace type      type of image interlacing scheme",
+      "-interpolate method  pixel color interpolation method",
       "-limit type value    pixel cache resource limit",
       "-log format          format of debugging information",
       "-loop iterations     loop images then exit",
@@ -137,8 +138,12 @@ static void AnimateUsage(void)
       "-monochrome          transform image to black and white",
       "-pause               seconds to pause before reanimating",
       "-page geometry       size and location of an image canvas (setting)",
+      "-quantize colorspace reduce colors in this colorspace",
+      "-quiet               suppress all error or warning messages",
+      "-regard-warnings     pay attention to warning messages",
       "-remote command      execute a command in an remote display process",
       "-repage geometry     size and location of an image canvas (operator)",
+      "-resample geometry   change the resolution of an image",
       "-resize geometry     resize the image",
       "-rotate degrees      apply Paeth rotation to the image",
       "-sampling-factor geometry",
@@ -148,6 +153,8 @@ static void AnimateUsage(void)
       "-size geometry       width and height of image",
       "-strip               strip image of all profiles and comments",
       "-support factor      resize support: > 1.0 is blurry, < 1.0 is sharp",
+      "-transparent-color color",
+      "                     transparent color",
       "-treedepth value     color tree depth",
       "-trim                trim image edges",
       "-verbose             print detailed information about the image",
@@ -200,6 +207,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
       XCloseDisplay(display); \
       display=(Display *) NULL; \
     } \
+  XDestroyResourceInfo(&resource_info); \
   for ( ; k >= 0; k--) \
     image_stack[k]=DestroyImageList(image_stack[k]); \
   for (i=0; i < (long) argc; i++) \
@@ -334,10 +342,10 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
     ParseMagickOption(MagickInterlaceOptions,MagickFalse,resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "verbose","False");
-  image_info->verbose=IsTrue(resource_value);
+  image_info->verbose=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "dither","True");
-  quantize_info->dither=IsTrue(resource_value);
+  quantize_info->dither=IsMagickTrue(resource_value);
   /*
     Parse command line.
   */
@@ -398,10 +406,19 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
         MogrifyImageStack(image_stack[k],MagickTrue,pend);
         for (scene=first_scene; scene <= last_scene ; scene++)
         {
+          const char
+            *filename;
+
           /*
             Read image.
           */
-          (void) CopyMagickString(image_info->filename,option,MaxTextExtent);
+          filename=option;
+          if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+            {
+              option=argv[++i];
+              filename=option;
+            }
+          (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
           if (first_scene != last_scene)
             {
               char
@@ -577,7 +594,6 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             long
               colorspace;
 
-            quantize_info->colorspace=UndefinedColorspace;
             if (*option == '+')
               break;
             i++;
@@ -588,12 +604,6 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             if (colorspace < 0)
               ThrowAnimateException(OptionError,"UnrecognizedColorspace",
                 argv[i]);
-            quantize_info->colorspace=(ColorspaceType) colorspace;
-            if (quantize_info->colorspace == GRAYColorspace)
-              {
-                quantize_info->number_colors=256;
-                quantize_info->colorspace=GRAYColorspace;
-              }
             break;
           }
         if (LocaleCompare("crop",option+1) == 0)
@@ -613,18 +623,19 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            LogEventType
-              event_mask;
+            long
+              event;
 
             if (*option == '+')
               break;
             i++;
             if (i == (long) argc)
               ThrowAnimateException(OptionError,"MissingArgument",option);
-            event_mask=SetLogEventMask(argv[i]);
-            if (event_mask == UndefinedEvents)
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
               ThrowAnimateException(OptionError,"UnrecognizedEventType",
                 argv[i]);
+            (void) SetLogEventMask(argv[i]);
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -816,6 +827,23 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("interpolate",option+1) == 0)
+          {
+            long
+              interpolate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowAnimateException(OptionError,"MissingArgument",option);
+            interpolate=ParseMagickOption(MagickInterpolateOptions,MagickFalse,
+              argv[i]);
+            if (interpolate < 0)
+              ThrowAnimateException(OptionError,"UnrecognizedInterpolateMethod",
+                argv[i]);
+            break;
+          }
         ThrowAnimateException(OptionError,"UnrecognizedOption",option);
       }
       case 'l':
@@ -882,7 +910,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             resource_info.map_type=(char *) NULL;
             if (*option == '+')
               break;
-            (void) CopyMagickString(argv[i]+1,"{1}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             i++;
             if (i == (long) argc)
               ThrowAnimateException(OptionError,"MissingArgument",option);
@@ -963,8 +991,33 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
           }
         ThrowAnimateException(OptionError,"UnrecognizedOption",option);
       }
+      case 'q':
+      {
+        if (LocaleCompare("quantize",option+1) == 0)
+          {
+            long
+              colorspace;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowAnimateException(OptionError,"MissingArgument",option);
+            colorspace=ParseMagickOption(MagickColorspaceOptions,
+              MagickFalse,argv[i]);
+            if (colorspace < 0)
+              ThrowAnimateException(OptionError,"UnrecognizedColorspace",
+                argv[i]);
+            break;
+          }
+        if (LocaleCompare("quiet",option+1) == 0)
+          break;
+        ThrowAnimateException(OptionError,"UnrecognizedOption",option);
+      }
       case 'r':
       {
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
         if (LocaleCompare("remote",option+1) == 0)
           {
             i++;
@@ -986,6 +1039,17 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
               ThrowAnimateInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("resample",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowAnimateException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowAnimateInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("resize",option+1) == 0)
           {
             if (*option == '+')
@@ -1117,6 +1181,15 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             resource_info.title=argv[i];
             break;
           }
+        if (LocaleCompare("transparent-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowAnimateException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("treedepth",option+1) == 0)
           {
             quantize_info->tree_depth=0;
