@@ -98,6 +98,7 @@ static void CompareUsage(void)
       "-depth value         image depth",
       "-extract geometry    extract area from image",
       "-format \"string\"     output formatted image characteristics",
+      "-fuzz distance       colors within this distance are considered equal",
       "-help                print program options",
       "-identify            identify the format and characteristics of the image",
       "-interlace type      type of image interlacing scheme",
@@ -108,10 +109,14 @@ static void CompareUsage(void)
       "-profile filename    add, delete, or apply an image profile",
       "-quality value       JPEG/MIFF/PNG compression level",
       "-quiet               suppress all error or warning messages",
+      "-quantize colorspace reduce colors in this colorspace",
+      "-regard-warnings     pay attention to warning messages",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-set attribute value set an image attribute",
       "-size geometry       width and height of image",
+      "-transparent-color color",
+      "                     transparent color",
       "-type type           image type",
       "-verbose             print detailed information about the image",
       "-version             print version information",
@@ -206,7 +211,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   channel=AllChannels;
   difference_image=NewImageList();
   distortion=0.0;
-  format="%w,%h,%m";
+  format=(char *) NULL;
   j=1;
   k=0;
   image_stack[k]=NewImageList();
@@ -262,6 +267,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         */
         MogrifyImageStack(image_stack[k],MagickTrue,pend);
         filename=argv[i];
+        if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+          filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
         image=ReadImage(image_info,exception);
         status&=(image != (Image *) NULL) &&
@@ -425,6 +432,17 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             format=argv[i];
             break;
           }
+        if (LocaleCompare("fuzz",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'h':
@@ -537,10 +555,33 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("quantize",option+1) == 0)
+          {
+            long
+              colorspace;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            colorspace=ParseMagickOption(MagickColorspaceOptions,
+              MagickFalse,argv[i]);
+            if (colorspace < 0)
+              ThrowCompareException(OptionError,"UnrecognizedColorspace",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("quiet",option+1) == 0)
           break;
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
+      case 'r':
+      {
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
+        ThrowCompareException(OptionError,"UnrecognizedOption",option)
+      }
       case 's':
       {
         if (LocaleCompare("sampling-factor",option+1) == 0)
@@ -581,6 +622,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       }
       case 't':
       {
+        if (LocaleCompare("transparent-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("type",option+1) == 0)
           {
             long
@@ -637,7 +687,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   }
   if (k != 0)
     ThrowCompareException(OptionError,"UnbalancedParenthesis",argv[i]);
-  if (i != (long) (argc-1))
+  if (i-- != (long) (argc-1))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
   if ((image_stack[k] == (Image *) NULL) ||
       (GetImageListLength(image_stack[k]) < 2))
@@ -654,7 +704,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       if (image_info->verbose != MagickFalse)
         (void) IsImagesEqual(image,reconstruct_image);
       status&=WriteImages(image_info,difference_image,argv[argc-1],exception);
-      if (metadata != (char **) NULL)
+      if ((metadata != (char **) NULL) && (format != (char *) NULL))
         {
           char
             *text;
@@ -676,7 +726,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         }
       difference_image=DestroyImageList(difference_image);
       if (metric != UndefinedMetric)
-        (void) fprintf(stdout,"%g dB\n",distortion);
+        (void) fprintf(stdout,"%g\n",distortion);
     }
   DestroyCompare();
   return(status != 0 ? MagickTrue : MagickFalse);
