@@ -257,6 +257,7 @@ static void CompositeUsage(void)
       "-help                print program options",
       "-identify            identify the format and characteristics of the image",
       "-interlace type      type of image interlacing scheme",
+      "-interpolate method  pixel color interpolation method",
       "-label name          assign a label to an image",
       "-limit type value    pixel cache resource limit",
       "-log format          format of debugging information",
@@ -267,8 +268,10 @@ static void CompositeUsage(void)
       "-page geometry       size and location of an image canvas (setting)",
       "-profile filename    add ICM or IPTC information profile to image",
       "-quality value       JPEG/MIFF/PNG compression level",
+      "-quantize colorspace reduce colors in this colorspace",
       "-quiet               suppress all error or warning messages",
       "-red-primary point   chromaticity red primary point",
+      "-regard-warnings     pay attention to warning messages",
       "-rotate degrees      apply Paeth rotation to the image",
       "-repage geometry     size and location of an image canvas (operator)",
       "-resize geometry     resize the image",
@@ -284,6 +287,8 @@ static void CompositeUsage(void)
       "-thumbnail geometry  create a thumbnail of the image",
       "-tile                repeat composite operation across and down image",
       "-transform           affine transform image",
+      "-transparent-color color",
+      "                     transparent color",
       "-treedepth value     color tree depth",
       "-type type           image type",
       "-units type          the units of image resolution",
@@ -462,6 +467,8 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         */
         MogrifyImageStack(image_stack[k],MagickTrue,pend);
         filename=argv[i];
+        if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+          filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
         image=ReadImage(image_info,exception);
         status&=(image != (Image *) NULL) &&
@@ -648,18 +655,19 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            LogEventType
-              event_mask;
+            long
+              event;
 
             if (*option == '+')
               break;
             i++;
             if (i == (long) argc)
               ThrowCompositeException(OptionError,"MissingArgument",option);
-            event_mask=SetLogEventMask(argv[i]);
-            if (event_mask == UndefinedEvents)
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
               ThrowCompositeException(OptionError,"UnrecognizedEventType",
                 argv[i]);
+            (void) SetLogEventMask(argv[i]);
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -914,6 +922,23 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
                 "UnrecognizedInterlaceType",argv[i]);
             break;
           }
+        if (LocaleCompare("interpolate",option+1) == 0)
+          {
+            long
+              interpolate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            interpolate=ParseMagickOption(MagickInterpolateOptions,MagickFalse,
+              argv[i]);
+            if (interpolate < 0)
+              ThrowCompositeException(OptionError,
+                "UnrecognizedInterpolateMethod",argv[i]);
+            break;
+          }
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 'l':
@@ -1021,6 +1046,23 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("quantize",option+1) == 0)
+          {
+            long
+              colorspace;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            colorspace=ParseMagickOption(MagickColorspaceOptions,
+              MagickFalse,argv[i]);
+            if (colorspace < 0)
+              ThrowCompositeException(OptionError,"UnrecognizedColorspace",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("quiet",option+1) == 0)
           break;
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
@@ -1038,6 +1080,8 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
         if (LocaleCompare("render",option+1) == 0)
           break;
         if (LocaleCompare("repage",option+1) == 0)
@@ -1166,11 +1210,20 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         if (LocaleCompare("tile",option+1) == 0)
           {
             option_info.tile=(*option == '-') ? MagickTrue : MagickFalse;
-            (void) CopyMagickString(argv[i]+1,"{0}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
             break;
           }
         if (LocaleCompare("transform",option+1) == 0)
           break;
+        if (LocaleCompare("transparent-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("treedepth",option+1) == 0)
           {
             if (*option == '+')
@@ -1309,38 +1362,28 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
   }
   if (k != 0)
     ThrowCompositeException(OptionError,"UnbalancedParenthesis",argv[i]);
-  if (i != (long) (argc-1))
+  if (i-- != (long) (argc-1))
     ThrowCompositeException(OptionError,"MissingAnImageFilename",argv[i]);
   if ((image_stack[k] == (Image *) NULL) ||
       (GetImageListLength(image_stack[k]) < 2))
     ThrowCompositeException(OptionError,"MissingAnImageFilename",argv[argc-1]);
   MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-  composite_image=GetImageFromList(image_stack[k],0);
-  image=GetImageFromList(image_stack[k],1);
-  mask_image=GetImageFromList(image_stack[k],2);
+  /*
+    Composite images.
+  */
+  composite_image=RemoveFirstImageFromList(image_stack+k);
+  image=RemoveFirstImageFromList(image_stack+k);
+  (void) TransformImage(&composite_image,(char *) NULL,
+    composite_image->geometry);
+  mask_image=RemoveFirstImageFromList(image_stack+k);
   if (mask_image != (Image *) NULL)
     {
-      (void) SetImageType(composite_image,TrueColorMatteType);
-      if (composite_image->matte == MagickFalse)
-        SetImageOpacity(composite_image,OpaqueOpacity);
-      status&=CompositeImage(composite_image,CopyOpacityCompositeOp,mask_image,
-        0,0);
-      if (status == MagickFalse)
-        GetImageException(composite_image,exception);
-    }
-  image=CloneImage(image,0,0,MagickTrue,exception);
-  if (image == (Image *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
-        message);
-      message=(char *) RelinquishMagickMemory(message);
+      image->clip_mask=mask_image;
+      NegateImage(image->clip_mask,MagickFalse);
     }
   status&=CompositeImageList(image_info,&image,composite_image,&option_info,
     exception);
+  composite_image=DestroyImage(composite_image);
   /*
     Write composite images.
   */
@@ -1365,6 +1408,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
       (void) ConcatenateString(&(*metadata),"\n");
       text=(char *) RelinquishMagickMemory(text);
     }
+  image=DestroyImage(image);
   RelinquishCompositeOptions(&option_info);
   DestroyComposite();
   return(status != 0 ? MagickTrue : MagickFalse);
