@@ -112,6 +112,7 @@ static void ConjureUsage(void)
       "-log format          format of debugging information",
       "-monitor             monitor progress",
       "-quiet               suppress all error or warning messages",
+      "-regard-warnings     pay attention to warning messages",
       "-verbose             print detailed information about the image",
       "-version             print version information",
       (char *) NULL
@@ -199,8 +200,6 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
         message);
       message=(char *) RelinquishMagickMemory(message);
     }
-  if (image_info->attributes == (Image *) NULL)
-    image_info->attributes=AllocateImage(image_info);
   for (i=1; i < (long) argc; i++)
   {
     option=argv[i];
@@ -208,13 +207,19 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowConjureException(OptionError,"MissingEventMask",option);
-                (void) SetLogEventMask(argv[i]);
-              }
+            long
+              event;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowConjureException(OptionError,"MissingArgument",option);
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
+              ThrowConjureException(OptionError,"UnrecognizedEventType",
+                argv[i]);
+            (void) SetLogEventMask(argv[i]);
             continue;
           }
         if ((LocaleCompare("help",option+1) == 0) ||
@@ -239,6 +244,8 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
           continue;
         if (LocaleCompare("quiet",option+1) == 0)
           continue;
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
         if (LocaleCompare("verbose",option+1) == 0)
           {
             image_info->verbose=(*option == '-') ? MagickTrue : MagickFalse;
@@ -256,8 +263,8 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
         /*
           Persist key/value pair.
         */
-        (void) DeleteImageAttribute(image_info->attributes,option+1);
-        status&=SetImageAttribute(image_info->attributes,option+1,argv[i+1]);
+        (void) DeleteImageOption(image_info,option+1);
+        status&=SetImageOption(image_info,option+1,argv[i+1]);
         if (status == MagickFalse)
           ThrowConjureException(ImageError,"UnableToPersistKey",option);
         i++;
@@ -266,8 +273,8 @@ WandExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
     /*
       Interpret MSL script.
     */
-    (void) DeleteImageAttribute(image_info->attributes,"filename");
-    status&=SetImageAttribute(image_info->attributes,"filename",argv[i]);
+    (void) DeleteImageOption(image_info,"filename");
+    status&=SetImageOption(image_info,"filename",argv[i]);
     if (status == MagickFalse)
       ThrowConjureException(ImageError,"UnableToPersistKey",argv[i]);
     (void) FormatMagickString(image_info->filename,MaxTextExtent,"msl:%s",
