@@ -141,6 +141,12 @@ static void ConvertUsage(void)
   static const char
     *options[]=
     {
+      "-adaptive-blur geometry",
+      "                     adaptively blur pixels; decrease effect near edges",
+      "-adaptive-resize geometry",
+      "                     adaptively resize image with data dependent triangulation",
+      "-adaptive-sharpen geometry",
+      "                     adaptively sharpen pixels; increase effect near edges",
       "-adjoin              join images into a single multi-image file",
       "-affine matrix       affine transform matrix",
       "-annotate geometry text",
@@ -148,11 +154,14 @@ static void ConvertUsage(void)
       "-antialias           remove pixel-aliasing",
       "-append              append an image sequence",
       "-authenticate value  decrypt image with this password",
+      "-auto-orient         automatically orient image",
       "-average             average an image sequence",
       "-background color    background color",
       "-bias value          add bias when convolving an image",
+      "-black-point-compensation",
+      "                     use black point compensation",
       "-black-threshold value",
-      "                     forces all pixels below the threshold into black",
+      "                     force all pixels below the threshold into black",
       "-blue-primary point  chromaticity blue primary point",
       "-blur geometry       blur the image",
       "-border geometry     surround image with a border of color",
@@ -161,6 +170,7 @@ static void ConvertUsage(void)
       "-charcoal radius     simulate a charcoal drawing",
       "-chop geometry       remove pixels from the image interior",
       "-clip                clip along the first path from the 8BIM profile",
+      "-clip-mask           clip as defined by an image mask",
       "-clip-path id        clip along a named path from the 8BIM profile",
       "-clone index         clone an image",
       "-coalesce            merge a sequence of images",
@@ -174,7 +184,7 @@ static void ConvertUsage(void)
       "-compress type       type of pixel compression when writing the image",
       "-contrast            enhance or reduce the image contrast",
       "-contrast-stretch geometry",
-      "                     improve the contrast in an image by `stretching' the range of intensity values",
+      "                     improve contrast by `stretching' the intensity range",
       "-convolve coefficients",
       "                     apply a convolution kernel to the image",
       "-crop geometry       cut out a rectangular region of the image",
@@ -206,10 +216,10 @@ static void ConvertUsage(void)
       "-fill color          color to use when filling a graphic primitive",
       "-filter type         use this filter when resizing an image",
       "-flatten             flatten a sequence of images",
-      "-flip                flip image in the vertical direction",
+      "-flip                flip image vertically",
       "-floodfill geometry color",
       "                     floodfill the image with color",
-      "-flop                flop image in the horizontal direction",
+      "-flop                flop image horizontally",
       "-font name           render text with this font",
       "-format \"string\"     output formatted image characteristics",
       "-frame geometry      surround image with an ornamental border",
@@ -226,8 +236,10 @@ static void ConvertUsage(void)
       "-insert index        insert last image into the image sequence",
       "-intent type         type of rendering intent when managing the image color",
       "-interlace type      type of image interlacing scheme",
+      "-interpolate method  pixel color interpolation method",
       "-label name          assign a label to an image",
       "-lat geometry        local adaptive thresholding",
+      "-layers method       optimize or compare image layers",
       "-level value         adjust the level of image contrast",
       "-limit type value    pixel cache resource limit",
       "-log format          format of debugging information",
@@ -257,14 +269,18 @@ static void ConvertUsage(void)
       "-pointsize value     font point size",
       "-posterize levels    reduce the image to a limited number of color levels",
       "-preview type        image preview type",
+      "-print string        interpret string and print to console",
+      "-process arguments   process the image with a custom image filter",
       "-profile filename    add, delete, or apply an image profile",
       "-quality value       JPEG/MIFF/PNG compression level",
+      "-quantize colorspace reduce colors in this colorspace",
       "-quiet               suppress all error or warning messages",
       "-radial-blur angle   radial blur the image",
       "-raise value         lighten/darken image edges to create a 3-D effect",
       "-random-threshold low,high",
       "                     random threshold the image",
       "-red-primary point   chromaticity red primary point",
+      "-regard-warnings     pay attention to warning messages",
       "-region geometry     apply options to a portion of the image",
       "-render              render vector graphics",
       "-repage geometry     size and location of an image canvas",
@@ -288,9 +304,10 @@ static void ConvertUsage(void)
       "-sharpen geometry    sharpen the image",
       "-shave geometry      shave pixels from the image edges",
       "-shear geometry      slide one edge of the image along the X or Y axis",
-      "-sigmodial-contrast geometry",
+      "-sigmoidal-contrast geometry",
       "                     lightness rescaling using sigmoidal contrast enhancement",
       "-size geometry       width and height of image",
+      "-sketch geometry     simulate a pencil sketch",
       "-solarize threshold  negate all pixels above the threshold level",
       "-splice geometry     splice the background color into the image",
       "-spread amount       displace image pixels by a random amount",
@@ -309,10 +326,15 @@ static void ConvertUsage(void)
       "-tint value          tint the image with the fill color",
       "-transform           affine transform image",
       "-transparent color   make this color transparent within the image",
+      "-transparent-color color",
+      "                     transparent color",
+      "-transpose           flip image vertically and rotate 90 degrees",
+      "-transverse          flop image horizontally and rotate 270 degrees",
       "-treedepth value     color tree depth",
       "-trim                trim image edges",
       "-type type           image type",
       "-undercolor color    annotation bounding box color",
+      "-unique-colors       discard all but one of any pixel color",
       "-units type          the units of image resolution",
       "-unsharp geometry    sharpen the image",
       "-verbose             print detailed information about the image",
@@ -325,7 +347,7 @@ static void ConvertUsage(void)
       "-weight type         render text with this font weight",
       "-white-point point   chromaticity white point",
       "-white-threshold value",
-      "                     forces all pixels above the threshold into white",
+      "                     force all pixels above the threshold into white",
       "-write filename      write images to this file",
       (char *) NULL
     };
@@ -473,6 +495,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
         */
         MogrifyImageStack(image_stack[k],MagickTrue,pend);
         filename=argv[i];
+        if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+          filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
         if (image_info->ping != MagickFalse)
           image=PingImage(image_info,exception);
@@ -490,6 +514,33 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
     {
       case 'a':
       {
+        if (LocaleCompare("adaptive-blur",option+1) == 0)
+          {
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConvertInvalidArgumentException(option,argv[i]);
+            break;
+          }
+        if (LocaleCompare("adaptive-resize",option+1) == 0)
+          {
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConvertInvalidArgumentException(option,argv[i]);
+            break;
+          }
+        if (LocaleCompare("adaptive-sharpen",option+1) == 0)
+          {
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConvertInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("adjoin",option+1) == 0)
           break;
         if (LocaleCompare("affine",option+1) == 0)
@@ -530,6 +581,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("auto-orient",option+1) == 0)
+          break;
         if (LocaleCompare("average",option+1) == 0)
           break;
         ThrowConvertException(OptionError,"UnrecognizedOption",option)
@@ -556,6 +609,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("black-point-compensation",option+1) == 0)
+          break;
         if (LocaleCompare("black-threshold",option+1) == 0)
           {
             if (*option == '+')
@@ -673,6 +728,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("clip",option+1) == 0)
           break;
+        if (LocaleCompare("clip-mask",option+1) == 0)
+          break;
         if (LocaleCompare("clip-path",option+1) == 0)
           {
             i++;
@@ -846,18 +903,20 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
           break;
         if (LocaleCompare("debug",option+1) == 0)
           {
-            LogEventType
-              event_mask;
+            long
+              event;
 
             if (*option == '+')
               break;
             i++;
             if (i == (long) argc)
               ThrowConvertException(OptionError,"MissingArgument",option);
-            event_mask=SetLogEventMask(argv[i]);
-            if (event_mask == UndefinedEvents)
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
               ThrowConvertException(OptionError,"UnrecognizedEventType",
                 argv[i]);
+            (void) SetLogEventMask(argv[i]);
+            break;
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -1297,6 +1356,23 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("interpolate",option+1) == 0)
+          {
+            long
+              interpolate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            interpolate=ParseMagickOption(MagickInterpolateOptions,MagickFalse,
+              argv[i]);
+            if (interpolate < 0)
+              ThrowConvertException(OptionError,"UnrecognizedInterpolateMethod",
+                argv[i]);
+            break;
+          }
         ThrowConvertException(OptionError,"UnrecognizedOption",option)
       }
       case 'l':
@@ -1321,6 +1397,22 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("layers",option+1) == 0)
+          {
+            long
+              type;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            type=ParseMagickOption(MagickLayersOptions,MagickFalse,argv[i]);
+            if (type < 0)
+              ThrowConvertException(OptionError,"UnrecognizedLayerType",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("level",option+1) == 0)
           {
             i++;
@@ -1536,8 +1628,6 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) (argc-1))
               ThrowConvertException(OptionError,"MissingArgument",option);
-            if (IsGeometry(argv[i]) == MagickFalse)
-              ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
         if (LocaleCompare("orient",option+1) == 0)
@@ -1553,8 +1643,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
             orientation=ParseMagickOption(MagickOrientationOptions,
               MagickFalse,argv[i]);
             if (orientation < 0)
-              ThrowConvertException(OptionError,
-                "UnrecognizedImageOrientation",argv[i]);
+              ThrowConvertException(OptionError,"UnrecognizedImageOrientation",
+                argv[i]);
             break;
           }
         ThrowConvertException(OptionError,"UnrecognizedOption",argv[i])
@@ -1622,6 +1712,15 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("print",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("process",option+1) == 0)
           {
             if (*option == '+')
@@ -1653,6 +1752,23 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("quantize",option+1) == 0)
+          {
+            long
+              colorspace;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            colorspace=ParseMagickOption(MagickColorspaceOptions,
+              MagickFalse,argv[i]);
+            if (colorspace < 0)
+              ThrowConvertException(OptionError,"UnrecognizedColorspace",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("quiet",option+1) == 0)
           break;
         ThrowConvertException(OptionError,"UnrecognizedOption",option)
@@ -1699,6 +1815,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
         if (LocaleCompare("region",option+1) == 0)
           {
             if (*option == '+')
@@ -1934,6 +2052,17 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("sketch",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConvertInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("solarize",option+1) == 0)
           {
             if (*option == '+')
@@ -2120,6 +2249,19 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("transparent-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            break;
+          }
+        if (LocaleCompare("transpose",option+1) == 0)
+          break;
+        if (LocaleCompare("transverse",option+1) == 0)
+          break;
         if (LocaleCompare("treedepth",option+1) == 0)
           {
             if (*option == '+')
@@ -2162,6 +2304,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("unique-colors",option+1) == 0)
+          break;
         if (LocaleCompare("units",option+1) == 0)
           {
             long
@@ -2301,7 +2445,7 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
   }
   if (k != 0)
     ThrowConvertException(OptionError,"UnbalancedParenthesis",argv[i]);
-  if (i != (long) (argc-1))
+  if (i-- != (long) (argc-1))
     ThrowConvertException(OptionError,"MissingAnImageFilename",argv[argc-1]);
   if (image_stack[k] == (Image *) NULL)
     ThrowConvertException(OptionError,"MissingAnImageFilename",argv[argc-1]);
