@@ -105,6 +105,7 @@ static void DisplayUsage(void)
     {
       "-antialias           remove pixel-aliasing",
       "-authenticate value  decrypt image with this password",
+      "-auto-orient         automatically orient image",
       "-backdrop            display image centered on a backdrop",
       "-border geometry     surround image with a border of color",
       "-channel type        apply option to select image channels",
@@ -144,6 +145,7 @@ static void DisplayUsage(void)
       "-identify            identify the format and characteristics of the image",
       "-immutable           displayed image cannot be modified",
       "-interlace type      type of image interlacing scheme",
+      "-interpolate method  pixel color interpolation method",
       "-label name          assign a label to an image",
       "-limit type value    pixel cache resource limit",
       "-log format          format of debugging information",
@@ -156,10 +158,13 @@ static void DisplayUsage(void)
       "-page geometry       size and location of an image canvas",
       "-profile filename    add, delete, or apply an image profile",
       "-quality value       JPEG/MIFF/PNG compression level",
+      "-quantize colorspace reduce colors in this colorspace",
       "-quiet               suppress all error or warning messages",
       "-raise value         lighten/darken image edges to create a 3-D effect",
+      "-regard-warnings     pay attention to warning messages",
       "-remote command      execute a command in an remote display process",
       "-repage geometry     size and location of an image canvas (operator)",
+      "-resample geometry   change the resolution of an image",
       "-resize geometry     resize the image",
       "-roll geometry       roll an image vertically or horizontally",
       "-rotate degrees      apply Paeth rotation to the image",
@@ -174,6 +179,8 @@ static void DisplayUsage(void)
       "-strip               strip image of all profiles and comments",
       "-support factor      resize support: > 1.0 is blurry, < 1.0 is sharp",
       "-texture filename    name of texture to tile onto the image background",
+      "-transparent-color color",
+      "                     transparent color",
       "-treedepth value     color tree depth",
       "-trim                trim image edges",
       "-update seconds      detect when image file is modified and redisplay",
@@ -183,7 +190,7 @@ static void DisplayUsage(void)
       "-virtual-pixel method",
       "                     virtual pixel access method",
       "-window id           display image to background of this window",
-      "-window_group id     exit program when this window id is destroyed",
+      "-window-group id     exit program when this window id is destroyed",
       "-write filename      write image to a file",
       (char *) NULL
     };
@@ -230,6 +237,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       XCloseDisplay(display); \
       display=(Display *) NULL; \
     } \
+  XDestroyResourceInfo(&resource_info); \
   for ( ; k >= 0; k--) \
     image_stack[k]=DestroyImageList(image_stack[k]); \
   if (image_marker != (unsigned long *) NULL) \
@@ -336,8 +344,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
         message);
       message=(char *) RelinquishMagickMemory(message);
     }
-  image_marker=(unsigned long *)
-    AcquireMagickMemory((argc+1)*sizeof(*image_marker));
+  image_marker=(unsigned long *) AcquireMagickMemory((argc+1)*
+    sizeof(*image_marker));
   if (image_marker == (unsigned long *) NULL)
     {
       char
@@ -402,16 +410,16 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
   image_info->quality=(unsigned long) atol(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "verbose","False");
-  image_info->verbose=IsTrue(resource_value);
+  image_info->verbose=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "dither","True");
-  quantize_info->dither=IsTrue(resource_value);
+  quantize_info->dither=IsMagickTrue(resource_value);
   /*
     Parse command line.
   */
   iteration=0;
   iterations=0;
-  for (i=1; ((i <= argc) && ((state & ExitState) == 0)); i++)
+  for (i=1; ((i <= (long) argc) && ((state & ExitState) == 0)); i++)
   {
     if (i < (long) argc)
       option=argv[i];
@@ -465,10 +473,19 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
         MogrifyImageStack(image_stack[k],MagickFalse,pend);
         for (scene=first_scene; scene <= last_scene ; scene++)
         {
+          const char
+            *filename;
+
           /*
             Read image.
           */
-          (void) CopyMagickString(image_info->filename,option,MaxTextExtent);
+          filename=option;
+          if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+            {
+              option=argv[++i];
+              filename=option;
+            }
+          (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
           if (first_scene != last_scene)
             {
               char
@@ -650,6 +667,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("auto-orient",option+1) == 0)
+          break;
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
       }
       case 'b':
@@ -781,7 +800,6 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             long
               colorspace;
 
-            quantize_info->colorspace=UndefinedColorspace;
             if (*option == '+')
               break;
             i++;
@@ -792,9 +810,6 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             if (colorspace < 0)
               ThrowDisplayException(OptionError,"UnrecognizedColorspace",
                 argv[i]);
-            quantize_info->colorspace=(ColorspaceType) colorspace;
-            if (quantize_info->colorspace == GRAYColorspace)
-              quantize_info->number_colors=256;
             break;
           }
         if (LocaleCompare("comment",option+1) == 0)
@@ -842,18 +857,19 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            LogEventType
-              event_mask;
+            long
+              event;
 
             if (*option == '+')
               break;
             i++;
             if (i == (long) argc)
               ThrowDisplayException(OptionError,"MissingArgument",option);
-            event_mask=SetLogEventMask(argv[i]);
-            if (event_mask == UndefinedEvents)
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
               ThrowDisplayException(OptionError,"UnrecognizedEventType",
                 argv[i]);
+            (void) SetLogEventMask(argv[i]);
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -1069,7 +1085,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             resource_info.image_geometry=(char *) NULL;
             if (*option == '+')
               break;
-            (void) CopyMagickString(argv[i]+1,"{1}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             i++;
             if (i == (long) argc)
               ThrowDisplayException(OptionError,"MissingArgument",option);
@@ -1131,6 +1147,23 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("interpolate",option+1) == 0)
+          {
+            long
+              interpolate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowDisplayException(OptionError,"MissingArgument",option);
+            interpolate=ParseMagickOption(MagickInterpolateOptions,MagickFalse,
+              argv[i]);
+            if (interpolate < 0)
+              ThrowDisplayException(OptionError,"UnrecognizedInterpolateMethod",
+                argv[i]);
+            break;
+          }
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
       }
       case 'l':
@@ -1211,7 +1244,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             resource_info.map_type=(char *) NULL;
             if (*option == '+')
               break;
-            (void) CopyMagickString(argv[i]+1,"{1}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             i++;
             if (i == (long) argc)
               ThrowDisplayException(OptionError,"MissingArgument",option);
@@ -1296,6 +1329,23 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("quantize",option+1) == 0)
+          {
+            long
+              colorspace;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowDisplayException(OptionError,"MissingArgument",option);
+            colorspace=ParseMagickOption(MagickColorspaceOptions,
+              MagickFalse,argv[i]);
+            if (colorspace < 0)
+              ThrowDisplayException(OptionError,"UnrecognizedColorspace",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("quiet",option+1) == 0)
           break;
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
@@ -1311,6 +1361,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
         if (LocaleCompare("remote",option+1) == 0)
           {
             i++;
@@ -1332,6 +1384,17 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("resample",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowDisplayException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowDisplayInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("resize",option+1) == 0)
           {
             if (*option == '+')
@@ -1389,6 +1452,11 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("sans",option+1) == 0)
+          {
+            i++;
+            break;
+          }
         if (LocaleCompare("scenes",option+1) == 0)
           {
             first_scene=0;
@@ -1473,7 +1541,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       }
       case 't':
       {
-        if (LocaleCompare("text_font",option+1) == 0)
+        if (LocaleCompare("text-font",option+1) == 0)
           {
             resource_info.text_font=(char *) NULL;
             if (*option == '+')
@@ -1516,6 +1584,15 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             resource_info.title=argv[i];
             break;
           }
+        if (LocaleCompare("transparent-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowDisplayException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("treedepth",option+1) == 0)
           {
             quantize_info->tree_depth=0;
@@ -1606,7 +1683,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             resource_info.window_id=argv[i];
             break;
           }
-        if (LocaleCompare("window_group",option+1) == 0)
+        if (LocaleCompare("window-group",option+1) == 0)
           {
             resource_info.window_group=(char *) NULL;
             if (*option == '+')
