@@ -93,6 +93,7 @@ static void IdentifyUsage(void)
   static const char
     *options[]=
     {
+      "-antialias           remove pixel-aliasing",
       "-authenticate value  decrypt image with this password",
       "-channel type        apply option to select image channels",
       "-crop geometry       cut out a rectangular region of the image",
@@ -106,6 +107,7 @@ static void IdentifyUsage(void)
       "-fuzz distance       colors within this distance are considered equal",
       "-help                print program options",
       "-interlace type      type of image interlacing scheme",
+      "-interpolate method  pixel color interpolation method",
       "-limit type value    pixel cache resource limit",
       "-list type           Color, Configure, Delegate, Format, Magic, Module,",
       "                     Resource, or Type",
@@ -114,6 +116,7 @@ static void IdentifyUsage(void)
       "-monitor             monitor progress",
       "-ping                efficiently determine image attributes",
       "-quiet               suppress all error or warning messages",
+      "-regard-warnings     pay attention to warning messages",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-set attribute value set an image attribute",
@@ -248,6 +251,9 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       }
     if (IsMagickOption(option) == MagickFalse)
       {
+        char
+          *filename;
+
         Image
           *image;
 
@@ -260,7 +266,10 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
         MogrifyImageStack(image_stack[k],MagickFalse,pend);
         identify_info=CloneImageInfo(image_info);
         identify_info->verbose=MagickFalse;
-        (void) CopyMagickString(identify_info->filename,argv[i],MaxTextExtent);
+        filename=argv[i];
+        if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+          filename=argv[++i];
+        (void) CopyMagickString(identify_info->filename,filename,MaxTextExtent);
         if ((image_info->verbose != MagickFalse) ||
             (identify_info->ping == MagickFalse))
           image=ReadImage(identify_info,exception);
@@ -311,6 +320,21 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
     pend=image_stack[k] != (Image *) NULL ? MagickTrue : MagickFalse;
     switch (*(option+1))
     {
+      case 'a':
+      {
+        if (LocaleCompare("antialias",option+1) == 0)
+          break;
+        if (LocaleCompare("authenticate",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            break;
+          }
+        ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
+      }
       case 'c':
       {
         if (LocaleCompare("cache",option+1) == 0)
@@ -326,20 +350,18 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("channel",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                long
-                  channel;
+            long
+              channel;
 
-                i++;
-                if (i == (long) (argc-1))
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                channel=ParseChannelOption(argv[i]);
-                if (channel < 0)
-                  ThrowIdentifyException(OptionError,"UnrecognizedChannelType",
-                    argv[i]);
-                break;
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            channel=ParseChannelOption(argv[i]);
+            if (channel < 0)
+              ThrowIdentifyException(OptionError,"UnrecognizedChannelType",
+                argv[i]);
             break;
           }
         if (LocaleCompare("crop",option+1) == 0)
@@ -360,18 +382,20 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            LogEventType
-              event_mask;
+            long
+              event;
 
             if (*option == '+')
               break;
             i++;
             if (i == (long) argc)
               ThrowIdentifyException(OptionError,"MissingArgument",option);
-            event_mask=SetLogEventMask(argv[i]);
-            if (event_mask == UndefinedEvents)
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
               ThrowIdentifyException(OptionError,"UnrecognizedEventType",
                 argv[i]);
+            (void) SetLogEventMask(argv[i]);
+            break;
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -467,6 +491,23 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
                 "UnrecognizedInterlaceType",argv[i]);
             break;
           }
+        if (LocaleCompare("interpolate",option+1) == 0)
+          {
+            long
+              interpolate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            interpolate=ParseMagickOption(MagickInterpolateOptions,MagickFalse,
+              argv[i]);
+            if (interpolate < 0)
+              ThrowIdentifyException(OptionError,
+                "UnrecognizedInterpolateMethod",argv[i]);
+            break;
+          }
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
       }
       case 'l':
@@ -545,6 +586,12 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
           break;
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
       }
+      case 'r':
+      {
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
+        ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
+      }
       case 's':
       {
         if (LocaleCompare("sampling-factor",option+1) == 0)
