@@ -133,6 +133,7 @@ static void ImportUsage(void)
       "-help                print program options",
       "-identify            identify the format and characteristics of the image",
       "-interlace type      None, Line, Plane, or Partition",
+      "-interpolate method  pixel color interpolation method",
       "-label name          assign a label to an image",
       "-limit type value    Area, Disk, Map, or Memory resource limit",
       "-log format          format of debugging information",
@@ -143,7 +144,9 @@ static void ImportUsage(void)
       "-pause value         seconds delay between snapshots",
       "-pointsize value     font point size",
       "-quality value       JPEG/MIFF/PNG compression level",
+      "-quantize colorspace reduce colors in this colorspace",
       "-quiet               suppress all error or warning messages",
+      "-regard-warnings     pay attention to warning messages",
       "-repage geometry     size and location of an image canvas",
       "-resize geometry     resize the image",
       "-rotate degrees      apply Paeth rotation to the image",
@@ -158,6 +161,8 @@ static void ImportUsage(void)
       "-support factor      resize support: > 1.0 is blurry, < 1.0 is sharp",
       "-thumbnail geometry  create a thumbnail of the image",
       "-transparent color   make this color transparent within the image",
+      "-transparent-color color",
+      "                     transparent color",
       "-treedepth value     color tree depth",
       "-trim                trim image edges",
       "-type type           image type",
@@ -332,7 +337,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
   quantize_info=resource_info.quantize_info;
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "border","False");
-  ximage_info.borders=IsTrue(resource_value);
+  ximage_info.borders=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "delay","0");
   resource_info.delay=(unsigned int) atoi(resource_value);
@@ -340,10 +345,10 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
     "density",(char *) NULL);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "descend","True");
-  ximage_info.descend=IsTrue(resource_value);
+  ximage_info.descend=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "frame","False");
-  ximage_info.frame=IsTrue(resource_value);
+  ximage_info.frame=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "interlace","none");
   image_info->interlace=UndefinedInterlace;
@@ -368,16 +373,16 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
   image_info->quality=(unsigned long) atol(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "screen","False");
-  ximage_info.screen=IsTrue(resource_value);
+  ximage_info.screen=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "silent","False");
-  ximage_info.silent=IsTrue(resource_value);
+  ximage_info.silent=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "verbose","False");
-  image_info->verbose=IsTrue(resource_value);
+  image_info->verbose=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "dither","True");
-  quantize_info->dither=IsTrue(resource_value);
+  quantize_info->dither=IsMagickTrue(resource_value);
   snapshots=1;
   status=MagickTrue;
   filename=(char *) NULL;
@@ -470,7 +475,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
         if (LocaleCompare("border",option+1) == 0)
           {
             ximage_info.borders=(*option == '-') ? MagickTrue : MagickFalse;
-            (void) CopyMagickString(argv[i]+1,"{0}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
             break;
           }
         if (LocaleCompare("bordercolor",option+1) == 0)
@@ -531,7 +536,6 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
             long
               colorspace;
 
-            quantize_info->colorspace=UndefinedColorspace;
             if (*option == '+')
               break;
             i++;
@@ -542,12 +546,6 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
             if (colorspace < 0)
               ThrowImportException(OptionError,"UnrecognizedColorspace",
                 argv[i]);
-            quantize_info->colorspace=(ColorspaceType) colorspace;
-            if (quantize_info->colorspace == GRAYColorspace)
-              {
-                quantize_info->number_colors=256;
-                quantize_info->colorspace=GRAYColorspace;
-              }
             break;
           }
         if (LocaleCompare("comment",option+1) == 0)
@@ -596,18 +594,18 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            LogEventType
-              event_mask;
+            long
+              event;
 
-            (void) SetLogEventMask("None");
             if (*option == '+')
               break;
             i++;
             if (i == (long) argc)
               ThrowImportException(OptionError,"MissingArgument",option);
-            event_mask=SetLogEventMask(argv[i]);
-            if (event_mask == UndefinedEvents)
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
               ThrowImportException(OptionError,"UnrecognizedEventType",argv[i]);
+            (void) SetLogEventMask(argv[i]);
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -735,7 +733,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
         if (LocaleCompare("frame",option+1) == 0)
           {
             ximage_info.frame=(*option == '-') ? MagickTrue : MagickFalse;
-            (void) CopyMagickString(argv[i]+1,"{0}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
             break;
           }
         if (LocaleCompare("format",option+1) == 0)
@@ -807,6 +805,23 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("interpolate",option+1) == 0)
+          {
+            long
+              interpolate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowImportException(OptionError,"MissingArgument",option);
+            interpolate=ParseMagickOption(MagickInterpolateOptions,MagickFalse,
+              argv[i]);
+            if (interpolate < 0)
+              ThrowImportException(OptionError,"UnrecognizedInterpolateMethod",
+                argv[i]);
+            break;
+          }
         ThrowImportException(OptionError,"UnrecognizedOption",option);
       }
       case 'l':
@@ -932,12 +947,31 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
               ThrowImportInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("quantize",option+1) == 0)
+          {
+            long
+              colorspace;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowImportException(OptionError,"MissingArgument",option);
+            colorspace=ParseMagickOption(MagickColorspaceOptions,
+              MagickFalse,argv[i]);
+            if (colorspace < 0)
+              ThrowImportException(OptionError,"UnrecognizedColorspace",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("quiet",option+1) == 0)
           break;
         ThrowImportException(OptionError,"UnrecognizedOption",option);
       }
       case 'r':
       {
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
         if (LocaleCompare("repage",option+1) == 0)
           {
             if (*option == '+')
@@ -1019,7 +1053,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("snaps",option+1) == 0)
           {
-            (void) CopyMagickString(argv[i]+1,"{1}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             i++;
             if (i == (long) argc)
               ThrowImportException(OptionError,"MissingArgument",option);
@@ -1065,6 +1099,15 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
               ThrowImportException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("transparent-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowImportException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("treedepth",option+1) == 0)
           {
             quantize_info->tree_depth=0;
