@@ -132,6 +132,7 @@ static void MontageUsage(void)
       "-help                print program options",
       "-identify            identify the format and characteristics of the image",
       "-interlace type      type of image interlacing scheme",
+      "-interpolate method  pixel color interpolation method",
       "-label name          assign a label to an image",
       "-limit type value    pixel cache resource limit",
       "-log format          format of debugging information",
@@ -144,8 +145,10 @@ static void MontageUsage(void)
       "-pointsize value     font point size",
       "-profile filename    add, delete, or apply an image profile",
       "-quality value       JPEG/MIFF/PNG compression level",
+      "-quantize colorspace reduce colors in this colorspace",
       "-quiet               suppress all error or warning messages",
       "-red-primary point   chromaticity red primary point",
+      "-regard-warnings     pay attention to warning messages",
       "-repage geometry     size and location of an image canvas (operator)",
       "-resize geometry     resize the image",
       "-rotate degrees      apply Paeth rotation to the image",
@@ -163,6 +166,8 @@ static void MontageUsage(void)
       "-tile geometry       number of tiles per row and column",
       "-transform           affine transform image",
       "-transparent color   make this color transparent within the image",
+      "-transparent-color color",
+      "                     transparent color",
       "-treedepth value     color tree depth",
       "-trim                trim image edges",
       "-type type           image type",
@@ -335,10 +340,16 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
         MogrifyImageStack(image_stack[k],MagickTrue,pend);
         for (scene=first_scene; scene <= last_scene ; scene++)
         {
+          char
+            *filename;
+
           /*
             Option is a file name: begin by reading image from specified file.
           */
-          (void) CopyMagickString(image_info->filename,argv[i],MaxTextExtent);
+          filename=argv[i];
+          if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+            filename=argv[++i];
+          (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
           if (first_scene != last_scene)
             {
               char
@@ -497,20 +508,18 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("channel",option+1) == 0)
           {
-            if (*option == '+')
-              {
-                long
-                  channel;
+            long
+              channel;
 
-                i++;
-                if (i == (long) (argc-1))
-                  ThrowMontageException(OptionError,"MissingArgument",option);
-                channel=ParseChannelOption(argv[i]);
-                if (channel < 0)
-                  ThrowMontageException(OptionError,"UnrecognizedChannelType",
-                    argv[i]);
-                break;
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            channel=ParseChannelOption(argv[i]);
+            if (channel < 0)
+              ThrowMontageException(OptionError,"UnrecognizedChannelType",
+                argv[i]);
             break;
           }
         if (LocaleCompare("clone",option+1) == 0)
@@ -560,7 +569,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             long
               colorspace;
 
-            quantize_info.colorspace=UndefinedColorspace;
             if (*option == '+')
               break;
             i++;
@@ -571,12 +579,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             if (colorspace < 0)
               ThrowMontageException(OptionError,"UnrecognizedColorspace",
                 argv[i]);
-            quantize_info.colorspace=(ColorspaceType) colorspace;
-            if (quantize_info.colorspace == GRAYColorspace)
-              {
-                quantize_info.colorspace=GRAYColorspace;
-                quantize_info.number_colors=256;
-              }
             break;
           }
         if (LocaleCompare("comment",option+1) == 0)
@@ -639,18 +641,19 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            LogEventType
-              event_mask;
+            long
+              event;
 
             if (*option == '+')
               break;
             i++;
             if (i == (long) argc)
               ThrowMontageException(OptionError,"MissingArgument",option);
-            event_mask=SetLogEventMask(argv[i]);
-            if (event_mask == UndefinedEvents)
+            event=ParseMagickOption(MagickLogEventOptions,MagickFalse,argv[i]);
+            if (event < 0)
               ThrowMontageException(OptionError,"UnrecognizedEventType",
                 argv[i]);
+            (void) SetLogEventMask(argv[i]);
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -822,7 +825,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("frame",option+1) == 0)
           {
-            (void) CopyMagickString(argv[i]+1,"{1}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             (void) CloneString(&montage_info->frame,(char *) NULL);
             if (*option == '+')
               break;
@@ -920,6 +923,23 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("interpolate",option+1) == 0)
+          {
+            long
+              interpolate;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            interpolate=ParseMagickOption(MagickInterpolateOptions,MagickFalse,
+              argv[i]);
+            if (interpolate < 0)
+              ThrowMontageException(OptionError,"UnrecognizedInterpolateMethod",
+                argv[i]);
+            break;
+          }
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
       }
       case 'l':
@@ -1088,6 +1108,23 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("quantize",option+1) == 0)
+          {
+            long
+              colorspace;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            colorspace=ParseMagickOption(MagickColorspaceOptions,
+              MagickFalse,argv[i]);
+            if (colorspace < 0)
+              ThrowMontageException(OptionError,"UnrecognizedColorspace",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("quiet",option+1) == 0)
           break;
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
@@ -1105,6 +1142,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("regard-warnings",option+1) == 0)
+          break;
         if (LocaleCompare("render",option+1) == 0)
           break;
         if (LocaleCompare("repage",option+1) == 0)
@@ -1184,7 +1223,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
         if (LocaleCompare("shadow",option+1) == 0)
           {
             montage_info->shadow=(*option == '-') ? MagickTrue : MagickFalse;
-            (void) CopyMagickString(argv[i]+1,"{0}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
             break;
           }
         if (LocaleCompare("sharpen",option+1) == 0)
@@ -1271,7 +1310,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("tile",option+1) == 0)
           {
-            (void) CopyMagickString(argv[i]+1,"{1}",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             (void) CloneString(&montage_info->tile,(char *) NULL);
             if (*option == '+')
               break;
@@ -1320,6 +1359,15 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             (void) CloneString(&transparent_color,argv[i]);
             break;
           }
+        if (LocaleCompare("transparent-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("treedepth",option+1) == 0)
           {
             quantize_info.tree_depth=0;
@@ -1429,13 +1477,14 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
   }
   if (k != 0)
     ThrowMontageException(OptionError,"UnbalancedParenthesis",argv[i]);
-  if (i != (long) (argc-1))
+  if (i-- != (long) (argc-1))
     ThrowMontageException(OptionError,"MissingAnImageFilename",argv[i]);
   if (image_stack[k] == (Image *) NULL)
     ThrowMontageException(OptionError,"MissingAnImageFilename",argv[argc-1]);
   MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
   (void) CopyMagickString(montage_info->filename,argv[argc-1],MaxTextExtent);
-  montage_image=MontageImages(image_stack[k],montage_info,exception);
+  montage_image=MontageImageList(image_info,montage_info,image_stack[k],
+    exception);
   if (montage_image == (Image *) NULL)
     status=MagickFalse;
   else
